<?php namespace Stage\Account;

use PDO;
use Exception;
use Stage\Paginator;
use Stage\Stage;
use Stage\Exception\StoredProcedure;

class Api
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $apikey;

    /**
     * @var
     */
    private $domains;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $isPublic = true;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'apikey','domains','active','description','isPublic');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getApiKey()
    {
        return $this->apikey;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getDomains()
    {
        return $this->domains;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function isPublic()
    {
        return $this->isPublic;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setApiKey($apikey)
    {
        $this->apikey = $apikey;
    }

    public function setDomains($name)
    {
        $this->domains = $name;
    }

    public function setDescription($name)
    {
        $this->description = $name;
    }

    public function setActive($active)
    {
        $this->active = (bool) $active;
    }

    public function setIsPublic($public)
    {
        $this->isPublic = (bool) $public;
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function all($idAccount)
    {
        try {
            $parameters = [[$idAccount, PDO::PARAM_STR]];

            $name = "ACCOUNT_APIS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($service) {
                return self::createInstance($service);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query api
     *
     * @param ApiQuery $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public static function query(ApiQuery $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idAccount = $q->idAccount;
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [[$idAccount, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "ACCOUNT_APIS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idAccount, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "ACCOUNT_APIS_NEW";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }
    /**
     * save
     *
     * Save a Api to the database and return a Api instance
     *
     * @return $this
     * @throws Exception
     */
    public function save($idAccount)
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                           [$idAccount, PDO::PARAM_INT],
                           [$this->getApiKey(), PDO::PARAM_STR],
                           [$this->getDescription(), PDO::PARAM_STR],
                           [$this->getDomains(), PDO::PARAM_STR],
                           [$this->getActive(), PDO::PARAM_BOOL],
                           [$this->isPublic(), PDO::PARAM_BOOL]];

            $name = "ACCOUNT_APIS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return Api::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Api
     *
     * @return bool
     * @throws Exception
     */
    public static function delete($idApi, $idAccount)
    {
        try {
            $parameters = [[$idApi, PDO::PARAM_STR],
                           [$idAccount, PDO::PARAM_INT]];


            $name = "ACCOUNT_API";
            $method = "DEL";
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        } catch (StoredProcedure $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * Finds a Api by Id
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
    }

    /**
     * domainExist
     *
     * @param $idAccount
     * @param $domain
     * @return bool
     * @throws Exception
     */
    public static function domainExist($idAccount, $domain, $isPublic = true)
    {
        try {

            // Filter apis public or private
            $apis = array_values(array_filter(self::all($idAccount), function ($api) use ($isPublic) {
                return $api->isPublic() == $isPublic;
            }));

            $domains = array_values(array_filter($apis, function ($api) {
                return $api->getDomains();
            }));

            $exist = false;
            foreach ($domains as $d) {
                $exploded = explode("\n", $d->getDomains());
                if (in_array($domain, $exploded) && !$exist) {
                    $exist = $d;
                }
            }

            return $exist;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($service)
    {
        $instance = new self();
        $instance->setId($service['id_api']);
        $instance->setApiKey($service['api_key']);
        $instance->setDomains($service['domains']);
        $instance->setDescription($service['description']);
        $instance->setActive($service['active']);
        $instance->setIsPublic($service['public']);

        return $instance;
    }
}
