<?php namespace Stage\Account;

use DateTime;
use PDO;
use Exception;
use Stage\Stage;

class Map
{
    const ADS = "ADS";
    const ARCHIVE = "ARCHIVE";
    const ASSETS = "ASSETS";
    const VIDEO = "VIDEO";

    private $idCdn;
    private $idStorage;
    private $storagePath;
    private $cdnPath;

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getStoragePath()
    {
        return $this->storagePath;
    }

    public function getCdnPath()
    {
        return $this->cdnPath;
    }

    /**
     * @param $id
     * @param $field
     * @return Map
     * @throws Exception
     */
    public static function getStorage($id, $field)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT],
                           [$field, PDO::PARAM_STR]];

            $name = "MAP_STORAGE";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $map = new self();
            if (isset($data[0]) &&  isset($data[0]['id_storage'])) {
                $map->idStorage = $data[0]['id_storage'];
                $map->storagePath = $data[0]['storage_path'];
            }
            return $map;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $id
     * @param $field
     * @return Map
     * @throws Exception
     */
    public static function getCdn($id, $field)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT],
                           [$field, PDO::PARAM_STR]];

            $name = "MAP_CDN";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $map = new self();
            if (isset($data[0]) &&  isset($data[0]['cdn_path'])) {
                $map->idCdn = $data[0]['id_cdn'];
                $map->cdnPath = $data[0]['cdn_path'];
            }
            return $map;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idAccount
     * @param $idCdn
     * @param $idStorage
     * @return array
     * @throws Exception
     */
    public static function getVideoAbrSettings($idAccount, $idCdn, $idStorage)
    {
        try {
            $parameters = [[$idAccount, PDO::PARAM_INT],
                           [$idCdn, PDO::PARAM_INT],
                           [$idStorage, PDO::PARAM_INT]];

            $name = "ACCOUNT_VIDEO_ABR_SETTINGS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idAccount
     * @param $idCdn
     * @param $idStorage
     * @return array
     * @throws Exception
     */
    public static function getAdsAbrSettings($idAccount, $idCdn, $idStorage)
    {
        try {
            $parameters = [[$idAccount, PDO::PARAM_INT],
                           [$idCdn, PDO::PARAM_INT],
                           [$idStorage, PDO::PARAM_INT]];

            $name = "ACCOUNT_ADS_ABR_SETTINGS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public static function parts(DateTime $date = null)
    {
        if (!$date) {
            $date = new DateTime();
        }

        $year = $date->format('Y');
        $month = $date->format('m');
        $day = $date->format('d');
        return [$year, $month, $day];
    }

    public static function partsToString($parts)
    {
        return implode(DIRECTORY_SEPARATOR, $parts);
    }
}
