<?php namespace Stage\Account;

use PDO;
use Exception;
use Stage\Stage;

class Service
{
    const ASSETS        = "assets";
    const POSTS         = "posts";
    const ACCOUNTS      = "accounts";
    const VIDEO         = "video";
    const LIVE          = "live";
    const ANALYTICS     = "analytics";
    const LEADS         = "leads";
    const PLAYERS       = "players";
    const ARCHIVE       = "archive";
    const ADS           = "ads";
    const SUPPORT       = "support";

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $active;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','description','active');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function __call($method, $args)
    {
        if (isset($this->$method)) {
            $func = $this->$method;
            return call_user_func_array($func, $args);
        }
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getActive()
    {
        return $this->active;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = trim($name);
    }

    public function setDescription($desc)
    {
        $this->description = trim($desc);
    }

    public function setActive($active)
    {
        $this->active = (bool) $active;
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "SERVICE";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (sizeof($data) <= 0) {
                throw new Exception("The Service id $id not exists.");
            }

            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function enable($type)
    {
        if ($type instanceof User) {
            $this->user = $type;
            $this->activateFlag = true;
            return $this;
        } elseif ($type instanceof Account) {
            try {
                $idAccount = $type->getId();
                $parameters = [[$idAccount, PDO::PARAM_INT],
                              [$this->getId(), PDO::PARAM_INT]];

                $name = "ACCOUNT_SERVICE";
                $method = "SET";
                $data = Stage::esp($parameters, $name, $method);

                // Set this service as active
                $this->setActive(true);

                return $data;
            } catch (Exception $e) {
                throw $e;
            }
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function disable($type)
    {
        if ($type instanceof User) {
            $this->user = $type;
            $this->activateFlag = false;
            return $this;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function in(Account $account)
    {
        try {
            $idUser = $this->user->getId();
            $idService = $this->getId();
            $idAccount = $account->getId();
            $parameters = [[$idUser, PDO::PARAM_INT],
                           [$idAccount, PDO::PARAM_INT],
                           [$idService, PDO::PARAM_INT]];

            $name = "ACCOUNT_USER_SERVICE";
            $method = $this->activateFlag === true ? "SET" : "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Service id $idService not found in account id $idAccount", $data[0]['return_code']);
            }/* else if( isset($data[0]['return_code']) && $data[0]['return_code'] == "0" ) {
                return self::createInstance($data[0]);
            }*/
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function all($idAccount)
    {
        try {
            $parameters = [[$idAccount, PDO::PARAM_INT]];

            $name = "SERVICES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($service) {
                return self::createInstance($service);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($service)
    {
        $instance = new self();
        $instance->setId($service['id_service']);
        $instance->setName(html_entity_decode($service['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setDescription(html_entity_decode($service['description'], ENT_COMPAT, 'UTF-8'));
        if (isset($service['enabled'])) {
            $instance->setActive($service['enabled']);
        }

        return $instance;
    }
}
