<?php namespace Stage\Account;

use PDO;
use Stage\Paginator;
use Stage\Stage;
use Exception;
use Stage\Role;

class Team
{
    const ID = "1";

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idAccount;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $position = 0;

    /**
     * @var
     */
    private $total = 0;

    /**
     * @var
     */
    private $active = false;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idAccount', 'name', 'nameStripped', 'description', 'position', 'total', 'active');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array();

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdAccount()
    {
        return $this->idAccount;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdAccount($id)
    {
        $this->idAccount = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($nameStripped)
    {
        $this->nameStripped = $nameStripped;
    }

    public function setDescription($desc)
    {
        $this->description = $desc;
    }

    public function setPosition($pos)
    {
        $this->position = $pos;
    }

    public function setTotal($total)
    {
        $this->total = $total;
    }

    public function setActive($active)
    {
        $this->active = $active == "1" || $active == true ? true : false;
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getIdAccount(), PDO::PARAM_INT],
            [$this->getId(), PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_STR],
            [$this->getNameStripped(), PDO::PARAM_STR],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getPosition(), PDO::PARAM_INT],
            [$this->getActive(), PDO::PARAM_INT]];

        // Make a request to Database
        try {
            $name = "ACCOUNT_TEAMS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            $idTeam = $data[0]['id_team'];
            if ($idTeam) { // Success
                $this->setId($idTeam);
                $this->setName(html_entity_decode($this->getName(), ENT_COMPAT, 'UTF-8'));
                $this->setDescription(html_entity_decode($this->getDescription(), ENT_COMPAT, 'UTF-8'));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query transfers
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function query(QueryTeam $q, $count = true, $page = 1, $perPage = 15)
    {

        $method = "GET";
        $idAccount = $q->idAccount;
        $field = $q->field;
        $term = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [
                    [$idAccount, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$term, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "ACCOUNT_TEAMS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idAccount, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "ACCOUNT_TEAMS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addUser
     *
     * @throws Exception
     */
    public static function addUser($idAccount, $idTeam, $idUser, $position, $active)
    {
        try {
            $parameters = [[$idAccount, PDO::PARAM_INT],
                [$idTeam, PDO::PARAM_INT],
                [$idUser, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT],
                [$active, PDO::PARAM_BOOL]];

            $name = "ACCOUNT_TEAMS_USERS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['id_user']) && $data[0]['id_user'] > 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addUser
     *
     * @throws Exception
     */
    public static function removeUser($idAccount, $idTeam, $idUser)
    {
        try {
            $parameters = [
                [$idAccount, PDO::PARAM_INT],
                [$idTeam, PDO::PARAM_INT],
                [$idUser, PDO::PARAM_INT]
            ];

            $name = "ACCOUNT_TEAMS_USER";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query transfers
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function users(QueryTeamUsers $q, $count = true, $page = 1, $perPage = 15)
    {

        $method = "GET";
        $idAccount = $q->idAccount;
        $idTeam = $q->idTeam;
        $field = $q->field;
        $term = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [
                    [$idAccount, PDO::PARAM_INT],
                    [$idTeam, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$term, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "ACCOUNT_TEAM_USERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idAccount, PDO::PARAM_INT],
                [$idTeam, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "ACCOUNT_TEAM_USERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return User::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Team
     *
     * @param $idAccount
     * @param $idTeam
     * @return bool
     * @throws Exception
     */
    public static function delete($idAccount, $idTeam)
    {
        $name = "ACCOUNT_TEAM";
        $parameters = [[$idAccount, PDO::PARAM_INT],
            [$idTeam, PDO::PARAM_INT]];
        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachCustomFieldVariant
     *
     * @param $idAccount
     * @param $idTeam
     * @param $name
     * @param $email
     * @param $position
     * @param $active
     * @return UserGuest
     * @throws Exception
     */
    public static function attachGuestUser($idAccount, $idTeam, $name, $email, $position, $active)
    {
        $spname = "ACCOUNT_TEAMS_GUESTS_USERS";
        $method = "SET";
        $parameters = [[$idAccount, PDO::PARAM_INT],
            [$idTeam, PDO::PARAM_INT],
            [$name, PDO::PARAM_STR],
            [$email, PDO::PARAM_STR],
            [$position, PDO::PARAM_INT],
            [$active, PDO::PARAM_INT]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The idAccount, idTeam not exists", $data[0]['return_code']);
            } else {
                return UserGuest::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachGuestUser
     *
     * @param $idAccount
     * @param $idTeam
     * @param $idGuest
     * @return bool
     * @throws Exception
     */
    public static function detachGuestUser($idAccount, $idTeam, $idGuest)
    {
        $name = "ACCOUNT_TEAMS_GUEST_USER";
        $method = "DEL";
        $parameters = [[$idAccount, PDO::PARAM_INT],
            [$idTeam, PDO::PARAM_INT],
            [$idGuest, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * guestUsers
     *
     * Query guestUsers
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function guestUsers(QueryTeamGuestUsers $q, $count = true, $page = 1, $perPage = 15)
    {

        $method = "GET";
        $idAccount = $q->idAccount;
        $idTeam = $q->idTeam;
        $field = $q->field;
        $term = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [
                    [$idAccount, PDO::PARAM_INT],
                    [$idTeam, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$term, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],

                ];

                $name = "ACCOUNT_TEAM_GUESTS_USERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idAccount, PDO::PARAM_INT],
                [$idTeam, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "ACCOUNT_TEAM_GUESTS_USERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return UserGuest::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "role" && $this->$attribute instanceof Role) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute != "role") {
                $arr[$attribute] = $attribute == "companyName" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return $this
     */
    public static function createInstance($data)
    {
        $accInstance = new self();
        $accInstance->setId($data['id_team']);
        $accInstance->setIdAccount($data['id_account']);
        $accInstance->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $accInstance->setNameStripped($data['name_stripped']);
        $accInstance->setDescription(html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8'));
        $accInstance->setTotal($data['total']);
        $accInstance->setPosition($data['position']);
        $accInstance->setActive($data['active']);
        return $accInstance;
    }

    /**
     * __toString
     *
     * @return string
     */
    public function __toString()
    {
        return "My id is: {$this->getId()}, My Token is: {$this->getToken()}, My Email is: {$this->getEmail()}\n";
    }
}
