<?php namespace Stage\Account;

use PDO;
use Stage\Paginator;
use Stage\Stage;
use Exception;
use Stage\Account\Account;
use Stage\Role;
use Stage\Account\Service;

class User
{
    const NIL = 0;
    const ID = '1';
    const EMAIL = '2';
    const TOKEN = '3';

    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $idTeam;

    /**
     * @var
     */
    private $token;

    /**
     * @var
     */
    private $firstName;

    /**
     * @var
     */
    private $lastName;

    /**
     * @var
     */
    private $email;

    /**
     * @var
     */
    private $password;

    /**
     * @var
     */
    private $verified;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $lang;

    /**
     * @var
     */
    private $role;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idTeam', 'token', 'firstName', 'lastName', 'email', 'password', 'active', 'verified', 'lang');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('password');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    /**
     * @var
     */
    private static $instance;

    public function __construct()
    {
    }

    public static function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdTeam()
    {
        return $this->idTeam;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getFirstName()
    {
        return $this->firstName;
    }

    public function getLastName()
    {
        return $this->lastName;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getPassword()
    {
        return $this->password;
    }

    public function getVerified()
    {
        return $this->verified;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function getLang()
    {
        return $this->lang;
    }

    public function getRole()
    {
        return $this->role;
    }

    private function setId($id)
    {
        if (!$this->getId()) {
            $this->id = (int)$id;
        }
    }

    private function setIdTeam($id)
    {
        $this->idTeam = (int)$id;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setFirstName($firstname)
    {
        $this->firstName = $firstname;
    }

    public function setLastName($lastName)
    {
        $this->lastName = $lastName;
    }

    public function setEmail($email)
    {
        $this->email = $email;
    }

    public function setPassword($password)
    {
        $this->password = $password;
    }

    public function setVerified($verified)
    {
        $this->verified = $verified == "1" || $verified == true ? true : false;
    }

    public function setActive($active)
    {
        $this->active = $active == "1" || $active == true ? true : false;
    }

    public function setLang($lang)
    {
        $this->lang = $lang;
    }

    public function setRole(Role $role)
    {
        $this->role = $role;
    }

    /**
     * services
     *
     * Returns all the services of the user
     *
     * @return $this
     * @throws Exception
     */
    public function services()
    {
        return $this;
    }

    /**
     * @param \Stage\Account $account
     * @return array
     * @throws Exception
     */
    public function in(Account $account)
    {
        $id_user = $this->getId();
        $parameters = [[$account->getId(), PDO::PARAM_INT],
            [$id_user, PDO::PARAM_INT]];

        try {
            $name = "ACCOUNT_USERS_SERVICES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($service) {
                return Service::createInstance($service);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * getAccounts
     *
     * Returns all the accounts that the user is inserted in
     *
     * @return array
     * @throws Exception
     */
    public function getAccounts()
    {
        if (!$this->getId()) {
            throw new Exception("This instance don't has a valid user id defined.");
        }

        $id_user = $this->getId();
        $parameters = [[$id_user, PDO::PARAM_INT]];

        try {
            $name = "USER_ACCOUNTS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($acc) {
                return Account::createInstance($acc);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a user to the database and return a User instance
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getToken(), PDO::PARAM_STR],
            [$this->getFirstName(), PDO::PARAM_STR],
            [$this->getLastName(), PDO::PARAM_STR],
            [$this->getEmail(), PDO::PARAM_INT],
            [$this->getPassword(), PDO::PARAM_STR],
            [$this->getVerified(), PDO::PARAM_INT],
            [$this->getActive(), PDO::PARAM_INT],
            [$this->getLang(), PDO::PARAM_STR]];

        try {
            $name = "USER";
            $method = "SET";
            $connectionName = "default";
            $data = Stage::esp($parameters, $name, $method, $connectionName);
            $return_code = $data[0]['return_code'];

            if ($return_code == "0") { // Success
                $this->setId($data[0]['id_user']);
            } else {
                throw new Exception("The email already exists", $return_code);
            }

            // Fire an event
            if (!$this->getId()) {
                \Stage\Events\Event::fire("user.created", $this);
            } else {
                \Stage\Events\Event::fire("user.updated", $this);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * delete
     *
     */
    public function delete()
    {
        // TODO
    }

    /**
     * find
     *
     * @param $value
     * @param string $field
     * @return User
     * @throws Exception
     */
    public static function find($value, $field = self::ID)
    {
        $parameters = [[$field, PDO::PARAM_STR],
            [$value, PDO::PARAM_STR]];

        try {
            $name = "USER";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        if (sizeof($data) > 0) {
            $userInstance = self::createInstance($data[0]);
            return $userInstance;
        } else {
            throw new Exception("User not found width id $value");
        }
    }

    /**
     * query
     *
     * Query users
     *
     * @param UserQuery $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public static function query(UserQuery $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "GET_USERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "GET_USERS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * accounts
     *
     * Query accounts
     *
     * @param AccountQuery $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public static function accounts(AccountQuery $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idUser = $q->idUser;
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [[$idUser, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "USER_ACCOUNTS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idUser, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "USER_ACCOUNTS_NEW";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return Account::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * authenticate
     *
     * @param $email
     * @param $password
     * @return array
     * @throws Exception
     */
    public static function authenticate($email, $hashedPassword)
    {
        try {
            $user = self::find($email, static::EMAIL);
        } catch (Exception $e) {
            throw $e;
        }

        // Binary string safe compare
        if (strcmp($user->getPassword(), $hashedPassword) !== 0) {
            throw new Exception("The password you entered is wrong.");
        }

        return $user;
    }

    /**
     * all
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $parameters = [[static::NIL, PDO::PARAM_STR],
            ["", PDO::PARAM_STR]];

        try {
            $name = "USER";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        $temp = [];
        foreach ($data as $user) {
            $userInstance = self::createInstance($user);
            array_push($temp, $userInstance);
        }

        return $temp;
    }

    /**
     * isVerified
     *
     * @return mixed
     */
    public function isVerified()
    {
        // Check if the user is verified or not
        return $this->getVerified();
    }

    /**
     * attachTo
     *
     * Associate a user to an account
     *
     * @param \Stage\Account $account
     */
    public function associateTo(Account $account, Role $role)
    {
        if (!($account instanceof Account)) {
            throw new Exception("You are not passing an instance of an Account");
        }

        if (!($role instanceof Role)) {
            throw new Exception("You are not passing an instance of Role");
        }

        $id_account = $account->getId();

        $parameters = [[$this->getId(), PDO::PARAM_STR],
            [$id_account, PDO::PARAM_STR],
            [$role->getId(), PDO::PARAM_STR]];

        try {
            $name = "USER_ACCOUNT";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            $return_code = $data[0]['return_code'];

            if ($return_code == "0") { // Success
                return $this;
            } elseif ($return_code == "-3") {
                throw new Exception("The user has already associated to the account $id_account", $return_code);
            } elseif ($return_code == "-2") {
                throw new Exception("The id of the user or id of the account was not found", $return_code);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachTo
     *
     * Dissociate a user of an account
     *
     * @param \Stage\Account $account
     */
    public function disassociateFrom(Account $account)
    {
        if (!($account instanceof Account)) {
            throw new Exception("You are not passing an instance of an Account");
        }

        $id_account = $account->getId();

        $parameters = [[$this->getId(), PDO::PARAM_STR],
            [$id_account, PDO::PARAM_STR]];

        try {
            $name = "USER_ACCOUNT";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "firstName" || $attribute == "lastName" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($user)
    {
        $userInstance = new self();
        $userInstance->setId($user['id_user']);
        if(isset($user['id_team'])){
            $userInstance->setIdTeam($user['id_team']);
        }
        $userInstance->setToken($user['user_token']);
        $userInstance->setFirstName(html_entity_decode($user['firstname'], ENT_COMPAT, 'UTF-8'));
        $userInstance->setLastName(html_entity_decode($user['lastname'], ENT_COMPAT, 'UTF-8'));
        $userInstance->setEmail($user['email']);
        $userInstance->setPassword(isset($user['password']) ? $user['password'] : null);
        $userInstance->setVerified($user['verified']);
        $userInstance->setActive($user['active']);
        $userInstance->setLang($user['lang']);

        return $userInstance;
    }

    /**
     * __toString
     *
     * @return string
     */
    public function __toString()
    {
        return "My id is: {$this->getId()}, My Token is: {$this->getToken()}, My Email is: {$this->getEmail()}\n";
    }
}
