<?php namespace Stage\Account;

use DateTime;
use Exception;
use Stage\Helper;
use Stage\Role;

class UserGuest
{
    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $idTeam;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $email;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idTeam', 'name', 'email', 'position', 'active', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('password');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    /**
     * @var
     */
    private static $instance;

    public function __construct()
    {
    }

    public static function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdTeam()
    {
        return $this->idTeam;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getActive()
    {
        return $this->active;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    private function setIdTeam($id)
    {
        $this->idTeam = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setPosition($position)
    {
        $this->position = (int)$position;
    }

    public function setEmail($email)
    {
        $this->email = $email;
    }

    public function setActive($active)
    {
        $this->active = $active == "1" || $active == true ? true : false;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * services
     *
     * Returns all the services of the user
     *
     * @return $this
     * @throws Exception
     */
    public function services()
    {
        return $this;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "firstName" || $attribute == "lastName" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $user
     */
    public static function createInstance($user)
    {
        $userInstance = new self();
        $userInstance->setId($user['id_guest']);
        $userInstance->setIdTeam($user['id_team']);
        $userInstance->setName($user['name']);
        $userInstance->setEmail($user['email']);
        if(isset($user['position'])){
            $userInstance->setPosition($user['position']);
        }
        $userInstance->setActive($user['active']);

        if (Helper::isValidDate($user['register_date'], "Y-m-d H:i:s")) {
            $userInstance->setRegisterDate(new DateTime($user['register_date']));
        }

        return $userInstance;
    }

    /**
     * __toString
     *
     * @return string
     */
    public function __toString()
    {
        return "My id is: {$this->getId()}, My Email is: {$this->getEmail()}\n";
    }
}
