<?php namespace Stage\Ad;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class Action
{

    /**
     * @var
     */
    public $id = -1;

    /**
     *
     * @var string
     */
    private $name;

    /**
     *
     * @var string
     */
    private $slug;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','name','slug');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getSlug()
    {
        return $this->slug;
    }

    public function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setSlug($slug)
    {
        $this->slug = $slug;
    }

    public function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    /**
     * @param int $field
     * @return array
     * @throws Exception
     */
    public static function all($field = 0)
    {
        try {
            $name = "ADS_ACTIONS_OPTIONS";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($action) {
                return self::createInstance($action);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Action
     */
    public static function createInstance($data)
    {
        $inst = new self();
        $inst->setId($data['id_action']);
        $inst->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $inst->setSlug($data['name_stripped']);

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $inst->setRegisterDate(new DateTime($data['register_date']));
        }

        return $inst;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
