<?php namespace Stage\Ad\Campaign;

use PDO;
use DateTime;
use Exception;
use Stage\Ad\Campaign\Group\Group;
use Stage\Stage;
use Stage\Helper;
use Stage\Ad\Tag\Tag;

class Campaign
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $beginDate;

    /**
     * @var
     */
    private $endDate = 0;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $hasAdVideos = false;

    /**
     * @var
     */
    private $hasExternalTag = false;

    /**
     * @var
     */
    private $tags = [];

    /**
     * @var
     */
    private $state;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'hasAdVideos', 'hasExternalTag','state', 'beginDate', 'endDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function hasAdVideos()
    {
        return $this->hasAdVideos;
    }

    public function hasExternalTag()
    {
        return $this->hasExternalTag;
    }

    public function getState()
    {
        return $this->state;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setBeginDate($date)
    {
        if ($date==null) {
            $this->beginDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        if ($date==null) {
            $this->endDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    public function setHasAdVideos($has)
    {
        $this->hasAdVideos = (bool) $has;
    }

    public function setHasExternalTag($has)
    {
        $this->hasExternalTag = (bool) $has;
    }

    public function setState($state)
    {
        $this->state = (int) $state;
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "ADS_CAMPAIGNS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * dates
     *
     * @param int $idQuery Se se passar o id album então vai buscar as dates dentro de um album
     * @return array
     */
    public static function dates(Query $q)
    {
        $field          = $q->by;
        $sndField       = $q->sndBy;
        $trdField       = $q->trdBy;
        $term           = $q->term;
        $filter         = $q->filter;
        $sndFilter      = $q->sndFilter;
        $trdFilter      = $q->trdFilter;
        $limit_sys_a    = $q->fromSize;
        $limit_sys_b    = $q->toSize;

        try {
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$sndField, PDO::PARAM_INT],
                            [$trdField, PDO::PARAM_INT],
                            [$term, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$sndFilter, PDO::PARAM_INT],
                            [$trdFilter, PDO::PARAM_INT],
                            [$limit_sys_a, PDO::PARAM_INT],
                            [$limit_sys_b, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_DATES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * @return $this
     * @throws Exception
     */
    public static function all()
    {
        try {
            $query = new Query();
            $query->by = Query::ALL;
            $query->from = 0;
            $query->to = -1;

            return self::query($query);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * Find a Asset
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ADS_CAMPAIGN";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Campaign id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Campaign
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
                        [$this->getName(), PDO::PARAM_STR],
                        [$this->getBeginDate(), PDO::PARAM_STR],
                        [$this->getEndDate(), PDO::PARAM_STR]];

        try {
            $name = "ADS_CAMPAIGNS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_campaign']);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Campain
     *
     * @throws Exception
     */
    public function delete()
    {
        $name = "ADS_CAMPAIGN";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * duplicate
     *
     * @return Campaign
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "ADS_CLONE_CAMPAIGN";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Campaigns by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field          = $q->by;
        $sndField       = $q->sndBy;
        $trdField       = $q->trdBy;
        $term           = $q->term;
        $filter         = $q->filter;
        $sndFilter      = $q->sndFilter;
        $trdFilter      = $q->trdFilter;
        $limit_sys_a    = $q->fromSize;
        $limit_sys_b    = $q->toSize;
        $from           = $q->from;
        $to             = $q->to;
        $orderBy        = $q->orderBy;
        $order          = $q->order;

        try {
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$sndField, PDO::PARAM_INT],
                            [$trdField, PDO::PARAM_INT],
                            [$term, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$sndFilter, PDO::PARAM_INT],
                            [$trdFilter, PDO::PARAM_INT],
                            [$limit_sys_a, PDO::PARAM_INT],
                            [$limit_sys_b, PDO::PARAM_INT],
                            [$from, PDO::PARAM_INT],
                            [$to, PDO::PARAM_INT],
                            [$orderBy, PDO::PARAM_INT],
                            [$order, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * @param Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        $field          = $q->by;
        $sndField       = $q->sndBy;
        $trdField       = $q->trdBy;
        $term           = $q->term;
        $filter         = $q->filter;
        $sndFilter      = $q->sndFilter;
        $trdFilter      = $q->trdFilter;
        $from           = $q->from;
        $to             = $q->to;
        $limit_sys_a    = $q->fromSize;
        $limit_sys_b    = $q->toSize;

        try {
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$sndField, PDO::PARAM_INT],
                            [$trdField, PDO::PARAM_INT],
                            [$term, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$sndFilter, PDO::PARAM_INT],
                            [$trdFilter, PDO::PARAM_INT],
                            [$limit_sys_a, PDO::PARAM_INT],
                            [$limit_sys_b, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag $tag
     * @throws Exception
     */
    public function addTag(Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [ [$idTag, PDO::PARAM_INT],
                            [$this->getId(), PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_TO_TAGS";
            $method = "SET";
            $result = Stage::esp($parameters, $name, $method);
            $this->tags = $this->tags();

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the Campaign tags
     * @return array
     * @throws Exception
     */
    public function removeAllTags(Query $q = null)
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * groups
     *
     * @return array
     * @throws Exception
     */
    public function groups()
    {
        try {
            return Group::inCampaign($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * groupVideos
     *
     * @param $idGroup
     * @return array
     * @throws Exception
     */
    public function groupVideos(Group $group)
    {
        try {
            return Group::videos($this->getId(), $group->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * groupCountries
     *
     * @param $idGroup
     * @return array
     * @throws Exception
     */
    public function groupCountries(Group $group)
    {
        try {
            return Group::countries($this->getId(), $group->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * groupAdTags
     *
     * @param $idGroup
     * @return array
     * @throws Exception
     */
    public function groupAdTags(Group $group)
    {
        try {
            return Group::adTags($this->getId(), $group->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllGroups
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllGroups()
    {
        try {
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            $name = "ADS_CAMPAIGN_GROUPS";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllGroupCountries
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllGroupCountries()
    {
        try {
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            $name = "ADS_CAMPAIGNS_GROUPS_COUNTRIES";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllGroupAdVideos
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllGroupAdVideos()
    {
        try {
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            $name = "ADS_CAMPAIGNS_GROUPS_ADS_VIDEOS";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllGroupAdTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllGroupAdTags()
    {
        try {
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            $name = "ADS_CAMPAIGNS_GROUPS_EXTERNAL_AD_TAGS";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function removeAllGroupMidRollOptions()
    {
        try {
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            $name = "ADS_CAMPAIGNS_GROUPS_MIDROLL_OPTIONS";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeThisCampaignFromAllComposes
     *
     * Apaga uma campanha de todos os composes
     * @return $this
     * @throws Exception
     */
    public function removeThisCampaignFromAllComposes()
    {
        try {
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            $name = "ADS_CAMPAIGNS_VIDEO_COMPOSED";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idCampaign
     * @return $this
     * @throws Exception
     */
    public function removeAllAdsVideosWeeklySchedule()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_GROUPS_ADS_VIDEOS_WEEKLY_SCHEDULE";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idCampaign
     * @return $this
     * @throws Exception
     */
    public function removeAllFrequencyCapOptions()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_GROUPS_FREQUENCY_CAP_OPTIONS";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the Campaign tags
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        try {
            $q = new Query();
            return self::allTagsRelatedWithCampaigns($this->getId(), $q);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * allTagsRelatedWithCampaigns
     *
     * Note: Passing $idCampaign -1 will return all the tags related with campaigns
     * @param $idCampaign
     * @param Query $q
     * @return array
     * @throws Exception
     */
    public static function allTagsRelatedWithCampaigns($idCampaign, Query $q)
    {
        try {
            $field          = $q->by;
            $sndField       = $q->sndBy;
            $trdField       = $q->trdBy;
            $term           = $q->term;
            $filter         = $q->filter;
            $sndFilter      = $q->sndFilter;
            $trdFilter      = $q->trdFilter;
            $limit_sys_a    = $q->fromSize;
            $limit_sys_b    = $q->toSize;

            $parameters = [[$idCampaign, PDO::PARAM_INT],
                            [$field, PDO::PARAM_INT],
                            [$sndField, PDO::PARAM_INT],
                            [$trdField, PDO::PARAM_INT],
                            [$term, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$sndFilter, PDO::PARAM_INT],
                            [$trdFilter, PDO::PARAM_INT],
                            [$limit_sys_a, PDO::PARAM_INT],
                            [$limit_sys_b, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * composes
     *
     * @return array
     * @throws Exception
     */
    public function composes()
    {
        try {
            $idCampaign = $this->getId();
            $parameters = [[$idCampaign, PDO::PARAM_INT]];
            $name = "ADS_VIDEO_COMPOSES_BY_CAMPAIGN";
            $method = "GET";
            $composes = Stage::esp($parameters, $name, $method);

            if (isset($composes[0]['return_code']) && ($composes[0]['return_code']) == "-2") {
                return [];
            } else {
                return  array_map(function ($compose) {
                    return $compose['id_composed'];
                }, $composes);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "beginDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($campaign)
    {
        $assetInstance = new self();
        $assetInstance->setId($campaign['id_campaign']);
        $assetInstance->setName(html_entity_decode($campaign['name'], ENT_COMPAT, 'UTF-8'));

        if (Helper::isValidDate($campaign['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($campaign['register_date']));
        }

        if (Helper::isValidDate($campaign['begin_date'], "Y-m-d H:i:s")) {
            $assetInstance->setBeginDate(new DateTime($campaign['begin_date']));
        }

        if (Helper::isValidDate($campaign['end_date'], "Y-m-d H:i:s")) {
            $assetInstance->setEndDate(new DateTime($campaign['end_date']));
        }

        $assetInstance->setHasAdVideos($campaign['has_ad_videos']);
        $assetInstance->setHasExternalTag($campaign['has_external_tag']);
        $assetInstance->setState($campaign['state']);

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
