<?php namespace Stage\Ad\Campaign\Code;

use Stage\Ad\Campaign\Exceptions\CampaignNotFound;
use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;

class Code
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idCampaign;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $code;

    /**
     * @var
     */
    private $ratio;

    /**
     * @var
     */
    private $autoplay;

    /**
     * @var
     */
    private $startExpanded;

    /**
     * @var
     */
    private $endCollapse;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idCampaign', 'name', 'code', 'ratio','autoplay', 'startExpanded', 'endCollapse', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdCamapaign($id)
    {
        $this->idCampaign = (int) $id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getCode()
    {
        return $this->code;
    }

    public function getRatio()
    {
        return $this->ratio;
    }

    public function autoplay()
    {
        return (boolean) $this->autoplay;
    }

    public function startExpanded()
    {
        return (boolean) $this->startExpanded;
    }

    public function endCollapse()
    {
        return (boolean) $this->endCollapse;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdCampaign($id)
    {
        $this->idCampaign = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setCode($code)
    {
        $this->code = $code;
    }

    public function setRatio($ratio)
    {
        $this->ratio = $ratio;
    }

    public function setAutoplay($autoplay)
    {
        $this->autoplay = (boolean) $autoplay;
    }

    public function setStartExpanded($expanded)
    {
        $this->startExpanded = (boolean) $expanded;
    }

    public function setEndCollapse($endCollapse)
    {
        $this->endCollapse = (boolean) $endCollapse;
    }

    public function setRegisterDate(DateTime $date)
    {
        if ($date==null) {
            $this->registerDate = 0;
        } else {
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * query
     *
     * @return array
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        $field          = $q->by;
        $term           = $q->term;
        $filter         = $q->filter;
        $from           = $q->from;
        $to             = $q->to;

        try {
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$term, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGN_CODES_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * @return array
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field          = $q->by;
        $term           = $q->term;
        $filter         = $q->filter;
        $from           = $q->from;
        $to             = $q->to;

        try {
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$term, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$from, PDO::PARAM_INT],
                            [$to, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGN_CODES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * @return $this
     * @throws Exception
     */
    public static function all()
    {
        try {
            $query = new Query();
            $query->by = Query::ALL;
            $query->from = 0;
            $query->to = -1;

            return self::query($query);
        } catch (Exception $e) {
            throw $e;
        }
    }

//    /**
//     * find
//     *
//     * Find a Asset
//     *
//     * @return $this
//     * @throws Exception
//     */
//    public static function find($id)
//    {
//        // TODO
//        try {
//
//            $name = "ADS_CAMPAIGN";
//            $method = "GET";
//            $parameters = [[$id, PDO::PARAM_INT]];
//
//            $data = Stage::esp($parameters, $name, $method);
//
//            if( isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
//                throw new Exception("The Campaign id $id not exists.", $data[0]['return_code']);
//            } else {
//                return self::createInstance($data[0]);
//            }
//
//        } catch(Exception $e){
//            throw $e;
//        }
//
//    }

    /**
     * save
     *
     * Save a code
     *
     * @return $this
     * @throws Exception
     */
    public function save($idCampaign)
    {
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
                        [$idCampaign, PDO::PARAM_INT],
                        [$this->getName(), PDO::PARAM_STR],
                        [$this->getCode(), PDO::PARAM_STR],
                        [$this->getRatio(), PDO::PARAM_STR],
                        [$this->autoplay(), PDO::PARAM_BOOL],
                        [$this->startExpanded(), PDO::PARAM_BOOL],
                        [$this->endCollapse(), PDO::PARAM_BOOL]];

        try {
            $name = "ADS_CAMPAIGN_CODE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_code'])) {
                $this->setId($data[0]['id_code']);
                return $this;
            } elseif (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new CampaignNotFound("Campaign $idCampaign not found", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * allOptions
     * Get all available options
     * @param $field
     * @return array
     * @throws Exception
     */
    public static function allOptions($field = null)
    {
        $name = "ADS_CAMPAIGNS_CODES_OPTIONS";
        $parameters = [[$field, PDO::PARAM_INT]];
        $method = "GET";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param null $idCode
     * @param null $idCampaign
     * @return array
     * @throws Exception
     */
    public static function optionsBy($idCode, $idCampaign)
    {
        $name = "ADS_CAMPAIGNS_CODES_OPTIONS_RELATED";
        $parameters = [[$idCode, PDO::PARAM_INT],
                       [$idCampaign, PDO::PARAM_INT]];
        $method = "GET";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idCode
     * @param $idCampaign
     * @param $idOption
     * @param $optionValue
     * @return array
     * @throws Exception
     */
    public static function setOptionsBy($idCode, $idCampaign, $idOption, $optionValue)
    {
        // -3 quer dizer que a (idOption, optionValue)Q ja existe
        $name = "ADS_CAMPAIGN_OPTIONS_TO_CODES";
        $parameters = [[$idCode, PDO::PARAM_INT],
                       [$idCampaign, PDO::PARAM_INT],
                       [$idOption, PDO::PARAM_INT],
                       [$optionValue, PDO::PARAM_INT]];

        $method = "SET";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idCode
     * @param $idCampaign
     * @return mixed
     * @throws Exception
     */
    public static function deleteAllOptionsFrom($idCode, $idCampaign)
    {
        $name = "ADS_CAMPAIGN_CODES_OPTIONS";
        $parameters = [[$idCode, PDO::PARAM_INT],
                       [$idCampaign, PDO::PARAM_INT]];
        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idCode
     * @param $idCampaign
     * @return Code
     * @throws Exception
     */
    public static function duplicate($idCode, $idCampaign, $code)
    {
        try {
            $parameters = [[$idCode, PDO::PARAM_INT],
                           [$idCampaign, PDO::PARAM_INT],
                           [$code, PDO::PARAM_STR]];

            $name = "ADS_CAMPAIGN_CODE_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_code'])) {
                return self::createInstance($data[0]);
            } else {
                return false;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Code from a Campaign
     *
     * @throws Exception
     */
    public static function delete($idCode, $idCampaign)
    {
        $name = "ADS_CAMPAIGN_CODE";
        $parameters = [[$idCode, PDO::PARAM_INT],
                       [$idCampaign, PDO::PARAM_INT]];
        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($campaign)
    {
        $assetInstance = new self();
        $assetInstance->setId($campaign['id_code']);
        $assetInstance->setIdCampaign($campaign['id_campaign']);
        $assetInstance->setName(html_entity_decode($campaign['name'], ENT_COMPAT, 'UTF-8'));
        $assetInstance->setCode($campaign['code']);
        $assetInstance->setRatio($campaign['ratio']);
        $assetInstance->setAutoplay($campaign['autoplay']);
        $assetInstance->setStartExpanded($campaign['start_expanded']);
        $assetInstance->setEndCollapse($campaign['end_collapse']);

        if (Helper::isValidDate($campaign['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($campaign['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
