<?php namespace Stage\Ad\Campaign\Group\Ad;

use PDO;
use DateTime;
use Exception;
use Stage\Ad\Action;
use Stage\Ad\Campaign\Campaign;
use Stage\Stage;
use Stage\Helper;

class Video
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idCampaignGroup;

    /**
     * @var
     */
    private $idCampaign;

    /**
     * @var
     */
    private $idAd;

    /**
     * @var
     */
    private $displayIn = false;

    /**
     * @var
     */
    private $beginDate;

    /**
     * @var
     */
    private $endDate;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $quantity;

    /**
     * @var
     */
    private $quantityType;

    /**
     * @var
     */
    private $action;

    /**
     * @var
     */
    private $canSkip = false;

    /**
     * @var
     */
    private $skipAfter;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idCampaignGroup', 'idCampaign', 'idAd', 'displayIn', 'beginDate', 'endDate','link','canSkip','skipAfter','registerDate','quantity','quantityType' ,'action');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return (int) $this->id;
    }

    public function getIdCampaignGroup()
    {
        return (int) $this->idCampaignGroup;
    }

    public function getIdCampaign()
    {
        return (int) $this->idCampaign;
    }

    public function getIdAd()
    {
        return (int) $this->idAd;
    }

    public function displayIn()
    {
        return $this->displayIn;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getQuantity()
    {
        return $this->quantity;
    }

    public function getQuantityType()
    {
        return $this->quantityType;
    }

    /**
     * @return Action
     */
    public function getAction()
    {
        return $this->action;
    }

    public function canSkip()
    {
        return (bool) $this->canSkip;
    }

    public function skipAfter()
    {
        return $this->skipAfter;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdCampaignGroup($id)
    {
        $this->idCampaignGroup = (int) $id;
    }

    public function setIdCampaign($id)
    {
        $this->idCampaign = (int) $id;
    }

    public function setIdAd($id)
    {
        $this->idAd = (int) $id;
    }

    public function setDisplayIn($in)
    {
        $this->displayIn = (int) $in;
    }

    public function setBeginDate($date)
    {
        if ($date==null) {
            $this->beginDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        if ($date==null) {
            $this->endDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setQuantity($qty)
    {
        $this->quantity = $qty;
    }

    public function setQuantityType($qty)
    {
        $this->quantityType = $qty;
    }

    public function setAction($action)
    {
        $this->action = $action;
    }

    public function setCanSkip($can)
    {
        $this->canSkip = (bool) $can;
    }

    public function setSkipAfter($after)
    {
        $this->skipAfter = $after;
    }

    public function setRegisterDate($date)
    {
        if ($date==null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function save()
    {
        $idCampaignGroup = $this->getIdCampaignGroup();
        $idCampaign = $this->getIdCampaign();
        $idAd = $this->getIdAd();
        $displayIn = $this->displayIn();
        $beginDate = $this->getBeginDate();
        $endDate = $this->getEndDate();
        $link = $this->getLink();
        $canSkip = $this->canSkip();
        $skipAfter = $this->skipAfter();
        $idAction = !($this->getAction() instanceof Action) ? 5 : $this->getAction()->getId();

        $quantityType = $this->getQuantityType();
        $quantity = $this->getQuantityType();

        $parameters = [ [$idCampaignGroup, PDO::PARAM_INT],
                        [$idCampaign , PDO::PARAM_INT],
                        [$idAd, PDO::PARAM_INT],
                        [$displayIn, PDO::PARAM_INT],
                        [$beginDate, PDO::PARAM_STR],
                        [$endDate, PDO::PARAM_STR],
                        [$link, PDO::PARAM_STR],
                        [$quantity, PDO::PARAM_INT],
                        [$quantityType, PDO::PARAM_INT],
                        [$idAction, PDO::PARAM_INT],
                        [$canSkip, PDO::PARAM_BOOL],
                        [$skipAfter, PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGNS_GROUPS_ADS_VIDEOS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            $actions = Action::all();
            $instance = self::createInstance($data[0], $actions);
            $instance->setIdCampaign($this->getIdCampaign());
            return $instance;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function weeklySchedule($idGroupAdVideo)
    {
        try {

            // $idGroupAdVideo = $this->getId();
            $parameters = [[$idGroupAdVideo, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_GROUPS_ADS_VIDEOS_WEEKLY_SCHEDULE";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($s) {
                return WeeklySchedule::createInstance($s);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "beginDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "campaign" && $this->$attribute instanceof Campaign) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute == "action" && $this->$attribute instanceof Action) {
                $arr[$attribute] = $this->$attribute->toArray();
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data, $actions = [])
    {
        $instance = new self();
        if (isset($data['id'])) {
            $instance->setId($data['id']);
        }
        $instance->setIdCampaignGroup($data['id_campaign_group']);
        if (isset($data['id_campaign'])) {
            $instance->setIdCampaign($data['id_campaign']);
        }
        $instance->setIdAd($data['id_ad']);
        $instance->setDisplayIn($data['display_in']);
        $instance->setCanSkip($data['can_skip']);
        $instance->setSkipAfter($data['skip_after']);
        $instance->setLink($data['link']);
        $instance->setQuantity($data['quantity']);
        $instance->setQuantityType($data['quantity_type']);

        $idAction = $data['id_action'];
        $exist = array_values(array_filter($actions, function (Action $action) use ($idAction) {
            return $action->getId() == $idAction;
        }));

        $instance->setAction($exist ? $exist[0] : null);

        if (Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $instance->setBeginDate(new DateTime($data['begin_date']));
        }

        if (Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $instance->setEndDate(new DateTime($data['end_date']));
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
