<?php namespace Stage\Ad\Campaign\Group\Ad;

use DateTime;
use Exception;
use PDO;
use Stage\Helper;
use Stage\Stage;

class WeeklySchedule
{

    /**
     * @var
     */
    private $id = -1;   // its the id_schedule

    /**
     * @var
     */
    private $idAd;

    /**
     * @var
     */
    private $exclude;

    /**
     * @var
     */
    private $beginAt;

    /**
     * @var
     */
    private $endAt;

    /**
     * @var
     */
    private $weekDay;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idAd', 'exclude', 'beginAt', 'endAt', 'weekDay', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdAd()
    {
        return (int) $this->idAd;
    }

    public function getExclude()
    {
        return (bool) $this->exclude;
    }

    public function beginAt()
    {
        return $this->beginAt;
    }

    public function endAt()
    {
        return $this->endAt;
    }

    public function getWeekDay()
    {
        return $this->weekDay;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdAd($id)
    {
        $this->idAd = (int) $id;
    }

    public function setExclude($exclude)
    {
        $this->exclude = (bool) $exclude;
    }

    public function setBeginAt($at)
    {
        $this->beginAt = $at;
    }

    public function setEndAt($at)
    {
        $this->endAt = $at;
    }

    public function setDayOfWeek($day)
    {
        $this->weekDay = $day;
    }

    public function setRegisterDate($date)
    {
        if ($date==null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save($idGgroupAdVideo)
    {
        // $idGgroupAdVideo its the collumn id from table ads_campaigns_groups_ads_videos
        // Note: This SP insert on table ads_campaigns_groups_ads_videos_weekly_schedule
        $parameters = [ [$idGgroupAdVideo, PDO::PARAM_INT],
                        [$this->getExclude() , PDO::PARAM_INT],
                        [$this->beginAt(), PDO::PARAM_STR],
                        [$this->endAt(), PDO::PARAM_STR],
                        [$this->getWeekDay(), PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGNS_GROUPS_ADS_VIDEOS_WEEKLY_SCHEDULE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return $this;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $instance = new self();
        if (isset($data['id_schedule'])) {
            $instance->setId($data['id_schedule']);
        }
        if (isset($data['id_group_ad_video'])) {
            $instance->setIdAd($data['id_group_ad_video']);
        }
        $instance->setExclude($data['exclude']);
        $instance->setBeginAt($data['begin_time']);
        $instance->setEndAt($data['end_time']);
        $instance->setDayOfWeek($data['day_of_week']);

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
