<?php namespace Stage\Ad\Campaign\Group;

use DateTime;
use Exception;
use PDO;
use Stage\Helper;
use Stage\Stage;

class FrequencyCap
{

    /**
     * @var
     */
    private $id = -1;   // Its the id_option

    /**
     * @var
     */
    private $idCampaignGroup;

    /**
     * @var
     */
    private $idCampaign;

    /**
     * @var
     */
    private $type;

    /**
     * @var
     */
    private $typeValue;

    /**
     * @var
     */
    private $value;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idCampaignGroup', 'idCampaign', 'type', 'value','typeValue', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdCampaignGroup()
    {
        return (int) $this->idCampaignGroup;
    }

    public function getIdCampaign()
    {
        return (int) $this->idCampaign;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getTypeValue()
    {
        return $this->typeValue;
    }

    public function getValue()
    {
        return $this->value;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdCampaignGroup($id)
    {
        $this->idCampaignGroup = (int) $id;
    }

    public function setIdCampaign($id)
    {
        $this->idCampaign = (int) $id;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function setTypeValue($type)
    {
        $this->typeValue = $type;
    }

    public function setValue($value)
    {
        $this->value = $value;
    }

    public function setRegisterDate($date)
    {
        if ($date==null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        // Note: This SP insert on table ads_campaigns_groups_frequency_cap_options
        $idCampaignGroup = $this->getIdCampaignGroup();
        $idCampaign = $this->getIdCampaign();
        $type = $this->getType();
        $value = $this->getValue();
        $typeValue = $this->getTypeValue();

        $parameters = [ [$idCampaignGroup, PDO::PARAM_INT],
                        [$idCampaign , PDO::PARAM_INT],
                        [$type, PDO::PARAM_INT],
                        [$typeValue, PDO::PARAM_INT],
                        [$value, PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGNS_GROUPS_FREQUENCY_CAP_OPTIONS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            $instance = self::createInstance($data[0]);
            $instance->setIdCampaign($this->getIdCampaign());
            return $instance;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idCampaign
     * @param $idCampaignGroup
     * @return array
     * @throws Exception
     */
    public static function options($idCampaign, $idCampaignGroup)
    {
        try {
            $parameters = [[$idCampaign, PDO::PARAM_INT],
                           [$idCampaignGroup, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_GROUP_FREQUENCY_CAP_OPTIONS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($option) {
                return self::createInstance($option);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $instance = new self();
        if (isset($data['id_option'])) {
            $instance->setId($data['id_option']);
        }
        if (isset($data['id_campaign'])) {
            $instance->setIdCampaign($data['id_campaign']);
        }
        $instance->setIdCampaignGroup($data['id_campaign_group']);
        $instance->setType($data['frequency_type']);
        $instance->setTypeValue($data['frequency_type_value']);
        $instance->setValue($data['frequency_value']);

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
