<?php namespace Stage\Ad\Campaign\Group;

use PDO;
use DateTime;
use Exception;
use Stage\Ad\Action;
use Stage\Ad\Campaign\Campaign;
use Stage\Ad\Campaign\Group\Ad\Tag;
use Stage\Ad\Campaign\Group\Ad\Video;
use Stage\Stage;
use Stage\Helper;

class Group
{
    const DISPLAY_IN_ALL = 1;
    const DISPLAY_IN_WEB = 2;
    const DISPLAY_IN_MOBILE = 3;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var Campaign
     */
    private $campaign;

    /**
     * @var
     */
    private $domains = "";

    /**
     * @var
     */
    private $exclude = false;

    /**
     * @var
     */
    private $playAdsDelivery = false;

    /**
     * @var 1 - PreRoll, 2 - MidRoll, 3 - PostRoll
     */
    private $rollIn;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'name', 'campaign', 'domains', 'exclude', 'playAdsDelivery', 'registerDate', 'rollIn');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDomains()
    {
        return $this->domains;
    }

    public function getExclude()
    {
        return (bool) $this->exclude;
    }

    public function getPlayAdsDelivery()
    {
        return (int) $this->playAdsDelivery;
    }

    public function getRollIn()
    {
        return (int) $this->rollIn;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDomains($domains)
    {
        $this->domains = $domains;
    }

    public function setExclude($exclude)
    {
        $this->exclude = (bool) $exclude;
    }

    public function setPlayAdsDelivery($delivery)
    {
        $this->playAdsDelivery = (int) $delivery;
    }

    public function setCampaign(Campaign $campaign)
    {
        $this->campaign = $campaign;
    }

    public function setRollIn($rollIn)
    {
        $this->rollIn = (int) $rollIn;
    }

    public function setRegisterDate($date)
    {
        if ($date==null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * find
     *
     * Find a Asset
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id, $idCampaign)
    {
        try {
            $idCampaignGroup = $id;
            $name = "ADS_CAMPAIGN_GROUP";
            $method = "GET";
            $parameters = [[$idCampaignGroup, PDO::PARAM_INT],
                            [$idCampaign, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Campaign id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Campaign
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
                        [$this->campaign->getId(), PDO::PARAM_INT],
                        [$this->getName(), PDO::PARAM_STR],
                        [$this->getDomains(), PDO::PARAM_STR],
                        [$this->getExclude(), PDO::PARAM_BOOL],
                        [$this->getPlayAdsDelivery(), PDO::PARAM_INT],
                        [$this->getRollIn(), PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGNS_GROUPS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_campaign_group']);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * inCampaign
     *
     * @param $idCampaign
     * @return array
     * @throws Exception
     */
    public static function inCampaign($idCampaign)
    {
        try {
            $parameters = [ [$idCampaign, PDO::PARAM_INT]];
            $name = "ADS_CAMPAIGN_GROUPS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($group) {
                return self::createInstance($group);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * videos
     *
     * @param $idCampaign
     * @param $idCampaignGroup
     * @return array
     * @throws Exception
     */
    public static function videos($idCampaign, $idCampaignGroup)
    {
        $parameters = [ [$idCampaign, PDO::PARAM_INT],
                        [$idCampaignGroup , PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGN_GROUP_ADS_VIDEOS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $actions = Action::all();
            return array_map(function ($group) use ($actions) {
                return Video::createInstance($group, $actions);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * countries
     *
     * @param $idCampaign
     * @param $idCampaignGroup
     * @return array
     * @throws Exception
     */
    public static function countries($idCampaign, $idCampaignGroup)
    {
        $parameters = [ [$idCampaign, PDO::PARAM_INT],
                        [$idCampaignGroup , PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGNS_GROUP_COUNTRIES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($group) {
                return Country::createInstance($group);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * adTags
     *
     * @param $idCampaign
     * @param $idCampaignGroup
     * @return array
     * @throws Exception
     */
    public static function adTags($idCampaign, $idCampaignGroup)
    {
        $parameters = [ [$idCampaign, PDO::PARAM_INT],
                        [$idCampaignGroup , PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGN_GROUP_EXTERNAL_AD_TAG";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($group) {
                return Tag::createInstance($group);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    public static function midRollOptions($idCampaign, $idCampaignGroup)
    {
        $parameters = [ [$idCampaign, PDO::PARAM_INT],
            [$idCampaignGroup , PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGNS_GROUP_MIDROLL_OPTIONS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($group) {
                return MidRollOption::createInstance($group);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "beginDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "campaign" && $this->$attribute instanceof Campaign) {
                $arr[$attribute] = $this->$attribute->toArray();
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($asset)
    {
        $instance = new self();
        $instance->setId($asset['id_campaign_group']);
        $instance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setDomains($asset['domains']);
        $instance->setExclude($asset['exclude']);
        $instance->setPlayAdsDelivery($asset['playads_delivery']);
        $instance->setRollIn($asset['roll_in']);

        $campaign = Campaign::find($asset['id_campaign']);
        $instance->setCampaign($campaign);

        if (Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
