<?php namespace Stage\Ad\Campaign\Group;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;
use Stage\Ad\Campaign\Campaign;

class Rules
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $field;

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $idCampaignGroup;

    /**
     * @var
     */
    private $idCampaign;

    /**
     * @var
     * $type = 0 ( vod and live )
     * $type = 1 ( vod  )
     * $type = 2 ( live )
     */
    private $type;

    /**
     * @var
     */
    private $condition;

    /**
     * @var
     */
    private $exclude = false;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'field', 'total', 'name', 'idCampaignGroup', 'idCampaign', 'type', 'condition', 'exclude', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getField()
    {
        return $this->field;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getIdCampaignGroup()
    {
        return (int) $this->idCampaignGroup;
    }

    public function getIdCampaign()
    {
        return (int) $this->idCampaign;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getCondition()
    {
        return $this->condition;
    }

    public function getExclude()
    {
        return $this->exclude;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setField($id)
    {
        $this->field = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setTotal($total)
    {
        $this->total = (int) $total;
    }

    public function setIdCampaignGroup($id)
    {
        $this->idCampaignGroup = (int) $id;
    }

    public function setIdCampaign($id)
    {
        $this->idCampaign = (int) $id;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function setCondition($cond)
    {
        $this->condition = $cond;
    }

    public function setExclude($exclude)
    {
        $this->exclude = (bool) $exclude;
    }

    public function setRegisterDate($date)
    {
        if ($date==null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * find
     *
     * Find a Rule
     *
     * @return []
     * @throws Exception
     */
    public static function find($idCampaign, $idCampaignGroup, $field = 0)
    {
        // $field = 0 Retorna tudo
        // $field = 1 Retorna todas as tags
        // $field = 2 Retorna todas as labels
        // $field = 3 Retorna todas as playlists
        // $field = 4 Retorna todas os channels

        try {
            $name = "ADS_CAMPAIGNS_GROUP_VIDEO_RULES";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT],
                            [$idCampaign, PDO::PARAM_INT],
                            [$idCampaignGroup, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Campaign id $idCampaign not exists.", $data[0]['return_code']);
            } else {
                return array_map(function ($rule) {
                    return self::createInstance($rule);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Campaign
     *
     * @return $this
     * @throws Exception
     */
    public static function save($field, $idCampaignGroup, $idCampaign, $id, $rule, $type, $exclude)
    {
        // $field = 1 ( id is id_tag )
        // $field = 2 ( id is id_label )
        // $field = 3 ( id is id_playlist )
        // $field = 4 ( id is id_channel )

        // $type = 0 ( both )
        // $type = 1 ( vod  )
        // $type = 2 ( live )

        $parameters = [ [$field, PDO::PARAM_INT],
                        [$idCampaignGroup, PDO::PARAM_INT],
                        [$idCampaign, PDO::PARAM_INT],
                        [$id, PDO::PARAM_INT],
                        [$rule, PDO::PARAM_INT],
                        [$type, PDO::PARAM_INT],
                        [$exclude, PDO::PARAM_INT]];

        try {
            $name = "ADS_CAMPAIGNS_GROUPS_VIDEO_RULES";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_campaign_group'])) {
                $data = $data[0];
                $instance = new self();
                $instance->setField($data['field']);
                $instance->setIdCampaignGroup($data['id_campaign_group']);
                $instance->setIdCampaign($idCampaign);
                $instance->setId($data['id']);
                $instance->setType($data['type']);
                $instance->setCondition($data['rule']);
                $instance->setExclude($data['exclude']);
                return $instance;
            }

            throw new Exception("Error saving video rules", $data[0]['return_code']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $field
     * @param $idCampaign
     * @return array
     * @throws Exception
     */
    public static function delete($field, $idCampaign)
    {
        // $field = 1 ( id is id_tag )
        // $field = 2 ( id is id_label )
        // $field = 3 ( id is id_playlist )
        // $field = 4 ( id is id_channel )

        $name = "ADS_CAMPAIGNS_GROUPS_VIDEO_RULES";
        $parameters = [[$field, PDO::PARAM_INT],
                       [$idCampaign, PDO::PARAM_INT]];

        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public static function resolveField($field)
    {
        $resolvers = [
          'none',
          'tag',
          'label',
          'playlist',
          'channel'
        ];
        // $field = 1 ( id is id_tag )
        // $field = 2 ( id is id_label )
        // $field = 3 ( id is id_playlist )
        // $field = 4 ( id is id_channel )

        return isset($resolvers[$field]) ? $resolvers[$field] : "";
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "beginDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "campaign" && $this->$attribute instanceof Campaign) {
                $arr[$attribute] = $this->$attribute->toArray();
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setField($data['field']);
        $instance->setIdCampaignGroup($data['id_campaign_group']);
        $instance->setIdCampaign($data['id_campaign']);
        $instance->setId($data['id']);
        $instance->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setTotal($data['total']);
        $instance->setType($data['type']);
        $instance->setCondition($data['rule']);
        $instance->setExclude($data['exclude']);

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
