<?php namespace Stage\Ad\External;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Stage;

class Tag
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $url;

    /**
     * @var
     */
    private $active = true;

    /**
     * @var
     */
    private $totalCampaigns = 0;


    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','url','active','totalCampaigns');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getUrl()
    {
        return $this->url;
    }

    public function getActive()
    {
        return (bool) $this->active;
    }

    public function getTotalCampaigns()
    {
        return (bool) $this->active;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setUrl($url)
    {
        $this->url = $url;
    }

    public function setActive($active)
    {
        $this->active = (bool) $active;
    }

    public function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    public function setTotalCampaigns($total)
    {
        $this->totalCampaigns = (int) $total;
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "ADS_EXTERNAL_AD_TAGS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * save
     *
     * Save a tag to the database and return a Tag instance
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "ADS_EXTERNAL_AD_TAGS";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$this->getName(), PDO::PARAM_STR],
                            [$this->getUrl(), PDO::PARAM_STR],
                            [$this->getActive(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_external_ad_tag']);

            // Fire an event
            if (!$this->getId()) {
                \Stage\Events\Event::fire("ads.external.tag.created", $this);
            } else {
                \Stage\Events\Event::fire("ads.external.tag.updated", $this);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * delete
     *
     * Delete a Tag
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ADS_EXTERNAL_AD_TAG";
            $method = "DEL";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * find
     *
     * Finds a Tag
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ADS_EXTERNAL_AD_TAG";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Tag id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }


    /**
     * all
     *
     * Returns all Tags
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->by = Query::ALL;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * query
     *
     * Search Tags
     *
     * @return Array
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_STR],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "ADS_EXTERNAL_AD_TAGS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = self::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_external_ad_tag']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setUrl($tag['tag']);
        $tagInstance->setActive($tag['active']);

        if (isset($tag['total_campaigns'])) {
            $tagInstance->setTotalCampaigns($tag['total_campaigns']);
        }

        if (Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        return $tagInstance;
    }
}
