<?php namespace Stage\Ad\Video\Vast\Tag;

use DateTime;
use PDO;
use Exception;
use Stage\Stage;
use Stage\Helper;

class Tag
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idAd = 0;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var string
     */
    private $registerDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id', 'idAd','name','registerDate');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdAd()
    {
        return $this->idAd;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    private function setIdAd($id)
    {
        $this->idAd = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    /**
     * @param $idAdTag
     * @param $idVideoAd
     * @return array
     * @throws Exception
     */
    public static function duplicate($idAdTag, $idVideoAd)
    {
        try {
            $parameters = [[$idAdTag, PDO::PARAM_INT],
                            [$idVideoAd, PDO::PARAM_INT]];

            $name = "ADS_VAST_TAG_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Tags
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->by = Query::ALL;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * @param Query $q
     * @return int
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_STR]];

            $name = "ADS_VAST_TAGS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int) isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Tags
     *
     * @return Array
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_STR],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "ADS_VAST_TAGS";
            $method = "GET";
            $results = Stage::esp($parameters, $name, $method);
            return $results;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idVast
     * @param $idVideoAd
     * @return array
     * @throws Exception
     */
    public static function find($idVast, $idVideoAd)
    {
        try {
            $parameters = [[$idVast, PDO::PARAM_INT],
                           [$idVideoAd, PDO::PARAM_INT]];
            $name = "ADS_VAST_OPTIONS_VIDEOS_RELATED";
            $method = "GET";
            $results = Stage::esp($parameters, $name, $method);
            if (isset($results[0]) && isset($results[0]['return_code']) && $results[0]['return_code'] == "-2") {
                return [];
            }
            return $results;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idVideoAd
     * @param $name
     * @return Tag
     * @throws Exception
     */
    public function save($idVideoAd)
    {
        try {
            $parameters = [[-1, PDO::PARAM_INT],
                           [$idVideoAd, PDO::PARAM_INT],
                           [$this->getName(), PDO::PARAM_STR]];
            $name = "ADS_VAST_TAG";
            $method = "SET";
            $result = Stage::esp($parameters, $name, $method);

            $this->setId($result[0]['id_vast']);
            $this->setIdAd($result[0]['id_ad']);

            if (Helper::isValidDate($result[0]['register_date'], "Y-m-d H:i:s")) {
                $this->setRegisterDate(new DateTime($result[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idVast
     * @param $idVideoAd
     * @param $idOption
     * @param $optionValue
     * @return bool
     * @throws Exception
     */
    public static function attachOption($idVast, $idVideoAd, $idOption, $optionValue)
    {
        try {
            $parameters = [[$idVast, PDO::PARAM_INT],
                            [$idVideoAd, PDO::PARAM_INT],
                            [$idOption, PDO::PARAM_INT],
                            [$optionValue, PDO::PARAM_INT]];

            $name = "ADS_VAST_OPTIONS_TO_VIDEOS";
            $method = "SET";
            $options = Stage::esp($parameters, $name, $method);
            return isset($options[0]) && isset($options[0]['return_code']) && $options[0]['return_code'] == 0 ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idVast
     * @param $idVideoAd
     * @return array
     * @throws Exception
     */
    public static function delete($idVast, $idVideoAd, $isOption = false)
    {
        try {
            if ($isOption) {
                $name = "ADS_VAST_VIDEO_OPTIONS";
            } else {
                $name = "ADS_VAST_TAG";
            }

            $parameters = [[$idVast, PDO::PARAM_INT],
                           [$idVideoAd, PDO::PARAM_INT]];


            $method = "DEL";
            $results = Stage::esp($parameters, $name, $method);
            return $results;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public static function addVastTagToAd($idVast, $idVideoAd, $name)
    {
        // TODO
        try {
            $parameters = [[$idVast, PDO::PARAM_INT],
                           [$idVideoAd, PDO::PARAM_INT],
                           [$name, PDO::PARAM_STR]];

            $name = "ADS_VAST_OPTIONS_TO_VIDEOS";
            $method = "SET";
            $videos = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $videos);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_vast']);
        $tagInstance->setIdAd($tag['id_ad']);

        if (Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        return $tagInstance;
    }
}
