<?php namespace Stage\Ad\Video;

use PDO;
use Exception;
use Stage\Stage;

class Version
{
    const WEB = 'w';            // Web
    const MOBILE = 'm';         // Mobile
    const LOW_MOBILE = 'lm';    // Low Mobile

    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $idVideo;

    /**
     * Video filename with extension
     *
     * @var array
     */
    private $filename;

    /**
     * Extension
     * @var array
     */
    private $extension;

    /**
     * Width
     * @var
     */
    private $width;

    /**
     * Height
     * @var
     */
    private $height;

    /**
     * Version
     * @var
     */
    private $version;

    /**
     * @var
     */
    private $versionType;

    /**
     * Size
     * @var
     */
    private $size;

    /**
     * State
     * @var
     */
    private $state;

    /**
     * isDefault
     * @var
     */
    private $defaultVersion;

    /**
     * @var
     */
    private $isDefault;

    /**
     * @var
     */
    private $bitrate;


    /**
     * @var
     */
    private $audioBitrate;

    /**
     * @var integer
     */
    private $idStorage = "";

    /**
     * @var integer
     */
    private $idCdn = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','idVideo','filename','extension','width','height','version','versionType','size','state','isDefault', 'bitrate', 'audioBitrate', 'idStorage', 'idCdn');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdVideo()
    {
        return $this->idVideo;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getExtension()
    {
        return $this->extension;
    }

    public function getWidth()
    {
        return (int) $this->width;
    }

    public function getHeight()
    {
        return (int) $this->height;
    }

    public function getVersion()
    {
        return $this->version;
    }

    public function getVersionType()
    {
        return $this->versionType;
    }

    public function getSize()
    {
        return $this->size;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getIsDefault()
    {
        return $this->isDefault;
    }

    public function getDefaultVersion()
    {
        return $this->defaultVersion;
    }

    public function getBitrate()
    {
        return $this->bitrate;
    }

    public function getAudioBitrate()
    {
        return $this->audioBitrate ? $this->audioBitrate : 0;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdVideo($idVideo)
    {
        $this->idVideo = $idVideo;
    }

    public function setFilename($filename)
    {
        $this->filename = trim($filename);
    }

    public function setExtension($extension)
    {
        $this->extension = strtolower(trim($extension));
    }

    public function setWidth($width)
    {
        $this->width = (int)$width;
    }

    public function setHeight($height)
    {
        $this->height = (int)$height;
    }

    public function setVersion($version)
    {
        $this->version = $version;
    }

    public function setVersionType($versionType)
    {
        $this->versionType = $versionType;
    }

    public function setSize($size)
    {
        $this->size = (int) $size;
    }

    public function setState($state)
    {
        $this->state = (int) $state;
    }

    public function setDefaultVersion($version)
    {
        $this->defaultVersion = $version;
    }

    public function setIsDefault($isDefault)
    {
        $this->isDefault = (bool) $isDefault;
    }

    public function setBitrate($bitrate)
    {
        $this->bitrate = (int) $bitrate;
    }

    public function setAudioBitrate($bitrate)
    {
        $this->audioBitrate = $bitrate;
    }

    public function setIdStorage($id)
    {
        return $this->idStorage = $id;
    }

    public function setIdCdn($id)
    {
        $this->idCdn = $id;
    }

    /**
     * versions
     *
     * @param $idVideo
     * @param $filterBy
     * @return array
     * @throws Exception
     */
    public static function versions($idVideo, $filterBy = [], $orderBy = null, $order = SORT_ASC, $type = 1)
    {
        try {

            //$type = 1;  // 1 - Web, 2 - Mobile
            if (in_array(Version::MOBILE, $filterBy)) {
                $type = 2;
            }

            $parameters = [[$idVideo, PDO::PARAM_INT],
                [$type, PDO::PARAM_INT]];
            $name = "ADS_VERSIONS";
            $method = "GET";
            $videos = Stage::esp($parameters, $name, $method);

            $versions = $videos;

            // Filter the versions
//            if( sizeof($filterBy) )
//            {
//                $versions = array_values(array_filter($videos, function ($item) use ($filterBy) {
//                    return in_array($item['version_type'], $filterBy);
//                }));
//            }

            // Sort the versions by field
            if ($orderBy) {
                $temp = [];
                foreach ($versions as $key => $row) {
                    $temp[$key] = $row[$orderBy];
                }

                // Sort the multi dimensional array
                array_multisort($temp, $order, $versions);
            }

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $versions);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * setDefaultVideoVersion
     *
     * @param $idVersion
     * @return Version
     * @throws Exception
     */
    public static function setDefaultVideoVersion($idVersion)
    {
        try {
            $parameters = [[$idVersion, PDO::PARAM_INT]];
            $name = "ADS_VIDEO_VERSION_DEFAULT";
            $method = "SET";
            $videos = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $videos)[0];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a video
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getIdVideo(), PDO::PARAM_INT],
                       [$this->getFilename(), PDO::PARAM_STR],
                       [$this->getExtension(), PDO::PARAM_STR],
                       [$this->getWidth(), PDO::PARAM_INT],
                       [$this->getHeight(), PDO::PARAM_INT],
                       [$this->getVersion(), PDO::PARAM_STR],
                       [$this->getVersionType(), PDO::PARAM_STR],
                       [$this->getSize(), PDO::PARAM_INT],
                       //[$this->getState(), PDO::PARAM_STR],
                       [$this->getBitrate(), PDO::PARAM_INT],
                       [$this->getAudioBitrate(), PDO::PARAM_INT],
                       [$this->getDefaultVersion(), PDO::PARAM_INT],
                       [$this->getIdStorage(), PDO::PARAM_INT],
                       [$this->getIdCdn(), PDO::PARAM_INT]];

        try {
            $name = "ADS_VIDEO_VERSIONS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_version']) && $data[0]['id_version'] >= 0) {
                $idVideo = $data[0]['id_version'];
                $this->setId($idVideo);

                // self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * @param $idAccount
     * @param $idStorage
     * @param $idCdn
     * @param $fallbackStoragePath
     * @return mixed
     * @throws Exception
     */
    public function getStoragePath($idAccount, $idStorage, $idCdn, $fallbackStoragePath)
    {
        $path = \Stage\Account\Map::getStorage($idStorage, \Stage\Account\Map::ADS)->getStoragePath();
        if ($path) {
            return $path;
        } else {
            $settings = \Stage\Account\Map::getVideoAbrSettings($idAccount, $idCdn, $idStorage);
            if (isset($settings[0]) && isset($settings[0]['path_versions'])) {
                foreach ($settings as $setting) {
                    if ($setting['path_versions'] == $idStorage) {
                        $fallbackStoragePath = $setting['path_versions'];
                    }
                }
            }
        }

        return $fallbackStoragePath;
    }

    /**
     * @param $idAccount
     * @param $idCdn
     * @param $idStorage
     * @param $fallbackCdnUrl
     * @return mixed
     * @throws Exception
     */
    public static function getCdnUrl($idAccount, $idCdn, $idStorage, $fallbackCdnUrl)
    {
        $settings = \Stage\Account\Map::getAdsAbrSettings($idAccount, $idCdn, $idStorage);
        if (isset($settings[0]) && isset($settings[0]['cdn_versions'])) {
            foreach ($settings as $setting) {
                if ($setting['cdn_versions'] == $idCdn) {
                    $fallbackCdnUrl = $setting['path_cdn_versions'];
                }
            }
        }

        return $fallbackCdnUrl;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Version
     */
    public static function createInstance($data)
    {
        $v = new self();
        $v->setId($data['id_version']);
        $v->setIdVideo($data['id_ad']);
        $v->setFilename($data['filename']);
        $v->setExtension($data['extension']);
        $v->setWidth($data['width']);
        $v->setHeight($data['height']);
        $v->setVersion($data['version']);
        $v->setVersionType($data['version_type']);
        $v->setSize($data['size']);
        $v->setState($data['state']);
        $v->setIsDefault($data['is_default']);
        $v->setBitrate($data['bitrate']);
        $v->setAudioBitrate($data['audio_bitrate']);

        if (isset($data['id_storage'])) {
            $v->setIdStorage($data['id_storage']);
        }

        if (isset($data['id_cdn'])) {
            $v->setIdCdn($data['id_cdn']);
        }

        return $v;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
