<?php namespace Stage\Archive\Clip\Thumbnail;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class Thumbnail
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idClip;

    /**
     * Filename of the thumbnail
     * @var array
     */
    private $filename;

    /**
     * Bool
     * @var array
     */
    private $isDefault;

    /**
     * @var
     */
    private $seconds = 0;

    /**
     * @var string
     */
    private $registerDate = "";

    /**
     * @var integer
     */
    private $idStorage = "";

    /**
     * @var integer
     */
    private $idCdn = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','idClip','filename','isDefault','seconds','registerDate', 'idStorage', 'idCdn');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdClip()
    {
        return $this->idClip;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function isDefault($default = null)
    {
        if (is_bool($default)) {
            $this->isDefault = $default;
        }

        return $this->isDefault;
    }

    public function getSeconds()
    {
        return $this->seconds;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdClip($id)
    {
        $this->idClip = (int) $id;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setSeconds($seconds)
    {
        $this->seconds = $seconds;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    public function setIdStorage($id)
    {
        return $this->idStorage = $id;
    }

    public function setIdCdn($id)
    {
        $this->idCdn = $id;
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$this->getIdClip(), PDO::PARAM_INT],
                       [$this->getFilename(), PDO::PARAM_STR],
                       [$this->isDefault(), PDO::PARAM_BOOL],
                       [$this->getSeconds(), PDO::PARAM_INT]];

        try {
            $name = "ARCHIVE_CLIP_THUMBNAIL";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && ($data[0]['return_code']) == "-2") {
                throw new Exception("The id clip " . $this->getIdClip() . " was not found.", $data[0]['return_code']);
            } elseif (isset($data[0]['id_thumbnail']) && $data[0]['id_thumbnail'] >= 0) {
                $idThumbnail = $data[0]['id_thumbnail'];
                $this->setId($idThumbnail);
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Thumbnail
     */
    public static function createInstance($data)
    {
        $thumbnail = new self();
        $thumbnail->setId($data['id_thumbnail']);
        $thumbnail->setIdClip($data['id_video']);
        $thumbnail->setFilename($data['thumbnail']);
        $thumbnail->isDefault((bool)$data['is_default']);
        $thumbnail->setSeconds($data['seconds']);

        if (isset($data['id_storage'])) {
            $thumbnail->setIdStorage($data['id_storage']);
        }

        if (isset($data['id_cdn'])) {
            $thumbnail->setIdCdn($data['id_cdn']);
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $thumbnail->setRegisterDate(new DateTime($data['register_date']));
        }

        return $thumbnail;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
