<?php namespace Stage\Archive\Clip\Transcode;

use PDO;
use Exception;
use Stage\Stage;

class Transcode
{

    /**
     * @var
     */
    private $idAccount;

    /**
     * @var
     */
    private $accountToken;

    /**
     * @var
     */
    private $idVideo;

    /**
     * @var
     */
    private $idClip;

    /**
     * @var
     */
    private $inputPath;

    /**
     * @var
     */
    private $outputPath;

    /**
     * @var
     */
    private $filename;

    /**
     * @var
     */
    private $source;

    /**
     * @var
     */
    private $clip;

    /**
     * @var boolean
     */
    private $muted;

    /**
     * @var
     */
    private $inSeconds;

    /**
     * @var
     */
    private $durationSeconds;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $clipRegisterDate;

    public function getIdAccount()
    {
        return $this->idAccount;
    }

    public function getAccountToken()
    {
        return $this->accountToken;
    }

    public function getIdVideo()
    {
        return $this->idVideo;
    }

    public function getIdClip()
    {
        return $this->idClip;
    }

    public function getInputPath()
    {
        return $this->inputPath;
    }

    public function getOutputPath()
    {
        return $this->outputPath;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getSource()
    {
        return $this->source;
    }

    public function getClip()
    {
        return $this->clip;
    }

    public function isMuted()
    {
        return $this->muted;
    }

    public function getInSeconds()
    {
        return $this->inSeconds;
    }

    public function getdurationSeconds()
    {
        return $this->durationSeconds;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getClipRegisterDate()
    {
        return $this->clipRegisterDate;
    }

    public function setIdAccount($idAccount)
    {
        return $this->idAccount = $idAccount;
    }

    public function setAccountToken($accountToken)
    {
        return $this->accountToken = $accountToken;
    }

    public function setIdVideo($idVideo)
    {
        return $this->idVideo = $idVideo;
    }

    public function setIdClip($idClip)
    {
        return $this->idClip = $idClip;
    }

    public function setInputPath($inputPath)
    {
        return $this->inputPath = $inputPath;
    }

    public function setOutputPath($outputPath)
    {
        return $this->outputPath = $outputPath;
    }

    public function setFilename($filename)
    {
        return $this->filename = $filename;
    }

    public function setSource($source)
    {
        return $this->source = $source;
    }

    public function setClip($clip)
    {
        return $this->clip = $clip;
    }

    public function setMuted($muted)
    {
        $this->muted = (bool) $muted;
    }

    public function setInSeconds($seconds)
    {
        return $this->inSeconds = $seconds;
    }

    public function setDurationSeconds($seconds)
    {
        return $this->durationSeconds = $seconds;
    }

    public function setRegisterDate($registerDate)
    {
        return $this->registerDate = $registerDate;
    }

    public function setClipRegisterDate($registerDate)
    {
        return $this->clipRegisterDate = $registerDate;
    }

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('idAccount','accountToken','idVideo','idClip', 'inputPath','outputPath','filename','source','clip','muted','inSeconds','durationSeconds');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "ARCHIVE_CREATE_CLIP";
            $method = "SET";
            $parameters = [[$this->getIdAccount(), PDO::PARAM_INT],
                           [$this->getAccountToken(), PDO::PARAM_STR],
                           [$this->getIdVideo(), PDO::PARAM_INT],
                           [$this->getIdClip(), PDO::PARAM_INT],
                           [$this->getInputPath(), PDO::PARAM_STR],
                           [$this->getOutputPath(), PDO::PARAM_STR],
                           [$this->getFilename(), PDO::PARAM_STR],
                           [$this->getSource(), PDO::PARAM_STR],
                           [$this->getClip(), PDO::PARAM_INT],
                           [$this->isMuted(), PDO::PARAM_BOOL],
                           [$this->getInSeconds(), PDO::PARAM_INT],
                           [$this->getDurationSeconds(), PDO::PARAM_INT],
                           [$this->getRegisterDate(), PDO::PARAM_STR],
                           [$this->getClipRegisterDate(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if (!sizeof($data)) {
                throw new Exception("Error saving Transcode.");
            } else {
                return $data;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * next
     *
     * Get the next transcode video
     *
     * @param int $field
     * @return mixed|Job
     * @throws Exception
     */
    public static function next($field = 0, $machine = "")
    {
        try {
            $name = "ARCHIVE_CREATE_CLIP_TO_PROCESS";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT],
                           [$machine, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            return isset($data[0]) && isset($data[0]['return_code']) ? [] : Job::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * done
     *
     * @param $idQueue
     * @param int $field
     * @return array
     * @throws Exception
     */
    public static function done($idQueue, $field = 1)
    {
        try {
            $name = "ARCHIVE_CREATE_CLIP_PROCESSED";
            $method = "SET";
            $parameters = [[$idQueue, PDO::PARAM_INT],
                           [$field, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) ? [] : $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * retry
     *
     * @param $idQueue
     * @param int $field
     * @return array
     * @throws Exception
     */
    public static function retry($idQueue, $field = 1)
    {
        try {
            $name = "ARCHIVE_CREATE_CLIP_NOT_PROCESSED";
            $method = "SET";
            $parameters = [[$idQueue, PDO::PARAM_INT],
                           [$field, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) ? [] : $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * softDelete
     *
     * @param $idAccount
     * @param $idVideo
     * @param bool $deleted
     * @return array
     * @throws Exception
     */
    public static function softDelete($idAccount, $idClip, $deleted = true)
    {
        try {
            $name = "ARCHIVE_CREATE_CLIP_DELETED";
            $method = "SET";
            $parameters = [[$idAccount, PDO::PARAM_INT],
                           [$idClip, PDO::PARAM_INT],
                           [$deleted, PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) ? [] : $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idAccount
     * @param $idVideo
     * @return array
     * @throws Exception
     */
    public static function delete($idAccount, $idClip)
    {
        try {
            $name = "ARCHIVE_CREATE_CLIP";
            $method = "DEL";
            $parameters = [[$idAccount, PDO::PARAM_INT],
                           [$idClip, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) ? [] : $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * @param $idAccount
     * @param $idVideo
     * @param bool $deleted
     * @return mixed|Job
     * @throws Exception
     */
    public static function query(Query $q)
    {
        try {
            $name = "TRANSCODER_ARCHIVE_CLIPS";
            $method = "GET";
            $field          = $q->by;
            $sndField       = $q->sndBy;
            $trdField       = $q->trdBy;
            $term           = $q->term;
            $filter         = $q->filter;
            $sndFilter      = $q->sndFilter;
            $trdFilter      = $q->trdFilter;
            $limit_a        = $q->from;
            $limit_b        = $q->to;
            $order          = $q->order;
            $orderBy        = $q->orderBy;

            $parameters = [[$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return Job::createInstance($video);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * Count videos by query
     * @return []
     * @throws Exception
     */
    public static function queryCount(Query $q = null, $idDirectory = 0)
    {
        $field          = $q->by;
        $sndField       = $q->sndBy;
        $trdField       = $q->trdBy;
        $term           = $q->term;
        $filter         = $q->filter;
        $sndFilter      = $q->sndFilter;
        $trdFilter      = $q->trdFilter;

        try {

            $parameters = [ [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_STR],
                [$trdFilter, PDO::PARAM_STR]];

            $name = "TRANSCODER_ARCHIVE_CLIPS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "account" ? $this->getAccount()->toArray() : $this->$attribute;
        }

        return $arr;
    }

    public function __toString()
    {
        $p = array_map(function ($var) {
            return "'" . $var . "'";
        }, get_object_vars($this));

        return implode(',', $p);
    }
}
