<?php namespace Stage\Archive\Tag\Key;

use PDO;
use Exception;
use Stage\Stage;

class Key
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $public = true;

    /**
     * @var
     */
    private $total;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','nameStripped','public','total');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getPublic()
    {
        return $this->public;
    }

    public function getTotal()
    {
        return $this->total;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setPublic($public)
    {
        $this->public = (bool) $public;
    }

    public function setTotal($total)
    {
        $this->total = (int) $total > 0 ? $total : 0;
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "ASSETS_TAGS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Tag
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ARCHIVE_TAG_KEY";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Key id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Tags
     *
     * @return Array|Int
     * @throws Exception
     */
    public static function query(Query $q, $count = false)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            if (!$count) {
                $parameters = [ [$field, PDO::PARAM_INT],
                                [$query, PDO::PARAM_STR],
                                [$filter, PDO::PARAM_STR],
                                [$limit_a, PDO::PARAM_INT],
                                [$limit_b, PDO::PARAM_INT]];
            } else {
                $parameters = [ [$field, PDO::PARAM_INT],
                                [$query, PDO::PARAM_STR],
                                [$filter, PDO::PARAM_STR]];
            }

            $name = !$count ? "ARCHIVE_TAGS_KEYS" : "ARCHIVE_TAGS_KEYS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            if ($count) {
                return (int) $data[0]['total'];
            } else {
                $temp = [];
                foreach ($data as $tag) {
                    $tagInstance = self::createInstance($tag);
                    array_push($temp, $tagInstance);
                }

                return $temp;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Tags
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->by = \Stage\Asset\Tag\Query::ALL;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * save
     *
     * Save a Tag
     *
     * @return $this
     * @throws Exception
     */
    public function save($ifExists = 0)
    {
        try {
            $name = "ARCHIVE_TAGS_KEYS";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$this->getName(), PDO::PARAM_STR],
                            [$this->getNameStripped(), PDO::PARAM_STR],
                            [$this->getPublic(), PDO::PARAM_BOOL],
                            [$ifExists, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_key']);

            // Fire an event
            if (!$this->getId()) {
                \Stage\Events\Event::fire("assets.tag.created", $this);
            } else {
                \Stage\Events\Event::fire("assets.tag.updated", $this);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Tag
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ARCHIVE_TAGS_KEY";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_key']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setPublic($tag['public']);
        $tagInstance->setTotal(isset($tag['total']) ? (int) $tag['total'] : 0);

        return $tagInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
