<?php namespace Stage\Archive\Tag;

use PDO;
use Exception;
use Stage\Stage;

class Tag
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    public $parentId;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $public = true;

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $totalClips;

    /**
     * @var
     */
    private $totalSegments;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','nameStripped','public','total','totalClips','totalSegments','parentId');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getParentId()
    {
        return $this->parentId;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getPublic()
    {
        return $this->public;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getTotalClips()
    {
        return $this->totalClips;
    }

    public function getTotalSegments()
    {
        return $this->totalSegments;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setParentId($id)
    {
        $this->parentId = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setPublic($public)
    {
        $this->public = (bool) $public;
    }

    public function setTotal($total)
    {
        $this->total = (int) $total > 0 ? $total : 0;
    }

    public function setTotalClips($total)
    {
        $this->totalClips = (int) $total > 0 ? $total : 0;
    }

    public function setTotalSegments($total)
    {
        $this->totalSegments = (int) $total > 0 ? $total : 0;
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "ARCHIVE_TAGS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * save
     *
     * Save a tag to the database and return a Tag instance
     *
     * @return $this
     * @throws Exception
     */
    public function save($ifExists = 0)
    {
        try {
            $name = "ARCHIVE_TAGS";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$this->getParentId(), PDO::PARAM_INT],
                            [$this->getName(), PDO::PARAM_STR],
                            [$this->getNameStripped(), PDO::PARAM_STR],
                            [$this->getPublic(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_tag']);

            // Fire an event
            if (!$this->getId()) {
                \Stage\Events\Event::fire("tag.created", $this);
            } else {
                \Stage\Events\Event::fire("tag.updated", $this);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * delete
     *
     * Delete a Tag
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ARCHIVE_TAG";
            $method = "DEL";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * @param $idKey
     * @param $variantValue
     * @return Variant
     * @throws Exception
     */
    public function addVariants($idKey, $variantValue)
    {
        try {
            $name = "ARCHIVE_TAGS_VARIANTS";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                            [$idKey, PDO::PARAM_INT],
                            [$variantValue, PDO::PARAM_INT]];
            $variant = Stage::esp($parameters, $name, $method);
            return Variant::createInstance($variant[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function variants()
    {
        try {
            // Get from table archive_tags_variants_related
            $name = "ARCHIVE_TAGS_VARIANTS_RELATED";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($variant) {
                return Variant::createInstance($variant);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idKey
     * @return $this|bool
     * @throws Exception
     */
    public function removeVariants($idKey)
    {
        try {
            // Remove from table archive_tags_variants_related
            $name = "ARCHIVE_TAGS_VARIANT";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                           [$idKey, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0) {
                return $this;
            } else {
                return false;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * Finds a Tag
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ARCHIVE_TAG";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Tag id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }


    /**
     * all
     *
     * Returns all Tags
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->by = Query::ALL;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * query
     *
     * Search Tags
     *
     * @return array
     * @throws Exception
     */
    public static function query(Query $q, $count = false)
    {
        $field = $q->by;
        $idGroup = $q->idGroup;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            if (!$count) {
                $parameters = [ [$field, PDO::PARAM_STR],
                    [$idGroup, PDO::PARAM_INT],
                    [$query, PDO::PARAM_INT],
                    [$filter, PDO::PARAM_INT],
                    [$limit_a, PDO::PARAM_INT],
                    [$limit_b, PDO::PARAM_INT]];
            } else {
                $parameters = [ [$field, PDO::PARAM_STR],
                    [$idGroup, PDO::PARAM_INT],
                    [$query, PDO::PARAM_INT],
                    [$filter, PDO::PARAM_INT]];
            }

            $name = !$count ? "ARCHIVE_TAGS" : "ARCHIVE_TAGS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            if ($count) {
                return (int) $data[0]['total'];
            } else {
                $temp = [];
                foreach ($data as $tag) {
                    $tagInstance = self::createInstance($tag);
                    array_push($temp, $tagInstance);
                }

                return $temp;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_tag']);
        if (isset($tag['id_group'])) {
            $tagInstance->setParentId($tag['id_group']);
        } else { // In case of /root folder
            if (isset($tagInstance->parentId)) {
                unset($tagInstance->parentId);
            }
        }
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setPublic($tag['public']);
        $tagInstance->setTotal(isset($tag['total_videos']) ? (int) $tag['total_videos'] : 0);
        $tagInstance->setTotalClips(isset($tag['total_clips']) ? (int) $tag['total_clips'] : 0);
        $tagInstance->setTotalSegments(isset($tag['total_segments']) ? (int) $tag['total_segments'] : 0);

        return $tagInstance;
    }
}
