<?php namespace Stage\Archive\Transfer;

use PDO;
use DateTime;
use Exception;
use Stage\Helper;
use Stage\Stage;
use Stage\Archive\Transfer\Content\Query as ContentQuery;

class Transfer
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * Original name of the video
     * @var array
     */
    private $code;

    /**
     * @var int
     */
    private $totalContent;

    /**
     * @var string
     */
    private $filename = '';

    /**
     * @var int
     */
    private $seconds;

    /**
     * @var boolean
     */
    private $isClip = false;

    /**
     * @var boolean
     */
    private $createXml = false;

    /**
     * Filename
     * @var array
     */
    private $password = '';

    /**
     * @var string
     */
    private $endDate = '';

    /**
     * @var boolean
     */
    private $active = true;

    /**
     * @var integer
     */
    private $idStorage = "";

    /**
     * @var integer
     */
    private $idCdn = "";

    /**
     * @var string
     */
    private $registerDate = '';

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'code', 'password', 'endDate','registerDate','totalContent','filename','seconds','isClip','active','createXml');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getCode()
    {
        return $this->code;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getPassword()
    {
        return $this->password;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getTotalContent()
    {
        return $this->totalContent;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getSeconds()
    {
        return $this->seconds;
    }

    public function isClip()
    {
        return (bool) $this->isClip;
    }

    public function createXml()
    {
        return (bool) $this->createXml;
    }

    public function isActive()
    {
        return $this->active;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setCode($code)
    {
        $this->code = $code;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setPassword($password)
    {
        $this->password = $password;
    }

    public function setEndDate($date)
    {
        if ($date==null) {
            $this->endDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    public function setTotalContent($total)
    {
        $this->totalContent = (int) $total;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setSeconds($seconds)
    {
        $this->seconds = $seconds;
    }

    public function setIsClip($isClip)
    {
        $this->isClip = (bool) $isClip;
    }

    public function setIsActive($active)
    {
        $this->active = (boolean) $active;
    }

    public function setCreateXml($active)
    {
        $this->createXml = (boolean) $active;
    }

    public function setIdStorage($id)
    {
        return $this->idStorage = $id;
    }

    public function setIdCdn($id)
    {
        $this->idCdn = $id;
    }

    /**
     *
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        $name = "ARCHIVE_TRANSFERS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * all
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        try {
            $query = new Query();
            $query->by = Query::ALL;
            $query->term = '';
            $query->from = 0;
            $query->to = -1;

            $transfers = self::query($query);
            return $transfers;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * Find a Transfer
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ARCHIVE_TRANSFER";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Transfer id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a transfer
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$this->getName(), PDO::PARAM_INT],
                       [$this->getCode(), PDO::PARAM_STR],
                       [$this->getPassword(), PDO::PARAM_STR],
                       [$this->getEndDate(), PDO::PARAM_INT],
                       [$this->isActive(), PDO::PARAM_BOOL],
                       [$this->createXml(), PDO::PARAM_BOOL]];

        try {
            $name = "ARCHIVE_TRANSFER";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_transfer']) && $data[0]['id_transfer'] >= 0) {
                $idVideo = $data[0]['id_transfer'];
                $this->setId($idVideo);

                self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * query
     *
     * @param Query $q
     * @return array
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $method = 'GET';
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "ARCHIVE_TRANSFERS";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($transfer) {
                return self::createInstance($transfer);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * @param \Stage\Post\Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            $method = 'GET';
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT]];

            $name = "ARCHIVE_TRANSFERS_COUNT";
            $data = Stage::esp($parameters, $name, $method);
            return (int) $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryContent
     *
     * @param Query $q
     * @return array
     * @throws Exception
     */
    public function queryContent(ContentQuery $q)
    {
        $query = $q->term;
        $field = $q->by;
        $filter = $q->filter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $method = 'GET';
            $parameters = [ [$this->getId(), PDO::PARAM_STR],
                            [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$limit_size_a, PDO::PARAM_INT],
                            [$limit_size_b, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "ARCHIVE_CONTENT_ON_TRANSFERS";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryContentCount
     *
     * @param Query $q
     * @return array
     * @throws Exception
     */
    public function queryContentCount(ContentQuery $q)
    {
        $query = $q->term;
        $field = $q->by;
        $filter = $q->filter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;

        try {
            $method = 'GET';
            $parameters = [ [$this->getId(), PDO::PARAM_STR],
                            [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_STR],
                            [$filter, PDO::PARAM_INT],
                            [$limit_size_a, PDO::PARAM_INT],
                            [$limit_size_b, PDO::PARAM_INT]];

            $name = "ARCHIVE_CONTENT_ON_TRANSFERS_COUNT";
            $data = Stage::esp($parameters, $name, $method);
            return $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * toTransfer
     *
     * @return array
     * @throws Exception
     */
    public function add($field, $filter, $position)
    {
        try {

            // $field = 0;     // 1 - videos, 2 - clips
            // $filter = 0;    // id_video or id_clip
            $method = 'SET';
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$field, PDO::PARAM_INT],
                            [$filter, PDO::PARAM_INT],
                            [$position, PDO::PARAM_INT]];

            $name = "ARCHIVE_CONTENT_TO_TRANSFER";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * remove
     *
     * @param $field
     * @param $filter
     * @return array
     * @throws Exception
     */
    public function remove($field, $filter)
    {
        try {

            // $field = 0;     // 1 - videos, 2 - clips
            // $filter = 0;    // id_video or id_clip
            $method = 'DEL';
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$field, PDO::PARAM_INT],
                            [$filter, PDO::PARAM_INT]];

            $name = "ARCHIVE_CONTENT_FROM_TRANSFER";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @return array
     * @throws Exception
     */
    public function delete()
    {
        try {
            $method = 'DEL';
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];

            $name = "ARCHIVE_TRANSFER";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Transfer
     */
    public static function createInstance($data)
    {
        $t = new self();
        $t->setId($data['id_transfer']);
        $t->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $t->setCode($data['code']);
        $t->setIsActive($data['active']);
        if (isset($data['password'])) {
            $t->setPassword($data['password']);
        }

        $t->setFilename($data['filename']);
        $t->setSeconds($data['seconds']);
        $t->setIsClip($data['is_clip']);
        $t->setCreateXml($data['create_xml']);
        $t->setTotalContent($data['total_content']);

        if (isset($data['id_storage'])) {
            $t->setIdStorage($data['id_storage']);
        }

        if (isset($data['id_cdn'])) {
            $t->setIdCdn($data['id_cdn']);
        }

        if (Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $t->setEndDate(new DateTime($data['end_date']));
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $t->setRegisterDate(new DateTime($data['register_date']));
        }

        return $t;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
