<?php namespace Stage\Archive\Video;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class Preview
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * Filename
     * @var array
     */
    private $filename;

    /**
     * Extension
     * @var array
     */
    private $extension;

    /**
     * Width
     * @var
     */
    private $width;

    /**
     * Height
     * @var
     */
    private $height;

    /**
     * Video filesize in bytes
     * @var
     */
    private $size;

    /**
     * @var
     */
    private $bitrate;

    /**
     * @var integer
     */
    private $idStorage = "";

    /**
     * @var integer
     */
    private $idCdn = "";

    /**
     * @var string
     */
    private $registerDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','filename','extension','width','height','size','bitrate','registerDate', 'idStorage', 'idCdn');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getExtension()
    {
        return $this->extension;
    }

    public function getWidth()
    {
        return (int) $this->width;
    }

    public function getHeight()
    {
        return (int) $this->height;
    }

    public function getSize()
    {
        return (int) $this->size;
    }

    public function getBitrate()
    {
        return $this->bitrate;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setExtension($extension)
    {
        $this->extension = strtolower(trim($extension));
    }

    public function setWidth($width)
    {
        $this->width = (int)$width;
    }

    public function setHeight($height)
    {
        $this->height = (int)$height;
    }

    public function setSize($size)
    {
        $this->size = (int)$size;
    }

    public function setBitrate($bitrate)
    {
        $this->bitrate = (int) $bitrate;
    }

    public function setIdStorage($id)
    {
        return $this->idStorage = $id;
    }

    public function setIdCdn($id)
    {
        $this->idCdn = $id;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    /**
     * save
     *
     * Save a video
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                           [$this->getFilename(), PDO::PARAM_INT],
                           [$this->getSize(), PDO::PARAM_INT],
                           [$this->getExtension(), PDO::PARAM_STR],
                           [$this->getBitrate(), PDO::PARAM_INT],
                           [$this->getWidth(), PDO::PARAM_INT],
                           [$this->getHeight(), PDO::PARAM_INT],
                           [$this->getIdStorage(), PDO::PARAM_INT],
                           [$this->getIdCdn(), PDO::PARAM_INT]];

            $name = "SET_ARCHIVE_VIDEO_PREVIEW";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * preview
     *
     * @param $id
     * @return Preview | array
     * @throws Exception
     */
    public static function preview($id)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEO_PREVIEW";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && ($data[0]['return_code']) == "-2") {
                return [];
            }

            return isset($data[0]) ? self::createInstance($data[0]) : [];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Preview
     */
    public static function createInstance($data)
    {
        $preview = new self();
        // $preview->setId($data['id_video']);
        $preview->setFilename($data['filename']);
        $preview->setExtension($data['extension']);
        $preview->setWidth($data['width']);
        $preview->setHeight($data['height']);
        $preview->setSize($data['size']);
        $preview->setBitrate($data['bitrate']);

        if (isset($data['id_storage'])) {
            $preview->setIdStorage($data['id_storage']);
        }

        if (isset($data['id_cdn'])) {
            $preview->setIdCdn($data['id_cdn']);
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $preview->setRegisterDate(new DateTime($data['register_date']));
        }

        return $preview;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
