<?php namespace Stage\Archive\Video\Room;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;

class Room
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $directoryId;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $tags = [];

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'directoryId', 'name','description','registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getDirectoryId()
    {
        return $this->directoryId;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setDirectoryId($id)
    {
        $this->directoryId = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setMimeType($mimeType)
    {
        $this->mimeType = strtolower(trim($mimeType));
    }

    public function setSize($size)
    {
        $this->size = (int) $size;
    }

//    public function setWidth($width)
//    {
//        $this->width = (int) $width;
//    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * @param $idasset
     * @param $room
     * @param $public
     * @param $idUser
     * @return Room
     * @throws Exception
     */
    public static function remove($idVideo, $room, $public, $idUser)
    {
        try {
            $name = "ARCHIVE_FROM_ROOM";
            $method = "DEL";
            $parameters = [[$idVideo, PDO::PARAM_INT],
                            [$room, PDO::PARAM_STR],
                            [$public, PDO::PARAM_BOOL],
                            [$idUser, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Video id $idVideo not exists.", $data[0]['return_code']);
            } else {
                return true;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idasset
     * @param $room
     * @param $public
     * @param $idUser
     * @return Bool
     * @throws Exception
     */
    public static function add($idVideo, $room, $public, $idUser)
    {
        try {
            $name = "ARCHIVE_TO_ROOM";
            $method = "SET";
            $parameters = [[$idVideo, PDO::PARAM_INT],
                            [$room, PDO::PARAM_STR],
                            [$public, PDO::PARAM_BOOL],
                            [$idUser, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Video id $idVideo not exists.", $data[0]['return_code']);
            } else {
                return true;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idUser
     * @return array
     * @throws Exception
     */
    public static function sizeof($idUser)
    {
        $name = "ARCHIVE_ROOMS_NUM";
        $method = "GET";
        $parameters = [[$idUser, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param $room
     * @param $public
     * @param $idUser
     * @return Bool
     * @throws Exception
     */
    public static function delete($room, $public, $idUser)
    {
        try {
            $name = "ARCHIVE_ROOM";
            $method = "DEL";
            $parameters = [[$room, PDO::PARAM_STR],
                            [$public, PDO::PARAM_BOOL],
                            [$idUser, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Room $room not exists.", $data[0]['return_code']);
            } else {
                return true;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idasset
     * @return array
     * @throws Exception
     */
    public static function removeAllTags($idVideo, $idUser)
    {
        try {
            $name = "ARCHIVE_FROM_ROOMS";
            $method = "DEL";
            $parameters = [[$idVideo, PDO::PARAM_INT],
                           [$idUser, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     * @throws Exception
     */
    public static function createInstance($asset)
    {
        $assetInstance = new self();
        $assetInstance->setId($asset['id_asset']);

        $assetInstance->directoryId = (int) $asset['id_directory'];
        $assetInstance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
//        $assetInstance->setMimeType($asset['mime_type']);

        if (Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
