<?php namespace Stage\Archive\Video;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;
use Stage\Archive\Tag\Tag;
use Stage\Archive\Video\Query as VideoQuery;
use Stage\Archive\Video\Thumbnail\Thumbnail;
use Stage\Archive\Video\Thumbnail\Query as VideoThumbnailQuery;

class Video
{
    const STATE2 = 2;
    const STATE3 = 3;
    const STATE4 = 4;
    const STATE5 = 5;

    const FILENAME = 1;
    const ID = 0;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idDirectory;

    /**
     * Original name of the video
     * @var array
     */
    private $name;

    /**
     * Filename
     * @var array
     */
    private $filename;

    /**
     * The video duration in seconds
     * @var
     */
    private $duration;

    /**
     * @var
     */
    private $extension;

    /**
     * @var
     */
    private $mimeType;

    /**
     * Video filesize in bytes
     * @var
     */
    private $size;

    /**
     * @var
     */
    private $bitrate;

    /**
     * @var
     */
    private $width;

    /**
     * @var
     */
    private $height;

    /**
     * @var
     */
    private $ratio;

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $codecAudio;

    /**
     * @var
     */
    private $codecVideo;

    /**
     * @var
     */
    private $audioBitrate;

    /**
     * @var string
     */
    private $defaultThumbnail = "";

    /**
     * @var string
     */
    private $registerDate = "";

    /**
     * @var boolean
     */
    private $hasPreview = false;

    /**
     * @var boolean
     */
    private $hasClips = false;

    /**
     * @var boolean
     */
    private $inTransfer = false;

    /**
     * @var
     */
    private $tags = [];

    /**
     * @var integer
     */
    private $framerate = 0;

    /**
     * @var integer
     */
    private $framecount = 0;

    /**
     * @var integer
     */
    private $idStorage = "";

    /**
     * @var integer
     */
    private $idCdn = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','idDirectory','name','filename','duration','extension','mimeType','size','bitrate','width','height','ratio','state','defaultThumbnail','registerDate','codecAudio','codecVideo','audioBitrate','hasPreview','hasClips','inTransfer','framerate', 'framecount', 'idStorage', 'idCdn');

    /**
 * The attributes that should be mutated to dates.
 *
 * @var array
 */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdDirectory()
    {
        return $this->idDirectory;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getDuration()
    {
        return $this->duration;
    }

    public function getExtension()
    {
        return $this->extension;
    }

    public function getMimeType()
    {
        return $this->mimeType;
    }

    public function getSize()
    {
        return (int) $this->size;
    }

    public function getBitrate()
    {
        return $this->bitrate;
    }

    public function getWidth()
    {
        return (int) $this->width;
    }

    public function getHeight()
    {
        return (int) $this->height;
    }

    public function getRatio()
    {
        return $this->ratio;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getCodecAudio()
    {
        return $this->codecAudio;
    }

    public function getCodecVideo()
    {
        return $this->codecVideo;
    }

    public function getAudioBitrate()
    {
        return $this->audioBitrate ? $this->audioBitrate : 0;
    }

    public function getDefaultThumbnail()
    {
        return $this->defaultThumbnail;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function hasPreview()
    {
        return $this->hasPreview;
    }

    public function hasClips()
    {
        return $this->hasClips;
    }

    public function inTransfer()
    {
        return $this->inTransfer;
    }

    public function getFramerate()
    {
        return $this->framerate;
    }

    public function getFramecount()
    {
        return $this->framecount;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdDirectory($id)
    {
        $this->idDirectory = $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setDuration($seconds)
    {
        $this->duration = $seconds;
    }

    public function setExtension($extension)
    {
        $this->extension = strtolower(trim($extension));
    }

    public function setMimeType($mimeType)
    {
        $this->mimeType = strtolower(trim($mimeType));
    }

    public function setSize($size)
    {
        $this->size = (int)$size;
    }

    public function setBitrate($bitrate)
    {
        $this->bitrate = $bitrate;
    }

    public function setWidth($width)
    {
        $this->width = (int)$width;
    }

    public function setHeight($height)
    {
        $this->height = (int)$height;
    }

    public function setRatio($ratio)
    {
        $this->ratio = $ratio;
    }

    public function setState($state)
    {
        $this->state = (int) $state;
    }

    public function setCodecAudio($codec)
    {
        $this->codecAudio = $codec;
    }

    public function setCodecVideo($codec)
    {
        $this->codecVideo = $codec;
    }

    public function setAudioBitrate($bitrate)
    {
        $this->audioBitrate = $bitrate;
    }

    public function setDefaultThumbnail($thumbnail)
    {
        $this->defaultThumbnail = $thumbnail;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    private function setHasPreview($has)
    {
        $this->hasPreview = (boolean) $has;
    }

    private function setHasClips($has)
    {
        $this->hasClips = (boolean) $has;
    }

    private function setInTransfer($in)
    {
        $this->inTransfer = (boolean) $in;
    }

    public function setFramerate($framerate)
    {
        $this->framerate = $framerate;
    }

    public function setFramecount($framecount)
    {
        $this->framecount = $framecount;
    }

    public function setIdStorage($id)
    {
        return $this->idStorage = $id;
    }

    public function setIdCdn($id)
    {
        $this->idCdn = $id;
    }

    /**
     *
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        $name = "ARCHIVE_VIDEOS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * dates
     *
     * @return array
     */
    public static function dates(VideoQuery $q)
    {
        $room           = $q->room;
        $idUser         = $q->idUser;
        $idDirectory    = $q->idDirectory;
        $field          = $q->by;
        $sndField       = $q->sndBy;
        $trdField       = $q->trdBy;
        $term           = $q->term;
        $filter         = $q->filter;
        $sndFilter      = $q->sndFilter;
        $trdFilter      = $q->trdFilter;
        $limit_sys_a    = $q->fromSize;
        $limit_sys_b    = $q->toSize;

        $parameters = [[$idDirectory, PDO::PARAM_INT],
            [$field, PDO::PARAM_INT],
            [$sndField, PDO::PARAM_INT],
            [$trdField, PDO::PARAM_INT],
            [$term, PDO::PARAM_STR],
            [$filter, PDO::PARAM_INT],
            [$sndFilter, PDO::PARAM_INT],
            [$trdFilter, PDO::PARAM_INT],
            [$limit_sys_a, PDO::PARAM_INT],
            [$limit_sys_b, PDO::PARAM_INT]];

        $name = "ARCHIVE_VIDEOS_DATES";
        $method = "GET";
        $data = Stage::esp($parameters, $name, $method);

        return $data;
    }

    /**
     * systemTags
     *
     * @param int $field
     * @param int $limit_a In bytes
     * @param int $limit_b In bytes
     * @return array
     */
    public static function systemTags(VideoQuery $q)
    {
        $room           = $q->room;
        $idUser         = $q->idUser;
        $idDirectory    = $q->idDirectory;
        $field          = $q->by;
        $sndField       = $q->sndBy;
        $trdField       = $q->trdBy;
        $term           = $q->term;
        $filter         = $q->filter;
        $sndFilter      = $q->sndFilter;
        $trdFilter      = $q->trdFilter;
        $limit_sys_a    = $q->fromSize;
        $limit_sys_b    = $q->toSize;

        $parameters = [[$idDirectory, PDO::PARAM_INT],
                        [$room, PDO::PARAM_STR],
                        [$idUser, PDO::PARAM_INT],
                        [$field, PDO::PARAM_INT],
                        [$sndField, PDO::PARAM_INT],
                        [$trdField, PDO::PARAM_INT],
                        [$term, PDO::PARAM_STR],
                        [$filter, PDO::PARAM_INT],
                        [$sndFilter, PDO::PARAM_INT],
                        [$trdFilter, PDO::PARAM_INT],
                        [$limit_sys_a, PDO::PARAM_INT],
                        [$limit_sys_b, PDO::PARAM_INT]];

        $name = "ARCHIVE_VIDEOS_SYSTEM_TAGS";
        $method = "GET";
        $tags = Stage::esp($parameters, $name, $method);

        return $tags;
    }

    /**
     * find
     *
     * Find a Video
     *
     * @return $this
     * @throws Exception
     */
    public static function find($value, $field = self::ID, $idUser = 0)
    {
        try {
            $name = "ARCHIVE_VIDEO";
            $method = "GET";
            $parameters = [[$idUser, PDO::PARAM_INT],
                           [$field, PDO::PARAM_INT],
                           [$value, PDO::PARAM_STR]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Video id $value not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Video from the database or mark it to be deleted.
     * The video is only removed from database if softDelete is true and the video was already marked to be deleted
     *
     * @param $idAsset
     * @param bool $softDelete
     * @param bool $deleted
     * @throws Exception
     */
    public static function delete($idVideo, $softDelete = true, $deleted = true)
    {
        if ($softDelete) {
            $name = "ARCHIVE_VIDEO_DELETED";    // Only mark the video has deleted and not remove from the database
            $parameters = [ [$idVideo, PDO::PARAM_INT],
                            [$deleted, PDO::PARAM_BOOL]];
            $method = "SET";
        } else {
            $name = "ARCHIVE_VIDEO";            // Remove from the database
            $parameters = [[$idVideo, PDO::PARAM_INT]];
            $method = "DEL";
        }
        try {
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Video id $idVideo not exists in database", $data[0]['return_code']);
            }

            return $idVideo;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a video
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$this->getIdDirectory(), PDO::PARAM_INT],
                       [$this->getName(), PDO::PARAM_STR],
                       [$this->getFilename(), PDO::PARAM_STR],
                       [$this->getDuration(), PDO::PARAM_INT],
                       [$this->getExtension(), PDO::PARAM_STR],
                       [$this->getMimeType(), PDO::PARAM_STR],
                       [$this->getSize(), PDO::PARAM_INT],
                       [$this->getCodecVideo(), PDO::PARAM_STR],
                       [$this->getBitrate(), PDO::PARAM_INT],
                       [$this->getAudioBitrate(), PDO::PARAM_INT],
                       [$this->getCodecAudio(), PDO::PARAM_STR],
                       [$this->getFramerate(), PDO::PARAM_INT],
                       [$this->getFramecount(), PDO::PARAM_INT],
                       [$this->getWidth(), PDO::PARAM_INT],
                       [$this->getHeight(), PDO::PARAM_INT],
                       [$this->getRatio(), PDO::PARAM_STR],
                       [$this->getIdStorage(), PDO::PARAM_INT],
                       [$this->getIdCdn(), PDO::PARAM_INT]];

        try {
            $name = "ARCHIVE_VIDEO";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_video']) && $data[0]['id_video'] >= 0) {
                $idVideo = $data[0]['id_video'];
                $this->setId($idVideo);

                if (Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * allTagsRelatedWithVideos
     *
     * @return array
     * @throws Exception
     */
    public static function allTagsRelatedWithVideos($idVideo, VideoQuery $q)
    {
        try {
            $room           = $q->room;
            $idUser         = $q->idUser;
            $idDirectory    = $q->idDirectory;
            $field          = $q->by;
            $sndField       = $q->sndBy;
            $trdField       = $q->trdBy;
            $term           = $q->term;
            $filter         = $q->filter;
            $sndFilter      = $q->sndFilter;
            $trdFilter      = $q->trdFilter;
            $limit_sys_a    = $q->fromSize;
            $limit_sys_b    = $q->toSize;

            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idVideo, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_sys_a, PDO::PARAM_INT],
                [$limit_sys_b, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the video tags
     * @return array
     * @throws Exception
     */
    public function tags(VideoQuery $q = null)
    {
        try {
            if (!$q) {
                $q = new VideoQuery();
            }

            $idVideo        = $this->getId();
            $room           = $q->room;
            $idUser         = $q->idUser;
            $idDirectory    = $q->idDirectory;
            $field          = $q->by;
            $sndField       = $q->sndBy;
            $trdField       = $q->trdBy;
            $term           = $q->term;
            $filter         = $q->filter;
            $sndFilter      = $q->sndFilter;
            $trdFilter      = $q->trdFilter;
            $limit_sys_a    = $q->fromSize;
            $limit_sys_b    = $q->toSize;

            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idVideo, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_sys_a, PDO::PARAM_INT],
                [$limit_sys_b, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag $tag
     * @throws Exception
     */
    public function addTag(Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [ [$idTag, PDO::PARAM_INT],
                            [$this->getId(), PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $this->tags = $this->tags();
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * markedToDelete
     *
     * Get the videos marked to be delete from database
     * @return array
     * @throws Exception
     */
    public static function markedToDelete()
    {
        try {
            $parameters = [];
            $name = "ARCHIVE_VIDEO_DELETED";
            $method = "GET";
            $videos = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $videos);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * thumbnails
     *
     * @param Query $q
     * @return array
     * @throws Exception
     */
    public static function thumbnails(VideoThumbnailQuery $q)
    {
        try {
            $field = $q->by;
            $term = $q->term;
            $limit_a = $q->from;
            $limit_b = $q->to;

            $parameters = [[$field, PDO::PARAM_INT],
                            [$term, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEO_THUMBNAILS";
            $method = "GET";
            $thumbs = Stage::esp($parameters, $name, $method);

            return array_map(function ($thumb) {
                return Thumbnail::createInstance($thumb);
            }, $thumbs);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * segmentTagsCount
     *
     * @param $field
     * @param $filter
     * @return mixed
     * @throws Exception
     */
    public static function segmentTagsCount($field, $filter)
    {
        try {
            $parameters = [[$field, PDO::PARAM_INT],
                           [$filter, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_SEGMENT_TAGS_RELATED_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * segmentTags
     *
     * @param $field
     * @param $filter
     * @return array
     * @throws Exception
     */
    public static function segmentTags($field, $filter, $limit_a, $limit_b)
    {
        try {
            $parameters = [[$field, PDO::PARAM_INT],
                           [$filter, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_SEGMENT_TAGS_RELATED";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    $tag['tag_name'] = html_entity_decode($tag['tag_name'], ENT_COMPAT, 'UTF-8');
                    $tag['video_name'] = html_entity_decode($tag['video_name'], ENT_COMPAT, 'UTF-8');

                    if (isset($tag['register_date']) && Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
                        $tag['register_date'] = (new DateTime($tag['register_date']))->format(DateTime::ATOM);
                    }

                    return $tag;
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addSegmentTag
     *
     * @param $idTag
     * @param $idVideo
     * @param $secondsIn
     * @param $secondsOut
     * @throws Exception
     */
    public static function addSegmentTag($idTag, $idVideo, $secondsIn, $secondsOut)
    {
        try {
            $parameters = [[$idTag, PDO::PARAM_INT],
                            [$idVideo, PDO::PARAM_INT],
                            [$secondsIn, PDO::PARAM_INT],
                            [$secondsOut, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_SEGMENT_TAGS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeSegmentTag
     *
     * @param $idVideo
     * @return bool
     * @throws Exception
     */
    public static function removeSegmentTags($idVideo)
    {
        try {
            $parameters = [[$idVideo, PDO::PARAM_INT]];

            $name = "ARCHIVE_VIDEOS_SEGMENT_TAGS";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Video
     */
    public static function createInstance($data)
    {
        $video = new self();
        $video->setId($data['id_video']);
        $video->setIdDirectory($data['id_directory']);
        $video->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $video->setFilename($data['filename']);
        $video->setDuration($data['seconds']);
        $video->setExtension($data['extension']);
        $video->setMimeType($data['mime_type']);
        $video->setSize($data['size']);
        $video->setBitrate($data['bitrate']);
        $video->setWidth($data['width']);
        $video->setHeight($data['height']);
        $video->setRatio($data['ratio']);
        $video->setState($data['state']);
        $video->setDefaultThumbnail($data['thumbnail']);
        $video->setCodecAudio($data['audio_codec']);
        $video->setCodecVideo($data['codec']);
        $video->setAudioBitrate($data['audio_bitrate']);
        $video->setHasPreview($data['has_preview']);
        $video->setHasClips($data['has_clips']);
        $video->setInTransfer($data['in_transfer']);
        $video->setFramerate($data['framerate']);
        $video->setFramecount($data['framecount']);

        if (isset($data['id_storage'])) {
            $video->setIdStorage($data['id_storage']);
        }

        if (isset($data['id_cdn'])) {
            $video->setIdCdn($data['id_cdn']);
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $video->setRegisterDate(new DateTime($data['register_date']));
        }

        return $video;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
