<?php namespace Stage\Asset\Album;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Asset\Asset;
use Stage\Asset\Album\Query\Album as AlbumQuery;
use Stage\Asset\Query;

class Album
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
//    private $directory;
    private $directoryId;

    /**
     * @var
     */
    private $position = 0;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $image;

    /**
     * @var
     */
    private $imageAlign;

    /**
     * @var
     */
    private $imageTitle;

    /**
     * @var
     */
    private $imageAlterText;

    /**
     * @var
     */
    private $metaTitle;

    /**
     * @var
     */
    private $metaDescription;

    /**
     * @var
     */
    private $metaImage;

    /**
     * @var
     */
    private $metaKeywords;

    /**
     * @var
     */
    private $metaUrl;

    /**
     * @var
     */
    private $metaWatermarkImage;

    /**
     * @var
     */
    private $totalImages;  // This field is not part of the table, is only returned in the SP

    /**
     * @var
     */
    private $coverImage;  // This field is not part of the table, is only returned in the SP

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $tags = [];

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'description', 'link', 'image', 'imageAlign', 'imageTitle', 'imageAlterText', 'metaTitle', 'metaDescription', 'metaImage', 'metaKeywords', 'metaUrl', 'metaWatermarkImage', 'totalImages', 'coverImage', 'registerDate', 'directoryId', 'position');

    public function __construct()
    {
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setDirectoryId($id)
    {
        $this->directoryId = (int)$id;
    }

    public function setPosition($pos)
    {
        $this->position = (int) $pos;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setImage($image)
    {
       $this->image = $image;
    }

    public function setImageAlign($imageAlign)
    {
       $this->imageAlign = $imageAlign;
    }

    public function setImageTitle($imageTitle)
    {
       $this->imageTitle = $imageTitle;
    }

    public function setImageAlterText($imageAlterText)
    {
       $this->imageAlterText = $imageAlterText;
    }

    public function setMetaTitle($metaTitle)
    {
       $this->metaTitle = $metaTitle;
    }

    public function setMetaDescription($metaDescription)
    {
       $this->metaDescription = $metaDescription;
    }

    public function setMetaImage($metaImage)
    {
       $this->metaImage = $metaImage;
    }

    public function setMetaKeywords($metaKeywords)
    {
       $this->metaKeywords = $metaKeywords;
    }

    public function setMetaUrl($metaUrl)
    {
       $this->metaUrl = $metaUrl;
    }

    public function setMetaWatermarkImage($metaWatermarkImage)
    {
       $this->metaWatermarkImage = $metaWatermarkImage;
    }

    private function setTotalImages($total)
    {
        $this->totalImages = (int)$total;
    }

    private function setCoverImage($image)
    {
        $this->coverImage = $image;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getDirectoryId()
    {
        return $this->directoryId;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getImageAlign()
    {
        return $this->imageAlign;
    }

    public function getImageTitle()
    {
        return $this->imageTitle;
    }

    public function getImageAlterText()
    {
        return $this->imageAlterText;
    }

    public function getMetaTitle()
    {
        return $this->metaTitle;
    }

    public function getMetaDescription()
    {
        return $this->metaDescription;
    }

    public function getMetaImage()
    {
        return $this->metaImage;
    }

    public function getMetaKeywords()
    {
        return $this->metaKeywords;
    }

    public function getMetaUrl()
    {
        return $this->metaUrl;
    }

    public function getMetaWatermarkImage()
    {
        return $this->metaWatermarkImage;
    }

    public function getTotalImages()
    {
        return $this->totalImages;
    }

    public function getCoverImage()
    {
        return $this->coverImage;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    /**
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        $name = "ASSETS_ALBUMS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Album
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ASSETS_ALBUM";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Album id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Albums
     *
     * @return Array
     * @throws Exception
     */
    public static function all($isPublish = false)
    {
        $query = new AlbumQuery();
        $query->term = "";
        $query->from = 0;
        $query->to = -1;

        try {
            return self::query($query, $isPublish);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Album
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $action = $this->getId() == -1 ? "CREATE" : "UPDATE";
        $idDirectory = $this->getDirectoryId();

        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idDirectory, PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_STR],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getLink(), PDO::PARAM_STR],
            [$this->getImage(), PDO::PARAM_STR],
            [$this->getImageAlign(), PDO::PARAM_STR],
            [$this->getImageTitle(), PDO::PARAM_STR],
            [$this->getImageAlterText(), PDO::PARAM_STR],
            [$this->getMetaTitle(), PDO::PARAM_STR],
            [$this->getMetaDescription(), PDO::PARAM_STR],
            [$this->getMetaImage(), PDO::PARAM_STR],
            [$this->getMetaKeywords(), PDO::PARAM_STR],
            [$this->getMetaUrl(), PDO::PARAM_STR],
            [$this->getMetaWatermarkImage(), PDO::PARAM_STR],];

        try {
            $name = "ASSETS_ALBUM";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if ($action == "CREATE") {
                $this->setId($data[0]['id_album']);
                \Stage\Events\Event::fire("assets.albums.created", $this);
            } elseif ($action == "UPDATE") {
                \Stage\Events\Event::fire("assets.albums.updated", $this);
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Album
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ASSETS_ALBUM";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }


    /**
     * query
     *
     * Query a Album
     *
     * @return Array
     * @throws Exception
     */
    public static function query(AlbumQuery $q, $isPublish = false)
    {
        $idDirectory = $q->idDirectory;
        $query = $q->term;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $name = $isPublish ? "PUBLISH_ASSETS_ALBUMS" : "ASSETS_ALBUMS";

            if ($isPublish) {
                $parameters = [[$query, PDO::PARAM_STR],
                    [$limit_a, PDO::PARAM_INT],
                    [$limit_b, PDO::PARAM_INT]];
            } else {
                $field = $q->by;
                $sndField = $q->sndBy;
                $trdField = $q->trdBy;
                $sndFilter = $q->sndFilter;
                $trdFilter = $q->trdFilter;
                $filter = $q->filter;
                $orderBy = $q->orderBy;
                $order = $q->order;

                $parameters = [[$idDirectory, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$sndField, PDO::PARAM_INT],
                    [$trdField, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                    [$sndFilter, PDO::PARAM_INT],
                    [$trdFilter, PDO::PARAM_INT],
                    [$limit_a, PDO::PARAM_INT],
                    [$limit_b, PDO::PARAM_INT],
                    [$orderBy, PDO::PARAM_INT],
                    [$order, PDO::PARAM_INT]];
            }

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $album) {
                $albumInstance = self::createInstance($album);
                array_push($temp, $albumInstance);
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param AlbumQuery $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(AlbumQuery $q)
    {
        try {
            $name = "ASSETS_ALBUMS_COUNT";

            $idDirectory = $q->idDirectory;
            $query = $q->term;
            $field = $q->by;
            $sndField = $q->sndBy;
            $trdField = $q->trdBy;
            $sndFilter = $q->sndFilter;
            $trdFilter = $q->trdFilter;
            $filter = $q->filter;

            $parameters = [[$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT]];

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assets
     *
     * Query assets inside a album
     *
     * @param Query|null $q
     * @return array
     * @throws Exception
     */
    public function assets(Query $q)
    {
        try {
            $field = $q->by;
            $query = $q->term;
            $filter = $q->filter;
            $limit_size_a = $q->fromSize;
            $limit_size_b = $q->toSize;
            $limit_a = $q->from;
            $limit_b = $q->to;

            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_INT],
                [$filter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "ASSETS_ON_ALBUMS";
            $method = "GET";

            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($asset) {
                return Asset::createInstance($asset);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assetsCount
     *
     * @param Query $q
     * @return array
     * @throws Exception
     */
    public function assetsCount(Query $q)
    {
        try {
            $field = $q->by;
            $query = $q->term;
            $filter = $q->filter;
            $limit_size_a = $q->fromSize;
            $limit_size_b = $q->toSize;

            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_INT],
                [$filter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT]];

            $name = "ASSETS_ON_ALBUMS_COUNT";
            $method = "GET";

            $data = Stage::esp($parameters, $name, $method);
            return $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAsset
     *
     * @param Asset $asset
     * @return $this|Asset
     * @throws Exception
     */
    public function removeAsset(Asset $asset)
    {
        try {
            $name = "ASSETS_FROM_ALBUMS";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$asset->getId(), PDO::PARAM_INT]];

            Stage::esp($parameters, $name, $method);
            return $asset;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * associatedGalleries
     * Return the associated galleries to this Album
     * @return array
     * @throws Exception
     */
    public function associatedGalleries()
    {
        try {
            $id = $this->getId();
            $name = "ASSETS_ALBUMS_GALLERIES";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Album id $id not exists.", $data[0]['return_code']);
            } else {
                return $data;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        try {
            $id = $this->getId();
            $name = "ASSETS_ALBUM_TAGS_RELATED";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Album id $id not exists.", $data[0]['return_code']);
            } else {
                return array_map(function ($data) {
                    return Tag\Tag::createInstance($data);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag\Tag $tag
     * @throws Exception
     */
    public function addTag(Tag\Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idTag, PDO::PARAM_INT]];

            $name = "ASSETS_ALBUM_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $this->tags = $this->tags();
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "ASSETS_ALBUM_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idDirectory
     * @return Album
     * @throws Exception
     */
    public function duplicate($idDirectory)
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT]];
            $name = "ASSETS_ALBUM_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_album'])) {
                return self::createInstance($data[0]);
            }
            return [];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idAlbuma
     * @param $idAlbumb
     * @param $type
     * @return Album
     * @throws Exception
     */
    public static function sync($idAlbuma, $idAlbumb, $type)
    {
        try {
            $parameters = [[$idAlbuma, PDO::PARAM_INT],
                [$idAlbumb, PDO::PARAM_INT],
                [$type, PDO::PARAM_INT]];
            $name = "ASSETS_ALBUM_SYNC";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_album'])) {
                return self::createInstance($data[0]);
            }
            return [];
        } catch (Exception $e) {
            throw $e;
        }
    }


    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table posts_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: albums_custom_fields_related
        $idL10n = $this->getId();
        $spname = "ALBUMS_CUSTOM_FIELDS_VARIANTS";
        $method = "SET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * customFieldsVariants
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        $idAsset = $this->getId();
        $name = "ALBUMS_CUSTOM_FIELDS_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idAsset, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        if (isset($data[0]) && isset($data[0]['id_variant'])) {
            $result->setCount(sizeof($data));
        }

        return $result;
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        $idAsset = $this->getId();
        $name = "ALBUMS_CUSTOM_FIELD_VARIANT";
        $method = "DEL";
        $parameters = [[$idAsset, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }


    /**
     * variantKeys
     *
     * Search Messages
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function variantKeys(CustomFieldQuery $q, $count = false, $page = 1, $perPage = 15, $all = false)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $name = 'ALBUMS_CUSTOM_FIELDS_KEYS_COUNT';
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = !$all ? (isset($p) ? $p->to() : $q->to) : -1;

            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "ALBUMS_CUSTOM_FIELDS_KEYS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = CustomFieldKey::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            $result = new MyArrayObject($temp);
            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachVariantKey
     *
     * @param $idKey
     * @param $name
     * @param $nameStripped
     * @param $public
     * @param $ifExists
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists)
    {
        // table: albums_custom_fields_keys
        $spName = "ALBUMS_CUSTOM_FIELDS_KEY";
        $method = "SET";
        $parameters = [[$idKey, PDO::PARAM_INT],
            [$name, PDO::PARAM_STR],
            [$nameStripped, PDO::PARAM_STR],
            [$public, PDO::PARAM_BOOL],
            [$ifExists, PDO::PARAM_BOOL]];

        try {
            $data = Stage::esp($parameters, $spName, $method);
            if (isset($data[0]) && isset($data[0]['id_key'])) {
                return CustomFieldKey::createInstance($data[0]);
            }

            throw new Exception("Error creating custom field key", $data[0]['return_code']);

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachVariantKey
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public static function detachVariantKey($idKey)
    {
        $name = "ALBUMS_CUSTOM_FIELD_KEY";
        $method = "DEL";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * variantKey
     *
     * @param $idKey
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function variantKey($idKey)
    {
        $name = "ALBUMS_CUSTOM_FIELD_KEY";
        $method = "GET";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
            throw new Exception("The Key not exists", $data[0]['return_code']);
        } else {
            return CustomFieldKey::createInstance($data[0]);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();
        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "description" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Album
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id_album']);
        $instance->setName(html_entity_decode(trim($data['name']), ENT_COMPAT, 'UTF-8'));
        $instance->setDescription($data['description']);
        $instance->setLink($data['link']);
        $instance->setImage($data['image']);
        $instance->setImageAlign($data['image_align']);
        $instance->setImageTitle($data['image_title']);
        $instance->setImageAlterText($data['image_alter_text']);
        $instance->setMetaTitle($data['meta_title']);
        $instance->setMetaDescription($data['meta_description']);
        $instance->setMetaImage($data['meta_image']);
        $instance->setMetaKeywords($data['meta_keywords']);
        $instance->setMetaUrl($data['meta_url']);
        $instance->setMetaWatermarkImage($data['meta_watermark_image']);
        $instance->setTotalImages($data['total_images']);
        $instance->setCoverImage($data['cover_image']);
        $instance->setDirectoryId((int)$data['id_directory']);
        if (isset($asset['position'])) {
            $instance->setPosition($asset['position']);
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
