<?php namespace Stage\Asset;

use ArrayObject;
use PDO;
use DateTime;
use Exception;
use Stage\Asset\Extensions\ItemFilter;
use Stage\Asset\Label\Label;
use Stage\Asset\Playlist\Playlist;
use Stage\Asset\Tag\Tag;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Helper;

class Asset
{
    const FILENAME = 0;
    const ID = 1;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
//    private $directory;
    private $directoryId;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $description = "";

    /**
     * @var
     */
    private $imageTitle = "";

    /**
     * @var
     */
    private $imageAlterText = "";

    /**
     * @var
     */
    private $link = "";

    /**
     * @var
     */
    private $filename = "";

    /**
     * @var
     */
    private $extension = ""; // The file extension, like png, jpg, tiff...

    /**
     * @var
     */
    private $mimeType = "";

    /**
     * @var
     */
    private $size = 0;      // The file size in bytes */

    /**
     * @var
     */
    private $width = 0;     // The width of image

    /**
     * @var
     */
    private $height = 0;    // The height of image

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $position = 0;

    /**
     * @var
     */
    private $rooms;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $createdDate;

    /**
     * @var
     */
    private $tags = [];

    /**
     * @var integer
     */
    private $idStorage = 0;

    /**
     * @var integer
     */
    private $idCdn = 0;

    /**
     * @var integer
     */
    private $public = true;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'directoryId', 'name', 'description', 'imageTitle', 'imageAlterText', 'link', 'filename', 'extension', 'size', 'width', 'height', 'state', 'position', 'rooms', 'mimeType', 'idStorage', 'idCdn', 'public', 'registerDate', 'createdDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getDirectoryId()
    {
        return $this->directoryId;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getImageTitle()
    {
        return $this->imageTitle;
    }

    public function getImageAlterText()
    {
        return $this->imageAlterText;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getFilename($fullPath = false, $accountToken = null)
    {
        if ($fullPath) {
            $idStorage = $this->getIdStorage();
            $assetDirname = \Stage\Account\Map::getStorage($idStorage, \Stage\Account\Map::ASSETS)->getStoragePath();
            return sprintf("%s/%s", $this->paths($assetDirname, $accountToken, $this->getRegisterDate()), $this->getFilename());
        }

        return $this->filename;
    }

    public function getExtension()
    {
        return $this->extension;
    }

    public function getMimeType()
    {
        return $this->mimeType;
    }

    public function getSize()
    {
        return $this->size;
    }

    public function getWidth()
    {
        return $this->width;
    }

    public function getHeight()
    {
        return $this->height;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getRooms()
    {
        return $this->rooms;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    public function isPublic()
    {
        return $this->public;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getCreatedDate()
    {
        return $this->createdDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setDirectoryId($id)
    {
        $this->directoryId = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setImageTitle($imageTitle)
    {
        $this->imageTitle = $imageTitle;
    }

    public function setImageAlterText($imageAlterText)
    {
        $this->imageAlterText = $imageAlterText;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setExtension($extension)
    {
        $this->extension = strtolower(trim($extension));
    }

    public function setMimeType($mimeType)
    {
        $this->mimeType = strtolower(trim($mimeType));
    }

    public function setSize($size)
    {
        $this->size = (int)$size;
    }

    public function setWidth($width)
    {
        $this->width = (int)$width;
    }

    public function setHeight($height)
    {
        $this->height = (int)$height;
    }

    public function setState($state)
    {
        $this->state = (int)$state;
    }

    public function setPosition($pos)
    {
        $this->position = (int)$pos;
    }

    public function setRooms($rooms)
    {
        $this->rooms = $rooms;
    }

    public function setPublic($public)
    {
        return $this->public = $public;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    private function setCreatedDate($date)
    {
        $this->createdDate = $date;
    }

    public function setIdStorage($id)
    {
        return $this->idStorage = $id;
    }

    public function setIdCdn($id)
    {
        $this->idCdn = $id;
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "ASSETS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * dates
     *
     * @param int $idQuery Se se passar o id album então vai buscar as dates dentro de um album
     * @return array
     */
    public static function dates(Query $q)
    {
        $room = $q->room;
        $idUser = $q->idUser;
        $idDirectory = $q->idDirectory;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $term = $q->term;
        $filter = $q->filter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;

        try {
            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT]];

            $name = "ASSETS_DATES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * storage
     *
     * @return array
     */
    public static function storage()
    {
        $name = "ASSETS_STORAGE";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * all
     *
     * @return $this
     * @throws Exception
     */
    public static function all()
    {
        try {
            $query = new Query();
            $query->by = Query::ALL;
            $query->from = 0;
            $query->to = -1;

            $idDirectory = 0;
            $assets = Directory::assets($query, $idDirectory);
            return $assets;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * Find a Asset
     *
     * @return $this
     * @throws Exception
     */
    public static function find($value, $field = self::ID, $idUser = 0)
    {
        try {
            $name = "ASSETS_ASSET";
            $method = "GET";
            $parameters = [[$idUser, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$value, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset id $value not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Asset
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $action = $this->getId() == -1 ? "CREATE" : "UPDATE";

        $idDirectory = $this->getDirectoryId();

        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idDirectory, PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_STR],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getImageTitle(), PDO::PARAM_STR],
            [$this->getImageAlterText(), PDO::PARAM_STR],
            [$this->getLink(), PDO::PARAM_STR],
            [$this->getFilename(), PDO::PARAM_STR],
            [$this->getExtension(), PDO::PARAM_STR],
            [$this->getMimeType(), PDO::PARAM_STR],
            [$this->getSize(), PDO::PARAM_INT],
            [$this->getWidth(), PDO::PARAM_INT],
            [$this->getHeight(), PDO::PARAM_INT],
            [$this->getIdStorage(), PDO::PARAM_INT],
            [$this->getIdCdn(), PDO::PARAM_INT],
            [$this->isPublic(), PDO::PARAM_BOOL]];

        try {
            $name = "ASSETS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if ($action == "CREATE") {
                $this->setId($data[0]['id_asset']);
                \Stage\Events\Event::fire("assets.created", $this);
            } elseif ($action == "UPDATE") {
                \Stage\Events\Event::fire("assets.updated", $this);
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Asset from the database or mark it to be deleted
     *
     * @param $idAsset
     * @param bool $softDelete
     * @param bool $deleted
     * @throws Exception
     */
    public static function delete($idAsset, $softDelete = true, $deleted = true)
    {
        if ($softDelete) {
            $name = "ASSET_DELETED";    // Only mark the asset has deleted and not remove from the database
            $parameters = [[$idAsset, PDO::PARAM_INT],
                [$deleted, PDO::PARAM_BOOL]];
            $method = "SET";
        } else {
            $name = "ASSET";            // Remove from the database
            $parameters = [[$idAsset, PDO::PARAM_INT]];
            $method = "DEL";
        }
        try {
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset id $idAsset not exists in database", $data[0]['return_code']);
            }

            return $idAsset;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the asset tags
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function tags(Query $q, $count = true, $page = 1, $perPage = 15)
    {

        $method = "GET";
        $room = $q->room;
        $idUser = $q->idUser;
        $idAsset = $q->idAsset;
        $idDirectory = $q->idDirectory;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $term = $q->term;
        $termTag = $q->termTag;
        $filter = $q->filter;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;

        try {
            if ($count == true) {
                $parameters = [
                    [$room, PDO::PARAM_STR],
                    [$idUser, PDO::PARAM_INT],
                    [$idAsset, PDO::PARAM_INT],
                    [$idDirectory, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$sndField, PDO::PARAM_INT],
                    [$trdField, PDO::PARAM_INT],
                    [$term, PDO::PARAM_STR],
                    [$termTag, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                    [$sndFilter, PDO::PARAM_INT],
                    [$trdFilter, PDO::PARAM_INT],
                    [$limit_size_a, PDO::PARAM_INT],
                    [$limit_size_b, PDO::PARAM_INT],
                ];

                $name = "ASSETS_TAGS_RELATED_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idAsset, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$termTag, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "ASSETS_TAGS_RELATED";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return Tag::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * systemTags
     *
     * @param int $field
     * @param int $limit_size_a In bytes
     * @param int $limit_size_b In bytes
     * @return array
     */
    public static function systemTags(Query $q)
    {
        $room = $q->room;
        $idUser = $q->idUser;
        $idDirectory = $q->idDirectory;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $term = $q->term;
        $filter = $q->filter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;

        try {
            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT]];

            $name = "ASSETS_SYSTEM_TAGS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addToAlbum
     *
     * @param Album\Album $tag
     * @throws Exception
     */
    public function addToAlbum(Album\Album $album, $position)
    {
        try {
            $idAlbum = $album->getId();
            $parameters = [[$idAlbum, PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT],
                [$position, PDO::PARAM_INT]];

            $name = "ASSETS_TO_ALBUMS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag $tag
     * @return Asset
     * @throws Exception
     */
    public function addTag(Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [[$idTag, PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT]];

            $name = "ASSETS_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $q = new Query();
            $q->idAsset = $this->getId();
            $this->tags = $this->tags($q);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "ASSETS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * markedToDelete
     *
     * Get then assets marked to delete from database
     * @return array
     * @throws Exception
     */
    public static function markedToDelete()
    {
        try {
            $parameters = [];
            $name = "ASSETS_MARKED_HAS_DELETED";
            $method = "GET";
            $assets = Stage::esp($parameters, $name, $method);

            return array_map(function ($asset) {
                return self::createInstance($asset);
            }, $assets);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function labels()
    {
        try {
            $name = "ASSETS_LABELS_RELATED";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return [];
            } else {
                $temp = [];
                foreach ($data as $tag) {
                    $tagInstance = Label::createInstance($tag);
                    array_push($temp, $tagInstance);
                }

                return $temp;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * checksum
     *
     * @param $idAsset
     * @param $checksum
     * @return bool
     * @throws Exception
     */
    public static function checksum($idAsset, $checksum)
    {
        try {
            $parameters = [
                [$idAsset, PDO::PARAM_INT],
                [$checksum, PDO::PARAM_STR]
            ];
            $name = "ASSET_CHECKSUM";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * inImageAdvertisement
     *
     * @return array|Asset[]
     * @throws Exception
     */
    public static function inImageAdvertisement($field, $filter)
    {
        try {
            $parameters = [
                [$field, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR]
            ];
            $name = "ASSETS_INIMAGE_ADVERTISEMENTS";
            $method = "GET";
            $results = Stage::esp($parameters, $name, $method);

            if (!is_array($results) || !isset($results[0]['id_option']))
                return [];

            $docs = [];
            $checkedItems = [];
            foreach ($results as $item) {
                $currentItem = $item['item'];

                if (!in_array($currentItem, $checkedItems)) {
                    $iterator = new ArrayObject($results);
                    $items = new ItemFilter($iterator->getIterator(), $currentItem);

                    $newItems = $items->translateToTypesFrom($items);
                    $docs[] = $items->transformToNestedStdClassFrom((object)$newItems);

                    $checkedItems[] = $currentItem;
                }
            }
            return $docs;

        } catch (Exception $e) {
            throw $e;
        }
    }
    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table posts_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: assets_custom_fields_related
        $idAsset = $this->getId();
        $spname = "ASSETS_CUSTOM_FIELDS_VARIANTS";
        $method = "SET";
        $parameters = [[$idAsset, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * customFieldsVariants
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        $idAsset = $this->getId();
        $name = "ASSETS_CUSTOM_FIELDS_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idAsset, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        if (isset($data[0]) && isset($data[0]['id_variant'])) {
            $result->setCount(sizeof($data));
        }

        return $result;
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        $idAsset = $this->getId();
        $name = "ASSETS_CUSTOM_FIELD_VARIANT";
        $method = "DEL";
        $parameters = [[$idAsset, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * variantKeys
     *
     * Search Messages
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function variantKeys(CustomFieldQuery $q, $count = false, $page = 1, $perPage = 15, $all = false)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $name = 'ASSETS_CUSTOM_FIELDS_KEYS_COUNT';
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = !$all ? (isset($p) ? $p->to() : $q->to) : -1;

            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "ASSETS_CUSTOM_FIELDS_KEYS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = CustomFieldKey::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            $result = new MyArrayObject($temp);
            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachVariantKey
     *
     * @param $idKey
     * @param $name
     * @param $nameStripped
     * @param $public
     * @param $ifExists
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists)
    {
        // table: assets_custom_fields_keys
        $spName = "ASSETS_CUSTOM_FIELDS_KEY";
        $method = "SET";
        $parameters = [[$idKey, PDO::PARAM_INT],
            [$name, PDO::PARAM_STR],
            [$nameStripped, PDO::PARAM_STR],
            [$public, PDO::PARAM_BOOL],
            [$ifExists, PDO::PARAM_BOOL]];

        try {
            $data = Stage::esp($parameters, $spName, $method);
            if (isset($data[0]) && isset($data[0]['id_key'])) {
                return CustomFieldKey::createInstance($data[0]);
            }

            throw new Exception("Error creating custom field key", $data[0]['return_code']);

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachVariantKey
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public static function detachVariantKey($idKey)
    {
        $name = "ASSETS_CUSTOM_FIELD_KEY";
        $method = "DEL";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * variantKey
     *
     * @param $idKey
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function variantKey($idKey)
    {
        $name = "ASSETS_CUSTOM_FIELD_KEY";
        $method = "GET";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
            throw new Exception("The Key not exists", $data[0]['return_code']);
        } else {
            return CustomFieldKey::createInstance($data[0]);
        }
    }

    /**
     * attachPlaylist
     *
     * @param $idPlaylist
     * @param $position
     * @return bool
     * @throws Exception
     */
    public function attachPlaylist($idPlaylist, $position)
    {
        try {
            $name = "ASSETS_TO_PLAYLISTS";
            $method = "SET";
            $parameters = [[$idPlaylist, PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT],
                [$position, PDO::PARAM_INT]];

            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachPlaylist
     *
     * @param $idPlaylist
     * @return bool
     * @throws Exception
     */
    public function detachPlaylist($idPlaylist)
    {
        try {
            $name = "ASSETS_FROM_PLAYLISTS";
            $method = "DEL";
            $parameters = [[$idPlaylist, PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT]];

            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "directory" && $this->$attribute instanceof Directory) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($asset)
    {
        $assetInstance = new self();
        $assetInstance->setId($asset['id_asset']);

//        if( $asset['id_directory'] == Directory::ROOT )
//        {
//            $directory = new Directory();
//        }
//        else
//        {
//            try {
//                $idDirectory = $asset['id_directory'];
//                $directory = Directory::find($idDirectory);
//            } catch(Exception $e){
//                throw new Exception("The directory with id $idDirectory was not found.");
//            }
//        }
//
//        $assetInstance->setDirectory($directory);
        $assetInstance->directoryId = (int)$asset['id_directory'];
        $assetInstance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $assetInstance->setDescription(html_entity_decode($asset['description'], ENT_COMPAT, 'UTF-8'));
        $assetInstance->setImageTitle($asset['image_title']);
        $assetInstance->setImageAlterText($asset['image_alter_text']);
        $assetInstance->setLink($asset['link']);
        $assetInstance->setFilename($asset['filename']);
        $assetInstance->setExtension($asset['extension']);
        $assetInstance->setMimeType($asset['mime_type']);
        $assetInstance->setSize($asset['size']);
        $assetInstance->setWidth($asset['width']);
        $assetInstance->setHeight($asset['height']);
        $assetInstance->setState($asset['state']);
        $assetInstance->setPublic($asset['public']);

        if (isset($asset['position'])) {
            $assetInstance->setPosition($asset['position']);
        }

        if (isset($asset['created_date']) && Helper::isValidDate($asset['created_date'], "Y-m-d H:i:s")) {
            $assetInstance->setCreatedDate(new DateTime($asset['created_date']));
        }

        if (isset($asset['id_storage'])) {
            $assetInstance->setIdStorage($asset['id_storage']);
        }

        if (isset($asset['id_cdn'])) {
            $assetInstance->setIdCdn($asset['id_cdn']);
        }


        if (isset($asset['rooms'])) {
            $assetInstance->setRooms($asset['rooms']);
        }

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }

    public function paths($prefix, $accountToken, \DateTime $registerDate)
    {
        $year = $registerDate->format('Y');
        $month = $registerDate->format('m');
        $day = $registerDate->format('d');

        // Build the path to the file
        $path = $prefix . DIRECTORY_SEPARATOR . $accountToken;
        return implode(DIRECTORY_SEPARATOR, [$path, $year, $month, $day]);
    }
}
