<?php namespace Stage\Asset\Embed;

use DateTime;
use Stage\Helper;

class Artwork
{
    /**
     * @var
     */
    private $idArtwork;

    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $idAsset;

    /**
     * @var
     */
    private $filename;

    /**
     * @var
     */
    private $position;


    /**
     * @var
     */
    private $headline;

    /**
     * @var
     */
    private $title;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $actionName;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idArtwork', 'idAsset', 'filename', 'headline', 'title', 'description', 'actionName', 'link', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getIdArtwork()
    {
        return $this->idArtwork;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdAsset()
    {
        return $this->idAsset;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getHeadline()
    {
        return $this->headline;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getActionName()
    {
        return $this->description;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setIdArtwork($id)
    {
        $this->idArtwork = (int)$id;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdAsset($id)
    {
        $this->idAsset = (int)$id;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setPosition($pos)
    {
        $this->position = $pos;
    }

    public function setHeadline($headline)
    {
        $this->headline = $headline;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setDescription($desc)
    {
        $this->description = $desc;
    }

    public function setActionName($name)
    {
        $this->actionName = $name;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        return $this->attributesToArray();
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "title" || $attribute == "headline" || $attribute == "description" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $asset
     */
    public static function createInstance($asset)
    {
        $assetInstance = new self();
        $assetInstance->setId($asset['id']);
        $assetInstance->setIdArtwork((int)$asset['id_artwork']);
        $assetInstance->setIdAsset((int)$asset['id_asset']);
        $assetInstance->setFilename($asset['filename']);
        $assetInstance->setPosition((int)$asset['position']);
        $assetInstance->setHeadline($asset['headline']);
        $assetInstance->setTitle($asset['title']);
        $assetInstance->setDescription($asset['description']);
        $assetInstance->setActionName($asset['action_name']);
        $assetInstance->setLink($asset['link']);

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
