<?php namespace Stage\Asset\Embed;

use PDO;
use DateTime;
use Exception;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Helper;

class Embed
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $type;

    /**
     * @var
     */
    private $variant;

    /**
     * @var
     */
    private $code;

    /**
     * @var
     */
    private $sourceType;

    /**
     * @var
     */
    private $sourceId;

    /**
     * @var
     */
    private $format;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'type', 'variant', 'code', 'sourceType', 'sourceId', 'format', 'name', 'active', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getVariant()
    {
        return $this->variant;
    }

    public function getCode()
    {
        return $this->code;
    }

    public function getSourceType()
    {
        return $this->sourceType;
    }

    public function getSourceId()
    {
        return $this->sourceId;
    }

    public function getFormat()
    {
        return $this->format;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getActive()
    {
        return (bool)$this->active;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function setVariant($variant)
    {
        $this->variant = $variant;
    }

    public function setCode($code)
    {
        $this->code = $code;
    }

    public function setSourceType($type)
    {
        $this->sourceType = $type;
    }

    public function setSourceId($id)
    {
        $this->sourceId = $id;
    }

    public function setFormat($format)
    {
        $this->format = $format;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * embeds
     *
     * @param $field
     * @param $filter
     * @return array|Embed[]
     * @throws Exception
     */
    public static function embeds($field, $filter)
    {
        try {
            // table: assets_embeds
            $name = "ASSETS_EMBED";
            $parameters = [[$field, PDO::PARAM_INT], [$filter, PDO::PARAM_INT]];
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return [];
            } else {
                return array_map(function ($embed) {
                    return self::createInstance($embed);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assets
     *
     * @param $field
     * @param $filter
     * @return MyArrayObject
     * @throws Exception
     */
    public static function assets($field, $filter)
    {
        try {
            // from table assets_embeds_assets_related, where id is the id_embed from assets_embed table
            $name = "ASSETS_EMBED_ASSETS_RELATED";
            $parameters = [
                [$field, PDO::PARAM_INT],
                [$filter, PDO::PARAM_INT]
            ];
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return Asset::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachAsset
     *
     * @param Asset $asset
     * @return bool
     * @throws Exception
     */
    public static function attachAsset(Asset $asset)
    {
        try {
            // table: assets_embeds_assets_related
            $name = "ASSETS_EMBED_ASSETS_RELATED";
            $parameters = [[$asset->getId(), PDO::PARAM_INT],
                [$asset->getIdAsset(), PDO::PARAM_INT],
                [$asset->getPosition(), PDO::PARAM_INT],
                [$asset->getHeadline(), PDO::PARAM_INT],
                [$asset->getTitle(), PDO::PARAM_STR],
                [$asset->getDescription(), PDO::PARAM_STR],
                [$asset->getActionName(), PDO::PARAM_STR],
                [$asset->getLink(), PDO::PARAM_STR]];

            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['id']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachAsset
     *
     * @param $idEmbed
     * @param $idAsset
     * @return bool
     * @throws Exception
     */
    public static function detachAsset($idEmbed, $idAsset)
    {
        try {
            // table: assets_embeds_assets_related
            $name = "ASSETS_EMBED_RELATED";
            $parameters = [[$idEmbed, PDO::PARAM_INT],
                [$idAsset, PDO::PARAM_INT]];

            $method = "DEL";
            $result = Stage::esp($parameters, $name, $method);
            return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachOption
     *
     * @param $idEmbed
     * @param $idOption
     * @param $optionValue
     * @return bool
     * @throws Exception
     */
    public static function attachOption($idEmbed, $idOption, $optionValue)
    {
        try {
            // table: assets_embeds_options_related
            $name = "ASSETS_EMBED_OPTIONS_RELATED";
            $parameters = [[$idEmbed, PDO::PARAM_INT],
                [$idOption, PDO::PARAM_INT],
                [$optionValue, PDO::PARAM_STR]];

            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['id']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * options
     *
     * @param $field
     * @param $filter
     * @return MyArrayObject
     * @throws Exception
     */
    public static function options($field, $filter)
    {
        try {
            $name = "ASSETS_EMBED_OPTIONS_RELATED";
            $parameters = [[$field, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR]];

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return Option::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * variants
     *
     * @param $type
     * @param $variant
     * @return MyArrayObject
     * @throws Exception
     */
    public static function variants($type, $variant)
    {
        try {
            $name = "ASSETS_EMBED_OPTIONS_VARIANTS_RELATED";
            $parameters = [[$type, PDO::PARAM_INT],
                [$variant, PDO::PARAM_STR]];

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return OptionVariant::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * Find an Embed
     *
     * @param $id
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ASSETS_EMBED";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Embed id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return Embed
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "ASSETS_EMBED";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getSourceType(), PDO::PARAM_INT],
                [$this->getSourceId(), PDO::PARAM_INT],
                [$this->getFormat(), PDO::PARAM_STR],
                [$this->getType(), PDO::PARAM_STR],
                [$this->getVariant(), PDO::PARAM_STR],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getActive(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id'])) {
                $this->setId($data[0]['id']);
                $this->setCode($data[0]['code']);
                if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
                return $this;
            } else {
                throw new Exception("Error saving Author L10n", $data[0]['return_code']);
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Embed by Query query
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return int|Embed[]|MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];
                $name = "ASSETS_EMBEDS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;
                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];
            $name = "ASSETS_EMBEDS";

            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($author) {
                return self::createInstance($author);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @param $idEmbed
     * @param bool $soft
     * @return bool
     * @throws Exception
     */
    public static function delete($idEmbed, $soft = true)
    {
        try {
            $name = "ASSET_EMBED_DELETED";
            $method = "SET";
            $parameters = [[$idEmbed, PDO::PARAM_INT],
                [$soft, PDO::PARAM_BOOL]];
            $result = Stage::esp($parameters, $name, $method);
            return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * embedsForPublish
     *
     * @param $field
     * @param $filter
     * @return MyArrayObject
     * @throws Exception
     */
    public static function embedsForPublish($field, $filter)
    {
        try {
            // table: assets_embeds
            $name = "ASSETS_EMBED_RELATED";
            $parameters = [[$field, PDO::PARAM_INT], [$filter, PDO::PARAM_INT]];
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return EmbedForPublish::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * settings
     *
     * @param $id
     * @return MyArrayObject
     * @throws Exception
     */
    public static function settings($id)
    {
        try {
            // table: assets_embeds_settings
            $name = "ASSETS_EMBEDS_SETTINGS";
            $parameters = [[$id, PDO::PARAM_INT]];
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return Setting::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachArtwork
     *
     * @param $id
     * @return MyArrayObject
     * @throws Exception
     */
    public static function attachArtwork($idArtwork, $id, $idAsset, $position, $headline, $title, $description, $actionName, $link)
    {
        try {

            // table: assets_embeds_artwork_related
            $name = "ASSETS_EMBED_ARTWORK_RELATED";
            $parameters = [[$idArtwork, PDO::PARAM_INT],
                [$id, PDO::PARAM_INT],
                [$idAsset, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT],
                [$headline, PDO::PARAM_STR],
                [$title, PDO::PARAM_STR],
                [$description, PDO::PARAM_STR],
                [$actionName, PDO::PARAM_STR],
                [$link, PDO::PARAM_STR]];

            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return Artwork::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachArtwork
     *
     * @param $idArtwork
     * @return bool
     * @throws Exception
     */
    public static function detachArtwork($idArtwork)
    {
        try {
            // table: assets_embeds_artwork_related
            $name = "ASSETS_EMBED_ARTWORK_RELATED";
            $parameters = [[$idArtwork, PDO::PARAM_INT]];

            $method = "DEL";
            $result = Stage::esp($parameters, $name, $method);
            return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * artwork
     *
     * @param $field
     * @param $filter
     * @return MyArrayObject
     * @throws Exception
     */
    public static function artwork($field, $filter)
    {
        try {
            $name = "ASSETS_EMBED_ARTWORK_RELATED";
            $parameters = [[$field, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR]];

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return Artwork::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachTrackinfo
     *
     * @param $id
     * @return MyArrayObject
     * @throws Exception
     */
    public static function attachTrackinfo($idTrackInfo, $id, $position, $headline, $title, $description, $author)
    {
        try {
            // table: assets_embeds_settings
            $name = "ASSETS_EMBED_TRACKINFO_RELATED";
            $parameters = [[$idTrackInfo, PDO::PARAM_INT],
                [$id, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT],
                [$headline, PDO::PARAM_STR],
                [$title, PDO::PARAM_STR],
                [$description, PDO::PARAM_STR],
                [$author, PDO::PARAM_STR]];
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return Trackinfo::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachTrackinfo
     *
     * @param $idTrackinfo
     * @return bool
     * @throws Exception
     */
    public static function detachTrackinfo($idTrackinfo)
    {
        try {
            // table: assets_embeds_trackinfo_related
            $name = "ASSETS_EMBED_TRACKINFO_RELATED";
            $parameters = [[$idTrackinfo, PDO::PARAM_INT]];

            $method = "DEL";
            $result = Stage::esp($parameters, $name, $method);
            return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * trackinfo
     *
     * @param $field
     * @param $filter
     * @return MyArrayObject
     * @throws Exception
     */
    public static function trackinfo($field, $filter)
    {
        try {
            $name = "ASSETS_EMBED_TRACKINFO_RELATED";
            $parameters = [[$field, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR]];

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return new MyArrayObject();
            } else {
                $result = new MyArrayObject(array_map(function ($asset) {
                    return Trackinfo::createInstance($asset);
                }, $data));
                $result->setCount(sizeof($data));
                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($asset)
    {
        $assetInstance = new self();
        $assetInstance->setId($asset['id']);
        $assetInstance->setVariant($asset['variant']);
        $assetInstance->setCode($asset['code']);
        $assetInstance->setSourceType($asset['source_type']);
        $assetInstance->setSourceId($asset['source_id']);
        $assetInstance->setFormat($asset['format']);
        $assetInstance->setName(html_entity_decode($asset['name'], ENT_COMPAT, 'UTF-8'));
        $assetInstance->setActive($asset['active']);

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
