<?php namespace Stage\Asset\Embed;

use DateTime;
use Stage\Helper;

class Trackinfo
{
    /**
     * @var
     */
    private $idTrackinfo;

    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $headline;

    /**
     * @var
     */
    private $title;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $author;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idTrackinfo', 'position', 'headline', 'title', 'description', 'author', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdTrackinfo()
    {
        return $this->idTrackinfo;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getHeadline()
    {
        return $this->headline;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getAuthor()
    {
        return $this->author;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdTrackinfo($id)
    {
        $this->idTrackinfo = (int)$id;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setPosition($pos)
    {
        $this->position = $pos;
    }

    public function setHeadline($headline)
    {
        $this->headline = $headline;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setDescription($desc)
    {
        $this->description = $desc;
    }

    public function setAuthor($author)
    {
        $this->author = $author;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        return $this->attributesToArray();
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "title" || $attribute == "headline" || $attribute == "description" || $attribute == "author" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $asset
     */
    public static function createInstance($asset)
    {
        $assetInstance = new self();
        $assetInstance->setId($asset['id']);
        $assetInstance->setIdTrackinfo((int)$asset['id_trackinfo']);
        $assetInstance->setPosition((int)$asset['position']);
        $assetInstance->setHeadline($asset['headline']);
        $assetInstance->setTitle($asset['title']);
        $assetInstance->setDescription($asset['description']);
        $assetInstance->setAuthor($asset['author']);


        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
