<?php namespace Stage\Asset\Extensions;

use Iterator;
use FilterIterator;
use stdClass;

class ItemFilter extends FilterIterator
{
    private $filter;

    public function __construct(Iterator $iterator, $filter)
    {
        parent::__construct($iterator);
        $this->filter = $filter;
    }

    public function accept()
    {
        $item = $this->getInnerIterator()->current();
        if ($item['item'] != $this->filter) {
            return false;
        }
        return true;
    }

    public function transformToNestedStdClassFrom($originalStdClass)
    {
        $nestedStdClass = new stdClass();

        foreach ($originalStdClass as $key => $value) {
            $dotPosition = strpos($key, '.');

            if ($dotPosition !== false) {
                // Handle properties with a dot in the name
                $namespace = substr($key, 0, $dotPosition);
                $nestedKey = substr($key, $dotPosition + 1);

                if (!isset($nestedStdClass->$namespace)) {
                    $nestedStdClass->$namespace = new stdClass();
                }

                $nestedStdClass->$namespace->$nestedKey = $value;
            } else {
                // Copy other properties directly
                $nestedStdClass->$key = $value;
            }
        }

        return $nestedStdClass;
    }

    public function translateToTypesFrom($items)
    {
        $newItem = [];
        foreach ($items as $item) {
            switch (($item['option_type'])) {
                case 0: // string
                    $newItem[$item['option_name']] = (string)$item['option_value'];
                    break;
                case 1: // int
                    $newItem[$item['option_name']] = (int)$item['option_value'];
                    break;
                case 2: // json
                    $newItem[$item['option_name']] = (object)json_decode($item['option_value']);
                    break;
                case 3: // bool
                    $newItem[$item['option_name']] = (bool)$item['option_value'];
                    break;
                default:
                    $newItem[$item['option_name']] = (string)$item['option_value'];
            }
        }
        return $newItem;
    }
}
