<?php namespace Stage\Asset\Gallery;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;
use Stage\Asset\Gallery\Tag;
use Stage\Asset\Album\Album;
use Stage\Asset\Gallery\Query\Gallery as GalleryQuery;
use Stage\Asset\Gallery\Query\Album as AlbumQuery;

class Gallery
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $image;

    /**
     * @var
     */
    private $imageAlign;

    /**
     * @var
     */
    private $imageTitle;

    /**
     * @var
     */
    private $imageAlterText;

    /**
     * @var
     */
    private $metaTitle;

    /**
     * @var
     */
    private $metaDescription;

    /**
     * @var
     */
    private $metaImage;

    /**
     * @var
     */
    private $metaKeywords;

    /**
     * @var
     */
    private $metaUrl;

    /**
     * @var
     */
    private $metaWatermarkImage;

    /**
     * @var
     */
    private $album; // Its the default Album when created

    /**
     * @var int
     */
    private $position = 1;

    /**
     * @var
     */
    private $tags = [];

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'album', 'name', 'description', 'image', 'imageAlign', 'imageTitle', 'imageAlterText', 'metaTitle', 'metaDescription', 'metaImage', 'metaKeywords', 'metaUrl', 'metaWatermarkImage', 'position', 'registerDate', 'updateDate');

    public function __construct()
    {
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    public function setImageAlign($imageAlign)
    {
        $this->imageAlign = $imageAlign;
    }

    public function setImageTitle($imageTitle)
    {
        $this->imageTitle = $imageTitle;
    }

    public function setImageAlterText($imageAlterText)
    {
        $this->imageAlterText = $imageAlterText;
    }

    public function setMetaTitle($metaTitle)
    {
        $this->metaTitle = $metaTitle;
    }

    public function setMetaDescription($metaDescription)
    {
        $this->metaDescription = $metaDescription;
    }

    public function setMetaImage($metaImage)
    {
        $this->metaImage = $metaImage;
    }

    public function setMetaKeywords($metaKeywords)
    {
        $this->metaKeywords = $metaKeywords;
    }

    public function setMetaUrl($metaUrl)
    {
        $this->metaUrl = $metaUrl;
    }

    public function setMetaWatermarkImage($metaWatermarkImage)
    {
        $this->metaWatermarkImage = $metaWatermarkImage;
    }

    public function setAlbum($album)
    {
        if ($album instanceof Album) {
            $this->album = $album;
        } else {
            $this->album = 0;
        }
    }

    public function setPosition($position)
    {
        $this->position = $position;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    private function setUpdateDate($date)
    {
        $this->updateDate = $date;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getImageAlign()
    {
        return $this->imageAlign;
    }

    public function getImageTitle()
    {
        return $this->imageTitle;
    }

    public function getImageAlterText()
    {
        return $this->imageAlterText;
    }

    public function getMetaTitle()
    {
        return $this->metaTitle;
    }

    public function getMetaDescription()
    {
        return $this->metaDescription;
    }

    public function getMetaImage()
    {
        return $this->metaImage;
    }

    public function getMetaKeywords()
    {
        return $this->metaKeywords;
    }

    public function getMetaUrl()
    {
        return $this->metaUrl;
    }

    public function getMetaWatermarkImage()
    {
        return $this->metaWatermarkImage;
    }

    public function getAlbum()
    {
        return $this->album;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    /**
     * @return array
     *
     */
    public static function sizeof()
    {
        $name = "ASSETS_GALLERIES_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Gallery
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ASSETS_GALLERY";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Gallery id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Galleries
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        try {
            $query = new GalleryQuery();
            $query->term = "";
            $query->from = 0;
            $query->to = -1;
            return self::query($query);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Gallery
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $action = $this->getId() == -1 ? "CREATE" : "UPDATE";

        if ($this->getAlbum() instanceof Album) {
            $idAlbum = $this->getAlbum()->getId();
        } else {
            $idAlbum = 0;
        }


        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_STR],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getImage(), PDO::PARAM_STR],
            [$this->getImageAlign(), PDO::PARAM_STR],
            [$this->getImageTitle(), PDO::PARAM_STR],
            [$this->getImageAlterText(), PDO::PARAM_STR],
            [$this->getMetaTitle(), PDO::PARAM_STR],
            [$this->getMetaDescription(), PDO::PARAM_STR],
            [$this->getMetaImage(), PDO::PARAM_STR],
            [$this->getMetaKeywords(), PDO::PARAM_STR],
            [$this->getMetaUrl(), PDO::PARAM_STR],
            [$this->getMetaWatermarkImage(), PDO::PARAM_STR],
            [$idAlbum, PDO::PARAM_INT]];

        try {
            $name = "ASSETS_GALLERIES";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if ($action == "CREATE") {
                $this->setId($data[0]['id_gallery']);
                \Stage\Events\Event::fire("assets.galleries.created", $this);
            } elseif ($action == "UPDATE") {
                \Stage\Events\Event::fire("assets.galleries.updated", $this);
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Gallery
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ASSETS_GALLERY";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }


    /**
     * query
     *
     * Query a Gallery
     *
     * @return Array
     * @throws Exception
     */
    public static function query(GalleryQuery $q)
    {
        $query = $q->term;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $parameters = [[$query, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "ASSETS_GALLERIES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($gallery) {
                return self::createInstance($gallery);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * albums
     *
     * @param AlbumQuery $q
     * @return array
     * @throws Exception
     */
    public function albums(AlbumQuery $q)
    {
        $query = $q->term;
        $order = $q->order;
        $orderBy = $q->orderBy;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "ASSETS_ALBUMS_ON_GALLERIES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($album) {
                return Album::createInstance($album);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addAlbum
     *
     * @param Album $album
     * @return $this
     * @throws Exception
     */
    public function addAlbum(Album $album, $position)
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$album->getId(), PDO::PARAM_INT],
                [$position, PDO::PARAM_INT]];

            $name = "ASSETS_ALBUMS_TO_GALLERIES";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAlbum
     *
     * @param Album $album
     * @return $this
     * @throws Exception
     */
    public function removeAlbum(Album $album)
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$album->getId(), PDO::PARAM_INT]];

            $name = "ASSETS_ALBUMS_FROM_GALLERIES";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            return $album;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        try {
            $id = $this->getId();
            $name = "ASSETS_GALLERY_TAGS_RELATED";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Gallery id $id not exists.", $data[0]['return_code']);
            } else {
                return array_map(function ($data) {
                    return \Stage\Asset\Gallery\Tag\Tag::createInstance($data);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag\Tag $tag
     * @throws Exception
     */
    public function addTag(Tag\Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idTag, PDO::PARAM_INT]];


            $name = "ASSETS_GALLERY_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $this->tags = $this->tags();
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "ASSETS_GALLERY_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();
        foreach ($attributes as $attribute) {
            if ($attribute == "album" && $this->$attribute instanceof Album) {
                $arr[$attribute] = $this->$attribute->toArray();
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Gallery
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id_gallery']);
        $instance->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setDescription($data['description']);
        $instance->setImage($data['image']);
        $instance->setImageAlign($data['image_align']);
        $instance->setImageTitle($data['image_title']);
        $instance->setImageAlterText($data['image_alter_text']);
        $instance->setMetaTitle($data['meta_title']);
        $instance->setMetaDescription($data['meta_description']);
        $instance->setMetaImage($data['meta_image']);
        $instance->setMetaKeywords($data['meta_keywords']);
        $instance->setMetaUrl($data['meta_url']);
        $instance->setMetaWatermarkImage($data['meta_watermark_image']);
        if (isset($data['position'])) {
            $instance->setPosition($data['position']);
        }

        try {
            $album = Album::find($data['default_id_album']);
        } catch (Exception $e) {
            $album = null;
        }

        $instance->setAlbum($album);

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
