<?php namespace Stage\Asset\Iptc;

use PDO;
use Exception;
use Stage\Stage;

class Iptc
{
    private static $instance;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $totalItems;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','totalItems');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $dates = array('DateCreated', 'ArtworkOrObjectAODateCreated');

    public function getName()
    {
        return $this->name;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setTotalItems($total)
    {
        return $this->totalItems = $total;
    }

    public function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public static function get($idAsset)
    {

//        $sps = Stage::getInstance()->getSps();
        $sp = "ASSETS_IPTC";
        $method = "GET";
        $parameters = [[$idAsset, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $sp, $method);

        if (isset($data[0]) && isset($data[0]['id_asset'])) {
            unset($data[0]['id_asset']);
            return array_map(function ($attribute) {
                return $attribute == "0000-00-00 00:00:00" ? null : html_entity_decode($attribute, ENT_COMPAT, 'UTF-8');
            }, $data[0]);
        }
        throw new Exception("The Asset $idAsset not exists");
    }

    /**
     * toCredits
     *
     *
     * @return array
     * @throws Exception
     */
    public static function toCredits($field, $filter)
    {
        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR]];
            $name = "ASSETS_IPTC_TO_CREDITS";
            $method = "GET";
            $assets = Stage::esp($parameters, $name, $method);
            return $assets;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function parameterify($idAsset, $fields, $data)
    {
        $parameters[] = [$idAsset, PDO::PARAM_INT];
        foreach ($fields as $field) {
            $parameters[] = [isset($data[$field]) ? $this->setAttribute($data[$field]) : "", PDO::PARAM_STR];
        }

        return $parameters;
    }

    private function setAttribute($attr)
    {
        if (in_array($attr, $this->dates)) {
            $dateTimeFormats = ["Y-m-dTH-i-s", "Y:m:d H:i:s"];
            if (\DateTime::createFromFormat("Y:m:d H:i:s", $attr) instanceof \DateTime) {
                return \DateTime::createFromFormat("Y:m:d H:i:s", $attr)->format("Y-m-d H:i:s");
            } else {
                return \DateTime::createFromFormat("Y-m-dTH-i-s", $attr)->format("Y-m-d H:i:s");
            }
        } else {
            if (is_array($attr)) {
                $attr = implode(" ", $attr);
            }

            return htmlentities(trim($attr), ENT_COMPAT, 'UTF-8');
        }
    }

    private function response($idAsset, $data)
    {
        if (isset($data[0]) && isset($data[0]['id_asset'])) {
            unset($data[0]['id_asset']);
            return $data[0];
        }
        throw new Exception("The Asset $idAsset not exists");
    }

    public function setCoreContact($idAsset, $data)
    {
        $fields = [
            'Creator',
            'CreatorJobtitle',
            'CiAdrExtadr',
            'CiAdrCity',
            'CiAdrRegion',
            'CiAdrPcode',
            'CiAdrCtry',
            'CiTelWork',
            'CiEmailWork',
            'CiUrlWork',
        ];

        $sp = "ASSETS_IPTC_CORE_CONTACT";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setCoreContent($idAsset, $data)
    {
        $fields = [
            'Headline',
            'Description',
            'Keywords',
            'SubjectCode',
            'DescriptionWriter'
        ];

        $sp = "ASSETS_IPTC_CORE_CONTENT";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setCoreImage($idAsset, $data)
    {
        $fields = [
            'DateCreated',
            'IntellectualGenre',
            'Scene',
            'Location',
            'City',
            'ProvinceState',
            'Country',
            'CountryCode',
        ];

        $sp = "ASSETS_IPTC_CORE_IMAGE";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setCoreStatus($idAsset, $data)
    {
        $fields = [
            'Title',
            'JobID',
            'Instructions',
            'Credit',
            'Source',
            'CopyrightNotice',
            'RightsUsageTerms',
        ];

        $sp = "ASSETS_IPTC_CORE_STATUS";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setExtensionArtwork($idAsset, $data)
    {
        $fields = [
            'ArtworkOrObject',
            'ArtworkOrObjectAOTitle',
            'ArtworkOrObjectAODateCreated',
            'ArtworkOrObjectAOCreator',
            'ArtworkOrObjectAOSource',
            'ArtworkOrObjectAOSourceInvNo',
            'ArtworkOrObjectAOCopyrightNotice',
        ];

        $sp = "ASSETS_IPTC_EXTENSION_ARTWORK";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setExtensionLocationCreated($idAsset, $data)
    {
        $fields = [
            'PersonInImage',
            'LocationCreatedSublocation',
            'LocationCreatedCity',
            'LocationCreatedProvinceState',
            'LocationCreatedCountryName',
            'LocationCreatedCountryCode',
            'LocationCreatedWorldRegion',
        ];

        $sp = "ASSETS_IPTC_EXTENSION_LOCATION_CREATED";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setExtensionLocationShown($idAsset, $data)
    {
        $fields = [
            'LocationShownSublocation',
            'LocationShownCity',
            'LocationShownProvinceState',
            'LocationShownCountryName',
            'LocationShownCountryCode',
            'LocationShownWorldRegion',
        ];

        $sp = "ASSETS_IPTC_EXTENSION_LOCATION_SHOWN";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setExtensionModelsImage($idAsset, $data)
    {
        $fields = [
            'AddlModelInfo',
            'ModelAge',
            'ModelReleaseID'
        ];

        $sp = "ASSETS_IPTC_EXTENSION_MODELS_IMAGE";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setExtensionOrganization($idAsset, $data)
    {
        $fields = [
            'OrganisationInImageName',
            'OrganisationInImageCode',
            'Event'
        ];

        $sp = "ASSETS_IPTC_EXTENSION_ORGANISATION";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setExtensionAdministrativeInformation($idAsset, $data)
    {
        $fields = ['ImageSupplier',
                   'ImageSupplierID',
                    'ImageSupplierImageID',
                    'RegistryID',
                    'RegistryIDRegOrgID',
                    'RegistryIDRegItemID',
                    'MaxAvailWidth',
                    'MaxAvailHeight',
                    'DigitalSourceType'];

        $sp = "ASSETS_IPTC_EXTENSION_ADMINISTRATIVE_INFORMATION";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }

    public function setExtensionRightsRelated($idAsset, $data)
    {
        $fields = ['ImageCreatorName',
                   'ImageCreatorID',
                    'CopyrightOwnerName',
                    'CopyrightOwnerID',
                    'LicensorName',
                    'LicensorID',
                    'LicensorTelephone1',
                    'LicensorTelephone2',
                    'LicensorEmail',
                    'PropertyReleaseID'];

        $sp = "ASSETS_IPTC_EXTENSION_RIGHTS_RELATED";
        $method = "SET";
        $parameters = $this->parameterify($idAsset, $fields, $data);
        $data = Stage::esp($parameters, $sp, $method);
        return $this->response($idAsset, $data);
    }
}
