<?php namespace Stage\Asset\Label;

use PDO;
use Exception;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Asset\Tag\Tag;

class Label
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $public = true;

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $totalProfile;

    /**
     * @var
     */
//    private $totalVideoCompose;

    /**
     * @var
     */
//    private $totalLiveCompose;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','nameStripped','public','total');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getPublic()
    {
        return $this->public;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getTotalProfile()
    {
        return $this->totalProfile;
    }

//    public function getTotalVideoCompose()
//    {
//        return $this->totalVideoCompose;
//    }
//
//    public function getTotalLiveCompose()
//    {
//        return $this->totalLiveCompose;
//    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setPublic($public)
    {
        $this->public = (bool) $public;
    }

    public function setTotal($total)
    {
        $this->total = (int) $total > 0 ? $total : 0;
    }

    public function setTotalProfile($total)
    {
        $this->totalProfile = (int) $total > 0 ? $total : 0;
    }

//    public function setTotalVideoCompose($total)
//    {
//        $this->totalVideoCompose = (int) $total > 0 ? $total : 0;
//    }
//
//    public function setTotalLiveCompose($total)
//    {
//        $this->totalLiveCompose = (int) $total > 0 ? $total : 0;
//    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "ASSETS_LABELS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * save
     *
     * Save a tag to the database and return a Tag instance
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "ASSETS_LABELS";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$this->getName(), PDO::PARAM_STR],
                            [$this->getNameStripped(), PDO::PARAM_STR],
                            [$this->getPublic(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_label']);

            // Fire an event
            if (!$this->getId()) {
                \Stage\Events\Event::fire("asset.label.created", $this);
            } else {
                \Stage\Events\Event::fire("asset.label.updated", $this);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * delete
     *
     * Delete a Tag
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ASSETS_LABEL";
            $method = "DEL";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * find
     *
     * Finds a Label
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ASSETS_LABEL";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Label id $id not exists.", $data[0]['return_code']);
            } else {
                return static::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Tags
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->by = Query::ALL;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return static::query($q);
    }

    /**
     * query
     *
     * Search Labels
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            if ($count == true) {
                $parameters = [[$field, PDO::PARAM_STR],
                    [$query, PDO::PARAM_INT],
                    [$filter, PDO::PARAM_INT],
                    [$limit_a, PDO::PARAM_INT],
                    [-1, PDO::PARAM_INT]];

                $name = "ASSETS_LABELS";
                $data = Stage::esp($parameters, $name, $method);
                $total = sizeof($data);

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [ [$field, PDO::PARAM_STR],
                [$query, PDO::PARAM_INT],
                [$filter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "ASSETS_LABELS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addAsset
     *
     * @param Tag $tag
     * @throws Exception
     */
    public static function addAsset($idLabel, $idAsset)
    {
        try {
            $parameters = [ [$idLabel, PDO::PARAM_INT],
                            [$idAsset, PDO::PARAM_INT]];

            $name = "ASSETS_ASSET_TO_LABELS";
            $method = "SET";
            $result = Stage::esp($parameters, $name, $method);

            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAsset
     *
     * @return $this
     * @throws Exception
     */
    public static function removeAsset($idAsset)
    {
        try {
            $parameters = [[$idAsset, PDO::PARAM_INT]];

            $name = "ASSETS_ASSET_FROM_LABELS";
            $method = "DEL";
            $result = Stage::esp($parameters, $name, $method);
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new static();
        $tagInstance->setId($tag['id_label']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setPublic($tag['public']);
        $tagInstance->setTotal(isset($tag['total_assets']) ? (int) $tag['total_assets'] : 0);
        // $tagInstance->setTotalVideoCompose( isset($tag['total_video_composed']) ? (int) $tag['total_video_composed'] : 0);
        // $tagInstance->setTotalLiveCompose( isset($tag['total_live_composed']) ? (int) $tag['total_live_composed'] : 0);

        return $tagInstance;
    }
}
