<?php namespace Stage\Asset\Playlist;

use PDO;
use Exception;
use Stage\Paginator;
use Stage\Stage;
//use Stage\Asset\Asset;
//use Stage\Asset\Query;

class Directory
{
    const CREATE = 0;
    const ROOT = 1;

    public static $instanceCounter = 0;

    /**
     * @var
     */
    private $id = self::CREATE;

    /**
     * @var
     */
    public $parentId;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $totalItems;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'description', 'link', 'parentId', 'totalItems');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    public function __construct()
    {
        self::$instanceCounter++;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getParentId()
    {
        return $this->parentId;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getTotalItems()
    {
        return $this->totalItems;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setTotalItems($total)
    {
        return $this->totalItems = $total;
    }

    /**
     * Find a Directory
     *
     * @param $id
     * @param int $toLevel The subdirectory return deep level
     * @return array|int|void
     * @throws Exception
     */
    public static function find($id, $toLevel = 10000)
    {
        // http://stackoverflow.com/questions/8656089/solution-for-fatal-error-maximum-function-nesting-level-of-100-reached-abor
        if (self::$instanceCounter > $toLevel) {
            return 1;
        }

        try {
            $name = "ASSETS_PLAYLISTS_DIRECTORY";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && ($data[0]['return_code']) == "-2") {
                throw new Exception("The Directory id $id not exists.", $data[0]['return_code']);
            } elseif (isset($data[0]['id_directory'])) {
                $temp = array_map(function ($directory) {
                    return self::createInstance($directory);
                }, $data);

                return $temp[0];
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * @param bool|false $raw If $raw is true then return the data directly from the SP
     * @return array
     * @throws Exception
     */
    public static function all($raw = false)
    {
        try {
            $q = new DirectoryQuery();
            $q->idDirectory = self::ROOT;
            $q->from = 0;
            $q->to = -1;
            $q->raw = $raw;
            return self::query($q,false, 0, -1);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * append
     *
     * Save a Directory
     *
     * @return $this
     * @throws Exception
     */
    private function append(Directory $targetDirectory = null, $isUpdating = false)
    {
        $targetIdDirectory = ($targetDirectory instanceof Directory) ? $targetDirectory->getId() : self::ROOT;

        if ($isUpdating) {
            $targetIdDirectory = $targetDirectory->parentId;
        }

        $action = $this->getId() == self::CREATE ? "CREATE" : "UPDATE";

        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$targetIdDirectory, PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_INT],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getLink(), PDO::PARAM_STR]];

        try {
            $name = "ASSETS_PLAYLISTS_DIRECTORIES";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if ($action == "CREATE") {
                $this->setId($data[0]['id_directory']);
            }

            // Update the parentId
            $this->parentId = $targetIdDirectory;

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * appendTo
     *
     * @param Directory $directory If not passing the directory than is inserted in the root
     * @return $this
     * @throws Exception
     */
    public function appendTo(Directory $targetDirectory = null)
    {
        try {
            return $this->append($targetDirectory);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return Directory
     * @throws Exception
     */
    public function save()
    {
        if ($this->getId() == self::ROOT) {
            throw new Exception("You are trying to updated the root(id=1) folder. Root folder cannot be updated.");
        }

        try {
            return $this->append($this, true);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * build
     *
     * Build the tree array of directories
     *
     * @return array
     * @throws Exception
     * @url http://stackoverflow.com/questions/4452472/category-hierarchy-php-mysql
     */
    public static function build()
    {
        $directories = self::all(true);
        $rootId = 1;
        $refs = array();
        $list = array();
        foreach ($directories as $row) {
            $ref = &$refs[$row['id_directory']];

            $ref['id'] = (int)$row['id_directory'];
            $ref['parentId'] = (int)$row['parent_id_directory'];
            $ref['name'] = html_entity_decode($row['name'], ENT_COMPAT, 'UTF-8');
            $ref['description'] = html_entity_decode($row['description'], ENT_COMPAT, 'UTF-8');
            $ref['link'] = $row['link'];
            $ref['files'] = $row['total_items'];

            if ($row['parent_id_directory'] == $rootId) {
                //$list[$row['id_directory']] = & $ref; // Put this if you want the array keys has directory id
                $list[] = &$ref;
            } else {
                // $refs[$row['parent_id_directory']]['children'][$row['id_directory']] = & $ref;   // Put this if you want the array keys has directory id
                $refs[$row['parent_id_directory']]['children'][] = &$ref;
            }
        }

        return $list;

//        Database Table: Items
//        Id	Parent Id	Name
//        1	    0	        North America
//        2	    1	        Canada
//        3	    7	        California
//        4	    0	        Europe
//        5	    0	        Antartica
//        6	    3	        Los Angeles
//        7	    1	        United States
//        8	    7	        Florida
//        9	    3	        Hollywood
//        10	2	        Quebec

//        One Pass Parent-Child Array Structure ( original code )
//        $refs = array();
//        $list = array();
//
//        $sql = "SELECT item_id, parent_id, name FROM items ORDER BY name";
//
//        /** @var $pdo \PDO */
//        $result = $pdo->query($sql);
//
//        foreach ($result as $row)
//        {
//            $ref = & $refs[$row['item_id']];
//
//            $ref['parent_id'] = $row['parent_id'];
//            $ref['name']      = $row['name'];
//
//            if ($row['parent_id'] == 0)
//            {
//                $list[$row['item_id']] = & $ref;
//            }
//            else
//            {
//                $refs[$row['parent_id']]['children'][$row['item_id']] = & $ref;
//            }
//        }
    }

    /**
     * findNode
     *
     * Find the node of some directory id
     * @param $tree
     * @param $idDirectory
     * @return array
     */
    public static function findNode($tree, $idDirectory)
    {
        $temp = [];
        foreach ($tree as $node) {
            if ($node['id'] == $idDirectory) {
                return $node;
            } elseif (isset($node['children'])) {
                $temp = self::findNode($node['children'], $idDirectory);
                if ($temp) {
                    return $temp;
                }
            }
        }

        return $temp;
    }

    /**
     * children
     *
     * @param $tree The parameter $tree must start with index 0
     * @return array
     */
    public static function children(array $tree)
    {
        $result = [];
        $ids = [];
        foreach ($tree as $object) {
            $ids = self::findIds($object, array());
            $result[] = array('id' => $object['id'], 'children' => $ids);
        }

        return sizeof($result) ? $result[0] : $result;
    }

    /**
     * findIds
     *
     * This is an helper method
     * @param $child
     * @param $ids
     * @return array
     * @see http://stackoverflow.com/questions/33800847/get-all-children-of-a-nested-tree-recursively
     */
    private static function findIds($child, $ids)
    {
        if (isset($child['children'])) {
            if (count($child['children']) > 0) {
                foreach ($child['children'] as $ch) {
                    $ids[] = ['id' => $ch['id'],
                        'name' => $ch['name'],
                        'description' => $ch['description'],
                        'link' => $ch['link'],
                        'parentId' => $ch['parentId']];

                    $ids = self::findIds($ch, $ids);
                }
            }
        }

        return $ids;
    }

    /**
     * query
     *
     * Query directories
     *
     * @return Array|Int
     * @throws Exception
     */
    public static function query(DirectoryQuery $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";

        try {

            $parameters = [];
            $name = "ASSETS_PLAYLISTS_DIRECTORIES";
            $data = Stage::esp($parameters, $name, $method);

            if ($q->raw) {
                return $data;
            }

            $total = sizeof($data);

            // Create a new Paginator
            $p = new Paginator($total, $perPage);
            $p->page($page);
            $q->from = $p->from();
            $q->to = $p->to();

            $result = new \Stage\Post\MyArrayObject(array_map(function ($directory) {
                return self::createInstance($directory);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

//    /**
//     * assets
//     *
//     * Query assets inside a directory. Pass $idDirectory=0 to query assets inside all directories
//     * @return $this
//     * @throws Exception
//     */
//    public static function assets(Query $q = null, $idDirectory = 0)
//    {
//        $room = $q->room;
//        $idUser = $q->idUser;
//        $field = $q->by;
//        $sndField = $q->sndBy;
//        $trdField = $q->trdBy;
//        $sndFilter = $q->sndFilter;
//        $trdFilter = $q->trdFilter;
//        $term = $q->term;
//        $filter = $q->filter;
//        $from = $q->from;
//        $to = $q->to;
//        $limit_size_a = $q->fromSize;
//        $limit_size_b = $q->toSize;
//        $orderBy = $q->orderBy;
//        $order = $q->order;
//
//        try {
//
////            $idDirectory = $this->getId() <= self::ROOT ? 1 : $this->getId();
////            $idDirectory = $idDirectory <= self::ROOT ? 0 : $idDirectory;
//
//            $parameters = [[$room, PDO::PARAM_STR],
//                [$idUser, PDO::PARAM_INT],
//                [$idDirectory, PDO::PARAM_INT],
//                [$field, PDO::PARAM_INT],
//                [$sndField, PDO::PARAM_INT],
//                [$trdField, PDO::PARAM_INT],
//                [$term, PDO::PARAM_STR],
//                [$filter, PDO::PARAM_INT],
//                [$sndFilter, PDO::PARAM_INT],
//                [$trdFilter, PDO::PARAM_INT],
//                [$limit_size_a, PDO::PARAM_INT],
//                [$limit_size_b, PDO::PARAM_INT],
//                [$from, PDO::PARAM_INT],
//                [$to, PDO::PARAM_INT],
//                [$orderBy, PDO::PARAM_INT],
//                [$order, PDO::PARAM_INT]];
//
//            $name = "ASSETS_IN_DIRECTORY";
//            $method = "GET";
//            $data = Stage::esp($parameters, $name, $method);
//
//            return array_map(function ($asset) {
//                return Asset::createInstance($asset);
//            }, $data);
//        } catch (Exception $e) {
//            throw $e;
//        }
//    }

//    /**
//     * assetsCount
//     *
//     * @param \Stage\Asset\Query|null $q
//     * @param int $idDirectory
//     * @return array
//     * @throws Exception
//     */
//    public static function assetsCount(Query $q = null, $idDirectory = 0)
//    {
//        $room = $q->room;
//        $idUser = $q->idUser;
//        $field = $q->by;
//        $sndField = $q->sndBy;
//        $trdField = $q->trdBy;
//        $sndFilter = $q->sndFilter;
//        $trdFilter = $q->trdFilter;
//        $term = $q->term;
//        $filter = $q->filter;
//        $limit_size_a = $q->fromSize;
//        $limit_size_b = $q->toSize;
//
//        try {
//            $parameters = [[$room, PDO::PARAM_STR],
//                [$idUser, PDO::PARAM_INT],
//                [$idDirectory, PDO::PARAM_INT],
//                [$field, PDO::PARAM_INT],
//                [$sndField, PDO::PARAM_INT],
//                [$trdField, PDO::PARAM_INT],
//                [$term, PDO::PARAM_STR],
//                [$filter, PDO::PARAM_STR],
//                [$sndFilter, PDO::PARAM_STR],
//                [$trdFilter, PDO::PARAM_STR],
//                [$limit_size_a, PDO::PARAM_INT],
//                [$limit_size_b, PDO::PARAM_INT]];
//
//            $name = "ASSETS_IN_DIRECTORY_COUNT";
//            $method = "GET";
//            $data = Stage::esp($parameters, $name, $method);
//            return isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;
//        } catch (Exception $e) {
//            throw $e;
//        }
//    }

    /**
     * delete
     *
     * Delete a Directory
     *
     * @return $this
     * @throws Exception
     */
    public static function delete($idDirectory, $softDelete = true, $deleted = true)
    {
        if ($softDelete) {
            $name = "ASSET_PLAYLISTS_DIRECTORY_DELETED";    // Only mark the asset has deleted and not remove from the database
            $parameters = [[$idDirectory, PDO::PARAM_INT],
                [$deleted, PDO::PARAM_BOOL]];
            $method = "SET";
        } else {
            $name = "ASSETS_PLAYLISTS_DIRECTORY";            // Remove from the database
            $parameters = [[$idDirectory, PDO::PARAM_INT]];
            $method = "DEL";
        }

        try {
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Directory id $idDirectory not exists in database", $data[0]['return_code']);
            }

            return $idDirectory;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * markedToDelete
     *
     * @return array
     * @throws Exception
     */
    public static function markedToDelete()
    {
        try {
            $parameters = [];
            $name = "ASSETS_PLAYLISTS_DIRECTORIES_DELETED";
            $method = "GET";
            $directories = Stage::esp($parameters, $name, $method);

            return array_map(function ($directory) {
                return self::createInstance($directory);
            }, $directories);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "parent" && $this->$attribute instanceof Directory) {
                $arr[$attribute] = $this->$attribute->toArray()['id'] == self::CREATE ? "/" : $this->$attribute->toArray();
            } elseif (isset($this->$attribute)) {
                $arr[$attribute] = $attribute == "name" || $attribute == "description" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($directory)
    {
        $directoryInstance = new self();
        $directoryInstance->setId($directory['id_directory']);
        if (isset($directory['parent_id_directory'])) {
            $directoryInstance->parentId = $directory['parent_id_directory'];
        } else { // In case of /root folder
            unset($directoryInstance->parentId);
        }

        $directoryInstance->setName(html_entity_decode($directory['name'], ENT_COMPAT, 'UTF-8'));
        $directoryInstance->setDescription(html_entity_decode($directory['description'], ENT_COMPAT, 'UTF-8'));
        $directoryInstance->setLink($directory['link']);
        $directoryInstance->setTotalItems($directory['total_items']);
        return $directoryInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
