<?php namespace Stage\Asset\Playlist;

use DateTime;
use PDO;
use Exception;
use Stage\Asset\Asset;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Video\Channel\Channel;
use Stage\Video\Playlist\Tag\Tag;
use Stage\Video\Sticker\Sticker;

class Playlist
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $directoryId = 1;

    /**
     * @var
     */
    private $token;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $description = "";

    /**
     * @var
     */
    private $link = "";

    /**
     * @var
     */
    private $image = "";

    /**
     * @var
     */
    private $imageAlign = "";

    /**
     * @var
     */
    private $imageTitle = "";

    /**
     * @var
     */
    private $imageAlterText = "";

    /**
     * @var
     */
    private $metaTitle = "";

    /**
     * @var
     */
    private $metaDescription = "";

    /**
     * @var
     */
    private $metaImage = "";

    /**
     * @var
     */
    private $metaKeywords = "";

    /**
     * @var
     */
    private $metaUrl = "";

    /**
     * @var
     */
    private $metaWatermarkImage = "";

    /**
     * @var
     */
    private $totalAssets = 0;

    /**
     * @var
     */
    private $registerDate = 0;

    /**
     * @var
     */
    private $updateDate = 0;


    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'directoryId', 'token', 'name', 'description', 'link', 'image', 'imageAlign', 'imageTitle', 'imageAlterText', 'metaTitle', 'metaDescription', 'metaImage', 'metaKeywords', 'metaUrl', 'metaWatermarkImage', 'totalAssets');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdDirectory()
    {
        return $this->directoryId;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getImageAlign()
    {
        return $this->imageAlign;
    }

    public function getImageTitle()
    {
        return $this->imageTitle;
    }

    public function getImageAlterText()
    {
        return $this->imageAlterText;
    }

    public function getMetaTitle()
    {
        return $this->metaTitle;
    }

    public function getMetaDescription()
    {
        return $this->metaDescription;
    }

    public function getMetaImage()
    {
        return $this->metaImage;
    }

    public function getMetaKeywords()
    {
        return $this->metaKeywords;
    }

    public function getMetaUrl()
    {
        return $this->metaUrl;
    }

    public function getMetaWatermarkImage()
    {
        return $this->metaWatermarkImage;
    }

    public function getTotalAssets()
    {
        return $this->totalAssets;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdDirectory($id)
    {
        $this->directoryId = (int)$id;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDescription($name)
    {
        $this->description = $name;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    public function setImageAlign($image)
    {
        $this->imageAlign = $image;
    }

    public function setImageTitle($image)
    {
        $this->imageTitle = $image;
    }

    public function setImageAlterText($image)
    {
        $this->imageAlterText = $image;
    }

    public function setMetaTitle($title)
    {
        $this->metaTitle = $title;
    }

    public function setMetaDescription($title)
    {
        $this->metaDescription = $title;
    }

    public function setMetaImage($title)
    {
        $this->metaImage = $title;
    }

    public function setMetaKeywords($title)
    {
        $this->metaKeywords = $title;
    }

    public function setMetaUrl($title)
    {
        $this->metaUrl = $title;
    }

    public function setMetaWatermarkImage($title)
    {
        $this->metaWatermarkImage = $title;
    }

    public function setTotalAssets($total)
    {
        $this->totalAssets = (int)$total;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }


    /**
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        $name = "ASSETS_PLAYLISTS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Playlist
     *
     * @return $this
     * @throws Exception
     */
    public static function find($field, $query)
    {
        try {
            $name = "ASSETS_PLAYLIST";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Playlist id $query not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Posts
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * query
     *
     * Search Playlist by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $idDirectory = $q->idDirectory;
        $field = $q->field;
        $sndField = $q->sndField;
        $trdField = $q->trdField;
        $query = $q->query;
        $filter = $q->filter;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$idDirectory, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$sndField, PDO::PARAM_INT],
                    [$trdField, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$sndFilter, PDO::PARAM_STR],
                    [$trdFilter, PDO::PARAM_STR]
                ];

                $name = "ASSETS_PLAYLISTS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_STR],
                [$trdFilter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
            ];

            $name = "ASSETS_PLAYLISTS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * duplicate
     *
     * Returns a new Playlist duplicated, with a new Playlist id
     *
     * @return $this
     * @throws Exception
     */
    public function duplicate($idDirectory)
    {
        try {
            $parameters = [
                [$this->getId(), PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT]
            ];
            $name = "ASSETS_PLAYLIST_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Returns a Playlist deleted
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ASSETS_PLAYLIST";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the channel tags
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEO_PLAYLIST_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag $tag
     * @throws Exception
     */
    public function addTag(Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [[$idTag, PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEO_PLAYLISTS_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $this->tags = $this->tags();
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEO_PLAYLISTS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Playlist
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "ASSETS_PLAYLIST";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getIdDirectory(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getDescription(), PDO::PARAM_STR],
                [$this->getLink(), PDO::PARAM_STR],
                [$this->getImage(), PDO::PARAM_STR],
                [$this->getImageAlign(), PDO::PARAM_STR],
                [$this->getImageTitle(), PDO::PARAM_STR],
                [$this->getImageAltertext(), PDO::PARAM_STR],
                [$this->getMetaTitle(), PDO::PARAM_STR],
                [$this->getMetaDescription(), PDO::PARAM_STR],
                [$this->getMetaImage(), PDO::PARAM_STR],
                [$this->getMetaKeywords(), PDO::PARAM_STR],
                [$this->getMetaUrl(), PDO::PARAM_STR],
                [$this->getMetaWaterMarkImage(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_playlist'])) {
                $this->setId($data[0]['id_playlist']);
                return $this;
            }

            throw new Exception("Could not create Playlist");
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * totalComposes
     *
     * Get the total composes inside a Playlist
     * @return int
     * @throws Exception
     */
    public function totalComposes()
    {
        $id = $this->getId();

        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "VIDEO_COMPOSES_ON_PLAYLISTS_NUM";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int)$data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * channels
     *
     * @return array
     * @throws Exception
     */
    public function channels()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEO_CHANNELS_BY_PLAYLIST";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($channel) {
                return Channel::createInstance($channel);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Assets in Playlist
     *
     * @return mixed
     * @throws Exception
     */
    public static function assets($idPlaylist, AssetsQuery $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $limit_size_a = $q->limitSizeA;
        $limit_size_b = $q->limitSizeB;

        try {
            if ($count == true) {
                $parameters = [
                    [$idPlaylist, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$limit_size_a, PDO::PARAM_INT],
                    [$limit_size_b, PDO::PARAM_INT],
                ];

                $name = "ASSETS_ON_PLAYLISTS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idPlaylist, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]
            ];

            $name = "ASSETS_ON_PLAYLISTS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return Asset::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * metaUrlExist
     *
     * Check if the post slug url exit
     *
     * @param $idPlaylist
     * @param $metaUrl
     * @return array
     * @throws Exception
     */
    public static function metaUrlExist($idPlaylist, $metaUrl)
    {
        $name = "ASSETS_PLAYLISTS_URL_NUM";
        $method = "GET";
        $parameters = [[$idPlaylist, PDO::PARAM_INT],
            [$metaUrl, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * metaUrlExist
     *
     * Check if the post slug url exit
     *
     * @param $idPlaylistA
     * @param $idPlaylistB
     * @param $type
     * @return MyArrayObject
     * @throws Exception
     */
    public static function sync($idPlaylistA, $idPlaylistB, $type)
    {
        $name = "ASSETS_PLAYLIST_SYNC";
        $method = "SET";
        $parameters = [
            [$idPlaylistA, PDO::PARAM_INT],
            [$idPlaylistB, PDO::PARAM_INT],
            [$type, PDO::PARAM_INT]
        ];
        $data = Stage::esp($parameters, $name, $method);

        if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
            return new \Stage\Post\MyArrayObject([]);
        }
        $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
            return self::createInstance($label);
        }, $data));

        return $result;
    }

    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table video_stickers_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: assets_playlists_custom_fields_related
        $idSticker = $this->getId();
        $spname = "ASSETS_PLAYLISTS_CUSTOM_FIELDS_VARIANTS";
        $method = "SET";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        // table: assets_playlists_custom_fields_related
        $idSticker = $this->getId();
        $name = "ASSETS_PLAYLISTS_CUSTOM_FIELD_VARIANT";
        $method = "DEL";
        $parameters = [[$idSticker, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * customFieldsVariants
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        // table: assets_playlists_custom_fields_related
        $idSticker = $this->getId();
        $name = "ASSETS_PLAYLISTS_CUSTOM_FIELDS_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idSticker, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * dates
     *
     * @param int $idQuery Se se passar o id album então vai buscar as dates dentro de um album
     * @return array
     */
    public static function dates(Query $q)
    {
        // TODO
        $room = $q->room;
        $idUser = $q->idUser;
        $idDirectory = $q->idDirectory;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $term = $q->term;
        $filter = $q->filter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;

        try {
            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT]];

            $name = "ASSETS_DATES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * systemTags
     *
     * @param int $field
     * @param int $limit_size_a In bytes
     * @param int $limit_size_b In bytes
     * @return array
     */
    public static function systemTags(Query $q)
    {
        // TODO
        $room = $q->room;
        $idUser = $q->idUser;
        $idDirectory = $q->idDirectory;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $term = $q->term;
        $filter = $q->filter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;

        try {
            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT]];

            $name = "ASSETS_SYSTEM_TAGS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name" || $attribute == "description" || $attribute == "imageTitle" || $attribute == "imageAlterText" || $attribute == "metaTitle" || $attribute == "metaDescription") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } elseif ($attribute == "l10n") {
                foreach ($this->$attribute as $l10n) {
                    $arr[$attribute][] = $l10n->toArray();
                }
            } else {
                if ($this->$attribute !== null) {
                    $arr[$attribute] = $this->$attribute;
                }
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Playlist
     */
    public static function createInstance($data)
    {
        $p = new self();
        $p->setId($data['id_playlist']);
        $p->setIdDirectory($data['id_directory']);
        $p->setToken($data['token']);
        $p->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $p->setDescription(html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8'));
        $p->setLink($data['link']);
        $p->setImage($data['image']);
        $p->setImageAlign($data['image_align']);
        $p->setImageTitle(html_entity_decode($data['image_title'], ENT_COMPAT, 'UTF-8'));
        $p->setImageAlterText(html_entity_decode($data['image_alter_text'], ENT_COMPAT, 'UTF-8'));
        $p->setMetaTitle(html_entity_decode($data['meta_title'], ENT_COMPAT, 'UTF-8'));
        $p->setMetaDescription(html_entity_decode($data['meta_description'], ENT_COMPAT, 'UTF-8'));
        $p->setMetaImage($data['meta_image']);
        $p->setMetaKeywords($data['meta_keywords']);
        $p->setMetaUrl($data['meta_url']);
        $p->setMetaWatermarkImage($data['meta_watermark_image']);
        $p->setTotalAssets($data['total_assets']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $p->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $p->setUpdateDate(new DateTime($data['update_date']));
        }


        return $p;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
