<?php namespace Stage\Asset\Tag\Group;

use PDO;
use Exception;
use Stage\Stage;

class Group
{
    const ROOT = 1;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $totalItems;

    /**
     * @var
     */
    private $parentId;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'totalItems', 'parentId');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getTotalItems()
    {
        return $this->totalItems;
    }

    public function getParentId()
    {
        return $this->parentId;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setTotalItems($total)
    {
        $this->totalItems = (int) $total;
    }

    public function setParentId($id)
    {
        $this->parentId = (int) $id;
    }

    /**
     * find
     *
     * Find a Group
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ASSETS_TAGS_GROUP";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Group id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * build
     *
     * Build the tree
     *
     * @return Array|Int
     * @throws Exception
     */
    public static function build()
    {
        try {
            $parameters = [];

            $name = "ASSETS_TAGS_GROUPS";
            $method = "GET";
            $groups = Stage::esp($parameters, $name, $method);

            $rootId = 1;
            $refs = array();
            $list = array();
            foreach ($groups as $row) {
                $ref = & $refs[$row['id_group']];

                $ref['id']          = (int) $row['id_group'];
                $ref['parentId']    = (int) $row['parent_id_group'];
                $ref['name']        = html_entity_decode($row['name'], ENT_COMPAT, 'UTF-8');
                // $ref['description'] = html_entity_decode($row['description'], ENT_COMPAT , 'UTF-8');
                // $ref['link']        = $row['link'];
                $ref['tags']       = $row['total_items'];

                if ($row['parent_id_group'] == $rootId) {
                    //$list[$row['id_directory']] = & $ref; // Put this if you want the array keys has directory id
                    $list[] = & $ref;
                } else {
                    // $refs[$row['parent_id_directory']]['children'][$row['id_directory']] = & $ref;   // Put this if you want the array keys has directory id
                    $refs[$row['parent_id_group']]['children'][] = & $ref;
                }
            }

            return $list;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Group
     *
     * @return $this
     * @throws Exception
     */
    public function save($parent = 1)
    {
        try {
            $name = "ASSETS_TAGS_GROUPS";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$parent, PDO::PARAM_INT],
                            [$this->getName(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_group']);
            $this->setParentId($parent);

            // Fire an event
            if (!$this->getId()) {
                \Stage\Events\Event::fire("assets.tag.created", $this);
            } else {
                \Stage\Events\Event::fire("assets.tag.updated", $this);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Group
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "ASSETS_TAGS_GROUP";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_group']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setTotalItems(isset($tag['total_items']) ? $tag['total_items'] : 0);

        if (isset($tag['parent_id_group'])) {
            $tagInstance->setParentId($tag['parent_id_group']);
        } else { //  In case of /root folder
            unset($tagInstance->parentId);
        }

        return $tagInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
