<?php namespace Stage\Asset\Transfer;

use PDO;
use Exception;
use Stage\Paginator;
use Stage\Stage;

class Directory
{
    const CREATE = 0;
    const ROOT = 1;

    public static $instanceCounter = 0;

    /**
     * @var
     */
    private $id = self::CREATE;

    /**
     * @var
     */
    public $parentId;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $totalItems;

    /**
     * @var
     */
    private $childDirectories;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'description', 'link', 'parentId', 'totalItems', 'childDirectories');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    public function __construct()
    {
        self::$instanceCounter++;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getParentId()
    {
        return $this->parentId;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getTotalItems()
    {
        return $this->totalItems;
    }

    public function getChildDirectories()
    {
        return $this->childDirectories;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    /*public function setParent($directory)
    {

        $directory = $directory instanceof Directory ? $directory : new Directory();

        // Only set the parent if the parent directory id is different from the instance directory id
        if( $directory->getId() != $this->getId() )
            $this->parent = $directory;
    }*/

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setTotalItems($total)
    {
        return $this->totalItems = $total;
    }

    public function setChildDirectories($childDirectories)
    {
        return $this->childDirectories = $childDirectories;
    }

    /**
     * Find a Directory
     *
     * @param $id
     * @param int $toLevel The subdirectory return deep level
     * @return array|int|void
     * @throws Exception
     */
    public static function find($id, $toLevel = 10000)
    {

        // http://stackoverflow.com/questions/8656089/solution-for-fatal-error-maximum-function-nesting-level-of-100-reached-abor
        if (self::$instanceCounter > $toLevel) {
            return 1;
        }

        try {
            $name = "ASSETS_TRANSFERS_DIRECTORY";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && ($data[0]['return_code']) == "-2") {
                throw new Exception("The Directory id $id not exists.", $data[0]['return_code']);
            } elseif (isset($data[0]['id_directory'])) {
                $temp = array_map(function ($directory) {
                    return self::createInstance($directory);
                }, $data);

                return $temp[0];
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * @param bool|false $raw If $raw is true then return the data directly from the SP
     * @return array
     * @throws Exception
     */
    public static function all($raw = false)
    {
        try {
            $q = new DirectoryQuery();
            $q->idDirectory = self::ROOT;
            $q->from = 0;
            $q->to = -1;
            $q->raw = $raw;
            return self::query($q,false, 0, -1);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * append
     *
     * Save a Directory
     *
     * @return $this
     * @throws Exception
     */
    private function append($targetIdDirectory)
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$targetIdDirectory, PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_INT],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getLink(), PDO::PARAM_STR]];

        try {
            $name = "ASSETS_TRANSFERS_DIRECTORIES";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            // Update the parentId
            $this->setId($data[0]['id_directory']);
            $this->parentId = $targetIdDirectory;

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * appendTo
     *
     * @param Directory $directory If not passing the directory than is inserted in the root
     * @return $this
     * @throws Exception
     */
    public function appendTo(Directory $targetDirectory = null)
    {
        try {
            if (!$targetDirectory) {
                $targetIdDirectory = self::ROOT;
            } else {
                $targetIdDirectory = $targetDirectory->getId();
            }
            return $this->append($targetIdDirectory);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return Directory
     * @throws Exception
     */
    public function save($targetIdDirectory = self::ROOT)
    {
        try {
            return $this->append($targetIdDirectory);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * build
     *
     * Build the tree array of directories
     *
     * @return array
     * @throws Exception
     * @url http://stackoverflow.com/questions/4452472/category-hierarchy-php-mysql
     */
    public static function build()
    {
        $directories = self::all(true);
        $rootId = 1;
        $refs = array();
        $list = array();
        foreach ($directories as $row) {
            $ref = &$refs[$row['id_directory']];

            $ref['id'] = (int)$row['id_directory'];
            $ref['parentId'] = (int)$row['parent_id_directory'];
            $ref['name'] = html_entity_decode($row['name'], ENT_COMPAT, 'UTF-8');
            $ref['description'] = html_entity_decode($row['description'], ENT_COMPAT, 'UTF-8');
            $ref['link'] = $row['link'];
            $ref['files'] = $row['total_items'];

            if ($row['parent_id_directory'] == $rootId) {
                //$list[$row['id_directory']] = & $ref; // Put this if you want the array keys has directory id
                $list[] = &$ref;
            } else {
                // $refs[$row['parent_id_directory']]['children'][$row['id_directory']] = & $ref;   // Put this if you want the array keys has directory id
                $refs[$row['parent_id_directory']]['children'][] = &$ref;
            }
        }

        return $list;
    }

    /**
     * findNode
     *
     * Find the node of some directory id
     * @param $tree
     * @param $idDirectory
     * @return array
     */
    public static function findNode($tree, $idDirectory)
    {
        $temp = [];
        foreach ($tree as $node) {
            if ($node['id'] == $idDirectory) {
                return $node;
            } elseif (isset($node['children'])) {
                $temp = self::findNode($node['children'], $idDirectory);
                if ($temp) {
                    return $temp;
                }
            }
        }

        return $temp;
    }

    /**
     * children
     *
     * @param $tree The parameter $tree must start with index 0
     * @return array
     */
    public static function children(array $tree)
    {
        $result = [];
        $ids = [];
        foreach ($tree as $object) {
            $ids = self::findIds($object, array());
            $result[] = array('id' => $object['id'], 'children' => $ids);
        }

        return sizeof($result) ? $result[0] : $result;
    }

    /**
     * findIds
     *
     * This is an helper method
     * @param $child
     * @param $ids
     * @return array
     * @see http://stackoverflow.com/questions/33800847/get-all-children-of-a-nested-tree-recursively
     */
    private static function findIds($child, $ids)
    {
        if (isset($child['children'])) {
            if (count($child['children']) > 0) {
                foreach ($child['children'] as $ch) {
                    $ids[] = ['id' => $ch['id'],
                        'name' => $ch['name'],
                        'description' => $ch['description'],
                        'link' => $ch['link'],
                        'parentId' => $ch['parentId']];

                    $ids = self::findIds($ch, $ids);
                }
            }
        }

        return $ids;
    }

    /**
     * query
     *
     * Query directories
     *
     * @return Array|Int
     * @throws Exception
     */
    public static function query(DirectoryQuery $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";

        try {

            $parameters = [];
            $name = "ASSETS_TRANSFERS_DIRECTORIES";
            $data = Stage::esp($parameters, $name, $method);

            if ($q->raw) {
                return $data;
            }

            $total = sizeof($data);

            // Create a new Paginator
            $p = new Paginator($total, $perPage);
            $p->page($page);
            $q->from = $p->from();
            $q->to = $p->to();

            $result = new \Stage\Post\MyArrayObject(array_map(function ($directory) {
                return self::createInstance($directory);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }

    }

    /**
     * delete
     *
     * Delete a Directory
     *
     * @return bool
     * @throws Exception
     */
    public static function delete($idDirectory, $softDelete = true, $deleted = true)
    {
        if ($softDelete) {
            $name = "ASSETS_TRANSFERS_DIRECTORY_DELETED";    // Only mark the asset has deleted and not remove from the database
            $parameters = [
                [$idDirectory, PDO::PARAM_INT],
                [$deleted, PDO::PARAM_BOOL]
            ];
            $method = "SET";
        } else {
            $name = "ASSETS_TRANSFERS_DIRECTORY";            // Remove from the database
            $parameters = [[$idDirectory, PDO::PARAM_INT]];
            $method = "DEL";
        }

        try {
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Directory id $idDirectory not exists in database", $data[0]['return_code']);
            }
            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * markedToDelete
     *
     * @return array
     * @throws Exception
     */
    public static function markedToDelete()
    {
        try {
            $parameters = [];
            $name = "ASSETS_TRANSFERS_DIRECTORIES_DELETED";
            $method = "GET";
            $directories = Stage::esp($parameters, $name, $method);

            return array_map(function ($directory) {
                return self::createInstance($directory);
            }, $directories);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "parent" && $this->$attribute instanceof Directory) {
                $arr[$attribute] = $this->$attribute->toArray()['id'] == self::CREATE ? "/" : $this->$attribute->toArray();
            } elseif (isset($this->$attribute)) {
                $arr[$attribute] = $attribute == "name" || $attribute == "description" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($directory)
    {
        $directoryInstance = new self();
        $directoryInstance->setId($directory['id_directory']);
        if (isset($directory['parent_id_directory'])) {
            $directoryInstance->parentId = $directory['parent_id_directory'];
        } else { // In case of /root folder
            unset($directoryInstance->parentId);
        }
        /*if( isset($directory['parent_id_directory']) )
        {
            $parentDirectory = $directory['parent_id_directory'] == self::CREATE ? self::ROOT : Directory::find($directory['parent_id_directory']);
            $directoryInstance->setParent($parentDirectory);
        }
        else
        {
            unset($directoryInstance->parent);
        }*/
        $directoryInstance->setName(html_entity_decode($directory['name'], ENT_COMPAT, 'UTF-8'));
        $directoryInstance->setDescription(html_entity_decode($directory['description'], ENT_COMPAT, 'UTF-8'));
        $directoryInstance->setLink($directory['link']);
        $directoryInstance->setTotalItems((int)$directory['total_items']);
        if(isset($directory['child_directories'])){
            $directoryInstance->setChildDirectories($directory['child_directories']);
        }
        return $directoryInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
