<?php namespace Stage\Asset\Transfer;

use PDO;
use DateTime;
use Exception;
use Stage\Paginator;
use Stage\Stage;
use Stage\Helper;

class Permission
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $imageOptimization = "";

    /**
     * @var
     */
    private $waterMarkImage = "";

    /**
     * @var
     */
    private $settings = "";

    /**
     * @var
     */
    private $viewOnly;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'imageOptimization', 'waterMarkImage', 'settings', 'viewOnly', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getImageOptimization()
    {
        return $this->imageOptimization;
    }

    public function getWaterMarkImage()
    {
        return $this->waterMarkImage;
    }

    public function getSettings()
    {
        return $this->settings;
    }

    public function getViewOnly()
    {
        return $this->viewOnly;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setImageOptimization($opt)
    {
        $this->imageOptimization = $opt;
    }

    public function setWaterMarkImage($image)
    {
        $this->waterMarkImage = $image;
    }

    public function setSettings($settings)
    {
        $this->settings = $settings;
    }

    public function setViewOnly($viewOnly)
    {
        $this->viewOnly = (bool) $viewOnly;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * find
     *
     * Find Transfer Permission
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ASSETS_TRANSFER_PERMISSION";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset Transfer id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Asset Transfer Permision
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_INT],
            [$this->getImageOptimization(), PDO::PARAM_STR],
            [$this->getWaterMarkImage(), PDO::PARAM_STR],
            [$this->getSettings(), PDO::PARAM_STR],
            [$this->getViewOnly(), PDO::PARAM_BOOL]];

        try {
            $name = "ASSETS_TRANSFER_PERMISSION";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_permission']) && $data[0]['id_permission'] >= 0) {
                $idTransfer = $data[0]['id_permission'];
                $this->setId($idTransfer);

                self::createInstance($data[0]);
                return $this;
            }

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset Transfer Permission not exists.", $data[0]['return_code']);
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Asset Transfer from the database
     *
     * @param $idPermission
     * @return bool
     * @throws Exception
     */
    public static function delete($idPermission)
    {
        $name = "ASSETS_TRANSFER_PERMISSION";            // Remove from the database
        $parameters = [[$idPermission, PDO::PARAM_INT]];
        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query transfers
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $field = $q->by;
        $term = $q->term;
        $filter = $q->filter;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$field, PDO::PARAM_INT],
                    [$term, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "ASSETS_TRANSFERS_PERMISSIONS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$field, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT]
            ];

            $name = "ASSETS_TRANSFERS_PERMISSIONS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "directory" && $this->$attribute instanceof Directory) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $transfer = new self();
        $transfer->setId($data['id_permission']);
        $transfer->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $transfer->setImageOptimization($data['image_optimization']);
        $transfer->setWaterMarkImage($data['watermark_image']);
        $transfer->setSettings($data['settings']);
        $transfer->setViewOnly($data['view_only']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $transfer->setRegisterDate(new DateTime($data['register_date']));
        }

        return $transfer;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
