<?php namespace Stage\Asset\Transfer;

use PDO;
use DateTime;
use Exception;
use Stage\Asset\Asset;
use Stage\Paginator;
use Stage\Stage;
use Stage\Helper;

class Transfer
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idDirectory;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $managementName = "";

    /**
     * @var
     */
    private $description = "";

    /**
     * @var
     */
    private $code = "";

    /**
     * @var
     */
    private $password = "";

    /**
     * @var
     */
    private $active = true;

    /**
     * @var
     */
    private $totalContent = 0;

    /**
     * @var
     */
    private $coverImage = "";

    /**
     * @var
     */
    private $imageOptimization = "";

    /**
     * @var
     */
    private $waterMarkImage = "";

    /**
     * @var
     */
    private $settings = "";

    /**
     * @var
     */
    private $viewOnly;

    /**
     * @var
     */
    private $verifyIdentity;

    /**
     * @var
     */
    private $compressEnabled;

    /**
     * @var
     */
    private $contentType = 1;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $endDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idDirectory',  'managementName', 'name', 'description', 'code', 'password', 'endDate', 'active', 'totalContent', 'coverImage', 'imageOptimization', 'waterMarkImage', 'settings', 'viewOnly', 'verifyIdentity', 'compressEnabled', 'contentType');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getDirectoryId()
    {
        return $this->idDirectory;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getCode()
    {
        return $this->code;
    }

    public function getPassword()
    {
        return $this->password;
    }

    public function isActive()
    {
        return $this->active;
    }

    public function getTotalContent()
    {
        return $this->totalContent;
    }

    public function getCoverImage()
    {
        return $this->coverImage;
    }

    public function getImageOptimization()
    {
        return $this->imageOptimization;
    }

    public function getWaterMarkImage()
    {
        return $this->waterMarkImage;
    }

    public function getSettings()
    {
        return $this->settings;
    }

    public function getViewOnly()
    {
        return $this->viewOnly;
    }

    public function getVerifyIdentity()
    {
        return $this->verifyIdentity;
    }

    public function getCompressEnabled()
    {
        return $this->compressEnabled;
    }

    public function getContentType()
    {
        return $this->contentType;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setDirectoryId($id)
    {
        $this->idDirectory = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setManagementName($name)
    {
        $this->managementName = $name;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setCode($code)
    {
        $this->code = $code;
    }

    public function setPassword($password)
    {
        $this->password = $password;
    }

    public function setIsActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setTotalContent($total)
    {
        $this->totalContent = (int)$total;
    }

    public function setCoverImage($image)
    {
        $this->coverImage = $image;
    }

    public function setImageOptimization($opt)
    {
        $this->imageOptimization = $opt;
    }

    public function setWaterMarkImage($image)
    {
        $this->waterMarkImage = $image;
    }

    public function setSettings($settings)
    {
        $this->settings = $settings;
    }

    public function setViewOnly($viewOnly)
    {
        $this->viewOnly = (bool) $viewOnly;
    }

    public function setVerifyIdentity($verifyIdentity)
    {
        $this->verifyIdentity = (bool) $verifyIdentity;
    }

    public function setCompressEnabled($compressEnabled)
    {
        $this->compressEnabled = (bool) $compressEnabled;
    }

    public function setContentType($contentType)
    {
        $this->contentType = $contentType;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    public function setEndDate($date)
    {
        $this->endDate = $date;
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "ASSETS_TRANSFERS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * all
     *
     * @return $this
     * @throws Exception
     */
    public static function all()
    {
        try {
            $query = new Query();
            $query->by = Query::ALL;
            $query->from = 0;
            $query->to = -1;

            $idDirectory = 0;
            return Directory::assets($query, $idDirectory);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * Find Transfer
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "ASSETS_TRANSFER";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset Transfer id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Asset
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $endDate = $this->getEndDate();
        if (!$endDate) {
            $endDate = "0000-00-00T00:00:00";
        }

        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getDirectoryId(), PDO::PARAM_INT],
            [$this->getManagementName(), PDO::PARAM_STR],
            [$this->getName(), PDO::PARAM_INT],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getCode(), PDO::PARAM_STR],
            [$this->getPassword(), PDO::PARAM_STR],
            [$endDate, PDO::PARAM_STR],
            [$this->isActive(), PDO::PARAM_BOOL],
            [$this->getImageOptimization(), PDO::PARAM_STR],
            [$this->getWaterMarkImage(), PDO::PARAM_STR],
            [$this->getSettings(), PDO::PARAM_STR],
            [$this->getViewOnly(), PDO::PARAM_BOOL],
            [$this->getVerifyIdentity(), PDO::PARAM_BOOL],
            [$this->getCompressEnabled(), PDO::PARAM_BOOL],
            [$this->getContentType(), PDO::PARAM_INT]];

        try {
            $name = "ASSETS_TRANSFER";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_transfer']) && $data[0]['id_transfer'] >= 0) {
                $idTransfer = $data[0]['id_transfer'];
                $this->setId($idTransfer);

                self::createInstance($data[0]);
                return $this;
            }

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset Transfer not exists.", $data[0]['return_code']);
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Asset Transfer from the database
     *
     * @param $idTransfer
     * @return bool
     * @throws Exception
     */
    public static function delete($idTransfer)
    {
        $name = "ASSETS_TRANSFER";            // Remove from the database
        $parameters = [[$idTransfer, PDO::PARAM_INT]];
        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query transfers
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idDirectory = $q->idDirectory;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $term = $q->term;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $filter = $q->filter;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$idDirectory, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$sndField, PDO::PARAM_INT],
                    [$trdField, PDO::PARAM_INT],
                    [$term, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                    [$sndFilter, PDO::PARAM_INT],
                    [$trdFilter, PDO::PARAM_INT],
                ];

                $name = "ASSETS_TRANSFERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT]
            ];

            $name = "ASSETS_TRANSFERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * markedToDelete
     *
     * Get then assets marked to delete from database
     * @return array
     * @throws Exception
     */
    public static function markedToDelete()
    {
        try {
            $parameters = [];
            $name = "ASSETS_MARKED_HAS_DELETED";
            $method = "GET";
            $assets = Stage::esp($parameters, $name, $method);

            return array_map(function ($asset) {
                return self::createInstance($asset);
            }, $assets);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachContent
     *
     * @return bool
     * @throws Exception
     */
    public static function attachContent($idTransfer, $field, $filter, $position, $childDirectories)
    {
        try {
            $parameters = [
                [$idTransfer, PDO::PARAM_INT],
                [$field, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT],
                [$childDirectories, PDO::PARAM_INT],
            ];
            $name = "ASSETS_CONTENT_TO_TRANSFER";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == "0";

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachContent
     *
     * @return bool
     * @throws Exception
     */
    public static function detachContent($idTransfer, $field, $filter)
    {
        try {
            $parameters = [
                [$idTransfer, PDO::PARAM_INT],
                [$field, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
            ];
            $name = "ASSETS_CONTENT_FROM_TRANSFER";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == "0";

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryAssets
     *
     * @return array
     * @throws Exception
     */
    public static function queryAssets(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $idTransfer = $q->idTransfer;
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $parameters = [
                    [$idTransfer, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "ASSETS_ASSETS_ON_TRANSFERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idTransfer, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "ASSETS_ASSETS_ON_TRANSFERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return Asset::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryDirectories
     *
     * @return array
     * @throws Exception
     */
    public static function queryDirectories(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $idTransfer = $q->idTransfer;
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $parameters = [
                    [$idTransfer, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "ASSETS_DIRECTORIES_ON_TRANSFERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idTransfer, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "ASSETS_DIRECTORIES_ON_TRANSFERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return Directory::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "directory" && $this->$attribute instanceof Directory) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" || $attribute == "imageTitle" || $attribute == "imageAlterText" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $transfer = new self();
        $transfer->setId($data['id_transfer']);
        $transfer->setDirectoryId($data['id_directory']);
        $transfer->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $transfer->setManagementName(html_entity_decode($data['management_name'], ENT_COMPAT, 'UTF-8'));
        $transfer->setDescription(html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8'));
        $transfer->setCode($data['code']);
        $transfer->setPassword($data['password']);
        $transfer->setIsActive($data['active']);
        $transfer->setTotalContent($data['total_content']);
        $transfer->setCoverImage($data['cover_image']);
        $transfer->setContentType($data['content_type']);
        $transfer->setImageOptimization($data['image_optimization']);
        $transfer->setWaterMarkImage($data['watermark_image']);
        $transfer->setSettings($data['settings']);
        $transfer->setViewOnly($data['view_only']);
        $transfer->setVerifyIdentity($data['verify_identity']);
        $transfer->setCompressEnabled($data['compress_enabled']);

        if (isset($data['end_date']) && Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $transfer->setEndDate(new DateTime($data['end_date']));
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $transfer->setRegisterDate(new DateTime($data['register_date']));
        }

        return $transfer;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
