<?php namespace Stage\Asset\Transfer;

use PDO;
use DateTime;
use Exception;
use Stage\Paginator;
use Stage\Stage;
use Stage\Helper;

class User
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idTransfer;

    /**
     * @var
     */
    private $idPermission;

    /**
     * @var
     */
    private $idNotice;

    /**
     * @var
     */
    private $idUser;

    /**
     * @var
     */
    private $userType;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $email = "";

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $lastVerifyDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idTransfer', 'idPermission', 'idNotice', 'idUser', 'userType', 'name', 'email', 'lastVerifyDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdTransfer()
    {
        return $this->idTransfer;
    }

    public function getIdPermission()
    {
        return $this->idPermission;
    }

    public function getIdNotice()
    {
        return $this->idNotice;
    }

    public function getIdUser()
    {
        return $this->idUser;
    }

    public function getUserType()
    {
        return $this->userType;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    private function setIdTransfer($id)
    {
        $this->idTransfer = (int)$id;
    }

    private function setIdPermission($id)
    {
        $this->idPermission = (int)$id;
    }

    private function setIdNotice($id)
    {
        $this->idNotice = (int)$id;
    }

    private function setIdUser($id)
    {
        $this->idUser = (int)$id;
    }

    private function setUserType($type)
    {
        $this->userType = $type;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setEmail($email)
    {
        $this->email = $email;
    }

    private function setLastVerifyDate($date)
    {
        $this->lastVerifyDate = $date;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * save
     *
     * Save a Asset Transfer User
     *
     * @return $this
     * @throws Exception
     */
    public function save($idTransfer, $idPermission, $idNotice, $idUser, $userType, $name, $email)
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idTransfer, PDO::PARAM_INT],
            [$idPermission, PDO::PARAM_INT],
            [$idNotice, PDO::PARAM_INT],
            [$idUser, PDO::PARAM_INT],
            [$userType, PDO::PARAM_INT],
            [$name, PDO::PARAM_STR],
            [$email, PDO::PARAM_STR]];

        try {
            $name = "ASSETS_TRANSFER_USERS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id']) && $data[0]['id'] >= 0) {
                return self::createInstance($data[0]);
            }

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset Transfer User not exists.", $data[0]['return_code']);
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Asset Transfer from the database
     *
     * @param $id
     * @return bool
     * @throws Exception
     */
    public static function delete($id)
    {
        $name = "ASSETS_TRANSFER_USER";            // Remove from the database
        $parameters = [[$id, PDO::PARAM_INT]];
        $method = "DEL";

        try {
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query transfers
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $field = $q->by;
        $term = $q->term;
        $filter = $q->filter;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$field, PDO::PARAM_INT],
                    [$term, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "ASSETS_TRANSFERS_USERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$field, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT]
            ];

            $name = "ASSETS_TRANSFERS_USERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "directory" && $this->$attribute instanceof Directory) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "lastVerifyDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $transfer = new self();
        $transfer->setId($data['id']);
        $transfer->setIdTransfer($data['id_transfer']);
        $transfer->setIdPermission($data['id_permission']);
        $transfer->setIdNotice($data['id_notice']);
        $transfer->setIdUser($data['id_user']);
        $transfer->setUserType($data['user_type']);
        $transfer->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $transfer->setEmail($data['email']);

        if (isset($data['last_verify_date']) && Helper::isValidDate($data['last_verify_date'], "Y-m-d H:i:s")) {
            $transfer->setLastVerifyDate(new DateTime($data['last_verify_date']));
        }
        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $transfer->setRegisterDate(new DateTime($data['register_date']));
        }

        return $transfer;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
