<?php namespace Stage\External;

use PDO;
use Exception;
use Stage\Asset\DirectoryQuery;
use Stage\Asset\Download\Download;
use Stage\Asset\Download\Query;
use Stage\Asset\Tag\Tag;
use Stage\Asset\Transfer\Directory;
use Stage\Asset\Transfer\Transfer;
use Stage\Asset\Transfer\User;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;

class Asset
{
    /**
     * find
     *
     * @param $idUser
     * @param $field
     * @param $query
     * @return \Stage\Asset\Asset|null
     * @throws Exception
     */
    public static function find($idUser, $field, $query)
    {
        try {
            $name = "SP_EXTERNAL_GET_ASSETS_ASSET";
            $method = "GET";
            $parameters = [[$idUser, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_asset'])) {
                return \Stage\Asset\Asset::createInstance($data[0]);
            }
            return null;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * createTag
     *
     * Save a Tag
     *
     * @return \Stage\Asset\Asset
     * @throws Exception
     */
    public static function create($idAsset, $idDirectory, $nameAsset, $filename, $extension, $mimetype, $size, $width, $height, $idStorage, $idCdn)
    {
        try {
            $name = "SP_EXTERNAL_SET_ASSETS";
            $method = "SET";
            $parameters = [[$idAsset, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$nameAsset, PDO::PARAM_STR],
                [$filename, PDO::PARAM_STR],
                [$extension, PDO::PARAM_STR],
                [$mimetype, PDO::PARAM_STR],
                [$size, PDO::PARAM_INT],
                [$width, PDO::PARAM_INT],
                [$height, PDO::PARAM_INT],
                [$idStorage, PDO::PARAM_INT],
                [$idCdn, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_asset'])) {
                return \Stage\Asset\Asset::createInstance($data[0]);
            }
            return null;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * createTag
     *
     * Save a Tag
     *
     * @return Tag
     * @throws Exception
     */
    public static function createTag($id, $parentId, $nameTag, $nameStripped, $public, $ifExists = 0)
    {
        try {
            $name = "SP_EXTERNAL_SET_ASSETS_TAGS";
            $method = "SET";
            $parameters = [[$id, PDO::PARAM_INT],
                [$parentId, PDO::PARAM_INT],
                [$nameTag, PDO::PARAM_STR],
                [$nameStripped, PDO::PARAM_STR],
                [$public, PDO::PARAM_BOOL],
                [$ifExists, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_tag'])) {
                return Tag::createInstance($data[0]);
            }
            return null;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param $idTag
     * @param $idAsset
     * @return boolean
     * @throws Exception
     */
    public static function addTag($idTag, $idAsset)
    {
        try {
            $name = "SP_EXTERNAL_SET_ASSETS_TO_TAGS";
            $method = "SET";
            $parameters = [[$idTag, PDO::PARAM_INT],
                [$idAsset, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * shareAndDownloadQuery
     *
     * Search Downloads by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function shareAndDownloadQuery(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $limit_a = $q->from;
        $limit_b = $q->to;
        $filter = $q->filter;

        try {
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_STR],
                [$query, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "EXTERNAL_GET_ASSETS_SHARE_AND_DOWNLOADS";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $download) {
                $downloadInstance = Download::createInstance($download);
                array_push($temp, $downloadInstance);
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * shareAndDownloadQueryCount
     *
     * @param \Stage\Asset\Download\Query $q
     * @return mixed
     * @throws Exception
     */
    public static function shareAndDownloadQueryCount(Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR]];

            $name = "EXTERNAL_GET_ASSETS_SHARE_AND_DOWNLOADS_COUNT";

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int)$data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * transfersQueryAssets
     *
     * @return array
     * @throws Exception
     */
    public static function transfersQueryAssets(\Stage\Asset\Transfer\Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $idTransfer = $q->idTransfer;
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $parameters = [
                    [$idTransfer, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "EXTERNAL_GET_ASSETS_ASSETS_ON_TRANSFERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idTransfer, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "EXTERNAL_GET_ASSETS_ASSETS_ON_TRANSFERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($label) {
                return \Stage\Asset\Asset::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * transfersQueryDirectories
     *
     * @return array
     * @throws Exception
     */
    public static function transfersQueryDirectories(\Stage\Asset\Transfer\Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $idTransfer = $q->idTransfer;
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $parameters = [
                    [$idTransfer, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT],
                ];

                $name = "EXTERNAL_GET_ASSETS_DIRECTORIES_ON_TRANSFERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idTransfer, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
            ];

            $name = "EXTERNAL_GET_ASSETS_DIRECTORIES_ON_TRANSFERS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($label) {
                return Directory::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assetsDirectoriesQuery
     *
     * Query directories
     *
     * @return Array|Int
     * @throws Exception
     */
    public static function assetsDirectoriesQuery(DirectoryQuery $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idDirectory = $q->idDirectory;
        $field = $q->by;
        $term = $q->term;
        $filter = $q->filter;
        $order_by = $q->orderBy;
        $order = $q->order;

        try {
            if ($count == true) {
                $parameters = [
                    [$idDirectory, PDO::PARAM_INT],
                    [$field, PDO::PARAM_STR],
                    [$term, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_INT]];

                $name = "ASSETS_DIRECTORIES_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();

            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order_by, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT]];


            $name = "ASSETS_DIRECTORIES";
            $data = Stage::esp($parameters, $name, $method);
            if ($q->raw) {
                return $data;
            }

            $result = new MyArrayObject(array_map(function ($label) {
                return \Stage\Asset\Directory::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * findTransferByCode
     *
     * Find Transfer
     *
     * @return Transfer
     * @throws Exception
     */
    public static function findTransferByCode($code)
    {
        try {
            $name = "EXTERNAL_GET_ASSETS_TRANSFER";
            $method = "GET";
            $parameters = [[$code, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Asset Transfer id $code not exists.", $data[0]['return_code']);
            } else {
                return Transfer::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assetsByDirectory
     *
     * Query directories
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function assetsByDirectory($idDirectory, $limitA = 0, $limitB = null)
    {
        $method = "GET";

        try {
            $parameters = [
                [$idDirectory, PDO::PARAM_INT]];

            $name = "EXTERNAL_GET_ASSETS_ON_DIRECTORIES_COUNT";
            $data = Stage::esp($parameters, $name, $method);
            $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;
            if ($total <= 0) {
                return new MyArrayObject([]);
            }

            $limitB = $limitB ?: $total;
            $parameters = [[$idDirectory, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_STR],
                [$limitB, PDO::PARAM_STR]];

            $name = "EXTERNAL_GET_ASSETS_ON_DIRECTORIES";
            $data = Stage::esp($parameters, $name, $method);

            return new MyArrayObject(array_map(function ($label) {
                return \Stage\Asset\Asset::createInstance($label);
            }, $data));
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assets
     *
     * Query assets inside a directory. Pass $idDirectory=0 to query assets inside all directories
     * @return $this
     * @throws Exception
     */
    public static function assets(\Stage\Asset\Query $q = null, $idDirectory = 0)
    {
        $room = $q->room;
        $idUser = $q->idUser;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $term = $q->term;
        $filter = $q->filter;
        $from = $q->from;
        $to = $q->to;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;
        $orderBy = $q->orderBy;
        $order = $q->order;

        try {

            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_INT],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT],
                [$from, PDO::PARAM_INT],
                [$to, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT]];

            $name = "EXTERNAL_GET_ASSETS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($asset) {
                return \Stage\Asset\Asset::createInstance($asset);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assetsCount
     *
     * @param \Stage\Asset\Query|null $q
     * @param int $idDirectory
     * @return array
     * @throws Exception
     */
    public static function assetsCount(Query $q = null, $idDirectory = 0)
    {
        $room = $q->room;
        $idUser = $q->idUser;
        $field = $q->by;
        $sndField = $q->sndBy;
        $trdField = $q->trdBy;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $term = $q->term;
        $filter = $q->filter;
        $limit_size_a = $q->fromSize;
        $limit_size_b = $q->toSize;

        try {
            $parameters = [[$room, PDO::PARAM_STR],
                [$idUser, PDO::PARAM_INT],
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$term, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_STR],
                [$trdFilter, PDO::PARAM_STR],
                [$limit_size_a, PDO::PARAM_INT],
                [$limit_size_b, PDO::PARAM_INT]];

            $name = "EXTERNAL_GET_ASSETS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * directories
     *
     * Query directories
     *
     * @return Array|Int
     * @throws Exception
     */
    public static function directories($idDirectory, $field, $query, $filter)
    {
        $method = "GET";

        try {

            $parameters = [
                [$idDirectory, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR]];


            $name = "EXTERNAL_GET_ASSETS_DIRECTORIES";
            return Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * build
     *
     * Build the tree array of directories
     *
     * @return array
     * @throws Exception
     * @url http://stackoverflow.com/questions/4452472/category-hierarchy-php-mysql
     */
    public static function build($idDirectory = 1, $field = 1, $query = "", $filter = 1)
    {
        $directories = self::directories($idDirectory, $field, $query, $filter);
        $rootId = 1;
        $refs = array();
        $list = array();
        foreach ($directories as $row) {
            $ref = &$refs[$row['id_directory']];

            $ref['id'] = (int)$row['id_directory'];
            $ref['parentId'] = (int)$row['parent_id_directory'];
            $ref['name'] = html_entity_decode($row['name'], ENT_COMPAT, 'UTF-8');
            $ref['description'] = html_entity_decode($row['description'], ENT_COMPAT, 'UTF-8');
            $ref['link'] = $row['link'];
            $ref['public'] = $row['public'];
            $ref['files'] = $row['total_items'];

            if ($row['parent_id_directory'] == $rootId) {
                //$list[$row['id_directory']] = & $ref; // Put this if you want the array keys has directory id
                $list[] = &$ref;
            } else {
                // $refs[$row['parent_id_directory']]['children'][$row['id_directory']] = & $ref;   // Put this if you want the array keys has directory id
                $refs[$row['parent_id_directory']]['children'][] = &$ref;
            }
        }

        return $list;
    }

    /**
     * externalShareAndDownloadAssetsQuery
     *
     * Search Downloads by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function externalShareAndDownloadAssetsQuery($id, $limitA, $limitB)
    {
        try {
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_STR]];

            $name = "EXTERNAL_GET_ASSETS_SHARE_AND_DOWNLOAD_CONTENT";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $download) {
                $downloadInstance = \Stage\Asset\Asset::createInstance($download);
                array_push($temp, $downloadInstance);
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * externalShareAndDownloadAssetsQueryCount
     *
     * @param $id
     * @return mixed
     * @throws Exception
     */
    public static function externalShareAndDownloadAssetsQueryCount($id)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "EXTERNAL_GET_ASSETS_SHARE_AND_DOWNLOAD_CONTENT_COUNT";

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int)$data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * externalAssetsTransfersUsers
     *
     * Search Users by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function externalAssetsTransfersUsers(\Stage\Asset\Transfer\Query $q)
    {
        $field = $q->by;
        $query = $q->term;
        $limit_a = $q->from;
        $limit_b = $q->to;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_STR],
                [$query, PDO::PARAM_INT],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],];

            $name = "EXTERNAL_GET_ASSETS_TRANSFERS_USERS";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $download) {
                $downloadInstance = User::createInstance($download);
                array_push($temp, $downloadInstance);
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * externalSetTransfersUserVerify
     *
     * @return bool
     * @throws Exception
     */
    public static function externalSetTransfersUserVerify($idTransfer, $email)
    {
        try {
            $name = "EXTERNAL_SET_ASSETS_TRANSFER_USERS_VERIFY";
            $method = "SET";
            $parameters = [[$idTransfer, PDO::PARAM_INT],
                [$email, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * externalTransferPermission
     *
     * @return array
     * @throws Exception
     */
    public static function externalTransferPermission($idPermission)
    {
        try {
            $name = "EXTERNAL_GET_ASSETS_TRANSFER_PERMISSION";
            $method = "GET";
            $parameters = [[$idPermission, PDO::PARAM_INT]];

            return Stage::esp($parameters, $name, $method);

        } catch (Exception $e) {
            throw $e;
        }
    }

}
