<?php namespace Stage\Lead;

use PDO;
use DateTime;
use Exception;
use Stage\Helper;
use Stage\Account\Account;
use Stage\Stage;
use Stage\Lead\Tag\Tag;
use Stage\Lead\HardTag\HardTag;

class Lead
{
    const CREATE = 0;           // Create a new Lead
    const UPDATE = 1;           // Faz update á Lead caso ela exista
    const FORCE_CREATE = -1;    // Cria sempre uma nova Lead

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $title = "";

    /**
     * @var
     */
    private $firstName = "";

    /**
     * @var
     */
    private $lastName = "";

    /**
     * @var
     */
    private $address = "";

    /**
     * @var
     */
    private $city = "";

    /**
     * @var
     */
    private $country = "";

    /**
     * @var
     */
    private $phone = "";

    /**
     * @var
     */
    private $mobilePhone = "";

    /**
     * @var
     */
    private $company = "";

    /**
     * @var
     */
    private $website = "";

    /**
     * @var
     */
    private $email = "";

    /**
     * @var
     */
    private $birthdate = "";

    /**
     * @var
     */
    private $notes = "";

    /**
     * @var
     */
    private $gender = 0;    // 0 - None, 1 - Male, 2 - Female

    /**
     * @var string
     */
    private $registerDate = "";

    /**
     * @var string
     */
    private $hardTag = null;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','title','firstName','lastName','address','city','country','phone','mobilePhone','company','website','email','birthdate','notes','registerDate','hardTag','gender');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function geFirstName()
    {
        return $this->firstName;
    }

    public function getLastName()
    {
        return $this->lastName;
    }

    public function getAddress()
    {
        return $this->address;
    }

    public function getCity()
    {
        return $this->city;
    }

    public function getCountry()
    {
        return $this->country;
    }

    public function getPhone()
    {
        return $this->phone;
    }

    public function getMobilePhone()
    {
        return $this->mobilePhone;
    }

    public function getCompany()
    {
        return $this->company;
    }

    public function getWebsite()
    {
        return $this->website;
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getBirthdate()
    {
        return $this->birthdate;
    }

    public function getNotes()
    {
        return $this->notes;
    }

    public function getGender()
    {
        return $this->gender;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getHardTag()
    {
        return $this->hardTag;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setFirstName($name)
    {
        $this->firstName = $name;
    }

    public function setLastName($name)
    {
        $this->lastName = $name;
    }

    public function setAddress($address)
    {
        $this->address = $address;
    }

    public function setCity($city)
    {
        $this->city = $city;
    }

    public function setCountry($country)
    {
        $this->country = $country;
    }

    public function setPhone($phone)
    {
        $this->phone = $phone;
    }

    public function setMobilePhone($mobilephone)
    {
        $this->mobilePhone = $mobilephone;
    }

    public function setCompany($company)
    {
        $this->company = $company;
    }

    public function setWebsite($website)
    {
        $this->website = $website;
    }

    public function setEmail($email)
    {
        $this->email = $email;
    }

    public function setBirthDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->birthdate = "";
        } else {
            $date = new DateTime($date);
            $this->birthdate = $date->format(DateTime::ATOM);
        }
    }

    public function setNotes($notes)
    {
        $this->notes = $notes;
    }

    public function setGender($gender)
    {
        return $this->gender = $gender;
    }

    public function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    public function setHardTag($hardTag)
    {
        $this->hardTag = $hardTag;
    }

    /**
     * @param $idAccount
     * @return array
     */
    public static function sizeof($idAccount)
    {
        $name = "LEADS_NUM";
        $method = "GET";
        $parameters = [[$idAccount, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Lead
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        $parameters = [[$id, PDO::PARAM_INT]];

        try {
            $name = "LEAD";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code'])) {
                throw new Exception("Lead with id $id not exists", $data[0]['return_code']);
            }

            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Lead
     *
     * @return $this
     * @throws Exception
     */
    public function save($idAccount, $overwrite)
    {
        $id = $overwrite == Lead::UPDATE ? $this->getId() : $overwrite;
        $parameters = [[$id, PDO::PARAM_INT],
                       [$idAccount, PDO::PARAM_INT],
                       [$overwrite, PDO::PARAM_INT],
                       [$this->getTitle(), PDO::PARAM_STR],
                       [$this->geFirstName(), PDO::PARAM_STR],
                       [$this->getLastName(), PDO::PARAM_STR],
                       [$this->getGender(), PDO::PARAM_INT],
                       [$this->getAddress(), PDO::PARAM_STR],
                       [$this->getCity(), PDO::PARAM_STR],
                       [$this->getCountry(), PDO::PARAM_STR],
                       [$this->getPhone(), PDO::PARAM_STR],
                       [$this->getMobilePhone(), PDO::PARAM_STR],
                       [$this->getCompany(), PDO::PARAM_STR],
                       [$this->getWebsite(), PDO::PARAM_STR],
                       [$this->getEmail(), PDO::PARAM_STR],
                       [$this->getBirthdate(), PDO::PARAM_STR],
                       [$this->getNotes(), PDO::PARAM_STR]];

        try {
            $name = "LEAD";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_lead'])) {
                $this->setId($data[0]['id_lead']);
                $this->setRegisterDate(new DateTime($data[0]['register_date']));
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query a Lead
     *
     * @return Array
     * @throws Exception
     */
    public static function query(Query $q, $idAccount)
    {
        $limit_a = $q->from;
        $limit_b = $q->to;
        try {
            $parameters = [ [$idAccount, PDO::PARAM_INT],
                            [$q->by, PDO::PARAM_INT],
                            [$q->term, PDO::PARAM_STR],
                            [$q->filter,PDO::PARAM_STR],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "LEADS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($lead) {
                return self::createInstance($lead);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Tag $tag
     * @return Tag
     * @throws Exception
     */
    public function addTag(Tag $tag)
    {
        $parameters = [[$tag->getId(), PDO::PARAM_INT],
                       [$this->getId(), PDO::PARAM_INT]];

        try {
            $name = "LEADS_TO_TAGS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return $tag;
            } elseif (isset($data[0]['return_code']) && $data[0]['return_code'] == "-1") {
                throw new Exception("You have already added this tag.", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addHardTag
     *
     * @param HardTag $tag
     * @return $this
     * @throws Exception
     */
    public function addHardTag($token, Account $account)
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$account->getId(), PDO::PARAM_INT],
                       [$token, PDO::PARAM_INT]];

        try {
            $name = "LEADS_TO_HARDTAGS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return $this;
            } elseif (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Hardtag not found with token $token.", $data[0]['return_code']);
            } else {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Tag $tag
     * @return Tag
     * @throws Exception
     */
    public function removeTag(Tag $tag)
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$tag->getId(), PDO::PARAM_INT]];

        try {
            $name = "LEADS_FROM_TAG";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "0") {
                return $tag;
            } elseif (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("You have already removed this tag.", $data[0]['return_code']);
            } else {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * deleteTags
     *
     * @throws Exception
     */
    public function deleteTags()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT]];

        try {
            $name = "LEADS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT]];

        try {
            $name = "LEADS_TAGS_RELATED";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code'])) {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag::createInstance($tag);
                }, $data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Lead
     * @param $idAccount
     * @param $idLead
     * @return $int
     * @throws Exception
     */
    public static function delete($idAccount, $idLead)
    {
        $parameters = [[$idAccount, PDO::PARAM_INT],
                       [$idLead, PDO::PARAM_INT]];

        try {
            $name = "LEAD";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Lead with id $idLead not found", $data[0]['return_code']);
            }

            return $idLead;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * campaign
     *
     * @param $idAccount
     * @param $field
     * @param $description
     * @return bool
     * @throws Exception
     */
    public function campaign($idAccount, $field, $description)
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$idAccount, PDO::PARAM_INT],
                       [$field, PDO::PARAM_STR],
                       [$description, PDO::PARAM_STR]];

        try {
            $name = "LEADS_TO_CAMPAIGNS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("Lead not found with id $this->getId().", $data[0]['return_code']);
            }

            return true;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Dossier
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id_lead']);
        $instance->setTitle(html_entity_decode($data['title'], ENT_COMPAT, 'UTF-8'));
        $instance->setFirstName(html_entity_decode($data['firstname'], ENT_COMPAT, 'UTF-8'));
        $instance->setLastName(html_entity_decode($data['lastname'], ENT_COMPAT, 'UTF-8'));
        $instance->setAddress(html_entity_decode($data['address'], ENT_COMPAT, 'UTF-8'));
        $instance->setCity(html_entity_decode($data['city'], ENT_COMPAT, 'UTF-8'));
        $instance->setCountry($data['country']);
        $instance->setPhone($data['phone']);
        $instance->setMobilePhone($data['mobilephone']);
        $instance->setCompany(html_entity_decode($data['company'], ENT_COMPAT, 'UTF-8'));
        $instance->setWebsite($data['website']);
        $instance->setEmail($data['email']);
        $instance->setGender($data['gender']);

        if (Helper::isValidDate($data['birthdate'], "Y-m-d")) {
            $instance->setBirthDate($data['birthdate']);
        }

        $instance->setNotes(html_entity_decode($data['notes'], ENT_COMPAT, 'UTF-8'));
        if (isset($data['register_date'])) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['id_hardtag']) && $data['id_hardtag'] != "" && isset($data['name_stripped']) && $data['name_stripped'] != "") {
            $instance->setHardTag(['id'=>$data['id_hardtag'],'nameStripped'=>html_entity_decode($data['name_stripped'], ENT_COMPAT, 'UTF-8'),'name'=>html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8')]);
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
