<?php namespace Stage\Lead;

use stdClass;

class Query
{
    const ALL                    = 0;  // SEARCH IN FIELDS: TAGS, TITLE, FIRST_NAME, LAST_NAME
    const TITLE                  = 1;  // SEARCH IN FIELDS: TITLE
    const FIRSTNAME              = 2;  // SEARCH IN FIELDS: FIRST_NAME
    const LASTNAME               = 3;  // SEARCH IN FIELDS: LAST_NAME
    const COUNTRY                = 4;  // SEARCH IN FIELDS: COUNTRY
    const EMAIL                  = 5;  // SEARCH IN FIELDS: EMAILS
    const NOTE                   = 6;  // SEARCH IN FIELDS: NOTES
    const TAG                    = 7;  // SEARCH IN FIELDS: TAG
    const HARDTAG                = 8;  // SEARCH IN FIELDS: HARDTAG
    const DATE                   = 9;  // SEARCH IN FIELDS: DATE ( When using this field must pass like YYYY-MM-DD )
    const STARTWITH              = 10; // FILTER IN FIRST_NAME, LAST_NAME, EMAIL starting with a char x
    const PHONE                  = 11; // FILTER IN PHONE

    const TITLE_IN_TAGS          = 12; // SEARCH IN FIELDS: TITLE with a tag filter
    const FIRSTNAME_IN_TAGS      = 13; // SEARCH IN FIELDS: FIRST:NAME with a tag filter
    const LASTNAME_IN_TAGS       = 14;
    const COUNTRY_IN_TAGS        = 15;
    const EMAIL_IN_TAGS          = 16;
    const NOTE_IN_TAGS           = 17;
    const DATE_IN_TAGS           = 24;
    const STARTWITH_IN_TAGS      = 26;
    const PHONE_IN_TAGS          = 27;

    const TITLE_IN_HARDTAGS      = 18;
    const FIRSTNAME_IN_HARDTAGS  = 19;
    const LASTNAME_IN_HARDTAGS   = 20;
    const COUNTRY_IN_HARDTAGS    = 21;
    const EMAIL_IN_HARDTAGS      = 22;
    const NOTE_IN_HARDTAGS       = 23;
    const DATE_IN_HARDTAGS       = 27;
    const STARTWITH_IN_HARDTAGS  = 28;
    const PHONE_IN_HARDTAGS      = 29;

    /**
     * @var
     */
    public $by;

    /**
     * @var
     */
    public $term = "";

    /**
     * @var
     */
    public $filter = "";

    /**
     * @var
     */
    public $operators = [
                'title',
                'firstname',
                'lastname',
                'country',
                'email',
                'note',
                'date',
                'startwith',
                'phone'
            ];

    /**
     * @var
     */
    public $separator = ":";

    /**
     * @var
     */
    public $from = 0;

    /**
     * @var
     * If you pass $to=-1 the results will be from=FIRST_ROW to=LAST_ROW
     */
    public $to = 15;

    public function __construct(stdClass $query)
    {
        $this->buildQuery($query);
    }

    public function __get($name)
    {
        return $this->$name;
    }

    public function __set($name, $value)
    {
        $this->$name = $value;
    }

    private function buildQuery($query)
    {
        $extracted  = $this->extract($query);
        $operator   = strtoupper($extracted['operator']);  // Operator could be title, firstname
        $term       = $extracted['term'];
        $filter     = isset($query->filter) ? $query->filter : "";

        $className = get_class($this);
        switch ($filter) {
            case 'tag':

                $const = !$operator ? 'TAG' : $operator . '_IN_TAGS';
                $by = constant($className . '::' . $const);
                break;

            case 'hardtag':

                $const = !$operator ? 'HARDTAG' : $operator . '_IN_HARDTAGS';
                $by = constant($className . '::' . $const);
                break;
                        
            default:

                $const = $operator;
                $by = constant($className . '::' . (!$const ? 'ALL' : $const));
                break;
        }
        
        // Data passed to SP
        $this->by       = trim($by);
        $this->term     = trim($term);
        $this->filter   = isset($query->tags) ? trim($query->tags) : $this->filter;
        $this->from     = isset($query->from) ? $query->from : $this->from;
        $this->to       = isset($query->to) ? $query->to : $this->to;
    }

    public function extract($query)
    {
        if (!isset($query->term)) {
            return ['term' => $this->term, 'operator' => null];
        }

        $found = false;
        $foundAt = -1;
        $i = 0;
        while (!$found && $i < sizeof($this->operators)) {
            $filterBy = $this->operators[$i].":";   // FilterBy= title: or email:
            $len = strlen($filterBy);
            
            if (substr(trim($query->term), 0, $len) === $filterBy) {
                $found = true;
                $foundAt = $i;
                $term = substr(trim($query->term), strpos(trim($query->term), $this->separator) + 1);
            }

            $i++;
        }
        
        return ['term' => $found ? $term : $query->term, 'operator' => $foundAt < 0 ? null : $this->operators[$foundAt]];
    }
}
