<?php namespace Stage\Live\Playlist;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Live\Playlist\Tag\Tag;
use Stage\Stage;
use Stage\Live\Channel\Channel;

class Playlist
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $managementName = "";

    /**
     * @var L10n[]
     */
    private $l10n = [];

    /**
     * @var
     */
    private $position = null;

    /**
     * @var
     */
    private $state;

    /**
     * Total composes inside this channel
     * @var array
     */
    private $totalComposesIn = 0;

    /**
     * @var
     */
    private $beginDate = 0;

    /**
     * @var
     */
    private $publishDate = 0;

    /**
     * @var
     */
    private $endDate = 0;

    /**
     * @var
     */
    private $registerDate = 0;

    /**
     * @var
     */
    private $updateDate = 0;

    /**
     *
     * @var bool
     */
    private $hasVod = false;

    /**
     *
     * @var bool
     */
    private $hasLive = false;

    /**
     *
     * @var
     */
    private $onChannel;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','state',/*'onChannel',*/'managementName', 'publishDate', 'beginDate', 'endDate','registerDate','updateDate','l10n','hasVod','hasLive', 'position', 'totalComposesIn');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getL10n()
    {
        return $this->l10n;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getPublishDate()
    {
        return $this->publishDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getOnChannel()
    {
        return $this->onChannel;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getTotalComposesIn()
    {
        return $this->totalComposesIn;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function hasVod()
    {
        return $this->hasVod;
    }

    public function hasLive()
    {
        return $this->hasLive;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setOnChannel($channel)
    {
        $this->onChannel = (int) $channel;
    }

    public function addL10n(L10n $l10n)
    {
        $this->l10n[] = $l10n;
    }

    public function setManagementName($managementName)
    {
        $this->managementName = $managementName;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->beginDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setPublishDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->publishDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->publishDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->endDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->updateDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    public function setHasVod($has)
    {
        $this->hasVod = (bool) $has;
    }

    public function setHasLive($has)
    {
        $this->hasLive = (bool) $has;
    }

    public function setPosition($position)
    {
        $this->position = (int) $position;
    }

    public function setState($state)
    {
        $this->state = (int) $state;
    }

    public function setTotalComposesIn($total)
    {
        $this->totalComposesIn = (int) $total;
    }

    /**
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        $name = "LIVE_PLAYLISTS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Playlist
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "LIVE_PLAYLIST";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Playlist id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Posts
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->term = "";
        $q->by = "-1";
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * query
     *
     * Search Playlist by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $query = $q->term;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $method = "GET";
            $parameters = [ [$query, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "LIVE_VIDEO_PLAYLISTS";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $playlist) {
                $playlistInstance = self::createInstance($playlist);
                array_push($temp, $playlistInstance);
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * @param \Stage\Video\Playlist\Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        try {
            $query = $q->term;
            $parameters = [[$query, PDO::PARAM_INT]];
            $name = "LIVE_VIDEO_PLAYLISTS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int) $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * duplicate
     *
     * Returns a new Playlist duplicated, with a new Playlist id
     *
     * @return $this
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "LIVE_PLAYLIST_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Returns a Playlist deleted
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "LIVE_PLAYLIST";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the channel tags
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "LIVE_PLAYLIST_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag $tag
     * @throws Exception
     */
    public function addTag(Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [ [$idTag, PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT]];

            $name = "LIVE_PLAYLISTS_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $this->tags = $this->tags();
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "LIVE_PLAYLISTS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Playlist
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $action = $this->getId() == -1 ? "CREATE" : "UPDATE";
            $name = "LIVE_PLAYLISTS";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$this->getManagementName(), PDO::PARAM_STR],
                            [$this->getPublishDate(), PDO::PARAM_STR],
                            [$this->getBeginDate(), PDO::PARAM_STR],
                            [$this->getEndDate(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if ($action == "CREATE") {
                $this->setId($data[0]['id_playlist']);
                \Stage\Events\Event::fire("playlist.created", $this);
            } elseif ($action == "UPDATE") {
                \Stage\Events\Event::fire("playlist.updated", $this);
            }

            // Remove all Playlist L10ns
            for ($i = 0 ; $i <= sizeof($this->getL10n()) -1 ; $i++) {
                if ($this->l10n[$i]->getId() != 0) {
                    $this->l10n[$i]->delete();
                    unset($this->l10n[$i]);
                }
            }

            // Sort array
            array_values($this->l10n);

            // Insert all again if there is any
            foreach ($this->getL10n() as $l10n) {
                $l10n->setIdPlaylist($data[0]['id_playlist']);
                $l10n->save();
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * totalComposes
     *
     * Get the total composes inside a Playlist
     * @return int
     * @throws Exception
     */
    public function totalComposes()
    {
        $id = $this->getId();

        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "LIVE_COMPOSES_ON_PLAYLISTS_NUM";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int)$data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * channels
     *
     * @return array
     * @throws Exception
     */
    public function channels()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "LIVE_CHANNELS_BY_PLAYLIST";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($channel) {
                return Channel::createInstance($channel);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute =="name" || $attribute == "managementName") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } elseif ($attribute == "l10n") {
                foreach ($this->$attribute as $l10n) {
                    $arr[$attribute][] = $l10n->toArray();
                }
            } else {
                if ($this->$attribute !== null) {
                    $arr[$attribute] = $this->$attribute;
                }
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Playlist
     */
    public static function createInstance($data)
    {
        $p = new self();
        $p->setId($data['id_playlist']);
        $p->setManagementName(html_entity_decode($data['management_name'], ENT_COMPAT, 'UTF-8'));
//        if( isset($data['on_channel']) )
//        {
//            $p->setOnChannel($data['on_channel']);
//        }

        if (isset($data['total_composes_in'])) {
            $p->setTotalComposesIn($data['total_composes_in']);
        }

        if (isset($data['position'])) {
            $p->setPosition($data['position']);
        }

        if (isset($data['state'])) {
            $p->setState($data['state']);
        }

        if (isset($data['has_live'])) {
            $p->setHasLive($data['has_live']);
        }

        if (isset($data['has_vod'])) {
            $p->setHasVod($data['has_vod']);
        }

        if (Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $p->setBeginDate(new DateTime($data['begin_date']));
        }

        if (Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $p->setEndDate(new DateTime($data['end_date']));
        }

        if (Helper::isValidDate($data['publish_date'], "Y-m-d H:i:s")) {
            $p->setPublishDate(new DateTime($data['publish_date']));
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $p->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $p->setUpdateDate(new DateTime($data['update_date']));
        }

        $l10ns = L10n::find($data['id_playlist']);
        foreach ($l10ns as $l10n) {
            $p->addL10n($l10n);
        }

        return $p;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
