<?php namespace Stage\Live\Sticker;

use PDO;
use Exception;
use Stage\Stage;

class Sticker
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
    * @var
    */
    private $totalVideo;

    /**
    * @var
    */
    private $totalLive;

    /**
     * @var
     */
    private $position;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','nameStripped','totalVideo', 'totalLive', 'position');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getTotalVideo()
    {
        return $this->totalVideo;
    }

    public function getTotalLive()
    {
        return $this->totalLive;
    }

    public function getPosition($position)
    {
        return $this->position;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setTotalVideo($total)
    {
        $this->totalVideo = (int) $total > 0 ? $total : 0;
    }

    public function setTotalLive($total)
    {
        $this->totalLive = (int) $total > 0 ? $total : 0;
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "LIVE_STICKERS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    public function setPosition($position)
    {
        $this->position = (int) $position;
    }

    /**
     * save
     *
     * Save a tag to the database and return a Tag instance
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "LIVE_STICKER";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$this->getName(), PDO::PARAM_STR],
                            [$this->getNameStripped(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_sticker']);

            // Fire an event
            if (!$this->getId()) {
                \Stage\Events\Event::fire("sticker.created", $this);
            } else {
                \Stage\Events\Event::fire("sticker.updated", $this);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * @param $idChannel
     * @param $idPlaylist
     * @param $idComposed
     * @param $beginDate
     * @param $endDate
     * @return $this
     * @throws Exception
     */
    public function saveRelated($idChannel, $idPlaylist, $idComposed, $position, $beginDate, $endDate)
    {
        try {
            $name = "LIVE_STICKER_RELATED";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$idChannel, PDO::PARAM_INT],
                            [$idPlaylist, PDO::PARAM_INT],
                            [$idComposed, PDO::PARAM_STR],
                            [$position, PDO::PARAM_INT],
                            [$beginDate, PDO::PARAM_INT],
                            [$endDate, PDO::PARAM_STR]];

            return Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Tag
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "LIVE_STICKER";
            $method = "DEL";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function deleteRelated()
    {
        try {
            $name = "LIVE_STICKERS_RELATED";
            $method = "DEL";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * find
     *
     * Finds a Label
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "LIVE_STICKER";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Sticker id $id not exists.", $data[0]['return_code']);
            } else {
                return static::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Tags
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->by = Query::ALL;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return static::query($q);
    }

    /**
     * query
     *
     * Search Labels
     *
     * @return array
     * @throws Exception
     */
    public static function query(Query $q, $count = false)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            if ($count) {
                $parameters = [ [$field, PDO::PARAM_STR],
                                [$query, PDO::PARAM_INT],
                                [$filter, PDO::PARAM_INT]];
                $name = "LIVE_STICKERS_COUNT";
            } else {
                $parameters = [ [$field, PDO::PARAM_STR],
                                [$query, PDO::PARAM_INT],
                                [$filter, PDO::PARAM_INT],
                                [$limit_a, PDO::PARAM_INT],
                                [$limit_b, PDO::PARAM_INT]];
                $name = "LIVE_STICKERS";
            }

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if ($count) {
                $temp = $data[0]['total'];
            } else {
                $temp = [];
                foreach ($data as $tag) {
                    $tagInstance = static::createInstance($tag);
                    array_push($temp, $tagInstance);
                }
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Query $q
     * @param bool $count
     * @return array
     * @throws Exception
     */
    public static function queryRelated(Query $q, $count = false)
    {
        $field = $q->by;
        $query = $q->term;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_INT],
                            [$filter, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];


            $name = "LIVE_STICKERS_RELATED";

            $method = "GET";
            return Stage::esp($parameters, $name, $method);

//            $temp = [];
//            foreach($data as $tag){
//                $tagInstance = static::createInstance($tag);
//                array_push($temp, $tagInstance);
//            }
//            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $url
     * @return array
     * @throws Exception
     */
    public static function slugExists($id, $url)
    {
        try {
            $name = "LIVE_STICKERS_NAME_NUM";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT],
                           [$url, PDO::PARAM_STR]];
            return Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new static();
        $tagInstance->setId($tag['id_sticker']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setTotalVideo(isset($tag['total_video']) ? (int) $tag['total_video'] : 0);
        $tagInstance->setTotalLive(isset($tag['total_live']) ? (int) $tag['total_live'] : 0);
        $tagInstance->setPosition(isset($tag['position']) ? (int) $tag['position'] : 0);

        return $tagInstance;
    }
}
