<?php namespace Stage\Live\Stream;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class Stream
{

    /**
     * @var
     */
    private $id = -1;

    /**
     *
     * @var array
     */
    private $cdn = "";

    /**
     *
     * @var array
     */
    private $name = "";

    /**
     *
     * @var
     */
    private $application = "";

    /**
     *
     * @var
     */
    private $server = "";

    /**
     *
     * @var
     */
    private $stream = "";

    /**
     *
     * @var
     */
    private $dvr = "";

    /**
     *
     * @var
     */
    private $record = "";

    /**
     *
     * @var
     */
    private $hasClipping = "";

    /**
     *
     * @var string
     */
    private $registerDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name','application','registerDate','stream','dvr','record','cdn','server','hasClipping');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getCdn()
    {
        return $this->cdn;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getApplication()
    {
        return $this->application;
    }

    public function getServer()
    {
        return $this->server;
    }

    public function getStream()
    {
        return $this->stream;
    }

    public function getDvr()
    {
        return (bool) $this->dvr;
    }

    public function getRecord()
    {
        return (bool) $this->record;
    }

    public function getHasClipping()
    {
        return (bool) $this->hasClipping;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setCdn($name)
    {
        $this->cdn = $name;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setApplication($app)
    {
        $this->application = $app;
    }

    public function setServer($name)
    {
        $this->server = $name;
    }

    public function setStream($name)
    {
        $this->stream = $name;
    }

    public function setDvr($dvr)
    {
        $this->dvr = (bool) $dvr;
    }

    public function setRecord($record)
    {
        $this->record = (bool) $record;
    }

    public function setHasClipping($hasClipping)
    {
        $this->hasClipping = (bool) $hasClipping;
    }

    private function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * sizeof
     *
     * @return array
     */
    public static function sizeof($idAccount)
    {
        $name = "ACCOUNT_LIVE_SERVER_SETTINGS_NUM";
        $method = "GET";
        $parameters = [[$idAccount, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

//    /**
//     * dates
//     *
//     * @return array
//     */
//    public static function dates($idQuery = 0)
//    {
//
//        $name = "LIVE_COMPOSES_DATES";
//        $method = "GET";
//        $parameters = [[$idQuery, PDO::PARAM_INT]];
//        $data = Stage::esp($parameters, $name, $method);
//        return $data;
//    }

    /**
     * find
     *
     * Find a Compose
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id, $idAccount = 0)
    {
        try {
            $q = new Query();
            $q->idStream = $id;
            $q->idAccount = $idAccount;
            $q->by = Query::ID_LIVE;
            $q->filter = $id;
            $data = self::query($q);

            return isset($data[0]) /*&& sizeof($data[0])*/ ? $data[0] : $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Stream by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $idAccount  = $q->idAccount;
        $idLive     = $q->idStream;
        $field      = $q->by;
        $query      = $q->term;
        $filter     = $q->filter;
        $limit_a    = $q->from;
        $limit_b    = $q->to;

        $parameters = [ [$idAccount, PDO::PARAM_INT],
            [$idLive, PDO::PARAM_INT],
            [$field, PDO::PARAM_INT],
            [$query, PDO::PARAM_STR],
            [$filter, PDO::PARAM_STR],
            [$limit_a, PDO::PARAM_INT],
            [$limit_b, PDO::PARAM_INT]];

        try {
            $method = "GET";
            $name = "ACCOUNT_LIVE_SERVER_SETTINGS";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($stream) {
                return self::createInstance($stream);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * @param \Stage\Video\Compose\Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        $idAccount  = $q->idAccount;
        $idLive     = $q->idStream;
        $field      = $q->by;
        $query      = $q->term;
        $filter     = $q->filter;

        $parameters = [ [$idAccount, PDO::PARAM_INT],
            [$idLive, PDO::PARAM_INT],
            [$field, PDO::PARAM_INT],
            [$query, PDO::PARAM_STR],
            [$filter, PDO::PARAM_STR]];

        try {
            $method = "GET";
            $name   = "ACCOUNT_LIVE_SERVER_SETTINGS_COUNT";
            $data = Stage::esp($parameters, $name, $method);
            return (int) $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save($idAccount)
    {
        try {
            $name = "ACCOUNT_LIVE_SERVER_SETTINGS";
            $method = "SET";
            $parameters = [ [$idAccount, PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getDvr(), PDO::PARAM_BOOL],
                [$this->getRecord(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_live']) && $data[0]['id_live'] >= 0) {
                $this->setId($data[0]['id_live']);

                if (isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }
            }
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

//    /**
//     * playlist
//     *
//     * @param Playlist|null $playlist
//     * @return $this
//     * @throws Exception
//     */
//    public function playlists(Playlist $playlist = null)
//    {
//
//        if( $playlist instanceof Playlist )
//        {
//            try {
//
//                $name = "LIVE_COMPOSED_TO_PLAYLISTS";
//                $method = "SET";
//                $parameters = [ [$playlist->getId(), PDO::PARAM_INT],
//                    [$this->getId(), PDO::PARAM_INT]];
//
//                $data = Stage::esp($parameters, $name, $method);
//
//                // Returns -3 in case of playlist already added
//                /*if( isset($data[0]['return_code']) && isset($data[0]['return_code']) != "0")
//                {
//                    throw new Exception("Error trying adding compose id " . $this->getId() . " to playlist " . $playlist->getId(), $data[0]['return_code']);
//                }*/
//
//                return $this;
//
//            } catch(Exception $e){
//                throw $e;
//            }
//        }
//        else
//        {
//            try {
//
//                $name = "LIVE_COMPOSES_PLAYLISTS";
//                $method = "GET";
//                $parameters = [[$this->getId(), PDO::PARAM_INT]];
//
//                $data = Stage::esp($parameters, $name, $method);
//
//                return (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") ? [] : array_map(function($playlist){
//                    return Playlist::createInstance($playlist);
//                }, $data);
//
//            } catch(Exception $e){
//                throw $e;
//            }
//        }
//
//    }

//    /**
//     * removePlaylist
//     *
//     * @param Playlist $playlist
//     * @return $this
//     * @throws Exception
//     */
//    public function removePlaylist(Playlist $playlist)
//    {
//        try {
//
//            $name = "LIVE_COMPOSED_FROM_PLAYLIST";
//            $method = "DEL";
//            $parameters = [[$playlist->getId(), PDO::PARAM_INT],
//                [$this->getId(), PDO::PARAM_INT]];
//
//            $data = Stage::esp($parameters, $name, $method);
//            return $this;
//        } catch(Exception $e){
//            throw $e;
//        }
//    }

//    /**
//     * delete
//     *
//     * Delete a compose
//     */
//    public function delete()
//    {
//        try {
//
//            $name = "LIVE_COMPOSED";
//            $method = "DEL";
//            $parameters = [[$this->getId(), PDO::PARAM_INT]];
//            Stage::esp($parameters, $name, $method);
//            return $this;
//
//        } catch(Exception $e) {
//            throw $e;
//        }
//    }

//    /**
//     * allTagsRelatedWithComposes
//     *
//     * @return array
//     */
//    public static function allTagsRelatedWithComposes()
//    {
//        $name = "LIVE_PROFILE_TAGS_RELATED";
//        $method = "GET";
//
//        $idQuery = 1;
//        $parameters = [[-1, PDO::PARAM_INT],
//            [$idQuery, PDO::PARAM_INT]];
//        $tags = Stage::esp($parameters, $name, $method);
//
//        if( isset($tags[0]['return_code']) )
//        {
//            return [];
//        }
//        else
//        {
//            $temp = [];
//            foreach ($tags as $tag)
//            {
//                $temp[] = Tag::createInstance($tag);
//            }
//
//            return $temp;
//        }
//    }

//    /**
//     * occurredIn
//     *
//     * @param $idComposed
//     * @return array
//     */
//    public static function occurredIn($idComposed)
//    {
//        $name = "LIVE_PLAYLISTS_BY_COMPOSED";
//        $method = "GET";
//
//        $parameters = [[$idComposed, PDO::PARAM_INT]];
//        $playlists = Stage::esp($parameters, $name, $method);
//
//        $playlists = array_map(function ($playlist) {
//            return Playlist::createInstance($playlist)->toArray();
//        }, $playlists);
//
//        for ($i = 0 ; $i <= sizeof($playlists) - 1 ; $i++)
//        {
//
//            // $name = "VIDEO_CHANNELS_BY_PLAYLIST";
//            // $method = "GET";
//
//            $idPlaylist = $playlists[$i]['id'];
//            $p = Playlist::find($idPlaylist);
//
//            // $parameters = [[$idPlaylist, PDO::PARAM_INT]];
//            // $channels = Stage::esp($parameters, $name, $method);
//
//            $playlists[$i]['channels'] = array_map(function ($channel) {
//                return $channel->toArray();
//            }, $p->channels());
//
//        }
//
//        return $playlists;
//    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Stream
     */
    public static function createInstance($data)
    {

//    [id_live] => 1
//    [cdn] => http://assets-oir.youongroup.uon
//    [name] => name1
//    [application] => application1
//    [server] => server1
//    [stream] => stream1
//    [dvr] => 1
//            [record] => 1

        if (isset($data[0]) && sizeof($data[0])) {
            $data = $data[0];
        }

        $stream = new self();
        $stream->setId($data['id_live']);
        if (isset($data['cdn'])) {
            $stream->setCdn($data['cdn']);
        }

        if (isset($data['cdn'])) {
            $stream->setName($data['name']);
        }

        if (isset($data['application'])) {
            $stream->setApplication($data['application']);
        }

        if (isset($data['server'])) {
            $stream->setServer($data['server']);
        }

        if (isset($data['stream'])) {
            $stream->setStream($data['stream']);
        }

        if (isset($data['dvr'])) {
            $stream->setDvr($data['dvr']);
        }

        if (isset($data['record'])) {
            $stream->setRecord($data['record']);
        }

        if (isset($data['has_clipping'])) {
            $stream->setHasClipping($data['has_clipping']);
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $stream->setRegisterDate(new DateTime($data['register_date']));
        }


        return $stream;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
