<?php namespace Stage\Post\Author;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Paginator;
use Stage\Stage;

class Author
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $registerDate = "";

    /**
     * @var
     */
    private $updateDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * sizeof
     *
     * @return int
     */
    public static function sizeof()
    {
        $name = "POSTS_AUTHORS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['total']) ? (int)$data[0]['total'] : 0;
    }

    /**
     * find
     *
     * Find an Author
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "POSTS_AUTHOR";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Author id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Posts
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * query
     *
     * Search Author by Query query
     *
     * @param Query $q
     * @return int|L10n[]|\Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->field;
        $sndField = $q->sndField;
        $trdField = $q->trdField;
        $query = $q->query;
        $filter = $q->filter;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $isoLang = $q->isoLanguage;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [[$field, PDO::PARAM_INT],
                    [$sndField, PDO::PARAM_INT],
                    [$trdField, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$sndFilter, PDO::PARAM_STR],
                    [$trdFilter, PDO::PARAM_STR],
                    [$isoLang, PDO::PARAM_STR]];
                $name = "POSTS_AUTHORS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_STR],
                [$trdFilter, PDO::PARAM_STR],
                [$isoLang, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];
            $name = "POSTS_AUTHORS";

            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($author) {
                return L10n::createInstance($author);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * innerLanguages
     *
     * Get all languages inside a Author
     *
     * @return array
     * @throws Exception
     */
    public function innerLanguages()
    {
        // TODO Review this method
        try {
            $name = "POSTS_AUTHORS_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return array_values(array_unique(array_map(function ($item) {
                return $item['iso_language'];
            }, $data)));
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Returns a Author deleted
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "POST_AUTHOR";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * l10ns
     *
     * @return array
     * @throws \Stage\Exception\StoredProcedure
     */
    public function l10ns()
    {
        return L10n::all($this->getId());
    }

    /**
     * detachAllL10n
     */
    public function detachAllL10n()
    {
        /** @var L10n[] $l10ns */
        $l10ns = $this->l10ns();
        $status = true;
        foreach ($l10ns as $l) {
            if(!L10n::delete($l->getId())){
                $status = false;
            }
        }
        return $status;
    }

    /**
     * save
     *
     * Save an author
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [];

        try {
            $name = "POSTS_AUTHOR";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_author'])) {
                $this->setId($data[0]['id_author']);
                if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * duplicate
     *
     * Returns a new Collection duplicated, with a new Collection id
     *
     * @return $this
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "POSTS_AUTHOR_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * metaUrlExist
     *
     * Check if the author slug url exit
     *
     * @param $idL10n
     * @param $metaUrl
     * @param $iso
     * @return array
     */
    public static function metaUrlExist($idL10n, $metaUrl, $iso)
    {
//        TODO
        $name = "POSTS_AUTHORS_URL_NUM";
        $method = "GET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$metaUrl, PDO::PARAM_STR],
            [$iso, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Author
     */
    public static function createInstance($data)
    {
        $author = new self();
        $author->setId($data['id_author']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $author->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $author->setUpdateDate(new DateTime($data['update_date']));
        }

        return $author;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
