<?php namespace Stage\Post;

use DateTime;
use Exception;
use Stage\Helper;

class BlockType
{

    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $widgetType;

    /**
     * @var
     */
    private $skey;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'nameStripped', 'position', 'active', 'widgetType', 'skey', 'registerDate', 'updateDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function getWidgetType()
    {
        return $this->widgetType;
    }

    public function getSkey()
    {
        return $this->skey;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setPosition($position)
    {
        $this->position = (int)$position;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setWidgetType($widgetType)
    {
        $this->widgetType = $widgetType;
    }

    public function setSkey($key)
    {
        $this->skey = $key;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name" || $attribute == "nameStripped") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $tag
     * @return Type
     * @throws Exception
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_type']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setPosition($tag['position']);
        $tagInstance->setActive($tag['active']);
        $tagInstance->setWidgetType($tag['widget_type']);
        $tagInstance->setSkey($tag['skey']);

        if (isset($tag['register_date']) && Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        if (isset($tag['update_date']) && Helper::isValidDate($tag['update_date'], "Y-m-d H:i:s")) {
            $tagInstance->setUpdateDate(new DateTime($tag['update_date']));
        }

        return $tagInstance;
    }
}
