<?php namespace Stage\Post\Collection;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\Collection\Tag\Tag;
use Stage\Post\Collection\Sticker\Sticker;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Post\Dossier\Dossier;

class Collection
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $managementName;

    /**
     * @var
     */
    private $publishDate;

    /**
     * @var
     */
    private $beginDate;

    /**
     * @var
     */
    private $endDate;

    /**
     * @var
     */
    private $domain;

    /**
     * @var
     */
    private $defaultIdDossier;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var int
     */
    private $totalDossiers = 0;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'managementName', 'publishDate', 'beginDate', 'endDate', 'domain', 'defaultIdDossier', 'position', 'state', 'registerDate', 'updateDate', 'totalDossiers');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setManagementName($name)
    {
        $this->managementName = $name;
    }

    public function setPublishDate($date)
    {

        // Define how is the date format
        if ($date == null) {
            $this->publishDate = "";
        } else {
            //$date = new DateTime($date);
            $this->publishDate = $date->format(DateTime::ATOM);
        }
        //$this->publishDate = $date->format(DateTime::ATOM);;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->beginDate = "";
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->endDate = "";
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setDomain($domain)
    {
        $this->domain = $domain;
    }

    public function setDefaultIdDossier($dossier)
    {
        $this->defaultIdDossier = $dossier;
    }

    public function setPosition($position)
    {
        $this->position = (int)$position;
    }

    public function setTotalDossiers($total)
    {
        $this->totalDossiers = $total;
    }

    public function setState($state)
    {
        $this->state = (int)$state;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    public function getId()
    {
        return $this->id;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getPublishDate()
    {
        return $this->publishDate;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getDomain()
    {
        return $this->domain;
    }

    public function getDefaultIdDossier()
    {
        return $this->defaultIdDossier;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getTotalDossiers()
    {
        return $this->totalDossiers;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    /**
     * duplicate
     *
     * Returns a new Collection duplicated, with a new Collection id
     *
     * @return $this
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "POSTS_COLLECTION_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * sizeof
     *
     * @return int
     * @throws Exception
     */
    public static function sizeof()
    {
        $name = "POSTS_COLLECTIONS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['total']) ? (int)$data[0]['total'] : 0;
    }

    /**
     * find
     *
     * Find a Collection
     *
     * @param $id
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "POSTS_COLLECTION";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code'])) {
                throw new Exception("The Collection id $id not exists or stored procedure error.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Collection
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->from = 0;
        $q->to = -1;

        return static::query($q);
    }

    /**
     * save
     *
     * Save a Collection
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getManagementName(), PDO::PARAM_STR],
            [$this->getPublishDate() == "" ? "0000-00-00" : $this->getPublishDate(), PDO::PARAM_STR],
            [$this->getBeginDate() == "" ? "0000-00-00" : $this->getBeginDate(), PDO::PARAM_STR],
            [$this->getEndDate() == "" ? "0000-00-00" : $this->getEndDate(), PDO::PARAM_STR],
            [$this->getDomain(), PDO::PARAM_STR],
            [$this->getDefaultIdDossier(), PDO::PARAM_INT],
            [$this->getPosition(), PDO::PARAM_INT]];

        try {
            $name = "POSTS_COLLECTIONS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_collection'])) {
                $this->setId($data[0]['id_collection']);
                if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
            } else {
                throw new Exception("Error saving Collection", $data[0]['return_code']);
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Collection
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "COLLECTION";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query a Collection
     *
     * @param Query $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $field = $q->field;
        $sndField = $q->sndField;
        $trdField = $q->trdField;
        $query = $q->query;
        $filter = $q->filter;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $limit_sys_a = $q->limitSysA;
        $limit_sys_b = $q->limitSysB;
        $order = $q->order;
        $orderBy = $q->orderBy;
        $isoLanguage = $q->isoLanguage;

        try {
            if ($count == true) {
                $parameters = [
                    [$field, PDO::PARAM_INT],
                    [$sndField, PDO::PARAM_INT],
                    [$trdField, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$sndFilter, PDO::PARAM_INT],
                    [$trdFilter, PDO::PARAM_INT],
                    [$limit_sys_a, PDO::PARAM_INT],
                    [$limit_sys_b, PDO::PARAM_INT],
                    [$isoLanguage, PDO::PARAM_STR],
                ];

                $name = "POSTS_COLLECTIONS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_sys_a, PDO::PARAM_INT],
                [$limit_sys_b, PDO::PARAM_INT],
                [$isoLanguage, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
            ];

            $name = "POSTS_COLLECTIONS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }

            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachDossier
     *
     * Remove a Dossier from a Collection
     * @param $idDossier
     * @return mixed
     * @throws Exception
     */
    public function detachDossier($idDossier)
    {
        try {
            $name = "DOSSIERS_FROM_COLLECTIONS";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idDossier, PDO::PARAM_INT]];

            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachDossier
     *
     * Add a Dossier to a Collection
     * @param $idDossier
     * @param int $position
     * @return bool
     * @throws Exception
     */
    public function attachDossier($idDossier, $position = 0)
    {
        try {
            $name = "DOSSIERS_TO_COLLECTIONS";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idDossier, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT]];

            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * dossiers
     *
     * Returns all the Dossiers withing a Collection
     *
     * @param Query $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public static function dossiers(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";

        $idCollection = $q->idCollection;
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [[$idCollection, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "POSTS_DOSSIERS_ON_COLLECTION_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $name = "POSTS_DOSSIERS_ON_COLLECTION";
            $parameters = [[$idCollection, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($dossier) {
                return Dossier::createInstance($dossier);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all tags related to this collection
     * @return \Stage\Post\MyArrayObject|array
     * @throws Exception
     */
    public function tags()
    {
        $name = "POSTS_COLLECTION_TAGS_RELATED";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        if (isset($data[0]) && isset($data[0]['id_tag'])) {
            $result = new \Stage\Post\MyArrayObject(array_map(function ($tag) {
                return Tag::createInstance($tag);
            }, $data));
            $result->setCount(sizeof($data));
            return $result;
        } else {
            return [];
        }
    }

    /**
     * innerLanguages
     *
     * Get all languages inside a collection
     *
     * @return array
     * @throws Exception
     */
    public function innerLanguages()
    {
        try {
            $name = "COLLECTIONS_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return array_values(array_unique(array_map(function ($item) {
                return $item['iso_language'];
            }, $data)));
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param L10n $l10n
     * @throws Exception
     */
    public function attachL10n(L10n $l10n)
    {
        try {
            $l10n->save($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * l10ns
     *
     * @return array|\Stage\Post\MyArrayObject
     * @throws \Stage\Exception\StoredProcedure
     */
    public function l10ns()
    {
        try {
            return L10n::all($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachL10n
     *
     * @throws Exception
     */
    public function detachL10n()
    {
        $l10ns = $this->l10ns();
        /** @var L10n $l10n */
        foreach ($l10ns as $l10n) {
            $l10n->delete();
        }
    }

    /**
     * attachSticker
     *
     * @param $idSticker
     * @param $position
     * @param DateTime $beginDate
     * @param DateTime $endDate
     * @param string $color
     * @return bool|mixed
     * @throws Exception
     */
    public function attachSticker($idSticker, $position, DateTime $beginDate = null, DateTime $endDate = null, $color = "")
    {
        $idDossier = $this->getId();

        $beginDate = $beginDate instanceof DateTime ? $beginDate->format(DateTime::ATOM) : "";
        $endDate = $endDate instanceof DateTime ? $endDate->format(DateTime::ATOM) : "";

        // write on table dossiers_stickers_related
        $name = "POSTS_COLLECTIONS_STICKER_RELATED";
        $method = "SET";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$idDossier, PDO::PARAM_INT],
            [$position, PDO::PARAM_INT],
            [$beginDate, PDO::PARAM_STR],
            [$endDate, PDO::PARAM_STR],
            [$color, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : $data[0]['return_code'];
    }

    /**
     * detachSticker
     *
     * Detach a Stickers from a Collection
     * @param $idSticker
     * @return bool
     * @throws Exception
     */
    public function detachSticker($idSticker)
    {
        $name = "POSTS_COLLECTION_FROM_STICKERS";
        $method = "DEL";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * stickers
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function stickers()
    {
        $q = new \Stage\Post\Collection\Sticker\Query();
        $q->field = 1000;
        $q->query = "";
        $q->filter = $this->getId();
        $q->from = 0;
        $q->to = -1;

        return Sticker::collections($q);
    }

    /**
     * detachAllStickers
     *
     * Detach all Stickers from a Dossier
     * @throws Exception
     */
    public function detachAllStickers()
    {
        $status = false;
        $sticksers = $this->stickers();
        /** @var Sticker $stickser */
        foreach ($sticksers as $stickser) {
            $status = $this->detachSticker($stickser->getId());
        }
        return $status;
    }

    /**
     * metaUrlExist
     *
     * Check if the dossier slug url exit
     *
     * @param $idL10n
     * @param $metaUrl
     * @param $iso
     * @return array
     */
    public static function metaUrlExist($idL10n, $metaUrl, $iso)
    {
//        TODO
        $name = "POSTS_COLLECTIONS_URL_NUM";
        $method = "GET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$metaUrl, PDO::PARAM_STR],
            [$iso, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * domains
     *
     * Check if the dossier slug url exit
     *
     * @return array
     */
    public static function domains()
    {
        $name = "SP_GET_POSTS_COLLECTIONS_DOMAINS";
        $method = "GET";
        $parameters = [];
        return Stage::esp($parameters, $name, $method);
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();
        foreach ($attributes as $attribute) {
            if ($attribute == "defaultDossier" && $this->$attribute instanceof Dossier) {
                $arr[$attribute] = $this->$attribute->toArray();
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Collection
     */
    public static function createInstance($data)
    {
        if (isset($data[0])) {
            $data = $data[0];
        }

        $instance = new self();
        $instance->setId($data['id_collection']);
        $instance->setState($data['state']);
        if (isset($data['position'])) {
            $instance->setPosition($data['position']);
        }
        $instance->setManagementName(html_entity_decode($data['management_name'], ENT_COMPAT, 'UTF-8'));
        if (isset($data['begin_date']) && Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $instance->setBeginDate(new DateTime($data['begin_date']));
        }

        if (isset($data['publish_date']) && Helper::isValidDate($data['publish_date'], "Y-m-d H:i:s")) {
            $instance->setPublishDate(new DateTime($data['publish_date']));
        }

        if (isset($data['end_date']) && Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $instance->setEndDate(new DateTime($data['end_date']));
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }

        $instance->setDomain($data['domain']);

        $idDossier = isset($data['default_id_dossier']) && $data['default_id_dossier'] ? $data['default_id_dossier'] : 0;
        $instance->setDefaultIdDossier($idDossier);

        if (isset($data['total'])) {
            $instance->setTotalDossiers((int)$data['total']);
        }
        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
