<?php namespace Stage\Post\Dossier;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\Tag\Tag;
use Stage\Post\MyArrayObject;
use Stage\Post\L10n\L10n as PostL10n;
use Stage\Post\Collection\Collection;
use Stage\Post\Dossier\Sticker\Sticker;

class Dossier
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $managementName;

    /**
     * @var
     */
    private $publishDate;

    /**
     * @var
     */
    private $beginDate;

    /**
     * @var
     */
    private $endDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * @var int
     */
    private $totalPosts = 0;

    /**
     * @var int
     */
    private $state;

    /**
     * @var int
     */
    private $position = 0;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'managementName', 'publishDate', 'beginDate', 'endDate', 'registerDate', 'updateDate', 'totalPosts', 'state', 'position');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array('publishDate', 'beginDate', 'endDate');

    public function __construct()
    {
    }

    public function __get($key)
    {
        return isset($this->$key) ? $this->$key : null;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getPublishDate()
    {
        return $this->publishDate;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getTotalPosts()
    {
        return $this->totalPosts;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getPosition()
    {
        return $this->position;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setManagementName($name)
    {
        $this->managementName = $name;
    }

    public function setPublishDate($date)
    {

        // Define how is the date format
        if ($date == null) {
            $this->publishDate = "";
        } else {
            //$date = new DateTime($date);
            $this->publishDate = $date->format(DateTime::ATOM);
        }
        //$this->publishDate = $date->format(DateTime::ATOM);;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->beginDate = "";
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->endDate = "";
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

//    public function setRandom($random)
//    {
//        if (is_bool($random)) {
//            $this->random = (bool)$random;
//        }
//    }

    public function setTotalPosts($total)
    {
        $this->totalPosts = $total;
    }

    public function setState($state)
    {
        $this->state = $state;
    }

    public function setPosition($position)
    {
        $this->position = (int) $position;
    }

    /**
     * duplicate
     *
     * Returns a new Collection duplicated, with a new Collection id
     *
     * @return $this
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "POSTS_DOSSIER_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * sizeof
     *
     * @return int
     */
    public static function sizeof()
    {
        $name = "POSTS_DOSSIERS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['total']) ? (int)$data[0]['total'] : 0;
    }

    /**
     * find
     *
     * Find a Dossier
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "POSTS_DOSSIER";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Dossier id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Dossiers
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->field = Query::ALL;
        $q->query = "";
        $q->from = 0;
        $q->to = -1;

        return static::query($q);
    }

    /**
     * save
     *
     * Save a dossier
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getManagementName(), PDO::PARAM_STR],
            [$this->getPublishDate(), PDO::PARAM_STR],
            [$this->getBeginDate(), PDO::PARAM_STR],
            [$this->getEndDate(), PDO::PARAM_STR]];

        try {
            $name = "POSTS_DOSSIERS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_dossier'])) {
                $this->setId($data[0]['id_dossier']);
                if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
                return $this;
            } else {
                throw new Exception("Error saving Dossier", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a dossier
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "DOSSIER";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query a Dossier
     *
     * @param \Stage\Post\Dossier\Query $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->field;
        $sndField = $q->sndField;
        $trdField = $q->trdField;
        $query = $q->query;
        $filter = $q->filter;
        $sndFilter = $q->sndFilter;
        $trdFilter = $q->trdFilter;
        $limit_sys_a = $q->limitSysA;
        $limit_sys_b = $q->limitSysB;
        $order = $q->order;
        $orderBy = $q->orderBy;
        $isoLanguage = $q->isoLanguage;

        try {
            if ($count == true) {
                $parameters = [
                    [$field, PDO::PARAM_INT],
                    [$sndField, PDO::PARAM_INT],
                    [$trdField, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$sndFilter, PDO::PARAM_INT],
                    [$trdFilter, PDO::PARAM_INT],
                    [$limit_sys_a, PDO::PARAM_INT],
                    [$limit_sys_b, PDO::PARAM_INT],
                    [$isoLanguage, PDO::PARAM_STR],
                ];

                $name = "POSTS_DOSSIERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_INT],
                [$trdFilter, PDO::PARAM_INT],
                [$limit_sys_a, PDO::PARAM_INT],
                [$limit_sys_b, PDO::PARAM_INT],
                [$isoLanguage, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
            ];

            $name = "POSTS_DOSSIERS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * innerLanguages
     *
     * Get all languages inside a collection
     *
     * @return array
     * @throws Exception
     */
    public function innerLanguages()
    {
        try {
            $name = "POSTS_DOSSIERS_ISOLANGUAGES";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return array_values(array_unique(array_map(function ($item) {
                return $item['iso_language'];
            }, $data)));
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * posts
     *
     * Returns all the Posts withing a Dossier
     *
     * @param \Stage\Post\Dossier\Query|null $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public function posts(Query $q = null, $count = true, $page = 1, $perPage = 15)
    {
        try {

            $method = "GET";
            if ($count == true) {
                $name = "POSTS_ON_DOSSIERS_COUNT";
                $parameters = [[$this->getId(), PDO::PARAM_INT],
                    [$q->field, PDO::PARAM_INT],
                    [$q->sndField, PDO::PARAM_INT],
                    [$q->trdField, PDO::PARAM_INT],
                    [$q->query, PDO::PARAM_STR],
                    [$q->filter, PDO::PARAM_STR],
                    [$q->sndFilter, PDO::PARAM_STR],
                    [$q->trdFilter, PDO::PARAM_STR]];
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();

            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $name = "POSTS_ON_DOSSIERS";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$q->field, PDO::PARAM_INT],
                [$q->sndField, PDO::PARAM_INT],
                [$q->trdField, PDO::PARAM_INT],
                [$q->query, PDO::PARAM_STR],
                [$q->filter, PDO::PARAM_STR],
                [$q->sndFilter, PDO::PARAM_STR],
                [$q->trdFilter, PDO::PARAM_STR],
                [$q->order, PDO::PARAM_INT],
                [$q->orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_map(function ($post) {
                return PostL10n::createInstance($post);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * postsSizeof
     *
     * Get the total number of posts inside a dossier, if iso passed the total number is by iso
     *
     * @return mixed
     * @throws Exception
     */
    public function postsSizeof($iso = "")
    {
        $id = $this->getId();

        try {
            $parameters = [[$id, PDO::PARAM_INT],
                [$iso, PDO::PARAM_STR]];

            $name = "POSTS_ON_DOSSIERS_NUM";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * collections
     *
     * Get all the Collections that the Dossier is insert in
     * @return \Stage\Post\MyArrayObject|array
     * @throws Exception
     */
    public function collections()
    {
        $id = $this->getId();
        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "POSTS_DOSSIER_COLLECTIONS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                return new \Stage\Post\MyArrayObject();
            }

            $result = new \Stage\Post\MyArrayObject(array_map(function ($collection) {
                return Collection::createInstance($collection);
            }, $data));
            $result->setCount(sizeof($data));
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all Tag's from a Dossier
     * @return \Stage\Post\MyArrayObject|array
     * @throws Exception
     */
    public function tags()
    {
        $id = $this->getId();
        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "POSTS_DOSSIER_TAGS_RELATED";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($tag) {
                return isset($tag['id_tag']) ? Tag::createInstance($tag) : [];
            }, $data), function ($item) {
                return $item;
            }));
            $result->setCount($result->count());
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachPost
     *
     * Remove a Post from a Dossier
     *
     * @param $idPost
     * @return bool
     * @throws Exception
     */
    public function detachPost($idPost)
    {
        try {
            $name = "POSTS_FROM_DOSSIERS";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idPost, PDO::PARAM_INT]];

            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachPost
     *
     * Add a Post to Dossier
     *
     * @param $idPost
     * @return bool
     * @throws Exception
     */
    public function attachPost($idPost, $position)
    {
        try {
            $name = "POSTS_TO_DOSSIERS";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idPost, PDO::PARAM_INT],
                [$position, PDO::PARAM_INT]];

            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param L10n $l10n
     * @throws Exception
     */
    public function attachL10n(L10n $l10n)
    {
        try {
            $l10n->save($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * l10ns
     *
     * @return array
     * @throws \Stage\Exception\StoredProcedure
     */
    public function l10ns()
    {
        return L10n::all($this->getId());
    }

    /**
     * detachL10n
     *
     * Detach all Dossier L10n
     * @throws Exception
     */
    public function detachL10n()
    {
        $l10ns = $this->l10ns();
        /** @var L10n $l10n */
        foreach ($l10ns as $l10n) {
            $l10n->delete();
        }
    }

    /**
     * attachSticker
     *
     * @param $idSticker
     * @param $position
     * @param DateTime $beginDate
     * @param DateTime $endDate
     * @param string $color
     * @return bool|mixed
     * @throws Exception
     */
    public function attachSticker($idSticker, $position, DateTime $beginDate = null, DateTime $endDate = null, $color = "")
    {
        $idDossier = $this->getId();

        $beginDate = $beginDate instanceof DateTime ? $beginDate->format(DateTime::ATOM) : "";
        $endDate = $endDate instanceof DateTime ? $endDate->format(DateTime::ATOM) : "";

        // write on table dossiers_stickers_related
        $name = "POSTS_DOSSIERS_STICKER_RELATED";
        $method = "SET";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$idDossier, PDO::PARAM_INT],
            [$position, PDO::PARAM_INT],
            [$beginDate, PDO::PARAM_STR],
            [$endDate, PDO::PARAM_STR],
            [$color, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : $data[0]['return_code'];
    }

    /**
     * detachSticker
     *
     * Detach a Stickers from a Dossier
     *
     * @param $idSticker
     * @return bool
     * @throws Exception
     */
    public function detachSticker($idSticker)
    {
        $name = "POSTS_DOSSIER_FROM_STICKERS";
        $method = "DEL";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * stickers
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function stickers()
    {
        $q = new \Stage\Post\Dossier\Sticker\Query();
        $q->field = 1000;
        $q->query = "";
        $q->filter = $this->getId();
        $q->from = 0;
        $q->to = -1;

        return Sticker::dossiers($q);
    }

    /**
     * detachAllStickers
     *
     * Detach all Stickers from a Dossier
     * @throws Exception
     */
    public function detachAllStickers()
    {
        $status = false;
        $sticksers = $this->stickers();
        /** @var Sticker $stickser */
        foreach ($sticksers as $stickser) {
            $status = $this->detachSticker($stickser->getId());
        }
        return $status;
    }

    /**
     * metaUrlExist
     *
     * Check if the dossier slug url exit
     *
     * @param $idL10n
     * @param $metaUrl
     * @param $iso
     * @return array
     */
    public static function metaUrlExist($idL10n, $metaUrl, $iso)
    {
        $name = "POSTS_DOSSIERS_URL_NUM";
        $method = "GET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$metaUrl, PDO::PARAM_STR],
            [$iso, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } elseif ($attribute == "l10n") {
                foreach ($this->$attribute as $l10n) {
                    $arr[$attribute][] = $l10n->toArray();
                }
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Dossier
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id_dossier']);
        $instance->setManagementName(html_entity_decode($data['management_name'], ENT_COMPAT, 'UTF-8'));
        if (Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $instance->setBeginDate(new DateTime($data['begin_date']));
        }

        if (Helper::isValidDate($data['publish_date'], "Y-m-d H:i:s")) {
            $instance->setPublishDate(new DateTime($data['publish_date']));
        }

        if (Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $instance->setEndDate(new DateTime($data['end_date']));
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }

        if (isset($data['total'])) {
            $instance->setTotalPosts((int)$data['total']);
        }

        if (isset($data['state'])) {
            $instance->setState($data['state']);
        }

        if (isset($data['position'])) {
            $instance->setPosition($data['position']);
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
