<?php namespace Stage\Post\Feed;

use PDO;
use DateTime;
use Exception;
use Stage\Paginator;
use Stage\Post\Feed\Schema\Schema;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Helper;

class Internal
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $token;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $domain;

    /**
     * @var
     */
    private $length;

    /**
     * @var
     */
    private $type;

    /**
     * @var
     */
    private $canOverwrite;

    /**
     * @var
     */
    private $trackingParameters;

    /**
     * @var
     */
    private $imageOptimization;

    /**
     * @var
     */
    private $title;

    /**
     * @var
     */
    private $description;

    /**
     * @var
     */
    private $image;

    /**
     * @var
     */
    private $copyright;

    /**
     * @var
     */
    private $numberOfBodies;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'token', 'name', 'domain', 'length', 'type', 'canOverwrite', 'trackingParameters', 'imageOptimization', 'title', 'description', 'image', 'copyright', 'numberOfBodies');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function __get($key)
    {
        return isset($this->$key) ? $this->$key : null;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDomain()
    {
        return $this->domain;
    }

    public function getLength()
    {
        return $this->length;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getCanOverwrite()
    {
        return $this->canOverwrite;
    }

    public function getTrackingParameters()
    {
        return $this->trackingParameters;
    }

    public function getImageOptimization()
    {
        return $this->imageOptimization;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getCopyright()
    {
        return $this->copyright;
    }

    public function getNumberOfBodies()
    {
        return $this->numberOfBodies;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    private function setToken($token)
    {
        $this->token = $token;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDomain($domain)
    {
        $this->domain = $domain;
    }

    public function setLength($length)
    {
        $this->length = $length;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function setCanOverwrite($can)
    {
        $this->canOverwrite = (bool)$can;
    }

    public function setTrackingParameters($parameters)
    {
        $this->trackingParameters = $parameters;
    }

    public function setImageOptimization($opts)
    {
        $this->imageOptimization = $opts;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setDescription($desc)
    {
        $this->description = $desc;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    public function setCopyright($copy)
    {
        $this->copyright = $copy;
    }

    public function setNumberOfBodies($total)
    {
        $this->numberOfBodies = (int)$total;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * sizeof
     *
     * @return int
     */
    public static function sizeof()
    {
        $name = "POSTS_FEEDS_INTERNAL_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['total']) ? (int)$data[0]['total'] : 0;
    }

    /**
     * find
     *
     * Find a Feed
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "POSTS_FEED_INTERNAL";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Feed id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachOption
     *
     * Add a Option to Feed
     *
     * @param $idFeed
     * @param $idOption
     * @param $optionValue
     * @return bool|int
     * @throws Exception
     */
    public static function attachOption($idFeed, $idOption, $optionValue)
    {
        // return_code = -3 in case of duplicated option
        try {
            $name = "POSTS_FEEDS_OPTIONS_RELATED";
            $method = "SET";
            $parameters = [[$idFeed, PDO::PARAM_INT],
                [$idOption, PDO::PARAM_INT],
                [$optionValue, PDO::PARAM_STR]];

            $status = Stage::esp($parameters, $name, $method);
//            return isset($status[0]) && isset($status[0]['id_feed']) && $status[0]['id_feed'] != 0 ? true : $status[0]['return_code'];
            return Option::createInstance($status[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachOption
     *
     * Remove a Option from a Feed
     *
     * @param $idFeed
     * @param $idOption
     * @return bool
     * @throws Exception
     */
    public static function detachOption($idFeed, $idOption)
    {
        try {
            $name = "POSTS_FEEDS_OPTION";
            $method = "DEL";
            $parameters = [[$idFeed, PDO::PARAM_INT],
                [$idOption, PDO::PARAM_INT]];

            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == 0 ? true : $status[0]['return_code'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * options
     *
     * Get feed options
     *
     * @param $idFeed
     * @return Option[]
     * @throws Exception
     */
    public static function options($idFeed)
    {
        try {
            $name = "POSTS_FEED_OPTIONS_RELATED";
            $method = "GET";
            $parameters = [[$idFeed, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Feed id $idFeed not exists.", $data[0]['return_code']);
            } else {
                if (sizeof($data)) {
                    return new MyArrayObject(array_map(function ($option) {
                        return Option::createInstance($option);
                    }, $data));
                } else {
                    return new MyArrayObject();
                }
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * allOptions
     *
     * Get all feed options
     *
     * @return Option[]
     * @throws Exception
     */
    public static function allOptions()
    {
        try {
            // table: posts_feeds_options
            $name = "POSTS_FEED_OPTIONS";
            $method = "GET";
            $parameters = [];
            $data = Stage::esp($parameters, $name, $method);

            if (sizeof($data)) {
                return new MyArrayObject(array_map(function ($option) {
                    return Option::createInstance($option);
                }, $data));
            } else {
                return new MyArrayObject();
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Feeds
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->field = Query::ALL;
        $q->query = "";
        $q->from = 0;
        $q->to = -1;

        return static::query($q);
    }

    /**
     * save
     *
     * Save a Feed
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_STR],
            [$this->getDomain(), PDO::PARAM_STR],
            [$this->getLength(), PDO::PARAM_INT],
            [$this->getType(), PDO::PARAM_STR],
            [$this->getCanOverwrite(), PDO::PARAM_BOOL],
            [$this->getTrackingParameters(), PDO::PARAM_STR],
            [$this->getImageOptimization(), PDO::PARAM_STR],
            [$this->getTitle(), PDO::PARAM_STR],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getImage(), PDO::PARAM_STR],
            [$this->getCopyright(), PDO::PARAM_STR],
            [$this->getNumberOfBodies(), PDO::PARAM_INT]];

        try {
            $name = "POSTS_FEEDS_INTERNAL";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_feed'])) {
                $this->setId($data[0]['id_feed']);
                if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
                return $this;
            } else {
                throw new Exception("Error saving Feed", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a feed
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "POSTS_FEEDS_INTERNAL";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query a Feed
     *
     * @param Query $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "POSTS_FEEDS_INTERNAL_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $name = "POSTS_FEEDS_INTERNAL";
            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($feed) {
                return self::createInstance($feed);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachSchema
     *
     * Add a Option to Feed
     *
     * @param $idFeed
     * @param $idSchema
     * @param $isoLanguage
     * @param $macroSchema
     * @return Schema
     * @throws Exception
     */
    public static function attachSchema($idFeed, $idSchema, $isoLanguage, $macroSchema)
    {
        try {
            $feed = new Schema();
            $feed->setId($idSchema);
            $feed->setIdFeed($idFeed);
            $feed->setIsoLanguage($isoLanguage);
            $feed->setMacroSchema($macroSchema);
            return $feed->save();
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachSchema
     *
     * Remove a Option from a Feed
     *
     * @param $idFeed
     * @param $idSchema
     * @return bool
     * @throws Exception
     */
    public static function detachSchema($idFeed, $idSchema)
    {
        try {
            return Schema::delete($idFeed, $idSchema);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * schemas
     *
     *
     * @param $idFeed
     * @return MyArrayObject
     * @throws Exception
     */
    public static function schemas($idFeed)
    {
        try {
            return Schema::find($idFeed);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * duplicate
     *
     *
     * @return $this|boolean
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "POSTS_FEEDS_CLONE_INTERNAL";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (!isset($data[0]) && isset($data[0]['id_feed']))
                return false;

            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name" || $attribute == "title" || $attribute == "description" || $attribute == "copyright") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Internal
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id_feed']);
        $instance->setToken($data['token']);
        $instance->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setDomain($data['domain']);
        $instance->setLength($data['feed_length']);
        $instance->setType($data['feed_type']);
        $instance->setCanOverwrite($data['can_overwrite']);
        $instance->setTrackingParameters($data['tracking_parameters']);
        $instance->setImageOptimization($data['image_optimization']);
        $instance->setTitle(html_entity_decode($data['feed_title'], ENT_COMPAT, 'UTF-8'));
        $instance->setDescription(html_entity_decode($data['feed_description'], ENT_COMPAT, 'UTF-8'));
        $instance->setImage(html_entity_decode($data['feed_image'], ENT_COMPAT, 'UTF-8'));
        $instance->setCopyright(html_entity_decode($data['feed_copyright'], ENT_COMPAT, 'UTF-8'));
        $instance->setNumberOfBodies($data['number_of_bodies']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
