<?php namespace Stage\Post\L10n;

use PDO;
use Exception;
use Stage\Stage;
use Stage\Exception\StoredProcedure as StoredProcedureException;

class Body
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $body = "";

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $settings;

    /**
     * @var
     */
    private $restricted;

    /**
     * @var
     */
    private $draft;

    /**
     * @var
     */
    private $blockType;

    /**
     * @var
     */
    private $idGroup;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'body', 'position', 'settings', 'restricted', 'draft', 'blockType', 'idGroup');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getBody()
    {
        return $this->body;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getSettings()
    {
        return $this->settings;
    }

    public function getRestricted()
    {
        return (bool)$this->restricted;
    }

    public function getDraft()
    {
        return (bool)$this->draft;
    }

    public function getBlockType()
    {
        return $this->blockType;
    }

    public function getIdGroup()
    {
        return $this->idGroup;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setBody($body)
    {
        $this->body = $body;
    }

    public function setPosition($pos)
    {
        $this->position = (int)$pos;
    }

    public function setSettings($settings)
    {
        $this->settings = $settings;
    }

    public function setRestricted($restricted)
    {
        $this->restricted = (bool)$restricted;
    }

    public function setDraft($draft)
    {
        $this->draft = (bool)$draft;
    }

    public function setBlockType($blockType)
    {
        $this->blockType = $blockType;
    }

    public function setIdGroup($idGroup)
    {
        $this->idGroup = $idGroup;
    }

    public static function find($idL10n, $field, $query, $filter)
    {
        $temp = [];
        $parameters = [
            [$idL10n, PDO::PARAM_INT],
            [$field, PDO::PARAM_INT],
            [$query, PDO::PARAM_STR],
            [$filter, PDO::PARAM_STR],
        ];

        try {
            $name = "POSTS_L10N_BODY";
            $method = "GET";
            $bodies = Stage::esp($parameters, $name, $method);

            if (isset($bodies[0]['return_code']) && isset($bodies[0]['return_code']) == "-2") {
                return new \Stage\Post\MyArrayObject($temp);
            }

            $result = new \Stage\Post\MyArrayObject(array_map(function ($b) {
                return self::createInstance($b);
            }, $bodies));
            $result->setCount(sizeof($bodies));
            return $result;
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idL10n
     * @param $body
     * @return $this
     * @throws Exception
     */
    public static function save($idL10n, $idBody, $body, $position, $settings, $private, $draft, $blockType, $idGroup)
    {
        $parameters = [
            [$idBody, PDO::PARAM_INT],
            [$idL10n, PDO::PARAM_INT],
            [$body, PDO::PARAM_STR],
            [$position, PDO::PARAM_INT],
            [$settings, PDO::PARAM_STR],
            [$private, PDO::PARAM_BOOL],
            [$draft, PDO::PARAM_BOOL],
            [$blockType, PDO::PARAM_STR],
            [$idGroup, PDO::PARAM_INT],
        ];

        try {
            $name = "POSTS_L10N_BODY";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_body'])) {
                return self::createInstance($data[0]);
            }
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAll
     *
     * @param $field
     * @param $idItem
     * @return $this
     * @throws Exception
     */
    public static function removeAll($field, $idItem)
    {
        $parameters = [
            [$field, PDO::PARAM_INT],
            [$idItem, PDO::PARAM_INT]
        ];

        try {
            $name = "POSTS_L10N_BODY";
            $method = "DEL";
            $status = Stage::esp($parameters, $name, $method);
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Body
     */
    public static function createInstance($data)
    {
        $body = new self();
        $body->setId($data['id_body']);
        $body->setBody($data['body']);
        $body->setPosition($data['position']);
        $body->setSettings($data['settings']);
        $body->setRestricted($data['restricted']);
        $body->setDraft($data['draft']);
        $body->setBlockType($data['block_type']);
        $body->setIdGroup(isset($data['id_group']) ? $data['id_group'] : 0);
        return $body;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
