<?php namespace Stage\Post\L10n\BodyGroup;

use PDO;
use DateTime;
use Exception;
use Stage\Post\L10n\Body;
use Stage\Stage;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Exception\StoredProcedure as StoredProcedureException;

class Group
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idPost;

    /**
     * @var
     */
    private $idL10n;

    /**
     * @var
     */
    private $title;

    /**
     * @var
     */
    private $publishDate;

    /**
     * @var
     */
    private $draft;

    /**
     * @var
     */
    private $restricted;

    /**
     * @var
     */
    private $settings;

    /**
     * @var
     */
    private $token;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idL10n', 'title', 'publishDate', 'draft', 'restricted', 'settings', 'token', 'registerDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdPost()
    {
        return $this->idPost;
    }

    public function getIdL10n()
    {
        return $this->idL10n;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getPublishDate()
    {
        return $this->publishDate;
    }

    public function getRestricted()
    {
        return (bool)$this->restricted;
    }

    public function getSettings()
    {
        return $this->settings;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getDraft()
    {
        return (bool)$this->draft;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdPost($id)
    {
        $this->idPost = (int)$id;
    }

    public function setIdL10n($id)
    {
        $this->idL10n = (int)$id;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setPublishDate($date)
    {

        // Define how is the date format
        if ($date == null) {
            $this->publishDate = "";
        } else {
            //$date = new DateTime($date);
            $this->publishDate = $date->format(DateTime::ATOM);
        }
        //$this->publishDate = $date->format(DateTime::ATOM);;
    }

    public function setSettings($settings)
    {
        $this->settings = $settings;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setRestricted($restricted)
    {
        $this->restricted = (bool)$restricted;
    }

    public function setDraft($draft)
    {
        $this->draft = (bool)$draft;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * find
     *
     * Find a Body Group
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "POSTS_L10N_BODY_GROUP";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Body Group id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @param $idGroup
     * @param $idL10n
     * @param $title
     * @param $publishDate
     * @param $draft
     * @param $restricted
     * @param $settings
     * @return $this
     * @throws StoredProcedureException
     */
    public static function save($idGroup, $idL10n, $title, $publishDate, $draft, $restricted, $settings)
    {
        // table: posts_l10n_body_groups
        $pDate = "0000-00-00";
        if ($publishDate instanceof DateTime) {
            try {
                $pDate = $publishDate->format(DateTime::ATOM);
            } catch (Exception $exception) {
            }
        }

        $parameters = [
            [$idGroup, PDO::PARAM_INT],
            [$idL10n, PDO::PARAM_INT],
            [$title, PDO::PARAM_STR],
            [$pDate, PDO::PARAM_STR],
            [$draft, PDO::PARAM_BOOL],
            [$restricted, PDO::PARAM_BOOL],
            [$settings, PDO::PARAM_STR],
        ];

        try {
            $name = "POSTS_L10N_BODY_GROUP";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_group'])) {
                return self::createInstance($data[0]);
            }
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Stickers
     *
     * @param Query $q
     * @param bool $count
     * @return MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        // table: posts_l10n_body_groups
        $method = "GET";
        $field = $q->field;
        $idL10n = $q->idL10n;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {

            if ($count == true) {
                $parameters = [[$idL10n, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "POSTS_L10N_BODY_GROUPS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$idL10n, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $name = "POSTS_L10N_BODY_GROUPS";

            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($sticker) {
                return self::createInstance($sticker);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachAuthor
     *
     * @param $idGroup
     * @param $idAuthor
     * @return MyArrayObject
     * @throws Exception
     */
    public static function attachAuthor($idGroup, $idAuthor)
    {
        $parameters = [
            [$idGroup, PDO::PARAM_INT],
            [$idAuthor, PDO::PARAM_INT],
        ];

        try {
            $name = "POSTS_L10N_BODY_GROUP_AUTHORS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachAllAuthors
     *
     * @param $idGroup
     * @return void
     * @throws Exception
     */
    public static function detachAllAuthors($idGroup)
    {
        $name = "POSTS_L10N_BODY_GROUPS_AUTHORS";
        $method = "DEL";
        $parameters = [[$idGroup, PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    /**
     * authors
     *
     * @param $idGroup
     * @return MyArrayObject
     * @throws Exception
     */
    public static function authors($idGroup)
    {
        $parameters = [
            [$idGroup, PDO::PARAM_INT],
        ];

        try {
            $name = "POSTS_L10N_BODY_GROUPS_AUTHORS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($sticker) {
                return \Stage\Post\Author\L10n::createInstance($sticker);
            }, $data));

            $result->setCount(sizeof($data));
            return $result;
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table posts_stickers_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: posts_l10n_body_groups_custom_fields_keys
        $idGroup = $this->getId();
        $spname = "POSTS_L10N_BODY_GROUP_CUSTOM_FIELDS_VARIANTS";
        $method = "SET";
        $parameters = [[$idGroup, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        // table: posts_l10n_body_groups_custom_fields_related
        $idGroup = $this->getId();
        $name = "POSTS_L10N_BODY_GROUPS_CUSTOM_FIELD_VARIANT";
        $method = "DEL";
        $parameters = [[$idGroup, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * customFieldsVariants
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        $idGroup = $this->getId();
        $name = "POSTS_L10N_BODY_GROUP_CUSTOM_FIELDS_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idGroup, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * attachCustomFieldVariantMatrix
     *
     * @param $idVariant
     * @param $nameRow
     * @param $nameColumn
     * @param $variantValue
     * @param $positionRow
     * @param $positionValue
     * @return CustomFieldVariantMatrix
     * @throws Exception
     */
    public function attachCustomFieldVariantMatrix($idVariant, $nameRow, $nameColumn, $variantValue, $positionRow, $positionColumn)
    {
        // table: posts_l10n_body_groups_custom_fields_variant_matrix
        $spname = "POSTS_L10N_BODY_GROUP_CUSTOM_FIELDS_VARIANT_MATRIX";
        $method = "SET";
        $parameters = [[$idVariant, PDO::PARAM_INT],
            [$nameRow, PDO::PARAM_STR],
            [$nameColumn, PDO::PARAM_STR],
            [$variantValue, PDO::PARAM_STR],
            [$positionRow, PDO::PARAM_INT],
            [$positionColumn, PDO::PARAM_INT]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariantMatrix::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachCustomFieldsVariantMatrix
     *
     * @param $idVariant
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariantMatrix($idVariant)
    {
        // table: posts_l10n_body_groups_custom_fields_variant_matrix
        $name = "POSTS_L10N_BODY_GROUPS_CUSTOM_FIELD_VARIANT_MATRIX";
        $method = "DEL";
        $parameters = [[$idVariant, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * customFieldsVariantsMatrix
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariantsMatrix($idVariant)
    {
        $name = "POSTS_L10N_BODY_GROUP_CUSTOM_FIELDS_VARIANTS_MATRIX";
        $method = "GET";
        $parameters = [[$idVariant, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariantMatrix::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * Delete a Body Group
     *
     */
    public static function delete($idBodyGroup)
    {
        $name = "POSTS_L10N_BODY_GROUP";
        $method = "DEL";
        $parameters = [[$idBodyGroup, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * bodies
     *
     *  Get all bodies of a body group
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function bodies($idBodyGroup)
    {
        $spname = "PUBLISH_POSTS_BODY_GROUPS_BODIES";
        $method = "GET";
        $parameters = [[$idBodyGroup, PDO::PARAM_INT]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                return new MyArrayObject([]);
            } else {
                $result = new MyArrayObject(array_map(function ($body) {
                    return Body::createInstance($body);
                }, $data));

                $total = sizeof($data);
                $result->setCount($total);

                return $result;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "title" || $attribute == "description" || $attribute == "imageCaption") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "publishDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return self
     */
    public static function createInstance($data)
    {
        $body = new self();
        $body->setId($data['id_group']);
        $body->setIdPost($data['id_post']);
        $body->setIdL10n($data['id_l10n']);
        $body->setTitle(html_entity_decode($data['title'], ENT_COMPAT, 'UTF-8'));

        if (Helper::isValidDate($data['publish_date'], "Y-m-d H:i:s")) {
            $body->setPublishDate(new DateTime($data['publish_date']));
        }

        $body->setDraft((bool)$data['draft']);
        $body->setRestricted($data['restricted']);
        $body->setSettings($data['settings']);
        $body->setToken($data['token']);


        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $body->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $body->setUpdateDate(new DateTime($data['update_date']));
        }

        return $body;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
