<?php namespace Stage\Post\L10n;

use PDO;
use DateTime;
use Exception;
use Stage\Post\Author\Author;
use Stage\Post\L10n\Key\Key;
use Stage\Post\L10n\Key\Variant;
use Stage\Post\Label\Label;
use Stage\Post\MyArrayObject;
use Stage\Post\Post;
use Stage\Stage;
use Stage\Post\Tag\Tag;
use Stage\Helper;
use Stage\Asset\Asset;
use Stage\Asset\Album\Album;
use Stage\Exception\StoredProcedure as StoredProcedureException;
use Stage\Video\Compose\Compose;

class L10n
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idPost;

    /**
     * @var
     */
    private $idType;

    /**
     * @var
     */
    private $isoLanguage;

    /**
     * @var
     */
    private $draft = false;

    /**
     * @var
     */
    private $title = "";

    /**
     * @var
     */
    private $description = "";

    /**
     * @var
     */
    private $state; // 5 - Online, 4 - Scheduled, 3 - Created

    /**
     * @var
     */
    private $image = "";

    /**
     * @var
     */
    private $imageCaption = "";

    /**
     * @var
     */
    private $imageAuthor = "";

    /**
     * @var
     */
    private $imageAlign = "";

    /**
     * @var
     */
    private $imageWidth = 0;

    /**
     * @var
     */
    private $imageHeight = 0;

    /**
     * @var
     */
    private $imageTitle = "";

    /**
     * @var
     */
    private $imageAlterText = "";

    /**
     * @var
     */
    private $metaKeywords = "";

    /**
     * @var
     */
    private $metaPlayMark = "";

    /**
     * @var
     */
    private $metaSync = "";

    /**
     * @var
     */
    private $metaTitle = "";

    /**
     * @var
     */
    private $metaDescription = "";

    /**
     * @var
     */
    private $metaImage = "";

    /**
     * @var
     */
    private $metaUrl = "";

    /**
     * @var
     */
    private $metaIdAsset = 0;

    /**
     * @var
     */
    private $idAsset = 0;

    /**
     * @var
     */
    private $position = 0;

    /**
     * @var
     */
    private $tagsEnumeration = "";

    /**
     * @var
     */
    private $isosEnumeration = "";

    /**
     * @var
     */
    private $headline = "";

    /**
     * @var
     */
    private $color = "";

    /**
     * @var
     */
    private $lastMessageAt;

    /**
     * @var
     */
    private $publishDate = "";

    /**
     * @var
     */
    private $beginDate = "";

    /**
     * @var
     */
    private $endDate = "";

    /**
     * @var
     */
    private $registerDate = "";

    /**
     * @var
     */
    private $updateDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idPost', 'idType', 'isoLanguage', 'title', 'description', 'image', 'imageCaption', 'imageAuthor', 'imageAlign', 'imageWidth', 'imageHeight', 'imageTitle', 'imageAlterText', 'metaKeywords', 'metaPlayMark', 'metaSync', 'metaTitle', 'metaDescription', 'metaImage', 'metaUrl', 'metaIdAsset', 'idAsset', 'draft', 'position', 'tagsEnumeration', 'isosEnumeration', 'headline', 'color', 'lastMessageAt', 'state', 'publishDate', 'beginDate', 'endDate', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array('publishDate', 'beginDate', 'endDate');

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdPost()
    {
        return $this->idPost;
    }

    public function getIdType()
    {
        return $this->idType;
    }

    public function getIsoLanguage()
    {
        return $this->isoLanguage;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getImageCaption()
    {
        return $this->imageCaption;
    }

    public function getImageAuthor()
    {
        return $this->imageAuthor;
    }

    public function getImageAlign()
    {
        return $this->imageAlign;
    }

    public function getImageWidth()
    {
        return $this->imageWidth;
    }

    public function getImageHeight()
    {
        return $this->imageHeight;
    }

    public function getMetaTitle()
    {
        return $this->metaTitle;
    }

    public function getMetaDescription()
    {
        return $this->metaDescription;
    }

    public function getMetaImage()
    {
        return $this->metaImage;
    }

    public function getMetaUrl()
    {
        return $this->metaUrl;
    }

    public function getMetaIdAsset()
    {
        return $this->metaIdAsset;
    }

    public function getIdAsset()
    {
        return $this->idAsset;
    }

    public function getDraft()
    {
        return (bool)$this->draft;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getTagsEnumeration()
    {
        return $this->tagsEnumeration;
    }

    public function getIsosEnumeration()
    {
        return $this->isosEnumeration;
    }

    public function getHeadline()
    {
        return $this->headline;
    }

    public function getColor()
    {
        return $this->color;
    }

    public function getLastMessageAt()
    {
        return $this->lastMessageAt;
    }

    public function getImageTitle()
    {
        return $this->imageTitle;
    }

    public function getImageAlterText()
    {
        return $this->imageAlterText;
    }

    public function getMetaKeywords()
    {
        return $this->metaKeywords;
    }

    public function getMetaPlayMark()
    {
        return $this->metaPlayMark;
    }

    public function getMetaSync()
    {
        return $this->metaSync;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getPublishDate()
    {
        return $this->publishDate;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdPost($idPost)
    {
        $this->idPost = (int)$idPost;
    }

    public function setIdType($idType)
    {
        $this->idType = (int)$idType;
    }

    public function setIsoLanguage($isoLanguage)
    {
        $this->isoLanguage = $isoLanguage;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    public function setImageCaption($image)
    {
        $this->imageCaption = $image;
    }

    public function setImageAuthor($image)
    {
        $this->imageAuthor = $image;
    }

    public function setImageAlign($imageAlign)
    {
        $this->imageAlign = $imageAlign;
    }

    public function setImageWidth($width)
    {
        $this->imageWidth = $width;
    }

    public function setImageHeight($height)
    {
        $this->imageHeight = $height;
    }

    public function setImageTitle($title)
    {
        $this->imageTitle = $title;
    }

    public function setImageAlterText($text)
    {
        $this->imageAlterText = $text;
    }

    public function setMetaKeywords($metaKeywords)
    {
        $this->metaKeywords = $metaKeywords;
    }

    public function setMetaPlayMark($playMark)
    {
        $this->metaPlayMark = (bool)$playMark;
    }

    public function setMetaSync($sync)
    {
        $this->metaSync = (bool)$sync;
    }

    public function setMetaTitle($title)
    {
        $this->metaTitle = $title;
    }

    public function setMetaDescription($desc)
    {
        $this->metaDescription = $desc;
    }

    public function setMetaImage($image)
    {
        $this->metaImage = $image;
    }

    public function setMetaUrl($metaUrl)
    {
        $this->metaUrl = $metaUrl;
    }

    public function setMetaIdAsset($metaIdAsset)
    {
        $this->metaIdAsset = $metaIdAsset;
    }

    public function setIdAsset($idAsset)
    {
        $this->idAsset = $idAsset;
    }

    public function setDraft($isDraft)
    {
        $this->draft = (bool)$isDraft;
    }

    public function setPosition($position)
    {
        $this->position = (int)$position;
    }

    public function setTagsEnumeration($tags)
    {
        $this->tagsEnumeration = $tags;
    }

    public function setIsosEnumeration($isos)
    {
        $this->isosEnumeration = $isos;
    }

    public function setHeadline($headline)
    {
        $this->headline = $headline;
    }

    public function setColor($color)
    {
        $this->color = $color;
    }

    private function setState($state)
    {
        $this->state = $state;
    }

    public function setPublishDate($date)
    {

        // Define how is the date format
        if ($date == null) {
            $this->publishDate = "";
        } else {
            //$date = new DateTime($date);
            $this->publishDate = $date->format(DateTime::ATOM);
        }
        //$this->publishDate = $date->format(DateTime::ATOM);;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->beginDate = "";
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->endDate = "";
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setLastMessageAt($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->lastMessageAt = "";
        } else {
            //$date = new DateTime($date);
            $this->lastMessageAt = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * @param $idPost
     * @param $isoLanguage
     * @return L10n
     */
    public static function find($idPost, $isoLanguage)
    {
        try {
            $name = "POSTS_L10N";
            $method = "GET";
            $parameters = [[$idPost, PDO::PARAM_INT],
                [$isoLanguage, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The L10n with iso language $isoLanguage not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    public static function all()
    {
        // TODO Return all l10n's from a post
    }

    /**
     * Delete a L10n
     *
     */
    public function delete()
    {
        $name = "POSTS_L10N";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $status = Stage::esp($parameters, $name, $method);
        return $this;
    }

    /**
     * save
     *
     * Save a L10n
     *
     * @return $this
     * @throws Exception
     */
    public function save($idPost, $idL10n = null)
    {
        $parameters = [[!$idL10n ? $this->getId() : $idL10n, PDO::PARAM_INT],
            [$idPost, PDO::PARAM_INT],
            [$this->getIsoLanguage(), PDO::PARAM_STR],
            [$this->getDraft(), PDO::PARAM_BOOL],
            [$this->getHeadline(), PDO::PARAM_STR],
            [$this->getTitle(), PDO::PARAM_STR],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getImage(), PDO::PARAM_STR],
            [$this->getImageCaption(), PDO::PARAM_STR],
            [$this->getImageAuthor(), PDO::PARAM_STR],
            [$this->getImageAlign(), PDO::PARAM_STR],
            [$this->getImageTitle(), PDO::PARAM_STR],
            [$this->getImageAlterText(), PDO::PARAM_STR],
            [$this->getMetaTitle(), PDO::PARAM_STR],
            [$this->getMetaDescription(), PDO::PARAM_STR],
            [$this->getMetaImage(), PDO::PARAM_STR],
            [$this->getMetaKeywords(), PDO::PARAM_STR],
            [$this->getMetaUrl(), PDO::PARAM_STR],
            [$this->getMetaPlayMark(), PDO::PARAM_BOOL],
            [$this->getMetaSync(), PDO::PARAM_BOOL],
            [$this->getColor(), PDO::PARAM_STR],
            [$this->getPublishDate() == "" ? "0000-00-00" : $this->getPublishDate(), PDO::PARAM_STR],
            [$this->getBeginDate() == "" ? "0000-00-00" : $this->getBeginDate(), PDO::PARAM_STR],
            [$this->getEndDate() == "" ? "0000-00-00" : $this->getEndDate(), PDO::PARAM_STR]];

        try {
            $name = "POSTS_L10N";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_l10n'])) {
                $this->setId($data[0]['id_l10n']);
            } else {
                throw new Exception("Error creating Post");
            }

        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    public function detachAllBodies()
    {
        $field = 1;
        $idItem = $this->getId();
        return \Stage\Post\L10n\Body::removeAll($field, $idItem);
    }

    public function detachBody($field, $idItem)
    {
        return \Stage\Post\L10n\Body::removeAll($field, $idItem);
    }

    public function detachAllTags()
    {
        $name = "POSTS_FROM_TAGS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    public function detachAllAssets()
    {
        $name = "POSTS_L10N_PHOTOS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    public function detachAllAlbums()
    {
        $name = "POSTS_L10N_ALBUMS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    public function detachAllAuthors()
    {
        $name = "POSTS_L10N_AUTHORS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    public function detachAllVideoComposes()
    {
        $name = "POSTS_L10N_VIDEOS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    public function detachAllLabels()
    {
        $name = "POSTS_L10N_FROM_LABELS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    /**
     * tags
     *
     * Get all Tag's
     * @return \Stage\Post\MyArrayObject
     */
    public function tags()
    {
        $name = "POSTS_TAGS_RELATED";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($tag) {
            return isset($tag['id_tag']) ? Tag::createInstance($tag) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * assets
     *
     * Get L10n assets
     * @return \Stage\Post\MyArrayObject
     */
    public function assets()
    {
        $name = "POSTS_L10N_PHOTOS";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $assets = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($asset) {
            return isset($asset['id_asset']) ? Asset::createInstance($asset) : [];
        }, $assets), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * albums
     *
     * Get L10n Albums
     * @return \Stage\Post\MyArrayObject
     */
    public function albums()
    {
        $name = "POSTS_L10N_ALBUMS";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $albums = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($album) {
            return isset($album['id_album']) ? Album::createInstance($album) : [];
        }, $albums), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * authors
     *
     * Get L10n Authors
     * @return \Stage\Post\MyArrayObject
     */
    public function authors()
    {
        $name = "POSTS_L10N_AUTHORS";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $authors = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($author) {
            return isset($author['id_author']) ? \Stage\Post\Author\L10n::createInstance($author) : [];
        }, $authors), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * videos
     *
     * Get L10n Videos
     * @return \Stage\Post\MyArrayObject
     */
    public function videos()
    {
        $name = "POSTS_L10N_VIDEOS";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $videos = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($video) {
            return isset($video['id_composed']) ? Compose::createInstance($video) : [];
        }, $videos), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * labels
     *
     * Get Labels
     * @return \Stage\Post\MyArrayObject
     */
    public function labels()
    {
        $name = "POSTS_L10N_LABELS_RELATED";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $labels = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($label) {
            return isset($label['id_label']) ? Label::createInstance($label) : [];
        }, $labels), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * bodies
     *
     * Return all L10n bodies
     * @param $field
     * @param $query
     * @param $filter
     * @return array|mixed|\Stage\Post\MyArrayObject
     * @throws StoredProcedureException
     */
    public function bodies($field = 0, $query = "", $filter = "")
    {
        return Body::find($this->getId(), $field, $query, $filter);
    }

    /**
     * attachTag
     *
     * Associate a Tag to a L10n
     * @param $idTag
     * @throws Exception
     */
    public function attachTag($idTag)
    {
        $name = "POSTS_TO_TAGS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idTag, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_tag']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachAsset
     *
     * @param $idAsset
     * @throws Exception
     */
    public function attachAsset($idAsset)
    {
        $name = "POSTS_L10N_PHOTOS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idAsset, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_asset']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachAlbum
     *
     * @param $idAlbum
     * @throws Exception
     */
    public function attachAlbum($idAlbum)
    {
        $name = "POSTS_L10N_ALBUMS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idAlbum, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_album']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachVideoCompose
     *
     * @param $idVideoCompose
     * @throws Exception
     */
    public function attachVideoCompose($idVideoCompose)
    {
        $name = "POSTS_L10N_VIDEOS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idVideoCompose, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_composed']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachAuthor
     *
     * @param $idAuthor
     * @throws Exception
     */
    public function attachAuthor($idAuthor)
    {
        $name = "POSTS_L10N_AUTHORS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idAuthor, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_author']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $body
     * @return bool
     * @throws Exception
     */
    public function attachBody($idBody, $body, $position, $settings, $private, $draft, $blockType, $idGroup)
    {
        try {
            return \Stage\Post\L10n\Body::save($this->getId(), $idBody, $body, $position, $settings, $private, $draft, $blockType, $idGroup) instanceof \Stage\Post\L10n\Body;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachLabel
     *
     * @param $idLabel
     * @return bool
     * @throws Exception
     */
    public function attachLabel($idLabel)
    {
        $name = "POSTS_L10N_TO_LABELS";
        $method = "SET";
        $parameters = [[$idLabel, PDO::PARAM_INT],
            [$this->getId(), PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_label']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachPost
     *
     * @param $idPost
     * @return bool
     * @throws Exception
     */
    public function attachPost($idPost)
    {
        $name = "POSTS_L10N_TO_POSTS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT], [$idPost, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['return_code']) && $status[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * audios
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function audios()
    {
        $name = "POSTS_L10N_AUDIOS";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];

        try {
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_asset'])) {
                $result = new \Stage\Post\MyArrayObject(array_map(function ($asset) {
                    return Asset::createInstance($asset);
                }, $data));
                $result->setCount($result->count());
                return $result;
            } else {
                return new \Stage\Post\MyArrayObject([]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachAllAudios
     *
     * Detach all Tag's attached to this L10n
     * @throws Exception
     */
    public function detachAllAudios()
    {
        $name = "POSTS_L10N_AUDIOS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $result = Stage::esp($parameters, $name, $method);
        return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
    }

    /**
     * attachAudio
     *
     * @param $idAsset
     * @return bool
     * @throws Exception
     */
    public function attachAudio($idAsset)
    {
        $name = "POSTS_L10N_AUDIOS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idAsset, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_asset']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * documents
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function documents()
    {
        $name = "POSTS_L10N_DOCUMENTS";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];

        try {
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_asset'])) {
                $result = new \Stage\Post\MyArrayObject(array_map(function ($asset) {
                    return Asset::createInstance($asset);
                }, $data));
                $result->setCount($result->count());
                return $result;
            } else {
                return new \Stage\Post\MyArrayObject([]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachAllAudios
     *
     * Detach all Tag's attached to this L10n
     * @throws Exception
     */
    public function detachAllDocuments()
    {
        $name = "POSTS_L10N_DOCUMENTS";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $result = Stage::esp($parameters, $name, $method);
        return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
    }

    /**
     * attachDocument
     *
     * @param $idAsset
     * @return bool
     * @throws Exception
     */
    public function attachDocument($idAsset)
    {
        $name = "POSTS_L10N_DOCUMENTS";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$idAsset, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id_asset']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * variantKeys
     *
     * Get all Variant Keys
     * @return \Stage\Post\MyArrayObject
     */
    public function variantKeys()
    {
        $idL10n = $this->getId();
        $name = "POSTS_L10N_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idL10n, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($variant) {
            return isset($variant['id_variant']) ? Variant::createInstance($variant) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * attachVariantKey
     *
     * Associate a Variant Key to a L10n
     * @param $idKey
     * @param $variantValue
     * @param $position
     * @return bool
     * @throws Exception
     */
    public function attachVariantKey($idKey, $headline, $title, $description, $image, $imageAlign, $imageTitle, $imageAlterText, $metaTitle, $metaDescription, $metaImage, $metaKeywords, $metaPlayMark, $idPosition)
    {
        // table: posts_l10n_variants_related
        $idL10n = $this->getId();
        $name = "POSTS_L10N_VARIANTS";
        $method = "SET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$headline, PDO::PARAM_STR],
            [$title, PDO::PARAM_STR],
            [$description, PDO::PARAM_STR],
            [$image, PDO::PARAM_STR],
            [$imageAlign, PDO::PARAM_STR],
            [$imageTitle, PDO::PARAM_STR],
            [$imageAlterText, PDO::PARAM_STR],
            [$metaTitle, PDO::PARAM_STR],
            [$metaDescription, PDO::PARAM_STR],
            [$metaImage, PDO::PARAM_STR],
            [$metaKeywords, PDO::PARAM_STR],
            [$metaPlayMark, PDO::PARAM_BOOL],
            [$idPosition, PDO::PARAM_INT]];

        try {
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['id_variant']) && $data[0]['id_variant'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachVariantKey
     *
     * @param $idAuthorL10n
     * @param $idKey
     * @throws Exception
     */
    public function detachVariantKey($idKey)
    {
        $idL10n = $this->getId();
        $name = "POSTS_L10N_VARIANT";
        $method = "DEL";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * detachAllVariantKeys
     *
     * @return bool
     * @throws Exception
     */
    public function detachAllVariantKeys()
    {
        $status = true;
        /** @var Variant[] $variants */
        $variants = $this->variantKeys();
        foreach ($variants as $variant) {
            $status = $this->detachVariantKey($variant->getIdKey());
        }
        return $status;
    }

    /**
     * detachAllPosts
     *
     * @return bool
     * @throws Exception
     */
    public function detachAllPosts()
    {
        $idL10n = $this->getId();
        $name = "POSTS_L10N_FROM_POSTS";
        $method = "DEL";
        $parameters = [[$idL10n, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * posts
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function posts()
    {
        $name = "POSTS_L10N_POSTS_RELATED";
        $method = "GET";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($post) {
            if (isset($post['id_post'])) {
                $p = Post::createInstance([$post]);
                $p->title = $post['title'];
                $p->position = $post['position'];
                return $p;
            } else {
                return [];
            }
        }, $data), function ($item) {
            return $item;
        }));

        $count = isset($data[0]) && isset($data[0]['id_post']) ? sizeof($data) : 0;
        $result->setCount($count);
        return $result;
    }

    /**
     * attachKey
     *
     * Associate a Variant Key to a L10n
     * @param $idKey
     * @param $name
     * @param $nameStripped
     * @param $public
     * @param $ifexists
     * @return Key
     * @throws Exception
     */
    public function attachKey($idKey, $name, $nameStripped, $public, $ifexists)
    {
        // table: posts_l10n_variants_keys
        $idL10n = $this->getId();
        $spname = "POSTS_L10N_KEYS";
        $method = "SET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$name, PDO::PARAM_STR],
            [$nameStripped, PDO::PARAM_STR],
            [$public, PDO::PARAM_BOOL],
            [$ifexists, PDO::PARAM_BOOL]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return Key::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachKey
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachKey($idKey)
    {
        $name = "POSTS_L10N_KEY";
        $method = "DEL";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * key
     *
     * @param $idKey
     * @return Key
     * @throws Exception
     */
    public function key($idKey)
    {
        $name = "POSTS_L10N_KEY";
        $method = "GET";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
            throw new Exception("The Key not exists", $data[0]['return_code']);
        } else {
            return Key::createInstance($data[0]);
        }
    }

    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table posts_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: posts_l10n_custom_fields_related
        $idL10n = $this->getId();
        $spname = "POSTS_L10N_CUSTOM_FIELDS_VARIANTS";
        $method = "SET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * customFieldsVariants
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        $idL10n = $this->getId();
        $name = "POSTS_L10N_CUSTOM_FIELDS_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idL10n, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new \Stage\Post\MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        $idL10n = $this->getId();
        $name = "POSTS_L10N_CUSTOM_FIELD_VARIANT";
        $method = "DEL";
        $parameters = [[$idL10n, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * attachImageOptions
     *
     * @param $ratio
     * @param $imageAlign
     * @param $imageOptimization
     * @param $settings
     * @return bool
     * @throws Exception
     */
    public function attachImageOptions($ratio, $imageAlign, $imageOptimization, $settings)
    {
        // table: posts_l10n_image_options
        $idL10n = $this->getId();
        $spname = "POSTS_L10N_IMAGE_OPTIONS";
        $method = "SET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$ratio, PDO::PARAM_STR],
            [$imageAlign, PDO::PARAM_STR],
            [$imageOptimization, PDO::PARAM_STR],
            [$settings, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            return isset($data[0]) && isset($data[0]['id_l10n']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * imageOptions
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function imageOptions()
    {
        $idL10n = $this->getId();
        $name = "POSTS_L10N_IMAGE_OPTIONS";
        $method = "GET";
        $parameters = [[$idL10n, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        $result = new \Stage\Post\MyArrayObject(array_map(function ($asset) {
            return ImageOption::createInstance($asset);
        }, $data));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * attachCustomFieldVariantMatrix
     *
     * @param $idVariant
     * @param $nameRow
     * @param $nameColumn
     * @param $variantValue
     * @param $positionRow
     * @param $positionValue
     * @return CustomFieldVariantMatrix
     * @throws Exception
     */
    public function attachCustomFieldVariantMatrix($idVariant, $nameRow, $nameColumn, $variantValue, $positionRow, $positionColumn)
    {
        // table: posts_l10n_custom_fields_variant_matrix
        $spname = "POSTS_L10N_CUSTOM_FIELDS_VARIANT_MATRIX";
        $method = "SET";
        $parameters = [[$idVariant, PDO::PARAM_INT],
            [$nameRow, PDO::PARAM_STR],
            [$nameColumn, PDO::PARAM_STR],
            [$variantValue, PDO::PARAM_STR],
            [$positionRow, PDO::PARAM_INT],
            [$positionColumn, PDO::PARAM_INT]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariantMatrix::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachCustomFieldsVariantMatrix
     *
     * @param $idVariant
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariantMatrix($idVariant)
    {
        // table: posts_l10n_custom_fields_variant_matrix
        $name = "POSTS_L10N_CUSTOM_FIELD_VARIANT_MATRIX";
        $method = "DEL";
        $parameters = [[$idVariant, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * customFieldsVariantsMatrix
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariantsMatrix($idVariant)
    {
        $name = "POSTS_L10N_CUSTOM_FIELDS_VARIANTS_MATRIX";
        $method = "GET";
        $parameters = [[$idVariant, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariantMatrix::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public
    function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public
    function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "title" || $attribute == "description" || $attribute == "imageCaption" || $attribute == "imageAuthor" ) {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "lastMessageAt" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return L10n
     */
    public
    static function createInstance($data)
    {
        $l10n = new self();
        $l10n->setId($data['id_l10n']);
        $l10n->setIdPost($data['id_post']);
        $l10n->setIdType($data['id_type']);
        $l10n->setTitle(html_entity_decode($data['title'], ENT_COMPAT, 'UTF-8'));
        $l10n->setDescription(html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8'));
        $l10n->setIsoLanguage($data['iso_language']);
        $l10n->setState($data['state']);

        $l10n->setMetaTitle(html_entity_decode($data['meta_title'], ENT_COMPAT, 'UTF-8'));
        $l10n->setMetaDescription(html_entity_decode($data['meta_description'], ENT_COMPAT, 'UTF-8'));
        $l10n->setMetaKeywords(html_entity_decode($data['meta_keywords'], ENT_COMPAT, 'UTF-8'));
        $l10n->setMetaPlayMark($data['meta_play_mark']);
        $l10n->setMetaSync($data['meta_sync']);
        $l10n->setMetaImage($data['meta_image']);
        $l10n->setImage($data['image']);
        if (isset($data['image_caption'])) {
            $l10n->setImageCaption(html_entity_decode($data['image_caption'], ENT_COMPAT, 'UTF-8'));
        }
        if (isset($data['image_author'])) {
            $l10n->setImageAuthor(html_entity_decode($data['image_author'], ENT_COMPAT, 'UTF-8'));
        }
        if (isset($data['image_align'])) {
            $l10n->setImageAlign($data['image_align']);
        }
        if (isset($data['image_width'])) {
            $l10n->setImageWidth($data['image_width']);
        }
        if (isset($data['image_height'])) {
            $l10n->setImageHeight($data['image_height']);
        }

        $l10n->setImageTitle(html_entity_decode($data['image_title'], ENT_COMPAT, 'UTF-8'));
        $l10n->setImageAlterText(html_entity_decode($data['image_alter_text'], ENT_COMPAT, 'UTF-8'));
        $l10n->setMetaUrl($data['meta_url']);
        $l10n->setMetaIdAsset($data['meta_id_asset']);
        $l10n->setIdAsset($data['id_asset']);
        $l10n->setDraft($data['draft']);

        if (isset($data['position'])) {
            $l10n->setPosition($data['position']);
        }

        // its the tags of the L10n separated by comma, like: tag1,tag2,tag3...
        if (isset($data['tags_grouped'])) {
            $l10n->setTagsEnumeration($data['tags_grouped']);
        }

        if (isset($data['iso_grouped'])) {
            $l10n->setIsosEnumeration($data['iso_grouped']);
        }

        if (isset($data['headline'])) {
            $l10n->setHeadline(html_entity_decode($data['headline'], ENT_COMPAT, 'UTF-8'));
        }

        if (isset($data['color'])) {
            $l10n->setColor($data['color']);
        }

        if (isset($data['last_message_at']) && Helper::isValidDate($data['last_message_at'], "Y-m-d H:i:s")) {
            $l10n->setLastMessageAt(new DateTime($data['last_message_at']));
        }

        if (Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $l10n->setBeginDate(new DateTime($data['begin_date']));
        }

        if (Helper::isValidDate($data['publish_date'], "Y-m-d H:i:s")) {
            $l10n->setPublishDate(new DateTime($data['publish_date']));
        }

        if (Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $l10n->setEndDate(new DateTime($data['end_date']));
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $l10n->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $l10n->setUpdateDate(new DateTime($data['update_date']));
        }

        return $l10n;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
