<?php namespace Stage\Post\Message\Group;

use PDO;
use DateTime;
use Exception;
use Stage\Post\Message\CustomFieldKey;
use Stage\Stage;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\MyArrayObject;

class Group
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idType;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $domain = "";

    /**
     * @var
     */
    private $trackingParameters = "";

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $beginDate = "";

    /**
     * @var
     */
    private $registerDate = "";

    /**
     * @var
     */
    private $updateDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idType', 'name', 'nameStripped', 'domain', 'trackingParameters', 'total', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array('publishDate', 'beginDate', 'endDate');

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdType()
    {
        return $this->idType;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getDomain()
    {
        return $this->domain;
    }

    public function getTrackingParameters()
    {
        return $this->trackingParameters;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdType($idType)
    {
        $this->idType = (int)$idType;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setDomain($domain)
    {
        $this->domain = $domain;
    }

    public function setTrackingParameters($trackingParameters)
    {
        $this->trackingParameters = $trackingParameters;
    }

    public function setTotal($total)
    {
        $this->total = (int)$total;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->beginDate = "";
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * find
     *
     * @param $idGroup
     * @return self
     * @throws Exception
     */
    public static function find($idGroup)
    {
        try {
            $name = "POSTS_MESSAGES_GROUP";
            $method = "GET";
            $parameters = [[$idGroup, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Message Group not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Message Group
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getIdType(), PDO::PARAM_INT],
            [$this->getName(), PDO::PARAM_STR],
            [$this->getNameStripped(), PDO::PARAM_STR],
            [$this->getDomain(), PDO::PARAM_STR],
            [$this->getTrackingParameters(), PDO::PARAM_STR]
        ];

        try {
            $name = "POSTS_MESSAGES_GROUPS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_group'])) {
                $this->setId($data[0]['id_group']);
            } else {
                throw new Exception("Error creating Message Group");
            }

        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * query
     *
     * Search Messages
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15, $all = false)
    {
        $method = "GET";
        $field = $q->field;
        $idType = $q->idType;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $name = 'POSTS_MESSAGES_GROUPS_COUNT';
                $parameters = [[$idType, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = !$all ? (isset($p) ? $p->to() : $q->to) : -1;

            $parameters = [[$idType, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $name = "POSTS_MESSAGES_GROUPS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = static::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            $result = new MyArrayObject($temp);
            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * variantKeys
     *
     * Search Messages
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function variantKeys(Query $q, $count = false, $page = 1, $perPage = 15, $all = false)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $name = 'POSTS_MESSAGES_GROUPS_ATTRIBUTES_KEYS_COUNT';
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = !$all ? (isset($p) ? $p->to() : $q->to) : -1;

            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "POSTS_MESSAGES_GROUPS_ATTRIBUTES_KEYS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = CustomFieldKey::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            $result = new MyArrayObject($temp);
            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachVariantKey
     *
     * @param $idKey
     * @param $name
     * @param $nameStripped
     * @param $public
     * @param $ifExists
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists)
    {
        // table: posts_messages_groups_attributes_keys
        $spName = "POSTS_MESSAGES_GROUPS_ATTRIBUTES_KEY";
        $method = "SET";
        $parameters = [[$idKey, PDO::PARAM_INT],
            [$name, PDO::PARAM_STR],
            [$nameStripped, PDO::PARAM_STR],
            [$public, PDO::PARAM_BOOL],
            [$ifExists, PDO::PARAM_BOOL]];

        try {
            $data = Stage::esp($parameters, $spName, $method);
            if (isset($data[0]) && isset($data[0]['id_key'])) {
                return CustomFieldKey::createInstance($data[0]);
            }

            throw new Exception("Error creating custom field key", $data[0]['return_code']);

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachVariantKey
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public static function detachVariantKey($idKey)
    {
        $name = "POSTS_MESSAGES_GROUPS_ATTRIBUTE_KEY";
        $method = "DEL";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }


    /**
     * variantKey
     *
     * @param $idKey
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function variantKey($idKey)
    {
        $name = "POSTS_MESSAGES_GROUPS_ATTRIBUTE_KEY";
        $method = "GET";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
            throw new Exception("The Key not exists", $data[0]['return_code']);
        } else {
            return CustomFieldKey::createInstance($data[0]);
        }
    }

    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table posts_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: posts_messages_groups_attributes_related
        $idMessage = $this->getId();
        $name = "POSTS_MESSAGES_GROUPS_ATTRIBUTES_VARIANTS";
        $method = "SET";
        $parameters = [[$idMessage, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $name, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * customFieldsVariants
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        $idGroup = $this->getId();
        $name = "POSTS_MESSAGES_GROUPS_ATTRIBUTES_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idGroup, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount(isset($result[0]) ? sizeof($result) : 0);
        return $result;
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        // table: posts_messages_groups_attributes_related
        $idGroup = $this->getId();
        $name = "POSTS_MESSAGES_GROUPS_ATTRIBUTE_VARIANT";
        $method = "DEL";
        $parameters = [[$idGroup, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * Delete a Message Group
     *
     */
    public function delete()
    {
        $name = "POSTS_MESSAGES_GROUP";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public
    function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name" || $attribute == "description" || $attribute == "imageCaption") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "beginDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return self
     */
    public static function createInstance($data)
    {
        $l10n = new self();
        $l10n->setId($data['id_group']);
        $l10n->setIdType($data['id_type']);
        $l10n->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $l10n->setNameStripped($data['name_stripped']);
        $l10n->setDomain($data['domain']);
        $l10n->setTrackingParameters($data['tracking_parameters']);
        $l10n->setTotal($data['total']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $l10n->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $l10n->setUpdateDate(new DateTime($data['update_date']));
        }

        return $l10n;
    }

    public
    function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
