<?php namespace Stage\Post\Message;

use PDO;
use DateTime;
use Exception;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Helper;
use Stage\Exception\StoredProcedure as StoredProcedureException;

class Message
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idGroup;

    /**
     * @var
     */
    private $idType;

    /**
     * @var
     */
    private $isoLanguage;

    /**
     * @var
     */
    private $managementName;

    /**
     * @var
     */
    private $title = "";

    /**
     * @var
     */
    private $message = "";

    /**
     * @var
     */
    private $image;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $beginDate = "";

    /**
     * @var
     */
    private $registerDate = "";

    /**
     * @var
     */
    private $updateDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idGroup', 'idType', 'isoLanguage', 'managementName', 'title', 'message', 'image', 'link', 'state', 'beginDate', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array('publishDate', 'beginDate', 'endDate');

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdGroup()
    {
        return $this->idGroup;
    }

    public function getIdType()
    {
        return $this->idType;
    }

    public function getIsoLanguage()
    {
        return $this->isoLanguage;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getMessage()
    {
        return $this->message;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdGroup($id)
    {
        $this->idGroup = (int)$id;
    }

    public function setIdType($idType)
    {
        $this->idType = (int)$idType;
    }

    public function setIsoLanguage($isoLanguage)
    {
        $this->isoLanguage = $isoLanguage;
    }

    public function setManagementName($name)
    {
        $this->managementName = $name;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setMessage($description)
    {
        $this->message = $description;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setState($state)
    {
        $this->state = $state;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->beginDate = "";
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * sizeof
     *
     * @return int
     */
    public static function sizeof()
    {
        $name = "POSTS_MESSAGES_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['total']) ? (int)$data[0]['total'] : 0;
    }

    /**
     * types
     *
     * @return MyArrayObject
     */
    public static function types($field, $filter)
    {
        $name = "POSTS_MESSAGES_TYPES";
        $method = "GET";
        $parameters = [[$field, PDO::PARAM_INT], [$filter, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        if (isset($data[0]) && isset($data[0]['id_type'])) {
            $result = new MyArrayObject(array_map(function ($asset) {
                return Type::createInstance($asset);
            }, $data));
            $result->setCount(sizeof($data));
            return $result;
        } else {
            return new MyArrayObject([]);
        }
    }

    /**
     * find
     *
     * @param $idMessage
     * @return self
     * @throws Exception
     */
    public static function find($idMessage)
    {
        try {
            $name = "POSTS_MESSAGE";
            $method = "GET";
            $parameters = [[$idMessage, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Message not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Messages
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15, $all = false)
    {
        $method = "GET";
        $field = $q->field;
        $idGroup = $q->idGroup;
        $idType = $q->idType;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $name = 'POSTS_MESSAGES_COUNT';
                $parameters = [[$field, PDO::PARAM_INT],
                    [$idGroup, PDO::PARAM_INT],
                    [$idType, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = !$all ? (isset($p) ? $p->to() : $q->to) : -1;

            $parameters = [[$field, PDO::PARAM_INT],
                [$idGroup, PDO::PARAM_INT],
                [$idType, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $name = "POSTS_MESSAGES";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = static::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            $result = new MyArrayObject($temp);
            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Delete a Message
     *
     */
    public function delete()
    {
        $name = "POSTS_MESSAGE";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * save
     *
     * Save a Message
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getIdGroup(), PDO::PARAM_INT],
            [$this->getIdType(), PDO::PARAM_INT],
            [$this->getIsoLanguage(), PDO::PARAM_STR],
            [$this->getManagementName(), PDO::PARAM_STR],
            [$this->getTitle(), PDO::PARAM_STR],
            [$this->getMessage(), PDO::PARAM_STR],
            [$this->getImage(), PDO::PARAM_STR],
            [$this->getLink(), PDO::PARAM_STR],
            [$this->getBeginDate() == "" ? "0000-00-00" : $this->getBeginDate(), PDO::PARAM_STR]
        ];

        try {
            $name = "POSTS_MESSAGE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_message'])) {
                $this->setId($data[0]['id_message']);
            } else {
                throw new Exception("Error creating Message");
            }

        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * attachPost
     *
     * @param $idPost
     * @return bool
     * @throws Exception
     */
    public function attachPost($idPost)
    {
        $name = "POSTS_MESSAGE_POST";
        $method = "SET";
        $parameters = [[$this->getId(), PDO::PARAM_INT], [$idPost, PDO::PARAM_INT]];

        try {
            $status = Stage::esp($parameters, $name, $method);
            return isset($status[0]) && isset($status[0]['id']) && $status[0]['id'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * posts
     *
     * @param MessagePostsQuery $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @param bool $all
     * @return MyArrayObject
     * @throws Exception
     */
    public static function posts(MessagePostsQuery $q, $count = false, $page = 1, $perPage = 15, $all = false)
    {
        $method = "GET";
        $field = $q->field;
        $idMessage = $q->idMessage;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            if ($count == true) {
                $name = 'POSTS_MESSAGES_POSTS';
                $parameters = [[$field, PDO::PARAM_INT],
                    [$idMessage, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [0, PDO::PARAM_INT],
                    [-1, PDO::PARAM_INT],
                    [$order, PDO::PARAM_INT],
                    [$orderBy, PDO::PARAM_INT]];
                $data = Stage::esp($parameters, $name, $method);
                $total = sizeof($data);

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = !$all ? (isset($p) ? $p->to() : $q->to) : -1;

            $parameters = [[$field, PDO::PARAM_INT],
                [$idMessage, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $name = "POSTS_MESSAGES_POSTS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = MessagePost::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            $result = new MyArrayObject($temp);
            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * variantKeys
     *
     * Search Messages
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function variantKeys(Query $q, $count = false, $page = 1, $perPage = 15, $all = false)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;

        try {
            if ($count == true) {
                $name = 'POSTS_MESSAGES_CUSTOM_FIELDS_KEYS_COUNT';
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && isset($data[0]['total']) ? $data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = !$all ? (isset($p) ? $p->to() : $q->to) : -1;

            $parameters = [[$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "POSTS_MESSAGES_CUSTOM_FIELDS_KEYS";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-1") {
                throw new Exception("Stored procedure error.", $data[0]['return_code']);
            }

            $temp = [];
            foreach ($data as $tag) {
                $tagInstance = CustomFieldKey::createInstance($tag);
                array_push($temp, $tagInstance);
            }

            $result = new MyArrayObject($temp);
            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachVariantKey
     *
     * @param $idKey
     * @param $name
     * @param $nameStripped
     * @param $public
     * @param $ifExists
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists)
    {
        // table: posts_messages_custom_fields_keys
        $spName = "POSTS_MESSAGES_CUSTOM_FIELDS_KEY";
        $method = "SET";
        $parameters = [[$idKey, PDO::PARAM_INT],
            [$name, PDO::PARAM_STR],
            [$nameStripped, PDO::PARAM_STR],
            [$public, PDO::PARAM_BOOL],
            [$ifExists, PDO::PARAM_BOOL]];

        try {
            $data = Stage::esp($parameters, $spName, $method);
            if (isset($data[0]) && isset($data[0]['id_key'])) {
                return CustomFieldKey::createInstance($data[0]);
            }

            throw new Exception("Error creating custom field key", $data[0]['return_code']);

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachVariantKey
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public static function detachVariantKey($idKey)
    {
        $name = "POSTS_MESSAGES_CUSTOM_FIELD_KEY";
        $method = "DEL";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * variantKey
     *
     * @param $idKey
     * @return CustomFieldKey
     * @throws Exception
     */
    public static function variantKey($idKey)
    {
        $name = "POSTS_MESSAGES_CUSTOM_FIELD_KEY";
        $method = "GET";
        $parameters = [[$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
            throw new Exception("The Key not exists", $data[0]['return_code']);
        } else {
            return CustomFieldKey::createInstance($data[0]);
        }
    }

    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table posts_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: posts_messages_custom_fields_related
        $idMessage = $this->getId();
        $spname = "POSTS_MESSAGES_CUSTOM_FIELDS_VARIANTS";
        $method = "SET";
        $parameters = [[$idMessage, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * customFieldsVariants
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        $idMessage = $this->getId();
        $name = "POSTS_MESSAGES_CUSTOM_FIELDS_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idMessage, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount(isset($result[0]) ? sizeof($result[0]) : 0);
        return $result;
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        $idMessage = $this->getId();
        $name = "POSTS_MESSAGES_CUSTOM_FIELD_VARIANT";
        $method = "DEL";
        $parameters = [[$idMessage, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * duplicate
     *
     * Returns a new Message duplicated, with a new Message id
     *
     * @return $this
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "POSTS_MESSAGES_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_message'])) {
                return self::createInstance($data[0]);
            }

            return null;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * setMessageState
     *
     * Set a message state
     *
     * @return bool
     * @throws Exception
     */
    public static function setMessageState($idMessage, $state)
    {
        $name = "POSTS_MESSAGE_STATE";
        $method = "SET";
        $parameters = [[$idMessage, PDO::PARAM_INT],
            [$state, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['state']) && $data[0]['state'] == "$state";
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public
    function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public
    function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "title" || $attribute == "description" || $attribute == "imageCaption") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "beginDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return self
     */
    public static function createInstance($data)
    {
        $l10n = new self();
        $l10n->setId($data['id_message']);
        $l10n->setIdGroup($data['id_group']);
        $l10n->setIdType($data['id_type']);
        $l10n->setIsoLanguage($data['iso_language']);
        $l10n->setManagementName(html_entity_decode($data['management_name'], ENT_COMPAT, 'UTF-8'));
        $l10n->setTitle(html_entity_decode($data['title'], ENT_COMPAT, 'UTF-8'));
        $l10n->setMessage(html_entity_decode($data['message'], ENT_COMPAT, 'UTF-8'));
        $l10n->setImage($data['image']);
        $l10n->setLink($data['link']);

        if (isset($data['state'])) {
            $l10n->setState($data['state']);
        }

        if (isset($data['begin_date']) && Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $l10n->setBeginDate(new DateTime($data['begin_date']));
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $l10n->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $l10n->setUpdateDate(new DateTime($data['update_date']));
        }

        return $l10n;
    }

    public
    function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
