<?php namespace Stage\Post\Message;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;
use Stage\Post\Post;
use Stage\Post\L10n\Key\Key;
use Stage\Post\L10n\Key\Variant;
use Stage\Exception\StoredProcedure as StoredProcedureException;

class Type
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $color;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $isDefault;

    /**
     * @var
     */
    private $idPostMandatory;

    /**
     * @var
     */
    private $registerDate = "";

    /**
     * @var
     */
    private $updateDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'nameStripped', 'color', 'position', 'active', 'isDefault', 'idPostMandatory', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array('publishDate', 'beginDate', 'endDate');

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getColor()
    {
        return $this->color;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function getIsDefault()
    {
        return $this->isDefault;
    }

    public function getIdPostMandatory()
    {
        return $this->idPostMandatory;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setColor($color)
    {
        $this->color = $color;
    }

    public function setPosition($pos)
    {
        $this->position = (int)$pos;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setIsDefault($default)
    {
        $this->isDefault = (bool)$default;
    }

    public function setIdPostMandatory($id)
    {
        $this->idPostMandatory = (int)$id;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public
    function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public
    function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "title" || $attribute == "description" || $attribute == "nameStripped") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return self
     */
    public static function createInstance($data)
    {
        $l10n = new self();
        $l10n->setId($data['id_type']);
        $l10n->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $l10n->setNameStripped($data['name_stripped']);
        $l10n->setColor($data['color']);
        $l10n->setPosition($data['position']);
        $l10n->setActive($data['active']);
        $l10n->setIsDefault($data['is_default']);
        $l10n->setIdPostMandatory($data['id_post_mandatory']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $l10n->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $l10n->setUpdateDate(new DateTime($data['update_date']));
        }

        return $l10n;
    }

    public
    function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
