<?php namespace Stage\Post;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\Sticker\Sticker;
use Stage\Stage;
use Stage\Post\L10n\L10n;
use Stage\Post\Dossier\Dossier;

class Post
{
    const CREATE = -1;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $type;

    /**
     *
     * @var array
     */
    public $isoLanguages = [];

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'type', 'isoLanguages', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getIsoLanguages()
    {
        return $this->isoLanguages;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    private function setType($type)
    {
        $this->type = (int)$type;
    }

    private function setIsoLanguages($languages)
    {
        if (!in_array($languages, $this->isoLanguages)) {
            $this->isoLanguages[] = $languages;
        }
    }

    public function unSetIsoLanguages($iso)
    {
        if (isset($this->isoLanguages[$iso])) {
            unset($this->isoLanguages[$iso]);
        }
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * sizeof
     *
     * @return int
     */
    public static function sizeof()
    {
        $name = "POSTS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['total']) ? (int)$data[0]['total'] : 0;
    }

    /**
     * types
     *
     * @return MyArrayObject
     */
    public static function types($field)
    {
        $name = "POSTS_TYPES";
        $method = "GET";
        $parameters = [[$field, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        $result = new \Stage\Post\MyArrayObject(array_map(function ($post) {
            return Type::createInstance($post);
        }, $data));
        $result->setCount(sizeof($result));
        return $result;
    }

    /**
     * dates
     *
     * @return array
     */
    public static function dates()
    {
        $name = "POSTS_DATES";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * getAllIsoLanguages
     *
     * @return array
     */
    public static function isoLanguages($idPost = 0)
    {
        $name = "POSTS_ISOLANGUAGES";
        $method = "GET";
        $parameters = [[$idPost, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * @param L10n $l10n
     * @throws Exception
     */
    public function attachL10n(L10n $l10n)
    {
        try {
            $l10n->save($this->getId());
            $this->setIsoLanguages($l10n->getIsoLanguage());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachL10n
     *
     * Remove all L10n's from a Post
     * @throws Exception
     */
    public function detachL10n()
    {
        try {
            $l10ns = $this->l10ns();
            /** @var L10n $l10n */
            foreach ($l10ns as $l10n) {
                $l10n->delete();
            }
            $this->isoLanguages = [];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachSticker
     *
     * @param $idSticker
     * @param $position
     * @param DateTime $beginDate
     * @param DateTime $endDate
     * @return bool|mixed
     * @throws Exception
     */
    public function attachSticker($idSticker, $position, DateTime $beginDate = null, DateTime $endDate = null, $color = "")
    {
        $idPost = $this->getId();

        $beginDate = $beginDate instanceof DateTime ? $beginDate->format(DateTime::ATOM) : "";
        $endDate = $endDate instanceof DateTime ? $endDate->format(DateTime::ATOM) : "";

        // write on table posts_stickers_related
        $name = "POSTS_STICKER_RELATED";
        $method = "SET";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$idPost, PDO::PARAM_INT],
            [$position, PDO::PARAM_INT],
            [$beginDate, PDO::PARAM_STR],
            [$endDate, PDO::PARAM_STR],
            [$color, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : $data[0]['return_code'];
    }

    /**
     * detachSticker
     *
     * Detach a Stickers from a Post
     * @throws Exception
     */
    public function detachSticker($idSticker)
    {
        $name = "POSTS_FROM_STICKERS";
        $method = "DEL";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * stickers
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function stickers()
    {
        $q = new \Stage\Post\Sticker\Query();
        $q->field = 1000;
        $q->query = "";
        $q->filter = $this->getId();
        $q->from = 0;
        $q->to = -1;

        return Sticker::posts($q);
    }

    /**
     * detachAllStickers
     *
     * Detach all Stickers from a Post
     * @throws Exception
     */
    public function detachAllStickers()
    {
        $status = false;
        $sticksers = $this->stickers();
        /** @var Sticker $stickser */
        foreach ($sticksers as $stickser) {
            $status = $this->detachSticker($stickser->getId());
        }
        return $status;
    }

    /**
     * find
     *
     * Find a Post
     *
     * @param $id
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "POST";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Post id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Posts
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * query
     *
     * Search Posts by Query query
     *
     * @param Query $q
     * @param bool $count
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        $field = $q->field;
        $fieldSnd = $q->sndField;
        $fieldTrd = $q->trdField;
        $query = $q->query;
        $filter = $q->filter;
        $filterSnd = $q->sndFilter;
        $filterTrd = $q->trdFilter;
        $limitSysA = $q->limitSysA;
        $limitSysB = $q->limitSysB;
        $limit_a = $q->from;
        $limit_b = $q->to;
        $isoLanguage = $q->isoLanguage;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {
            $method = "GET";
            if ($q->idCollection || $q->idDossier) {
                $idDossier = $q->idDossier;
                $idCollection = $q->idCollection;

                if (!$count) {
                    $parameters = [[$idCollection, PDO::PARAM_STR],
                        [$idDossier, PDO::PARAM_STR],
                        [$field, PDO::PARAM_STR],
                        [$query, PDO::PARAM_INT],
                        [$filter, PDO::PARAM_STR],
                        [$isoLanguage, PDO::PARAM_STR],
                        [$limit_a, PDO::PARAM_INT],
                        [$limit_b, PDO::PARAM_INT]];

                    $name = "PUBLISH_POSTS_SEARCH";
                    $data = Stage::esp($parameters, $name, $method);

                    if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                        throw new Exception("The dossier id or collection id not found", $data[0]['return_code']);
                    }
                } else {
                    $parameters = [[$idCollection, PDO::PARAM_STR],
                        [$idDossier, PDO::PARAM_STR],
                        [$field, PDO::PARAM_STR],
                        [$query, PDO::PARAM_INT],
                        [$filter, PDO::PARAM_STR],
                        [$isoLanguage, PDO::PARAM_STR]];

                    $name = "PUBLISH_POSTS_SEARCH_COUNT";
                    $data = Stage::esp($parameters, $name, $method);
                    return (int)$data[0]['total'];
                }
            } else {

                if ($count == true) {
                    $parameters = [[$field, PDO::PARAM_INT],
                        [$fieldSnd, PDO::PARAM_INT],
                        [$fieldTrd, PDO::PARAM_INT],
                        [$query, PDO::PARAM_STR],
                        [$filter, PDO::PARAM_STR],
                        [$filterSnd, PDO::PARAM_STR],
                        [$filterTrd, PDO::PARAM_STR],
                        [$limitSysA, PDO::PARAM_INT],
                        [$limitSysB, PDO::PARAM_INT],
                        [$isoLanguage, PDO::PARAM_STR]];

                    $name = "POSTS_COUNT";
                    $data = Stage::esp($parameters, $name, $method);
                    $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                    // Create a new Paginator
                    $p = new Paginator($total, $perPage);
                    $p->page($page);
                    $q->from = $p->from();
                    $q->to = $p->to();
                }

                $limit_a = isset($p) ? $p->from() : $q->from;
                $limit_b = isset($p) ? $p->to() : $q->to;

                $parameters = [[$field, PDO::PARAM_INT],
                    [$fieldSnd, PDO::PARAM_INT],
                    [$fieldTrd, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR],
                    [$filterSnd, PDO::PARAM_STR],
                    [$filterTrd, PDO::PARAM_STR],
                    [$limitSysA, PDO::PARAM_INT],
                    [$limitSysB, PDO::PARAM_INT],
                    [$isoLanguage, PDO::PARAM_STR],
                    [$limit_a, PDO::PARAM_INT],
                    [$limit_b, PDO::PARAM_INT],
                    [$order, PDO::PARAM_INT],
                    [$orderBy, PDO::PARAM_INT]];

                $name = "POSTS";
                $data = Stage::esp($parameters, $name, $method);
                $result = new \Stage\Post\MyArrayObject(array_map(function ($post) {
                    return L10n::createInstance($post);
                }, $data));

                if (isset($p)) {
                    $result->setCount($total);
                    $result->setPaginator($p);
                }
                return $result;
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * duplicate
     *
     * Returns a new Post duplicated, with a new Post id
     *
     * @return $this
     * @throws Exception
     */
    public function duplicate()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $name = "POSTS_CLONE";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            return self::createInstance($data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Returns a Post deleted
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "POST";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * dossiers
     *
     * Get all the dossiers that the Post in inserted in
     * @return array|MyArrayObject
     * @throws Exception
     */
    public function dossiers()
    {
        try {
            $name = "POSTS_DOSSIER_BY_POST";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                return [];
            } else {
                return new \Stage\Post\MyArrayObject(array_map(function ($dossier) {
                    return Dossier::createInstance($dossier);
                }, $data));
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachDossier
     *
     * @param $idDossier
     * @return bool
     * @throws Exception
     */
    public function attachDossier($idDossier, $position)
    {
        $dossier = Dossier::find($idDossier);
        return $dossier->attachPost($this->getId(), $position);
    }

    /**
     * detachDossier
     *
     * Detach all Dossier from the Post
     * @return false
     * @throws Exception
     */
    public function detachAllDossiers()
    {
        $status = false;
        $dossiers = $this->dossiers();
        /** @var Dossier $dossier */
        foreach ($dossiers as $dossier) {
            $dossier->detachPost($this->getId());
        }
        return $status;
    }

    /**
     * save
     *
     * Save a post
     *
     * @param int $idPost -1 para criar 0 para fazer update
     * @param int $idType
     * @return $this
     * @throws Exception
     */
    public function save($idPost = -1, $idType = 1)
    {
        try {
            $parameters = [
                [$idPost, PDO::PARAM_INT],
                [$idType, PDO::PARAM_INT]
            ];
            $name = "POSTS";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_post']) && $data[0]['id_post'] >= 0) {
                $idPost = $data[0]['id_post'];
                $this->setId($idPost);
                if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
                return $this;
            } else {
                throw new Exception("Error creating Post");
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * metaUrlExist
     *
     * Check if the post slug url exit
     *
     * @param $idL10n
     * @param $metaUrl
     * @param $iso
     * @return array
     */
    public static function metaUrlExist($idL10n, $metaUrl, $iso)
    {
        $name = "POSTS_URL_NUM";
        $method = "GET";
        $parameters = [[$idL10n, PDO::PARAM_INT],
            [$metaUrl, PDO::PARAM_STR],
            [$iso, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * existIn
     *
     * Get all the collections, dossiers where this post is inserted in
     * @param $idPost
     * @return array
     * @throws Exception
     */
    public static function existIn($idPost)
    {
        try {
            $name = "POST_DOSSIERS_AND_COLLECTIONS_IDS_RELATED";
            $method = "GET";
            $parameters = [[$idPost, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * l10ns
     *
     * Get all Post L10n's
     * @throws Exception
     */
    public function l10ns()
    {
        $idPost = $this->getId();
        return new \Stage\Post\MyArrayObject(array_map(function ($iso) use ($idPost) {
            return L10n::find($idPost, $iso);
        }, $this->getIsoLanguages()));
    }

    public static function systemTags($field, $idQuery, $limitA, $limitB)
    {
        try {
            $name = "POSTS_SYSTEM_TAGS";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT],
                [$idQuery, PDO::PARAM_INT],
                [$limitA, PDO::PARAM_INT],
                [$limitB, PDO::PARAM_INT]];

            return Stage::esp($parameters, $name, $method);

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * blockTypes
     *
     * @param $idPostType
     * @return MyArrayObject
     * @throws Exception
     */
    public static function blockTypes($idPostType)
    {
        try {
            $name = "POSTS_BLOCK_TYPES";
            $method = "GET";
            $parameters = [[$idPostType, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($type) {
                return BlockType::createInstance($type);
            }, $data));
            $result->setCount(sizeof($result));
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * imageRations
     *
     * @param int $field
     * @return array
     * @throws Exception
     */
    public static function imageRations($field)
    {
        try {
            $name = "POSTS_IMAGES_RATIOS";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT]];

            return array_map(function ($ratio) {
                unset($ratio['register_date']);
                $ratio['isDefault'] = $ratio['is_default'];
                unset($ratio['is_default']);
                $ratio['name'] = html_entity_decode($ratio['name'], ENT_COMPAT, 'UTF-8');
                return $ratio;
            }, Stage::esp($parameters, $name, $method));
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * saveStats
     *
     * @param $idPost
     * @param $idDossier
     * @param $idCollection
     * @param $day
     * @param $impressions
     * @return bool
     * @throws Exception
     */
    public static function saveStats($idPost, $idDossier, $idCollection, $day, $impressions)
    {
        try {
            $name = "POSTS_STATS";
            $method = "SET";
            $parameters = [[$idPost, PDO::PARAM_INT],
                [$idDossier, PDO::PARAM_INT],
                [$idCollection, PDO::PARAM_INT],
                [$day, PDO::PARAM_STR],
                [$impressions, PDO::PARAM_INT]];

            $result = Stage::esp($parameters, $name, $method);
            return isset($result[0]) && isset($result[0]['id']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * typeColors
     *
     * @param $idPostType
     * @return MyArrayObject
     * @throws Exception
     */
    public static function typeColors($idPostType)
    {
        try {
            $name = "POSTS_TYPES_COLORS";
            $method = "GET";
            $parameters = [[$idPostType, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($post) {
                return TypeColor::createInstance($post);
            }, $data));
            $result->setCount(sizeof($result));
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * idsRelated
     *
     * @param $ids
     * @return array
     * @throws Exception
     */
    public static function idsRelated($ids)
    {
        try {
            $name = "POSTS_IDS_RELATED";
            $method = "GET";
            $parameters = [[$ids, PDO::PARAM_STR]];

            $result = Stage::esp($parameters, $name, $method);
            return array_map(function ($item) {
                $std = new \stdClass();
                $std->idPost = $item['id_post'];
                $std->idType = $item['id_type'];
                $std->idDossier = $item['id_dossier'];
                $std->idCollection = $item['id_collection'];
                return $std;
            }, $result);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

//        foreach ($this->getL10n() as $l10n) {
//            $attributes['l10n'][] = $l10n->toArray();
//        }

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Post
     */
    public static function createInstance($data)
    {
        // $data is an array of rows of posts_l10n
        $p = new self();
        $p->setId($data[0]['id_post']);
        $p->setType($data[0]['id_type']);

        $languages = self::isoLanguages($data[0]['id_post']);
        foreach ($languages as $l) {
            $p->setIsoLanguages($l['iso_language']);
        }

        if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
            $p->setRegisterDate(new DateTime($data[0]['register_date']));
        }

        if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
            $p->setUpdateDate(new DateTime($data[0]['update_date']));
        }
        return $p;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
