<?php namespace Stage\Post\Sticker\Group;

use DateTime;
use Stage\Helper;

class Country
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idGroup;

    /**
     * @var
     */
    private $iso = "";

    /**
     * @var
     */
    private $exclude = false;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idGroup', 'iso', 'exclude', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdGroup()
    {
        return (int)$this->idGroup;
    }

    public function getIso()
    {
        return $this->iso;
    }

    public function getExclude()
    {
        return (bool)$this->exclude;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdGroup($id)
    {
        $this->idGroup = (int)$id;
    }

    public function setIso($iso)
    {
        $this->iso = $iso;
    }

    public function setExclude($exclude)
    {
        $this->exclude = (bool)$exclude;
    }

    public function setRegisterDate($date)
    {
        if ($date == null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $instance = new self();
        if (isset($data['id'])) {
            $instance->setId($data['id']);
        }
//        if (isset($data['id_group'])) {
//            $instance->setIdGroup($data['id_group']);
//        }
        $instance->setIso($data['country_iso']);
        $instance->setExclude($data['exclude']);

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
