<?php namespace Stage\Post\Sticker;

use DateTime;
use Stage\Helper;

class SourceType
{
    const SOURCE_TYPE_POST = 1;
    const SOURCE_TYPE_POST_TAG = 10;
    const SOURCE_TYPE_DOSSIER = 2;
    const SOURCE_TYPE_DOSSIER_TAG = 20;
    const SOURCE_TYPE_COLLECTION = 3;
    const SOURCE_TYPE_COLLECTION_TAG = 30;
    const SOURCE_TYPE_AUTHOR = 4;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $type;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'type', 'name', 'total', 'registerDate', 'updateDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = $id;
    }

    public function setType($itype)
    {
        $this->type = (int)$itype;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setTotal($total)
    {
        $this->total = (int)$total;
    }

    public function setRegisterDate($date)
    {
        if ($date == null) {
            $this->registerDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id']);
        $instance->setType($data['source_type']);
        $instance->setTotal($data['total']);
        $instance->setName($data['name']);
        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
