<?php namespace Stage\Post;

use DateTime;
use Exception;
use Stage\Helper;

class TypeColor
{

    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $idType;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $color;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $isDefault;

    /**
     * @var
     */
    private $colorSet;

    /**
     * @var
     */
    private $type;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idType', 'name', 'nameStripped', 'color', 'position', 'isDefault', 'colorSet', 'type', 'registerDate', 'updateDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdType()
    {
        return $this->idType;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getColor()
    {
        return $this->color;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getIsDefault()
    {
        return $this->isDefault;
    }

    public function getColorSet()
    {
        return $this->colorSet;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    private function setIdType($idType)
    {
        $this->idType = (int)$idType;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setColor($color)
    {
        $this->color = $color;
    }

    public function setPosition($position)
    {
        $this->position = (int)$position;
    }

    public function setIsDefault($isDefault)
    {
        $this->isDefault = (bool)$isDefault;
    }

    public function setColorSet($colorSet)
    {
        $this->colorSet = $colorSet;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name" || $attribute == "nameStripped") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $tag
     * @return TypeColor
     * @throws Exception
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id']);
        $tagInstance->setIdType($tag['id_type']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setColor($tag['color']);
        $tagInstance->setPosition($tag['position']);
        $tagInstance->setIsDefault($tag['is_default']);
        $tagInstance->setColorSet($tag['color_set']);
        $tagInstance->setType($tag['type']);

        if (isset($tag['register_date']) && Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        if (isset($tag['update_date']) && Helper::isValidDate($tag['update_date'], "Y-m-d H:i:s")) {
            $tagInstance->setUpdateDate(new DateTime($tag['update_date']));
        }

        return $tagInstance;
    }
}
