<?php namespace Stage;

use PDO;
use Exception;

class Role
{
    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $description;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'description');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array();

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array();    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    /**
     * @return mixed
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return mixed
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param $id
     */
    private function setId($id)
    {
        $this->id = (int) $id;
    }

    /**
     * @param $description
     */
    private function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "ROLE";
            $method = "GET";
            $role = Stage::esp($parameters, $name, $method);

            if (sizeof($role) > 0) {
                return self::createInstance($role[0]);
            } else {
                throw new Exception("Role id $id not found");
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $parameters = [];
        try {
            $name = "ROLES";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $role) {
                $roleInstance = new self();
                $roleInstance->setId($role['id_role']);
                $roleInstance->setDescription($role['description']);

                array_push($temp, $roleInstance);
            }

            return $temp;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public static function role($roleName)
    {
        $role = null;

        try {
            $roles =  self::all();
            foreach ($roles as $r) {
                if (trim(strtoupper($r->getDescription())) === trim(strtoupper($roleName))) {
                    $role = $r;
                    return $role;
                }
            }

            throw new Exception("You are not passing a valid role name. Please check available roles with Roles::all()");
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($role)
    {
        $r = new self();
        $r->setId($role['id_role']);
        $r->setDescription($role['description']);

        return $r;
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }
}
