<?php namespace Stage\Support;

use PDO;
use Exception;
use Stage\Asset\Tag\Tag;
use Stage\Paginator;
use Stage\Stage;

class Asset
{

    /**
     * assetTags
     *
     * @param $idAsset
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function assetTags($idAsset)
    {
        try {
            $name = "PUBLISH_ASSETS_TAGS_RELATED";
            $method = "GET";
            $parameters = [[$idAsset, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['return_code'])) {
                return new \Stage\Post\MyArrayObject([]);
            }

            return new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return Tag::createInstance($label);
            }, $data));

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assetsAlbumsTags
     *
     * Return all the tags inside an Album
     * @param $idAlbum
     * @return array
     * @throws Exception
     */
    public static function assetsAlbumsTags($idAlbum)
    {
        try {
            $name = "PUBLISH_ASSETS_ALBUMS_TAGS";
            $method = "GET";
            $parameters = [[$idAlbum, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assetsGalleriesTags
     *
     * Return all the Tags inside a gallery
     * @param $idGallery
     * @return array
     * @throws Exception
     */
    public static function assetsGalleriesTags($idGallery)
    {
        try {
            $name = "PUBLISH_ASSETS_TAGS";
            $method = "GET";
            $parameters = [[$idGallery, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * changed
     * Get all the objects to sync with couchdb
     * @return array
     * @throws Exception
     */
    public static function changed($page = 1, $perPage = 20)
    {
        $from = ($page - 1) * $perPage;
        $to = $perPage;

        try {
            $name = "PUBLISH_ASSETS";
            $method = "GET";
            $parameters = [[$from, PDO::PARAM_INT],
                [$to, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * publishEnd
     *
     * @return array
     * @throws Exception
     */
    public static function publishEnd($syncType = null)
    {
        try {
            $name = "PUBLISH_ASSETS_ENDS";
            $method = "SET";
            $parameters = [];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * sync
     *
     * @param $id
     * @param $idSync
     * @param $type
     * @return array
     * @throws Exception
     */
    public static function sync($id, $idSync, $idSyncRev, $type)
    {
        try {
            $name = "PUBLISH_ASSETS_SYNC";
            $method = "SET";
            $parameters = [[$id, PDO::PARAM_INT],
                [$idSync, PDO::PARAM_STR],
                [$idSyncRev, PDO::PARAM_STR],
                [$type, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * publishStart
     *
     * Is doing some stuff in the table assets_publish_log
     * @return array
     * @throws Exception
     */
    public static function publishStart($syncType = null)
    {
        try {
            $name = "PUBLISH_ASSETS";
            $method = "SET";
            $parameters = [];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * existIn
     *
     * Give all the pairs channel,playlist where the video composed exist
     * @param $idPost
     * @return array
     * @throws Exception
     */
    public static function existIn($idAsset)
    {
        try {
            $name = "PUBLISH_ASSETS_ALBUMS_GALLERIES_IDS_RELATED";
            $method = "GET";
            $parameters = [[$idAsset, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * assetsToExecute
     *
     * @return array
     * @throws Exception
     */
    public static function assetsToExecute()
    {
        try {
            $name = "PUBLISH_ASSETS_TO_EXECUTE_MGO";
            $method = "GET";
            $parameters = [];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }
}
