<?php namespace Stage\Upload;

use Stage\Account\Account;
use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class Upload
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idDirectory;

    /**
     * @var
     */
    private $active;

    /**;
     * @var
     */
    private $account;

    /**
     * @var integer
     */
    private $idService;

    /**
     * @var
     */
    private $code;

    /**
     * @var
     */
    private $password;

    /**
     * @var
     */
    private $uploadMaxSize;

    /**
     * @var
     */
    private $uploadTypes;

    /**
     * @var
     */
    private $shared = 0;

    /**
     * @var
     */
    private $endDate;

    /**
     * @var
     */
    private $maintenance;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    // private $fromService;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','account','code','password','uploadMaxSize','uploadTypes','shared','endDate','idService', 'maintenance','idDirectory','active');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getAccount()
    {
        return $this->account;
    }

    public function getIdService()
    {
        return $this->idService;
    }

    public function getCode()
    {
        return $this->code;
    }

    public function getPassword()
    {
        return $this->password;
    }

    public function getUploadMaxSize()
    {
        return $this->uploadMaxSize;
    }

    public function getUploadTypes()
    {
        return $this->uploadTypes;
    }

    public function getShared()
    {
        return $this->shared;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getIdDirectory()
    {
        return $this->idDirectory;
    }

    public function isActive()
    {
        return (bool) $this->active;
    }

    public function inMaintenance()
    {
        return (bool) $this->maintenance;
    }

    /*
    public function getFromService()
    {
        return $this->fromService;
    }*/

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setAccount(Account $account)
    {
        $this->account = $account;
    }

    public function setCode($code)
    {
        $this->code = $code;
    }

    public function setPassword($password)
    {
        $this->password = $password;
    }

    public function setUploadMaxSize($uploadMaxSize)
    {
        $this->uploadMaxSize = $uploadMaxSize;
    }

    public function setUploadTypes($uploadTypes)
    {
        $this->uploadTypes = $uploadTypes;
    }

    public function setShared($shared)
    {
        $this->shared = $shared;
    }

    public function setEndDate($date)
    {
        if ($date==null) {
            $this->endDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /*public function setFromService($service)
    {
        $this->fromService = $service;
    }*/

    public function setIdService($idService)
    {
        $this->idService = (int) $idService;
    }

    public function setIdDirectory($idDirectory)
    {
        $this->idDirectory = (int) $idDirectory;
    }

    public function setIsActive($active)
    {
        $this->active = $active;
    }

    public function setInMaintenance($in)
    {
        $this->maintenance = (bool) $in;
    }

    /**
     * find
     *
     * Find a Upload by code
     *
     * @return $this
     * @throws Exception
     */
    public static function find($code)
    {
        try {
            $name = "ACCOUNT_BULK_UPLOAD";
            $method = "GET";
            $parameters = [[$code, PDO::PARAM_STR]];
            $data = Stage::esp($parameters, $name, $method);

            if (!sizeof($data)) {
                throw new Exception("The Upload with code $code not exists");
            } else {
                return self::createInstance($data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "ACCOUNT_BULK_UPLOAD";
            $method = "SET";
            $parameters = [[$this->getAccount()->getId(), PDO::PARAM_INT],
                            [$this->getIdService(), PDO::PARAM_INT],
                            [$this->getCode(), PDO::PARAM_STR],
                            [$this->getPassword(), PDO::PARAM_STR],
                            [$this->getUploadMaxSize(), PDO::PARAM_INT],
                            [$this->getUploadTypes(), PDO::PARAM_INT],
                            [$this->getShared(), PDO::PARAM_INT],
                            [$this->getIdDirectory(), PDO::PARAM_INT],
                            [$this->isActive(), PDO::PARAM_BOOL],
                            [$this->getEndDate(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_upload'])) {
                self::createInstance($data);
                return $this;
            } else {
                throw new Exception("Error", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }
    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "account" ? $this->getAccount()->toArray() : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Upload
     */
    public static function createInstance($data)
    {

        // $data is an array of rows of posts_l10n
        $upload = new self();
        $upload->setId($data[0]['id_upload']);

        $account = Account::find($data[0]['id_account'], Account::ID);
        $upload->setAccount($account);

        $upload->setCode($data[0]['code']);
        $upload->setPassword($data[0]['password']);
        $upload->setUploadMaxSize($data[0]['upload_max_size']);
        $upload->setUploadTypes($data[0]['uptypes']);
        $upload->setShared($data[0]['shared']);
        $upload->setIdService($data[0]['id_service']);
        $upload->setIdDirectory($data[0]['id_directory']);
        $upload->setIsActive($data[0]['active']);
        $upload->setInMaintenance($data[0]['maintenance']);

        if (Helper::isValidDate($data[0]['end_date'], "Y-m-d H:i:s")) {
            $upload->setEndDate(new DateTime($data[0]['end_date']));
        }

        if (Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
            $upload->setRegisterDate(new DateTime($data[0]['register_date']));
        }

        return $upload;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
