<?php namespace Stage\Video\Channel;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Video\Channel\Tag\Tag;
use Stage\Video\Playlist\Playlist;
use Stage\Video\Sticker\Sticker;

class Channel
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $managementName = "";

    /**
     * @var
     */
    private $position = null;

    /**
     * @var
     */
    private $state;

    /**
     * @var L10n[]
     */
    private $l10n = [];

    /**
     *
     * @var array
     */
    private $domain = "";

    /**
     *
     * @var bool
     */
    private $hasVod = false;

    /**
     *
     * @var bool
     */
    private $hasLive = false;

    /**
     * Total number of playlists inside this channel
     * @var array
     */
    private $total = 0;

    /**
     * Total composes inside this channel
     * @var array
     */
    private $totalComposesIn = 0;

    /**
     * @var Playlist
     */
    private $defaultPlaylist;

    /**
     * @var
     */
    private $beginDate = 0;

    /**
     * @var
     */
    private $publishDate = 0;

    /**
     * @var
     */
    private $endDate = 0;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'managementName','position','state', 'domain', 'registerDate', 'defaultPlaylist', 'total', 'hasVod', 'hasLive', 'l10n', 'publishDate', 'beginDate', 'endDate', 'totalComposesIn');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getL10n()
    {
        return $this->l10n;
    }

    public function getManagementName()
    {
        return $this->managementName;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getPublishDate()
    {
        return $this->publishDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getDomain()
    {
        return $this->domain;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getTotalComposesIn()
    {
        return $this->totalComposesIn;
    }

    public function hasVod()
    {
        return $this->hasVod;
    }

    public function hasLive()
    {
        return $this->hasLive;
    }

    public function addL10n(L10n $l10n)
    {
        $this->l10n[] = $l10n;
    }

    public function getDefaultPlaylist()
    {
        return $this->defaultPlaylist;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setDomain($domain)
    {
        $this->domain = $domain;
    }

    public function setTotal($total)
    {
        $this->total = (int) $total;
    }

    public function setTotalComposesIn($total)
    {
        $this->totalComposesIn = (int) $total;
    }

    public function setHasVod($has)
    {
        $this->hasVod = (bool) $has;
    }

    public function setHasLive($has)
    {
        $this->hasLive = (bool) $has;
    }

    public function setDefaultPlaylist($playlist)
    {
        $this->defaultPlaylist = $playlist;
    }

    public function setManagementName($managementName)
    {
        $this->managementName = $managementName;
    }

    public function setPosition($position)
    {
        $this->position = (int) $position;
    }

    public function setState($state)
    {
        $this->state = (int) $state;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->beginDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setPublishDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->publishDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->publishDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->endDate = 0;
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        //
        $name = "VIDEO_CHANNELS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Channel
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "VIDEO_CHANNEL";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Channel id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Posts
     *
     * @return Array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->term = "";
        $q->by = "-1";
        $q->from = 0;
        $q->to = -1;

        return self::query($q);
    }

    /**
     * query
     *
     * Search Channels by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $query = $q->term;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $parameters = [ [$query, PDO::PARAM_STR],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "VIDEO_CHANNELS";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($channel) {
                return self::createInstance($channel);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * @param \Stage\Video\Channel\Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q)
    {
        try {
            $query = $q->term;
            $parameters = [ [$query, PDO::PARAM_STR]];
            $name = "VIDEO_CHANNELS_COUNT";

            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int) $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * totalPlaylists
     *
     * Get the total composes inside a Channel
     * @return int
     * @throws Exception
     */
    public function totalPlaylists()
    {
        $id = $this->getId();

        try {
            $parameters = [[$id, PDO::PARAM_INT]];

            $name = "VIDEO_PLAYLISTS_ON_CHANNELS_NUM";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int)$data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Returns the Channel deleted
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "VIDEO_CHANNEL";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the channel tags
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEO_CHANNEL_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag $tag
     * @throws Exception
     */
    public function addTag(Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [ [$idTag, PDO::PARAM_INT],
                            [$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEO_CHANNELS_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $this->tags = $this->tags();
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEO_CHANNELS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Channel
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            if ($this->getDefaultPlaylist() instanceof Playlist) {
                $defaultPlaylist = $this->getDefaultPlaylist()->getId();
            } else {
                $defaultPlaylist = 0;
            }

            $name = "VIDEO_CHANNELS";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$this->getManagementName(), PDO::PARAM_STR],
                            [$this->getPublishDate(), PDO::PARAM_STR],
                            [$this->getBeginDate(), PDO::PARAM_STR],
                            [$this->getEndDate(), PDO::PARAM_STR],
                            [$this->getDomain(), PDO::PARAM_STR],
                            [$defaultPlaylist, PDO::PARAM_STR],
                            [$this->getPosition(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_channel']) && $data[0]['id_channel'] >= 0) {
                $this->setId($data[0]['id_channel']);
                if (Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }
            }

            // Remove all Channel L10ns
            for ($i = 0 ; $i <= sizeof($this->getL10n()) -1 ; $i++) {
                if ($this->l10n[$i]->getId() != 0) {
                    $this->l10n[$i]->delete();
                    unset($this->l10n[$i]);
                }
            }

            // Sort array
            array_values($this->l10n);

            // Insert all again if there is any
            foreach ($this->getL10n() as $l10n) {
                $l10n->setIdChannel($data[0]['id_channel']);
                $l10n->save();
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * removePlaylist
     *
     * Remove a Playlist from a Channel
     * @param Playlist $d
     */
    public function removePlaylist(Playlist $d)
    {
        try {
            $name = "VIDEO_PLAYLISTS_FROM_CHANNELS";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                            [$d->getId(), PDO::PARAM_INT]];

            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $d;
    }

    /**
     * addPlaylist
     *
     * Add a Playlist to a Channel
     * @param Playlist $d
     */
    public function addPlaylist(Playlist $d, $position)
    {
        try {
            $name = "VIDEO_PLAYLISTS_TO_CHANNELS";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                           [$d->getId(), PDO::PARAM_INT],
                           [$position, PDO::PARAM_INT]];

            Stage::esp($parameters, $name, $method);
            return $d;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * playlists
     *
     * Returns all the Playlists withing a Channel
     *
     * @return mixed
     * @throws Exception
     */
    public function playlists(Query $q)
    {
        try {
            $name = "PLAYLISTS_ON_CHANNELS";
            $method = "GET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$q->term, PDO::PARAM_STR],
                            [0, PDO::PARAM_INT],
                            [0, PDO::PARAM_INT],
                            [$q->from, PDO::PARAM_INT],
                            [$q->to, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            $playlists = [];
            foreach ($data as $playlist) {
                $playlists[] = Playlist::createInstance($playlist);
            }

            return $playlists;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * playlistsCount
     *
     * @param Query $q
     * @return int
     * @throws Exception
     */
    public function playlistsCount(Query $q)
    {
        $term = $q->term;

        try {
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                            [$term, PDO::PARAM_STR]];

            $name = "PLAYLISTS_ON_CHANNELS_COUNT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return (int)$data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * innerLanguages
     *
     * Get all languages inside a channel
     *
     * @return array
     * @throws Exception
     */
    public function innerLanguages()
    {
        try {
            $name = "CHANNELS_ISOLANGUAGES";
            $method = "GET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return array_values(array_unique(array_map(function ($item) {
                return $item['iso_language'];
            }, $data)));
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachSticker
     *
     * @param $idSticker
     * @param $position
     * @param DateTime $beginDate
     * @param DateTime $endDate
     * @return bool|mixed
     * @throws Exception
     */
    public function attachSticker($idSticker, $position, DateTime $beginDate = null, DateTime $endDate = null, $color = "")
    {
        $idPlaylist = $this->getId();

        $beginDate = $beginDate instanceof DateTime ? $beginDate->format(DateTime::ATOM) : "";
        $endDate = $endDate instanceof DateTime ? $endDate->format(DateTime::ATOM) : "";

        // write on table video_stickers_related
        $name = "VIDEO_CHANNELS_STICKER_RELATED";
        $method = "SET";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$idPlaylist, PDO::PARAM_INT],
            [$position, PDO::PARAM_INT],
            [$beginDate, PDO::PARAM_STR],
            [$endDate, PDO::PARAM_STR],
            [$color, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : $data[0]['return_code'];
    }

    /**
     * detachSticker
     *
     * Detach a Stickers from a Channel
     * @throws Exception
     */
    public function detachSticker($idSticker)
    {
        $name = "VIDEO_CHANNEL_FROM_STICKERS";
        $method = "DEL";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * stickers
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function stickers()
    {
        $q = new \Stage\Video\Sticker\Query();
        $q->field = 1000;
        $q->query = "";
        $q->filter = $this->getId();
        $q->from = 0;
        $q->to = -1;

        return Sticker::channels($q);
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute =="defaultPlaylist" && $this->$attribute instanceof Playlist) {
                $arr[$attribute] = $this->$attribute->toArray();
            } elseif ($attribute == "l10n") {
                foreach ($this->$attribute as $l10n) {
                    $arr[$attribute][] = $l10n->toArray();
                }
            } else {
                if ($this->$attribute !== null) {
                    $arr[$attribute] = $attribute == "managementName" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
                }
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Channel
     */
    public static function createInstance($data)
    {
        $p = new self();
        $p->setId($data['id_channel']);
        $p->setManagementName(html_entity_decode($data['management_name'], ENT_COMPAT, 'UTF-8'));
        if (isset($data['position'])) {
            $p->setPosition($data['position']);
        }

        if (isset($data['total_composes_in'])) {
            $p->setTotalComposesIn($data['total_composes_in']);
        }

        if (isset($data['state'])) {
            $p->setState($data['state']);
        }
        $p->setDomain($data['domain']);

        if (isset($data['has_vod'])) {
            $p->setHasVod($data['has_vod']);
        }

        if (isset($data['has_live'])) {
            $p->setHasLive($data['has_live']);
        }

        if (isset($data['total'])) {
            $p->setTotal($data['total']);
        }

        $playlist = 0;
        try {
            $playlist = Playlist::find($data['default_id_playlist']);
        } catch (Exception $e) {
        }

        $p->setDefaultPlaylist($playlist);

        if (Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $p->setBeginDate(new DateTime($data['begin_date']));
        }

        if (Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $p->setEndDate(new DateTime($data['end_date']));
        }

        if (Helper::isValidDate($data['publish_date'], "Y-m-d H:i:s")) {
            $p->setPublishDate(new DateTime($data['publish_date']));
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $p->setRegisterDate(new DateTime($data['register_date']));
        }

        $l10ns = L10n::find($data['id_channel']);
        foreach ($l10ns as $l10n) {
            $p->addL10n($l10n);
        }

        return $p;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
