<?php namespace Stage\Video\Channel;

use DateTime;
use PDO;
use Exception;
use Stage\Helper;
use Stage\Stage;
use Stage\Exception\StoredProcedure as StoredProcedureException;
use Stage\Video\Channel\Metadata\Metadata;
use Stage\Video\Channel\Tag\Tag;

class L10n
{
    /**
     * @var
     */
    private $id = 0;

    /**
     * @var
     */
    private $idChannel;

    /**
     * @var
     */
    private $isoLanguage;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $description = "";

    /**
     * @var
     */
    private $image = "";

    /**
     * @var
     */
    private $registerDate = "";

    /**
     * @var
     */
    private $metadata;

    /**
     * @var
     */
    private $tags = [];

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'isoLanguage','name','description', 'tags','registerDate','image');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdChannel()
    {
        return $this->idChannel;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getTags()
    {
        return $this->tags;
    }

    public function getIsoLanguage()
    {
        return $this->isoLanguage;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getMetadata()
    {
        return $this->metadata;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdChannel($id)
    {
        $this->idChannel = $id;
    }

    public function setIsoLanguage($isoLanguage)
    {
        $this->isoLanguage = $isoLanguage;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setDescription($desc)
    {
        $this->description = $desc;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    public function addTag(Tag $tag)
    {
        $this->tags[] = $tag;
    }

    public function setRegisterDate($registerDate)
    {
        $this->registerDate = $registerDate;
    }

    public function addMetadata(Metadata $meta)
    {
        $this->metadata = $meta;
    }

    public static function find($idChannel)
    {
        $parameters = [ [$idChannel, PDO::PARAM_INT] ];

        try {
            $name = "VIDEO_CHANNEL_L10N";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            $temp = [];
            foreach ($data as $l10n) {
                $temp[] = self::createInstance($l10n);
            }
            return $temp;
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Dossier L10n
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $metadata = $this->getMetadata() instanceof Metadata ? $this->getMetadata() : new Metadata();

        $parameters = [ [$this->getId(), PDO::PARAM_INT],
                        [$this->getIdChannel(), PDO::PARAM_INT],
                        [$this->getIsoLanguage(), PDO::PARAM_STR],
                        [$this->getName(), PDO::PARAM_STR],
                        [$this->getDescription(), PDO::PARAM_STR],
                        [$this->getImage(), PDO::PARAM_STR],
                        [$metadata->getTitle(),PDO::PARAM_STR],
                        [$metadata->getDescription(),PDO::PARAM_STR],
                        [$metadata->getImage(),PDO::PARAM_STR],
                        [$metadata->getImageTitle(),PDO::PARAM_STR],
                        [$metadata->getImageAlt(),PDO::PARAM_STR],
                        [$metadata->getKeywords(),PDO::PARAM_STR]];

        try {
            $name = "VIDEO_CHANNELS_L10N";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_l10n'])) {
                $this->setId($data[0]['id_l10n']);
            }

            // Save the tags of the post
            $tags = $this->getTags();
            $this->removeAllTags();
            foreach ($tags as $tag) {
                $this->associateTag($tag);
            }
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * @return array
     */
    public function getL10nTags()
    {
        $name = "VIDEO_CHANNEL_L10N_TAGS_RELATED";
        $method = "GET";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $tags = [];
        foreach ($data as $b) {
            if (isset($b['id_tag'])) {
                $tags[] = Tag::createInstance($b);
            }
        }

        return $tags;
    }

    public function removeAllTags()
    {
        $name = "VIDEO_CHANNELS_L10N_FROM_TAGS";
        $method = "DEL";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    /**
     * associateTag
     *
     * Associate a Tag to a L10n
     * @param Tag $tag
     */
    public function associateTag(Tag $tag)
    {
        // -3 Ja existe
        $name = "VIDEO_CHANNELS_L10N_TO_TAGS";
        $method = "SET";
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
                        [$tag->getId(), PDO::PARAM_INT]];

        Stage::esp($parameters, $name, $method);
    }

    /**
     * delete
     *
     * @return $this
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "VIDEO_CHANNEL_L10N";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        if ($this->getMetadata() instanceof Metadata) {
            $attributes['metadata'] = $this->getMetadata()->toArray();
        }

        $attributes['tags'] = [];
        foreach ($this->getL10nTags() as $tag) {
            $attributes['tags'][] = $tag->toArray();
        }

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "name" || $attribute == "description") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else {
                $arr[$attribute] =  $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return L10n
     */
    public static function createInstance($data)
    {
        $l10n = new self();

        $l10n->setId($data['id_l10n']);
        $l10n->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $l10n->setDescription(html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8'));
        $l10n->setIsoLanguage($data['iso_language']);
        $l10n->setImage($data['image']);

        // Add the Tag
        $tags = $l10n->getL10nTags();
        foreach ($tags as $tag) {
            $l10n->addTag($tag);
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $l10n->setRegisterDate(new DateTime($data['register_date']));
        }

        return $l10n;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
