<?php namespace Stage\Video\Compose\Campaign;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;

class Campaign
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $idCampaign;

    /**
     * @var
     */
    private $idComposed;

    /**
     * @var
     */
    private $rollIn;

    /**
     * @var
     */
    private $secondsIn;

    /**
     * @var
     */
    private $beginDate;

    /**
     * @var
     */
    private $endDate;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $hasAdVideos = false;

    /**
     * @var
     */
    private $hasExternalTag = false;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'idCampaign', 'idComposed', 'rollIn', 'secondsIn', 'hasAdVideos', 'hasExternalTag', 'beginDate', 'endDate', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getIdCampaign()
    {
        return $this->idCampaign;
    }

    public function getIdComposed()
    {
        return $this->idComposed;
    }

    public function getSecondsIn()
    {
        return $this->secondsIn;
    }

    public function getRollIn()
    {
        return $this->rollIn;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function hasAdVideos()
    {
        return $this->registerDate;
    }

    public function hasExternalTag()
    {
        return $this->hasExternalTag;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    private function setName($name)
    {
        $this->name = $name;
    }

    public function setIdCampaign($id)
    {
        $this->idCampaign = (int) $id;
    }

    public function setIdComposed($id)
    {
        $this->idComposed = (int) $id;
    }

    public function setBeginDate($date)
    {
        if ($date==null) {
            $this->beginDate = 0;
        } else {
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        if ($date==null) {
            $this->endDate = 0;
        } else {
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    public function setHasAdVideos($has)
    {
        $this->hasAdVideos = (bool) $has;
    }

    public function setHasExternalTag($has)
    {
        $this->hasExternalTag = (bool) $has;
    }

    public function setSecondsIn($in)
    {
        $this->secondsIn = $in;
    }

    public function setRollIn($in)
    {
        $this->rollIn = (int) $in;
    }

    /**
     * query
     *
     * Search Compose Campaigns by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $query)
    {
        try {
            $field = $query->filter;
            $filter = $query->by;
            $limit_a = $query->from;
            $limit_b = $query->to;

            $parameters = [ [$field, PDO::PARAM_INT],
                            [$filter, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "ADS_CAMPAIGNS_VIDEO_COMPOSED_RELATED";
            $method = "GET";
            $campaigns = Stage::esp($parameters, $name, $method);

            if (isset($campaigns[0]['return_code']) && $campaigns[0]['return_code'] == "-2") {
                return [];
            } else {
                return array_map(function ($campaign) {
                    return self::createInstance($campaign);
                }, $campaigns);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $idComposed
     * @param $field
     * @return array
     * @throws Exception
     */
    public static function removeComposed($idComposed, $field)
    {
        try {
            $name = "ADS_VIDEO_COMPOSED_CAMPAIGNS";
            $method = "DEL";

//            $field = 1; // prerrol
//            $field = 2; // midroll
//            $field = 3; // postroll
//            $field = 4; // resetpostroll

            $parameters = [ [$idComposed, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT]];

            return Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "beginDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } elseif ($attribute == "endDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($campaign)
    {
        $instance = new self();
        $instance->setId($campaign['id_related']);
        $instance->setName(html_entity_decode($campaign['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setIdCampaign($campaign['id_campaign']);
        $instance->setIdComposed($campaign['id_composed']);

        if (Helper::isValidDate($campaign['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($campaign['register_date']));
        }

        if (Helper::isValidDate($campaign['begin_date'], "Y-m-d H:i:s")) {
            $instance->setBeginDate(new DateTime($campaign['begin_date']));
        }

        if (Helper::isValidDate($campaign['end_date'], "Y-m-d H:i:s")) {
            $instance->setEndDate(new DateTime($campaign['end_date']));
        }

        $instance->setRollIn($campaign['roll_in']);
        $instance->setSecondsIn($campaign['seconds_in']);
        $instance->setHasAdVideos($campaign['has_ad_videos']);
        $instance->setHasExternalTag($campaign['has_external_tag']);

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
