<?php namespace Stage\Video\Compose;

use DOMDocument;
use Stage\Ad\Campaign\Campaign as AdCampaign;
use Stage\Post\MyArrayObject;
use Stage\Video\Compose\Campaign\Campaign as ComposeCampaign;
use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;
use Stage\Video\Channel\Channel;
use Stage\Video\Label\Label;
use Stage\Video\Playlist\Playlist;
use Stage\Video\Profile\Profile;
use Stage\Video\Sticker\Sticker;
use Stage\Video\Tag\Tag;
use Stage\Video\Compose\Campaign\Query as ComposeCompaignQuery;
use Stage\Video\Video;

class Compose
{

    /**
     * @var
     */
    private $id = -1;

    /**
     *
     * @var array
     */
    private $beginDate = "";

    /**
     *
     * @var
     */
    private $endDate = "";

    /**
     *
     * @var
     */
    private $publishDate = "";

    /**
     *
     * @var
     */
    private $unpublishDate = "";

    /**
     *
     * @var string
     */
    private $registerDate = "";

    /**
     * Its a Boolean
     * @var
     */
    private $thumbnailChoice = 0;

    /**
     * Its a boolean
     * @var
     */
    private $hasInsideSocial = false;

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $autoplay;

    /**
     * @var array
     */
    private $profile = [];

    /**
     * @var array
     */
    private $video = [];

    /**
     * @var bool
     */
    private $playInline = false;

    /**
     * @var bool
     */
    private $playMark = false;

    /**
     * @var bool
     */
    private $webvtt = false;

    /**
     * @var bool
     */
    private $type = 0;

    /**
     * @var
     */
    private $position = null;

    /**
     * @var
     */
    private $onSticker = false;

    /**
     *
     * @var string
     */
    private $color = "";

    /**
     *
     * @var string
     */
    private $osGraphicsImage = "";

    /**
     *
     * @var string
     */
    private $osGraphicsPosition = "";

    /**
     *
     * @var string
     */
    private $osGraphicsSize = "";

    /**
     *
     * @var string
     */
    private $osGraphicsPadding = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','beginDate','endDate','registerDate','thumbnailChoice','state','autoplay','hasInsideSocial','profile','video','playInline','playMark','webvtt','type','publishDate','unpublishDate','position', 'onSticker', 'color', 'osGraphicsImage', 'osGraphicsPosition', 'osGraphicsSize', 'osGraphicsPadding');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdVideo()
    {
        // return $this->idVideo;
        return $this->video['id'];
    }

    public function getIdProfile()
    {
        // return $this->idProfile;
        return $this->profile['id'];
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getPublishDate()
    {
        return $this->publishDate;
    }

    public function getUnPublishDate()
    {
        return $this->unpublishDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getThumbnailChoice()
    {
        return $this->thumbnailChoice;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getAutoplay()
    {
        return $this->autoplay;
    }

    public function playInline()
    {
        return $this->playInline;
    }

    public function playMark()
    {
        return $this->playMark;
    }

    public function webvtt()
    {
        return $this->webvtt;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getOnSticker()
    {
        return $this->onSticker;
    }

    public function color()
    {
        return $this->color;
    }

    public function osGraphicsImage()
    {
        return $this->osGraphicsImage;
    }

    public function osGraphicsPosition()
    {
        return $this->osGraphicsPosition;
    }

    public function osGraphicsSize()
    {
        return $this->osGraphicsSize;
    }

    public function osGraphicsPadding()
    {
        return $this->osGraphicsPadding;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdVideo($id)
    {
        //$this->idVideo = (int) $id;
        $this->video['id'] = (int) $id;
    }

    public function setIdProfile($id)
    {
        //$this->idProfile = (int) $id;
        $this->profile['id'] = (int) $id;
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->beginDate = "";
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setPublishDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->publishDate = "";
        } else {
            //$date = new DateTime($date);
            $this->publishDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUnPublishDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->unpublishDate = "";
        } else {
            //$date = new DateTime($date);
            $this->unpublishDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->endDate = "";
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    private function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setPlayInline($playInline)
    {
        $this->playInline = (bool)  $playInline;
    }

    public function setPlayMark($playMark)
    {
        $this->playMark = (bool) $playMark;
    }

    public function setWebvtt($webvtt)
    {
        $this->webvtt = (bool) $webvtt;
    }

    public function setType($type)
    {
        $this->type = (int) $type;
    }

    public function setPosition($position)
    {
        $this->position = $position;
    }

    public function setOnSticker($onSticker)
    {
        $this->onSticker = (bool) $onSticker;
    }

    public function setColor($color)
    {
        $this->color = $color;
    }

    public function setOsGraphicsImage($image)
    {
        $this->osGraphicsImage = $image;
    }

    public function setOsGraphicsPosition($position)
    {
        $this->osGraphicsPosition = $position;
    }

    public function setOsGraphicsSize($size)
    {
        $this->osGraphicsSize = $size;
    }

    public function setOsGraphicsPadding($padding)
    {
        $this->osGraphicsPadding = $padding;
    }

    /**
     * setThumbnail
     *
     * If $choice equal to 0, then is the video thumbnail, else is the profile thumbnail
     * @param $thumbnail
     */
    public function setThumbnailChoice($choice = 0)
    {
        $this->thumbnailChoice = (int) $choice;
    }

    public function hasInsideSocial($insideSocial = null)
    {
        if (!$insideSocial) {
            return $this->hasInsideSocial;
        } else {
            $this->hasInsideSocial = (bool) $insideSocial;
        }
    }

    public function setState($state)
    {
        $this->state = (int) $state;
    }

    public function setAutoplay($autoplay)
    {
        return $this->autoplay = (bool) $autoplay;
    }

    /**
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        $name = "VIDEO_COMPOSES_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * dates
     *
     * @return array
     */
    public static function dates($idQuery = 0)
    {
        $name = "VIDEO_COMPOSES_DATES";
        $method = "GET";
        $parameters = [[$idQuery, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * find
     *
     * Find a Compose
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "VIDEO_COMPOSED";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Compose id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Search Composes by Query query
     *
     * @return mixed
     * @throws Exception
     */
    public static function query(Query $q, $isPublish = false)
    {
        if ($isPublish === true && $q instanceof PublishQuery) {
            $idChannel = $q->idChannel;
            $idPlaylist = $q->idPlaylist;
            $field = $q->by;
            $query = $q->term;
            $filter = $q->filter;
            $limit_sys_a = $q->fromSize;
            $limit_sys_b = $q->toSize;
            $limit_a = $q->from;
            $limit_b = $q->to;
            $order = $q->order;
            $orderBy = $q->orderBy;

            $parameters = [ [$idChannel, PDO::PARAM_INT],
                [$idPlaylist, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_sys_a, PDO::PARAM_STR],
                [$limit_sys_b, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $method = "GET";
            $name = "PUBLISH_VIDEO_SEARCH";
            $data = Stage::esp($parameters, $name, $method);
            return $data;
        }

        if (isset($q->idPlaylist) && is_numeric($q->idPlaylist)) {
            $id = $q->idPlaylist;
            $name = "VIDEO_COMPOSES_ON_PLAYLISTS";

            $field = $q->by;
            $query = $q->term;
            $limit_a = $q->from;
            $limit_b = $q->to;
            $order = $q->order;
            $orderBy = $q->orderBy;

            $parameters = [ [$id, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];
        } else {
            $id = $q->idDirectory;
            $name = "VIDEO_COMPOSES";

            $field      = $q->by;
            $sndField   = $q->sndBy;
            $trdField   = $q->trdBy;
            $query      = $q->term;
            $filter     = $q->filter;
            $sndFilter  = $q->sndFilter;
            $trdFilter  = $q->trdFilter;
            $limit_sys_a= $q->fromSize;
            $limit_sys_b= $q->toSize;
            $iso_lang   = $q->isoLanguage;
            $limit_a    = $q->from;
            $limit_b    = $q->to;
            $order      = $q->order;
            $orderBy    = $q->orderBy;

            $parameters = [ [$id, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_STR],
                [$trdFilter, PDO::PARAM_STR],
                [$limit_sys_a, PDO::PARAM_STR],
                [$limit_sys_b, PDO::PARAM_STR],
                [$iso_lang, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];
        }

        try {
            $method = "GET";

            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($compose) {
                return self::createInstance([$compose]);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * queryCount
     *
     * @param \Stage\Video\Compose\Query $q
     * @return mixed
     * @throws Exception
     */
    public static function queryCount(Query $q, $isPublish = false)
    {
        if ($isPublish === true && $q instanceof PublishQuery) {
            $idChannel = $q->idChannel;
            $idPlaylist = $q->idPlaylist;
            $field = $q->by;
            $query = $q->term;
            $filter = $q->filter;
            $limit_sys_a = $q->from;
            $limit_sys_b = $q->to;

            $parameters = [ [$idChannel, PDO::PARAM_INT],
                [$idPlaylist, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_sys_a, PDO::PARAM_STR],
                [$limit_sys_b, PDO::PARAM_STR]];

            $method = "GET";
            $name = "PUBLISH_VIDEO_SEARCH_COUNT";
            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("Channel $idChannel ou Playlist $idPlaylist not found", $data[0]['return_code']);
            } else {
                return (int) $data[0]['total'];
            }
        }

        if (isset($q->idPlaylist) && is_numeric($q->idPlaylist)) {
            $name = "VIDEO_COMPOSES_ON_PLAYLISTS_COUNT";

            $id = $q->idPlaylist;
            $field = $q->by;
            $query = $q->term;

            $parameters = [ [$id, PDO::PARAM_INT],
                [$field, PDO::PARAM_STR],
                [$query, PDO::PARAM_INT]];
        } else {
            $name       = "VIDEO_COMPOSES_COUNT";
            $id         = $q->idDirectory;
            $field      = $q->by;
            $sndField   = $q->sndBy;
            $trdField   = $q->trdBy;
            $query      = $q->term;
            $filter     = $q->filter;
            $sndFilter  = $q->sndFilter;
            $trdFilter  = $q->trdFilter;
            $limit_sys_a    = $q->fromSize;
            $limit_sys_b    = $q->toSize;
            $iso_lang   = $q->isoLanguage;

            $parameters = [ [$id, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$sndField, PDO::PARAM_INT],
                [$trdField, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$sndFilter, PDO::PARAM_STR],
                [$trdFilter, PDO::PARAM_STR],
                [$limit_sys_a, PDO::PARAM_INT],
                [$limit_sys_b, PDO::PARAM_INT],
                [$iso_lang, PDO::PARAM_STR]];
        }

        try {
            $method = "GET";

            $data = Stage::esp($parameters, $name, $method);
            return (int) $data[0]['total'];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "VIDEO_COMPOSED";
            $method = "SET";
            $parameters = [ [$this->getId(), PDO::PARAM_INT],
                [$this->getIdVideo(), PDO::PARAM_INT],
                [$this->getIdProfile(), PDO::PARAM_INT],
                [$this->getPublishDate(), PDO::PARAM_STR],
                [$this->getUnPublishDate(), PDO::PARAM_STR],
                [$this->getBeginDate(), PDO::PARAM_STR],
                [$this->getEndDate(), PDO::PARAM_STR],
                [$this->getThumbnailChoice(), PDO::PARAM_STR],
                [$this->hasInsideSocial(), PDO::PARAM_STR],
                [$this->getAutoplay(), PDO::PARAM_INT],
                [$this->playInline(), PDO::PARAM_BOOL],
                [$this->playMark(), PDO::PARAM_BOOL],
                [$this->webvtt(), PDO::PARAM_BOOL],
                [$this->color(), PDO::PARAM_STR],
                [$this->osGraphicsImage(), PDO::PARAM_STR],
                [$this->osGraphicsPosition(), PDO::PARAM_STR],
                [$this->osGraphicsSize(), PDO::PARAM_STR],
                [$this->osGraphicsPadding(), PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]['id_composed']) && $data[0]['id_composed'] >= 0) {
                $this->setId($data[0]['id_composed']);

                if (Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }
            }
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * playlist
     *
     * @param Playlist|null $playlist
     * @return $this
     * @throws Exception
     */
    public function playlists(Playlist $playlist = null, $position = 0)
    {
        if ($playlist instanceof Playlist) {
            try {
                if ($position === null) {
                    $position = 0;
                }

                $name = "VIDEO_COMPOSED_TO_PLAYLISTS";
                $method = "SET";
                $parameters = [ [$playlist->getId(), PDO::PARAM_INT],
                                [$this->getId(), PDO::PARAM_INT],
                                [$position, PDO::PARAM_INT]];

                $data = Stage::esp($parameters, $name, $method);

                // Returns -3 in case of playlist already added
                /*if( isset($data[0]['return_code']) && isset($data[0]['return_code']) != "0")
                {
                    throw new Exception("Error trying adding compose id " . $this->getId() . " to playlist " . $playlist->getId(), $data[0]['return_code']);
                }*/

                return $this;
            } catch (Exception $e) {
                throw $e;
            }
        } else {
            try {
                $name = "VIDEO_COMPOSES_PLAYLISTS";
                $method = "GET";
                $parameters = [[$this->getId(), PDO::PARAM_INT]];

                $data = Stage::esp($parameters, $name, $method);

                return (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") ? [] : array_map(function ($playlist) {
                    return Playlist::createInstance($playlist);
                }, $data);
            } catch (Exception $e) {
                throw $e;
            }
        }
    }

    /**
     * removePlaylist
     *
     * @param Playlist $playlist
     * @return $this
     * @throws Exception
     */
    public function removePlaylist(Playlist $playlist)
    {
        try {
            $name = "VIDEO_COMPOSED_FROM_PLAYLIST";
            $method = "DEL";
            $parameters = [[$playlist->getId(), PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a compose
     */
    public function delete()
    {
        try {
            $name = "VIDEO_COMPOSED";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * allTagsRelatedWithComposes
     *
     * @return array
     */
    public static function allTagsRelatedWithComposes()
    {
        $name = "VIDEO_PROFILE_TAGS_RELATED";
        $method = "GET";

        $idQuery = 1;
        $parameters = [[-1, PDO::PARAM_INT],
            [$idQuery, PDO::PARAM_INT]];
        $tags = Stage::esp($parameters, $name, $method);

        if (isset($tags[0]['return_code'])) {
            return [];
        } else {
            $temp = [];
            foreach ($tags as $tag) {
                $temp[] = Tag::createInstance($tag);
            }

            return $temp;
        }
    }

    /**
     * occurredIn
     *
     * @param $idComposed
     * @return array
     */
    public static function occurredIn($idComposed)
    {
        $name = "VIDEO_PLAYLISTS_BY_COMPOSED";
        $method = "GET";

        $parameters = [[$idComposed, PDO::PARAM_INT]];
        $playlists = Stage::esp($parameters, $name, $method);

        $playlists = array_map(function ($playlist) {
            return Playlist::createInstance($playlist)->toArray();
        }, $playlists);

        for ($i = 0 ; $i <= sizeof($playlists) - 1 ; $i++) {

            // $name = "VIDEO_CHANNELS_BY_PLAYLIST";
            // $method = "GET";

            $idPlaylist = $playlists[$i]['id'];
            $p = Playlist::find($idPlaylist);

            // $parameters = [[$idPlaylist, PDO::PARAM_INT]];
            // $channels = Stage::esp($parameters, $name, $method);

            $playlists[$i]['channels'] = array_map(function ($channel) {
                return $channel->toArray();
            }, $p->channels());
        }

        return $playlists;
    }

    /**
     * iframe
     *
     * @param $src
     * @return DOMDocument
     */
    public static function iframe($src, $width = null, $height = null)
    {
        // Create a DOM iframe element
        $dom = new DOMDocument('1.0');

        // Create an iframe Dom element
        $iframe = $dom->createElement('iframe');
        $iframe->setAttribute('src', $src);
        $iframe->setAttribute('allowfullscreen', true);
        $iframe->setAttribute('webkitallowfullscreen', true);
        $iframe->setAttribute('mozallowfullscreen', true);
        $iframe->setAttribute('allow', 'autoplay; fullscreen');
        $iframe->setAttribute('scrolling', false);
        $iframe->setAttribute('marginwidth', '0');   // Obsolete since HTML5
        $iframe->setAttribute('marginheight', '0');  // Obsolete since HTML5
        $iframe->setAttribute('frameborder', '0');   // Obsolete since HTML5
        $iframe->setAttribute('vspace', '0');        // Obsolete since HTML5
        $iframe->setAttribute('hspace', '0');        // Obsolete since HTML5

        $iframe->setAttribute('width', isset($width) ? $width : '100%');
        $iframe->setAttribute('height', isset($height) ? $height : '100%');

        if (!$width && !$height) {
            $iframe->setAttribute('class', 'fluid');
        }

        $iframe->setAttribute('style', 'position: absolute; top: 0; left: 0; width: 100%; height: 100%;');
        $dom->appendChild($iframe);

        return $dom;
    }

    /**
     * addCampaign
     *
     * @param AdCampaign $campaign
     * @return ComposeCampaign
     * @throws Exception
     */
    public function addCampaign(AdCampaign $campaign, $rollIn, $secondsIn)
    {
        try {

            // Note: This is is writing on table ads_campaigns_video_composed_related
            // Note: If already exist the rollIn and secondsIn for a campaign, -3 will be returned
            $name = "ADS_CAMPAIGNS_VIDEO_COMPOSED";
            $method = "SET";
            $parameters = [[$campaign->getId(), PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT],
                [$rollIn, PDO::PARAM_INT],
                [$secondsIn, PDO::PARAM_INT]];

            $result = Stage::esp($parameters, $name, $method);

            if (isset($result[0]['id_related'])) {
                return ComposeCampaign::createInstance($result[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeCampaigns
     *
     * @param AdCampaign $campaign
     * @return bool
     * @throws Exception
     */
    public function removeCampaigns($field)
    {
        // field = 4 Apaga todas as camapnhas do compose
        // field = 3 Apaga so post roll campaign de um compose
        // field = 2 Apaga so middle roll campaign de um compose
        // field = 1 Apaga so pre roll campaign de um compose

        try {
            $name = "ADS_VIDEO_COMPOSED_CAMPAIGNS";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$field, PDO::PARAM_INT]];

            $result = Stage::esp($parameters, $name, $method);
            if (isset($result[0]['return_code']) && $result[0]['return_code'] == 0) {
                return true;
            } else {
                return false;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * campaigns
     *
     * @return array
     * @throws Exception
     */
    public function composedCampaigns(ComposeCompaignQuery $query)
    {
        try {
            return ComposeCampaign::query($query);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Label $label
     * @return Label
     * @throws Exception
     */
    public function addLabel(Label $label)
    {
        try {
            // -3 Label already exist
            $name = "VIDEO_COMPOSED_TO_LABELS";
            $method = "SET";
            $parameters = [[$label->getId(), PDO::PARAM_INT],
                            [$this->getId(), PDO::PARAM_INT]];

            $result = Stage::esp($parameters, $name, $method);

            if (!isset($result[0]['return_code'])) {
                return Label::createInstance($result[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function removeLabels()
    {
        try {
            $name = "VIDEO_COMPOSED_FROM_LABELS";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $result = Stage::esp($parameters, $name, $method);
            if (isset($result[0]['return_code']) && $result[0]['return_code'] == 0) {
                return true;
            } else {
                return false;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function labels()
    {
        try {
            $name = "VIDEO_COMPOSES_LABELS_RELATED";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == -2) {
                return [];
            } else {
                $temp = [];
                foreach ($data as $tag) {
                    $tagInstance = Label::createInstance($tag);
                    array_push($temp, $tagInstance);
                }

                return $temp;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachSticker
     *
     * @param $idSticker
     * @param $position
     * @param DateTime $beginDate
     * @param DateTime $endDate
     * @return bool|mixed
     * @throws Exception
     */
    public function attachSticker($idSticker, $position, DateTime $beginDate = null, DateTime $endDate = null, $color = "")
    {
        $idVideo = $this->getId();

        $beginDate = $beginDate instanceof DateTime ? $beginDate->format(DateTime::ATOM) : "";
        $endDate = $endDate instanceof DateTime ? $endDate->format(DateTime::ATOM) : "";

        // write on table video_stickers_related
        $name = "VIDEO_STICKER_RELATED";
        $method = "SET";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$idVideo, PDO::PARAM_INT],
            [$position, PDO::PARAM_INT],
            [$beginDate, PDO::PARAM_STR],
            [$endDate, PDO::PARAM_STR],
            [$color, PDO::PARAM_STR]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : $data[0]['return_code'];
    }

    /**
     * detachSticker
     *
     * Detach a Stickers from a Compose
     * @throws Exception
     */
    public function detachSticker($idSticker)
    {
        $name = "VIDEO_FROM_STICKERS";
        $method = "DEL";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$this->getId(), PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
    }

    /**
     * stickers
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function stickers()
    {
        $q = new \Stage\Video\Sticker\Query();
        $q->field = 1000;
        $q->query = "";
        $q->filter = $this->getId();
        $q->from = 0;
        $q->to = -1;

        return Sticker::composes($q);
    }

    /**
     * detachAllStickers
     *
     * Detach all Stickers from a Post
     * @throws Exception
     */
    public function detachAllStickers()
    {
        $status = false;
        $sticksers = $this->stickers();
        /** @var Sticker $stickser */
        foreach ($sticksers as $stickser) {
            $status = $this->detachSticker($stickser->getId());
        }
        return $status;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($this->$attribute != null) {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Compose
     */
    public static function createInstance($data)
    {
        if (isset($data[0])) {
            $data = $data[0];
        }

        if (isset($data['id_live']) && !empty($data['id_live'])) {
            return \Stage\Live\Compose\Compose::find($data['id_composed']);
        }

        $compose = new self();
        $compose->setId($data['id_composed']);

        if (isset($data['position'])) {
            $compose->setPosition($data['position']);
        }

        $compose->setIdVideo($data['id_video']);

        $compose->setThumbnailChoice($data['set_thumbnail']);
        $compose->hasInsideSocial($data['set_inside_social']);

        if (Helper::isValidDate($data['begin_date'], "Y-m-d H:i:s")) {
            $compose->setBeginDate(new DateTime($data['begin_date']));
        }

        if (Helper::isValidDate($data['end_date'], "Y-m-d H:i:s")) {
            $compose->setEndDate(new DateTime($data['end_date']));
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $compose->setRegisterDate(new DateTime($data['register_date']));
        }

        if (Helper::isValidDate($data['publish_date'], "Y-m-d H:i:s")) {
            $compose->setPublishDate(new DateTime($data['publish_date']));
        }

        if (Helper::isValidDate($data['unpublish_date'], "Y-m-d H:i:s")) {
            $compose->setUnPublishDate(new DateTime($data['unpublish_date']));
        }


        $profile = Profile::find($data['id_profile']);

        $video = Video::find($data['id_video'])->toArray();
        $compose->video = $video;
//        $compose->video['name'] = html_entity_decode($data['name'], ENT_COMPAT , 'UTF-8');
//        $compose->video['seconds'] = $data['seconds'];
//        $compose->video['thumbnail'] = $data['thumbnail'];

        //unset($profile['videos']);
        //unset($profile['playlists']);
        $compose->profile = ['l10n' => array_map(function ($l10n) {
            return [
                'iso' => $l10n->getIsoLanguage(),
                'image' => $l10n->getImage(),
                'title' => $l10n->getTitle(),
                'description' => $l10n->getDescription()
            ];
        }, $profile->getL10n())];
        $compose->setIdProfile($data['id_profile']);
//        $compose->profile['image'] = $data['image'];
//        $compose->profile['title'] = html_entity_decode($data['title'], ENT_COMPAT , 'UTF-8');
//        $compose->profile['description'] = html_entity_decode($data['description'], ENT_COMPAT , 'UTF-8');

        $compose->setAutoplay($data['set_autoplay']);
        $compose->setState($data['state']);
        $compose->setPlayInline($data['set_play_inline']);
        $compose->setPlayMark($data['set_play_mark']);
        $compose->setWebvtt($data['set_webvtt']);
        $compose->setType($data['type']);   // 0 - Type VOD, 1 - Type Live

        if (isset($data['color'])) {
            $compose->setColor($data['color']);
        }

        if (isset($data['os_graphics_image'])) {
            $compose->setOsGraphicsImage($data['os_graphics_image']);
        }

        if (isset($data['os_graphics_position'])) {
            $compose->setOsGraphicsPosition($data['os_graphics_position']);
        }

        if (isset($data['os_graphics_size'])) {
            $compose->setOsGraphicsSize($data['os_graphics_size']);
        }

        if (isset($data['os_graphics_padding'])) {
            $compose->setOsGraphicsPadding($data['os_graphics_padding']);
        }

        if (isset($data['on_sticker'])) {
            $compose->setOnSticker($data['on_sticker']);
        }

        return $compose;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
