<?php namespace Stage\Video\Compose\Security;

use PDO;
use Stage\Stage;
use Exception;

class Security
{

    /**
     * @var
     */
    private $idComposed;

    /**
     * @var
     */
    private $domains;

    /**
     * @var
     */
    private $exclude;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('idComposed','domains','exclude');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getIdComposed()
    {
        // return $this->idVideo;
        return (int) $this->idComposed;
    }

    public function getDomains()
    {
        return $this->domains;
    }

    public function getExclude()
    {
        return $this->exclude;
    }

    public function setIdComposed($idComposed)
    {
        $this->idComposed =  (int) $idComposed;
    }

    public function setDomains($domains)
    {
        $this->domains = $domains;
    }

    public function setExclude($exclude)
    {
        $this->exclude = $exclude;
    }

    /**
     * find
     *
     * Find a Security
     *
     * @return $this
     * @throws Exception
     */
    public static function find($idComposed)
    {
        try {
            $name = "VIDEO_COMPOSED_SECURITY";
            $method = "GET";
            $parameters = [[$idComposed, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Security for id composed $idComposed not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "VIDEO_COMPOSED_SECURITY";
            $method = "SET";
            $parameters = [ [$this->getIdComposed(), PDO::PARAM_INT],
                            [$this->getDomains(), PDO::PARAM_STR],
                            [$this->getExclude(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * countries
     *
     * @param $iso
     * @return array
     * @throws Exception
     */
    public static function countries($idComposed)
    {
        try {
            $name = "VIDEO_COMPOSED_SECURITY_COUNTRIES";
            $method = "GET";
            $parameters = [ [$idComposed, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return $data;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $iso
     * @return $this
     * @throws Exception
     */
    public function saveCountries($iso, $exclude = false)
    {
        try {
            $name = "VIDEO_COMPOSED_SECURITY_COUNTRIES";
            $method = "SET";
            $parameters = [ [$this->getIdComposed(), PDO::PARAM_INT],
                            [$iso, PDO::PARAM_STR],
                            [$exclude, PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * deleteCountries
     *
     * @return $this
     * @throws Exception
     */
    public function deleteCountries()
    {
        try {
            $name = "VIDEO_COMPOSED_SECURITY_COUNTRIES";
            $method = "DEL";
            $parameters = [[$this->getIdComposed(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Security
     */
    public static function createInstance($data)
    {
        $data = $data[0];

        $s = new self();
        $s->setIdComposed($data['id_composed']);
        $s->setDomains($data['domains']);
        $s->setExclude($data['exclude']);

        return $s;
    }

    public function __toString()
    {
        return "My id is: {$this->getIdComposed()}\n";
    }
}
