<?php namespace Stage\Video\Embed;

use DateTime;
use Stage\Helper;

class OptionVariant
{
    /**
     * @var
     */
    private $idOption;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $optionName;

    /**
     * @var
     */
    private $optionType;

    /**
     * @var
     */
    private $type;

    /**
     * @var
     */
    private $variant;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $isMandatory;

    /**
     * @var
     */
    private $defaultOptionValue;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('idOption', 'name', 'optionName', 'optionType', 'type', 'variant', 'position', 'isMandatory', 'defaultOptionValue', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');

    public function __construct()
    {
    }

    public function getIdOption()
    {
        return $this->idOption;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getOptionName()
    {
        return $this->optionName;
    }

    public function getOptionType()
    {
        return $this->optionType;
    }

    public function getType()
    {
        return $this->type;
    }

    public function getVariant()
    {
        return $this->variant;
    }

    public function getPosition()
    {
        return (int)$this->position;
    }

    public function isMandatory()
    {
        return (bool)$this->isMandatory;
    }

    public function defaultOptionValue()
    {
        return (bool)$this->defaultOptionValue;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setIdOption($id)
    {
        $this->idOption = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setOptionName($name)
    {
        $this->optionName = $name;
    }

    public function setOptionType($type)
    {
        $this->optionType = $type;
    }

    public function setType($val)
    {
        $this->type = $val;
    }

    public function setVariant($val)
    {
        $this->variant = $val;
    }

    public function setPosition($val)
    {
        $this->position = (int)$val;
    }

    public function setIsMandatory($val)
    {
        $this->isMandatory = (bool)$val;
    }

    public function setDefaultOptionValue($val)
    {
        $this->defaultOptionValue = (bool)$val;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        return $this->attributesToArray();
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "title" || $attribute == "description" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $asset
     */
    public static function createInstance($asset)
    {
        $assetInstance = new self();
        $assetInstance->setIdOption($asset['id_option']);
        $assetInstance->setName($asset['name']);
        $assetInstance->setOptionName($asset['option_name']);
        $assetInstance->setOptionType($asset['option_type']);
        $assetInstance->setType($asset['type']);
        $assetInstance->setVariant($asset['variant']);
        $assetInstance->setPosition($asset['position']);
        $assetInstance->setIsMandatory($asset['is_mandatory']);
        $assetInstance->setDefaultOptionValue($asset['default_option_value']);

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getIdOption()}\n";
    }
}
