<?php namespace Stage\Video\Embed;

use DateTime;
use Stage\Helper;

class Setting
{
    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $version;

    /**
     * @var
     */
    private $container;

    /**
     * @var
     */
    private $url;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $preview;

    /**
     * @var
     */
    private $registerDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'version', 'container', 'url', 'link', 'preview', 'registerDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getVersion()
    {
        return $this->version;
    }

    public function getMacroContainer()
    {
        return $this->container;
    }

    public function getMacroUrl()
    {
        return $this->url;
    }

    public function getMacroLink()
    {
        return $this->link;
    }

    public function getMacroPreview()
    {
        return $this->preview;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setVersion($id)
    {
        $this->version = (int)$id;
    }

    public function setMacroContainer($macroContainer)
    {
        $this->container = $macroContainer;
    }

    public function setMacroUrl($macroUrl)
    {
        $this->url = $macroUrl;
    }

    public function setMacroLink($macroLink)
    {
        $this->link = $macroLink;
    }

    public function setMacroPreview($macroPreview)
    {
        $this->preview = $macroPreview;
    }

    private function setRegisterDate($date)
    {
        $this->registerDate = $date;
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        return $this->attributesToArray();
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $attribute == "name" || $attribute == "title" || $attribute == "description" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $asset
     */
    public static function createInstance($asset)
    {
        $assetInstance = new self();
        $assetInstance->setId($asset['id']);
        $assetInstance->setVersion($asset['version']);
        $assetInstance->setMacroContainer($asset['container']);
        $assetInstance->setMacroUrl($asset['url']);
        $assetInstance->setMacroLink($asset['link']);
        $assetInstance->setMacroPreview($asset['preview']);

        if (isset($asset['register_date']) && Helper::isValidDate($asset['register_date'], "Y-m-d H:i:s")) {
            $assetInstance->setRegisterDate(new DateTime($asset['register_date']));
        }

        return $assetInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
