<?php namespace Stage\Video\Feed;

use DateTime;
use Stage\Helper;

class Option
{
    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $idFeed;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $optionName;

    /**
     * @var
     */
    private $macro;

    /**
     * @var
     */
    private $optionCombined;

    /**
     * @var
     */
    private $optionType;

    /**
     * @var
     */
    private $idOption;

    /**
     * @var
     */
    private $optionValue;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idFeed', 'name', 'optionName', 'macro', 'optionCombined', 'optionType', 'idOption', 'optionValue', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function __get($key)
    {
        return isset($this->$key) ? $this->$key : null;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdFeed()
    {
        return $this->idFeed;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getOptionName()
    {
        return $this->optionName;
    }

    public function getMacro()
    {
        return $this->macro;
    }

    public function getOptionCombined()
    {
        return $this->optionCombined;
    }

    public function getOptionType()
    {
        return $this->optionType;
    }

    public function getIdOption()
    {
        return $this->idOption;
    }

    public function getOptionValue()
    {
        return $this->optionValue;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdFeed($id)
    {
        $this->idFeed = (int)$id;
    }

    private function setName($name)
    {
        $this->name = $name;
    }

    private function setOptionName($name)
    {
        $this->optionName = $name;
    }

    private function setMacro($name)
    {
        $this->macro = $name;
    }

    private function setOptionCombined($combined)
    {
        $this->optionCombined = $combined;
    }

    private function setOptionType($type)
    {
        $this->optionType = $type;
    }

    private function setIdOption($idOption)
    {
        $this->idOption = (int)$idOption;
    }

    public function setOptionValue($optionValue)
    {
        $this->optionValue = $optionValue;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name" || $attribute == "optionName") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Option
     */
    public static function createInstance($data)
    {
        $instance = new self();
        if (isset($data['id'])) {
            $instance->setId($data['id']);
        }

        if (isset($data['id_feed'])) {
            $instance->setIdFeed($data['id_feed']);
        }

        if (isset($data['name'])) {
            $instance->setName($data['name']);
        }

        if (isset($data['option_name'])) {
            $instance->setOptionName($data['option_name']);
        }

        if (isset($data['macro'])) {
            $instance->setMacro($data['macro']);
        }

        if (isset($data['option_combined'])) {
            $instance->setOptionCombined($data['option_combined']);
        }

        if (isset($data['option_type'])) {
            $instance->setOptionType($data['option_type']);
        }

        $instance->setIdOption($data['id_option']);

        if (isset($data['option_value'])) {
            $instance->setOptionValue($data['option_value']);
        }

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }
        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
