<?php namespace Stage\Video\Feed\Schema;

use DateTime;
use Stage\Helper;

class Option
{
    /**
     * @var
     */
    private $id;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $macro;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'macro', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function __get($key)
    {
        return isset($this->$key) ? $this->$key : null;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getMacro()
    {
        return $this->macro;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    private function setName($name)
    {
        $this->name = $name;
    }

    public function setMacro($macro)
    {
        $this->macro = $macro;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Option
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id_option']);
        $instance->setName($data['name']);
        $instance->setMacro($data['macro']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }
        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
