<?php namespace Stage\Video\Message;

use DateTime;
use Exception;
use Stage\Helper;
use Stage\Post\MyArrayObject;

class CustomFieldKey
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $public = true;

    /**
     * @var
     */
    private $regex;

    /**
     * @var
     */
    private $fixedField;

    /**
     * @var
     */
    private $readOnly;

    /**
     * @var
     */
    private $defaultValue;

    /**
     * @var
     */
    private $isMandatory;

    /**
     * @var
     */
    private $autofill;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'nameStripped', 'public', 'regex', 'fixedField', 'readOnly', 'defaultValue', 'isMandatory', 'autofill', 'registerDate', 'updateDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getPublic()
    {
        return $this->public;
    }

    public function getRegex()
    {
        return $this->regex;
    }

    public function getFixedField()
    {
        return $this->fixedField;
    }

    public function getReadOnly()
    {
        return $this->readOnly;
    }

    public function getDefaultValue()
    {
        return $this->defaultValue;
    }

    public function getIsMandatory()
    {
        return $this->isMandatory;
    }

    public function getAutofill()
    {
        return $this->autofill;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setPublic($public)
    {
        $this->public = (bool)$public;
    }

    public function setRegex($regex)
    {
        $this->regex = $regex;
    }

    public function setFixedField($value)
    {
        $this->fixedField = $value;
    }

    public function setReadOnly($value)
    {
        $this->readOnly = (bool)$value;
    }

    public function setDefaultValue($value)
    {
        $this->defaultValue = $value;
    }

    public function setIsMandatory($value)
    {
        $this->isMandatory = (bool)$value;
    }

    public function setAutofill($value)
    {
        $this->autofill = (bool)$value;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * all
     *
     * Returns all custom field keys
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->field = Query::ALL;
        $q->query = "";
        $q->from = 0;
        $q->to = -1;

        return Message::variantKeys($q);
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name" || $attribute == "nameStripped") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } else if ($attribute == "registerDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else if ($attribute == "updateDate" && $this->$attribute instanceof DateTime) {
                $arr[$attribute] = $this->$attribute->format(DATE_ATOM);
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_key']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setPublic($tag['public']);
        $tagInstance->setRegex($tag['regex']);
        $tagInstance->setFixedField($tag['fixed_field']);
        $tagInstance->setReadOnly((bool)$tag['read_only']);
        $tagInstance->setDefaultValue($tag['default_value']);
        $tagInstance->setIsMandatory((bool)$tag['is_mandatory']);
        $tagInstance->setAutofill((bool)$tag['autofill']);

        if (isset($tag['register_date']) && Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        if (isset($tag['update_date']) && Helper::isValidDate($tag['update_date'], "Y-m-d H:i:s")) {
            $tagInstance->setUpdateDate(new DateTime($tag['update_date']));
        }

        return $tagInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
