<?php namespace Stage\Video\Message\Group;

use DateTime;
use Stage\Helper;

class CustomFieldVariant
{
    /**
     * @var
     */
    private $idVariant;

    /**
     * @var
     */
    private $idGroup;

    /**
     * @var
     */
    private $idKey;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $variantValue;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('idVariant', 'idGroup', 'idKey', 'name', 'nameStripped', 'variantValue', 'position', 'registerDate', 'updateDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getIdVariant()
    {
        return $this->idVariant;
    }

    public function getIdGroup()
    {
        return $this->idGroup;
    }

    public function getIdKey()
    {
        return $this->idKey;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getVariantValue()
    {
        return $this->variantValue;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setIdVariant($id)
    {
        $this->idVariant = (int)$id;
    }

    private function setIdGroup($id)
    {
        $this->idGroup = (int)$id;
    }

    private function setIdKey($id)
    {
        $this->idKey = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setVariantValue($value)
    {
        $this->variantValue = $value;
    }

    public function setPosition($pos)
    {
        $this->position = (int)$pos;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setIdVariant($tag['id_variant']);
        $tagInstance->setIdGroup($tag['id_group']);
        $tagInstance->setIdKey($tag['id_key']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setVariantValue(html_entity_decode($tag['variant_value'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setPosition($tag['position']);

        if (isset($tag['register_date']) && Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        if (isset($tag['update_date']) && Helper::isValidDate($tag['update_date'], "Y-m-d H:i:s")) {
            $tagInstance->setUpdateDate(new DateTime($tag['update_date']));
        }

        return $tagInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
