<?php namespace Stage\Video\Message\Group;

use PDO;
use DateTime;
use Exception;
use Stage\Post\MyArrayObject;
use Stage\Stage;
use Stage\Helper;

class Schema
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idGroup;

    /**
     * @var
     */
    private $isoLanguage;

    /**
     * @var
     */
    private $macroSchema;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idGroup', 'isoLanguage', 'macroSchema', 'registerDate', 'updateDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function __get($key)
    {
        return isset($this->$key) ? $this->$key : null;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdGroup()
    {
        return $this->idGroup;
    }

    public function getIsoLanguage()
    {
        return $this->isoLanguage;
    }

    public function getMacroSchema()
    {
        return $this->macroSchema;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    public function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdGroup($id)
    {
        $this->idGroup = (int)$id;
    }

    public function setIsoLanguage($iso)
    {
        $this->isoLanguage = $iso;
    }

    public function setMacroSchema($macro)
    {
        $this->macroSchema = $macro;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * find
     *
     * Find a Schema
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function find($id, $field = 0)
    {
        try {
            $name = "VIDEO_MESSAGES_GROUP_SCHEMAS";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT], [$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Schema id $id not exists.", $data[0]['return_code']);
            } else {
                return new MyArrayObject(array_map(function ($option) {
                    return self::createInstance($option);
                }, $data));
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a schema
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        // table: video_messages_groups_schema
        $parameters = [[$this->getId(), PDO::PARAM_INT],
            [$this->getIdGroup(), PDO::PARAM_STR],
            [$this->getIsoLanguage(), PDO::PARAM_STR],
            [$this->getMacroSchema(), PDO::PARAM_STR]];

        try {
            $name = "VIDEO_MESSAGES_GROUPS_SCHEMA";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['id_schema'])) {
                $this->setId($data[0]['id_schema']);
                if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
                return $this;
            } else {
                throw new Exception("Error saving Schema", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Schema
     *
     * @param $idGroup
     * @param $idSchema
     * @return bool
     * @throws Exception
     */
    public static function delete($idGroup, $idSchema)
    {
        try {
            $name = "VIDEO_MESSAGES_GROUP_SCHEMA";
            $method = "DEL";
            $parameters = [[$idGroup, PDO::PARAM_INT],
                [$idSchema, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            if ($attribute == "name") {
                $arr[$attribute] = html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8');
            } elseif ($attribute == "l10n") {
                foreach ($this->$attribute as $l10n) {
                    $arr[$attribute][] = $l10n->toArray();
                }
            } else {
                $arr[$attribute] = $this->$attribute;
            }
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Schema
     */
    public static function createInstance($data)
    {
        $instance = new self();
        $instance->setId($data['id_schema']);
        $instance->setIdGroup($data['id_group']);
        $instance->setIsoLanguage($data['iso_language']);
        $instance->setMacroSchema($data['macro_schema']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }
        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
