<?php namespace Stage\Video\Profile\L10n;

use PDO;
use Exception;
use Stage\Stage;
use Stage\Exception\StoredProcedure as StoredProcedureException;

class Body
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $body = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','body');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array('id', 'body');

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getBody()
    {
        return $this->body;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setBody($body)
    {
        $this->body = $body;
    }


    public static function find($idL10n)
    {
        $temp = [];
        $parameters = [ [$idL10n, PDO::PARAM_INT]];

        try {
            $name = "VIDEO_PROFILE_L10N_BODY";
            $method = "GET";
            $bodies = Stage::esp($parameters, $name, $method);

            if (isset($bodies[0]['return_code']) && isset($bodies[0]['return_code']) == "-2") {
                return $temp;
                //throw new Exception("The body for the L10n id $idL10n not exists", $bodies[0]['return_code']);
            }

            foreach ($bodies as $b) {
                $temp[] = self::createInstance($b);
            }
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }

        return $temp;
    }

    /**
     * @param $idL10n
     * @return $this
     * @throws Exception
     * @throws StoredProcedureException
     */
    public function save($idL10n)
    {
        $parameters = [ [$idL10n, PDO::PARAM_INT],
                        [$this->getBody(), PDO::PARAM_INT]];

        try {
            $name = "VIDEO_PROFILE_L10N_BODY";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);
            $this->setId($data[0]['id_body']);
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * @param $idL10n
     * @return $this
     * @throws Exception
     * @throws StoredProcedureException
     */
    public static function removeAll($idL10n)
    {
        $parameters = [ [$idL10n, PDO::PARAM_INT]];

        try {
            $name = "VIDEO_PROFILE_L10N_BODY";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Body
     */
    public static function createInstance($data)
    {
        $body = new self();
        $body->setId($data['id_body']);
        $body->setBody($data['body']);

        return $body;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
