<?php namespace Stage\Video\Profile\L10n;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Video\Profile\Metadata\Metadata;
use Stage\Video\Tag\Tag;
use Stage\Helper;
use Stage\Asset\Asset;
use Stage\Asset\Album\Album;
use Stage\Exception\StoredProcedure as StoredProcedureException;

class L10n
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idProfile;

    /**
     * @var
     */
    private $isoLanguage;

    /**
     * @var
     */
    private $title = "";

    /**
     * @var
     */
    private $description = "";

    /**
     * @var
     */
    private $state; // 5 - Online, 4 - Scheduled, 3 - Created

    /**
     * @var
     */
    private $image = "";

    /**
     * @var
     */
    private $updateDate = "";

    /**
     * @var
     */
    private $registerDate = "";

    /**
     * @var
     */
    private $metadata;

    /**
     * @var
     */
    private $body = [];

    /**
     * @var
     */
    private $tags = [];

    /**
     * @var Asset[]
     */
    private $assets = [];

    /**
     * @var Album[]
     */
    private $albums = [];

    /**
     * @var bool
     */
    private $updating = false;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','isoLanguage','title','description','image','registerDate', 'updateDate','body','tags','assets','albums');
    //protected $attributes = array('id','title','description','thumbnail','state','registerDate','metadata','body','tags','albums','assets','videos','playlists');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdProfile()
    {
        return $this->idProfile;
    }

    public function getTitle()
    {
        return $this->title;
    }

    public function getIsoLanguage()
    {
        return $this->isoLanguage;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getUpdateDate()
    {
        $this->updateDate;
    }

    public function getRegisterDate()
    {
        $this->registerDate;
    }

    public function getMetadata()
    {
        return $this->metadata;
    }

    public function getBody()
    {
        return $this->body;
    }

    public function getTags()
    {
        return $this->tags;
    }

    public function getAssets()
    {
        return $this->assets;
    }

    public function getAlbums()
    {
        return $this->albums;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdProfile($id)
    {
        $this->idProfile = $id;
    }

    public function setIsoLanguage($isoLanguage)
    {
        $this->isoLanguage = $isoLanguage;
    }

    public function setTitle($title)
    {
        $this->title = $title;
    }

    public function setDescription($description)
    {
        $this->description = $description;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    private function setState($state)
    {
        $this->state = (int) $state;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date==null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    public function addMetadata(Metadata $meta)
    {
        $this->metadata = $meta;
    }

    public function addBody(Body $body)
    {
        $this->body[] = $body;
    }

    public function addTag(Tag $tag)
    {
        $this->tags[] = $tag;
    }

    public function addAsset(Asset $asset)
    {
        $this->assets[] = $asset;
    }

    public function addAlbum(Album $album)
    {
        $this->albums[] = $album;
    }

    public function discardTags()
    {
        $this->tags = [];
    }

    public function discardAssets()
    {
        $this->assets = [];
    }

    public function discardAlbums()
    {
        $this->albums = [];
    }

    /**
     * @param $idProfile
     * @param $isoLanguage
     * @return L10n
     */
    public static function find($idProfile, $isoLanguage)
    {
        try {
            $name = "VIDEO_PROFILE_L10N";
            $method = "GET";
            $parameters = [ [$idProfile, PDO::PARAM_INT],
                            [$isoLanguage, PDO::PARAM_STR]];

            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The L10n with iso language $isoLanguage not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Delete a L10n
     *
     */
    public function delete()
    {
        $name = "VIDEO_PROFILE_L10N";
        $method = "DEL";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
        return $this;
    }

    /**
     * save
     *
     * Save a L10n
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $metadata = $this->getMetadata() instanceof Metadata ? $this->getMetadata() : new Metadata();
        //echo $this->getId()."|".$this->getIdProfile()."|".$this->getIsoLanguage()."##";
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
            [$this->getIdProfile(), PDO::PARAM_INT],
            [$this->getIsoLanguage(), PDO::PARAM_STR],
            [$this->getTitle(), PDO::PARAM_STR],
            [$this->getDescription(), PDO::PARAM_STR],
            [$this->getImage(), PDO::PARAM_STR],
            [$metadata->getTitle(),PDO::PARAM_STR],
            [$metadata->getDescription(),PDO::PARAM_STR],
            [$metadata->getImage(),PDO::PARAM_STR],
            [$metadata->getImageTitle(),PDO::PARAM_STR],
            [$metadata->getImageAlt(),PDO::PARAM_STR],
            [$metadata->getKeywords(),PDO::PARAM_STR]];

        try {
            $name = "VIDEO_PROFILE_L10N";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_l10n']);

            $bodies = $this->getBody();
            if (sizeof($bodies) > 0) {
                $clonedBodies = array_merge(array(), $bodies);

                $this->body = [];

                Body::removeAll($this->getId());    // Remove all body of the l10n in the database

                foreach ($clonedBodies as $body) {
                    if ($body->getId() == -1) {
                        $b = $body->save($this->getId());    // Save the bodies of the post
                        $this->addBody($b);
                    }
                }
            }

            // Save the tags of the post
            $tags = $this->getTags();
            $this->removeAllTags();
            foreach ($tags as $tag) {
                $this->associateTag($tag);
            }

            // Save the assets of the post
            $this->removeAllAssets();
            foreach ($this->getAssets() as $asset) {
                $this->associateAsset($asset);
            }

            // Save albums of the post
            $this->removeAllAlbums();
            foreach ($this->getAlbums() as $album) {
                $this->associateAlbum($album);
            }
        } catch (StoredProcedureException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    public function removeAllTags()
    {
        $name = "VIDEO_PROFILE_FROM_TAGS";
        $method = "DEL";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    public function removeAllAssets()
    {
        $name = "VIDEO_PROFILE_L10N_PHOTOS";
        $method = "DEL";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    public function removeAllAlbums()
    {
        $name = "VIDEO_PROFILE_L10N_ALBUMS";
        $method = "DEL";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        Stage::esp($parameters, $name, $method);
    }

    /**
     * @return array
     */
    public function getL10nTags()
    {
        $name = "VIDEO_PROFILE_TAGS_RELATED";
        $method = "GET";
        $idQuery = 0;
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
                        [$idQuery, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $tags = [];
        foreach ($data as $b) {
            if (isset($b['id_tag'])) {
                $tags[] = Tag::createInstance($b);
            }
        }

        return $tags;
    }

    /**
     * getL10nAssets
     *
     * @return array
     */
    public function getL10nAssets()
    {
        $name = "VIDEO_PROFILE_L10N_PHOTOS";
        $method = "GET";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        $assets = Stage::esp($parameters, $name, $method);

        return array_map(function ($asset) {
            return isset($asset['id_asset']) ? Asset::createInstance($asset) : [];
        }, $assets);
    }

    /**
     * getL10nAssets
     *
     * @return array
     */
    public function getL10nAlbums()
    {
        $name = "VIDEO_PROFILE_L10N_ALBUMS";
        $method = "GET";
        $parameters = [ [$this->getId(), PDO::PARAM_INT]];
        $albums = Stage::esp($parameters, $name, $method);

        return array_map(function ($album) {
            return isset($album['id_album']) ? Album::createInstance($album) : [];
        }, $albums);
    }

    /**
     * associateTag
     *
     * Associate a Tag to a L10n
     * @param Tag $tag
     */
    public function associateTag(Tag $tag)
    {
        // -3 Ja existe
        $name = "VIDEO_PROFILE_TO_TAGS";
        $method = "SET";
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
                        [$tag->getId(), PDO::PARAM_INT]];

        Stage::esp($parameters, $name, $method);
    }

    /**
     * associateAsset
     *
     * @param Asset $asset
     */
    public function associateAsset(Asset $asset)
    {
        $name = "VIDEO_PROFILE_L10N_PHOTOS";
        $method = "SET";
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
            [$asset->getId(), PDO::PARAM_INT]];

        Stage::esp($parameters, $name, $method);
    }

    /**
     * associateAlbum
     *
     * @param Asset $asset
     */
    public function associateAlbum(Album $album)
    {
        $name = "VIDEO_PROFILE_L10N_ALBUMS";
        $method = "SET";
        $parameters = [ [$this->getId(), PDO::PARAM_INT],
            [$album->getId(), PDO::PARAM_INT]];

        Stage::esp($parameters, $name, $method);
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        //$attributes['image'] = ($attributes['image'])->toArray();
        if ($this->getMetadata() instanceof Metadata) {
            $attributes['metadata'] = $this->getMetadata()->toArray();

            $attributes['body'] = [];
            foreach ($this->getBody() as $body) {
                $attributes['body'][] = $body->toArray(); //['id'=>$body->getId(),'body'=>$body->getBody()];
            }
        }

        $attributes['tags'] = [];
        foreach ($this->getL10nTags() as $tag) {
            $attributes['tags'][] = $tag->toArray();
        }

        $attributes['assets'] = [];
        foreach ($this->getAssets() as $asset) {
            $attributes['assets'][] = $asset->toArray();
        }

        $attributes['albums'] = [];
        foreach ($this->getAlbums() as $album) {
            $attributes['albums'][] = $album->toArray();
        }

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "title" || $attribute == "description" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return L10n
     */
    public static function createInstance($data)
    {
        $l10n = new self();
        $metadata = new Metadata();
        $l10n->updating = true;

        $l10n->setId($data['id_l10n']);
        $l10n->setIdProfile($data['id_profile']);
        $l10n->setTitle(html_entity_decode($data['title'], ENT_COMPAT, 'UTF-8'));
        $l10n->setDescription(html_entity_decode($data['description'], ENT_COMPAT, 'UTF-8'));
        $l10n->setIsoLanguage($data['iso_language']);
        $l10n->setImage($data['image']);
        $l10n->setState($data['state']);

        $metadata->setTitle(html_entity_decode($data['meta_title'], ENT_COMPAT, 'UTF-8'));
        $metadata->setDescription(html_entity_decode($data['meta_description'], ENT_COMPAT, 'UTF-8'));
        $metadata->setKeywords(html_entity_decode($data['meta_keywords'], ENT_COMPAT, 'UTF-8'));
        $metadata->setImage($data['meta_image']);
        $metadata->setImageTitle(html_entity_decode($data['image_title'], ENT_COMPAT, 'UTF-8'));
        $metadata->setImageAlt(html_entity_decode($data['image_alter_text'], ENT_COMPAT, 'UTF-8'));

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $l10n->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $l10n->setUpdateDate(new DateTime($data['update_date']));
        }

        // Add Metadata
        $l10n->addMetadata($metadata);

        // Add the Body
        $bodies = Body::find($data['id_l10n']);
        foreach ($bodies as $body) {
            $l10n->addBody($body);
        }

        // Add the Tag
        $tags = $l10n->getL10nTags();
        foreach ($tags as $tag) {
            $l10n->addTag($tag);
        }

        // Add photo
        $l10n->discardAssets();
        $assets = $l10n->getL10nAssets();
        foreach ($assets as $asset) {
            if ($asset instanceof Asset) {
                $l10n->addAsset($asset);
            }
        }

        // Add album
        $l10n->discardAlbums();
        $albums = $l10n->getL10nAlbums();
        foreach ($albums as $album) {
            if ($album instanceof Album) {
                $l10n->addAlbum($album);
            }
        }

        return $l10n;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
