<?php namespace Stage\Video;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class SneakPeek
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name = "";

    /**
     * @var
     */
    private $secondsIn;

    /**
     * Original name of the video
     * @var array
     */
    private $secondsOut;

    /**
     *
     * @var array
     */
    private $isDefault = false;

    /**
     * @var
     */
    private $registerDate = 0;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','isDefault','name','secondsIn','secondsOut','registerDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getSecondsIn()
    {
        return $this->secondsIn;
    }

    public function getSecondsOut()
    {
        return $this->secondsOut;
    }

    public function isDefault()
    {
        return (bool) $this->isDefault;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setSecondsIn($sec)
    {
        $this->secondsIn = $sec;
    }

    public function setSecondsOut($sec)
    {
        $this->secondsOut = $sec;
    }

    public function setIsDefault($is)
    {
        $this->isDefault = (bool) $is;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    /**
     * save
     *
     * @param $idVideo
     * @throws Exception
     * @return SneakPeek
     */
    public function save($idVideo)
    {
        try {
            $parameters = [[$idVideo, PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getSecondsIn(), PDO::PARAM_INT],
                [$this->getSecondsOut(), PDO::PARAM_INT],
                [$this->isDefault(), PDO::PARAM_BOOL]];

            $name = "VIDEO_SNEAK_PEEK";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            return self::createInstance($data[0]);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * @param $idVideo
     * @throws Exception
     */
    public static function find($idVideo)
    {
        try {
            $parameters = [[$idVideo, PDO::PARAM_INT]];
            $name = "VIDEO_SNEAK_PEEK";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($sp) {
                return self::createInstance($sp);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @param $idVideo
     * @return bool
     * @throws Exception
     */
    public static function delete($idVideo)
    {
        try {
            $parameters = [[$idVideo, PDO::PARAM_INT]];
            $name = "VIDEO_SNEAK_PEEK";
            $method = "DEL";
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : false;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return SneakPeek
     */
    public static function createInstance($data)
    {
        $video = new self();
        $video->setId($data['id_sneak_peek']);
        $video->setName($data['name']);
        $video->setSecondsIn($data['seconds_in']);
        $video->setSecondsOut($data['seconds_out']);
        $video->setIsDefault($data['is_default']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $video->setRegisterDate(new DateTime($data['register_date']));
        }

        return $video;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
