<?php namespace Stage\Video\Sticker;

use PDO;
use DateTime;
use Exception;
use Stage\Stage;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\MyArrayObject;

class CustomFieldKey
{
    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $public = true;

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $regex;

    /**
     * @var
     */
    private $fixedField;

    /**
     * @var
     */
    private $readOnly;

    /**
     * @var
     */
    private $fieldType;

    /**
     * @var
     */
    private $defaultValue;

    /**
     * @var
     */
    private $isMandatory;

    /**
     * @var
     */
    private $autofill;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'name', 'nameStripped', 'public', 'total', 'regex', 'fixedField', 'readOnly', 'fieldType', 'defaultValue', 'isMandatory', 'autofill', 'registerDate', 'updateDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getPublic()
    {
        return $this->public;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getRegex()
    {
        return $this->regex;
    }

    public function getFixedField()
    {
        return $this->fixedField;
    }

    public function getReadOnly()
    {
        return $this->readOnly;
    }

    public function getFieldType()
    {
        return $this->fieldType;
    }

    public function getDefaultValue()
    {
        return $this->defaultValue;
    }

    public function isMandatory()
    {
        return $this->isMandatory;
    }

    public function getAutofill()
    {
        return $this->autofill;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setPublic($public)
    {
        $this->public = (bool)$public;
    }

    public function setTotal($total)
    {
        $this->total = (int)$total > 0 ? $total : 0;
    }

    public function setRegex($regex)
    {
        $this->regex = $regex;
    }

    public function setFixedField($fixed)
    {
        $this->fixedField = (bool)$fixed;
    }

    public function setReadOnly($rd)
    {
        $this->readOnly = (bool)$rd;
    }

    public function setFieldType($type)
    {
        $this->fieldType = $type;
    }

    public function setDefaultValue($value)
    {
        $this->defaultValue = $value;
    }

    public function setIsMandatory($is)
    {
        $this->isMandatory = (bool)$is;
    }

    public function setAutofill($is)
    {
        $this->autofill = (bool)$is;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * find
     *
     * Find a Custom Key
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "VIDEO_STICKERS_CUSTOM_FIELD_KEY";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The CustomKey id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Dossiers
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public static function all()
    {
        $q = new CustomFieldKeyQuery();
        $q->field = CustomFieldKeyQuery::ALL;
        $q->query = "";
        $q->from = 0;
        $q->to = -1;

        return static::query($q);
    }

    /**
     * query
     *
     * Query
     *
     * @param CustomFieldKeyQuery $q
     * @param bool $count
     * @param int $page
     * @param int $perPage
     * @return MyArrayObject
     * @throws Exception
     */
    public static function query(CustomFieldKeyQuery $q, $count = true, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;

        try {
            if ($count) {
                $parameters = [[$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "VIDEO_STICKERS_CUSTOM_FIELDS_KEYS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "VIDEO_STICKERS_CUSTOM_FIELDS_KEYS";
            $data = Stage::esp($parameters, $name, $method);

            $result = new MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Custom Key
     *
     * @return $this
     * @throws Exception
     */
    public function save($ifExists = 0)
    {
        // table: video_stickers_custom_fields_keys
        try {
            $name = "VIDEO_STICKERS_CUSTOM_FIELDS_KEY";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->getPublic(), PDO::PARAM_BOOL],
                [$ifExists, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            $this->setId($data[0]['id_key']);
            return $this;

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a custom key
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "VIDEO_STICKERS_CUSTOM_FIELD_KEY";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new self();
        $tagInstance->setId($tag['id_key']);
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setPublic($tag['public']);
        $tagInstance->setTotal(isset($tag['total']) ? (int)$tag['total'] : 0);
        $tagInstance->setRegex($tag['regex']);
        $tagInstance->setFixedField($tag['fixed_field']);
        $tagInstance->setReadOnly($tag['read_only']);
        $tagInstance->setFieldType($tag['field_type']);
        $tagInstance->setDefaultValue($tag['default_value']);
        $tagInstance->setIsMandatory($tag['is_mandatory']);
        $tagInstance->setAutofill($tag['autofill']);

        if (isset($tag['register_date']) && Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        if (isset($tag['update_date']) && Helper::isValidDate($tag['update_date'], "Y-m-d H:i:s")) {
            $tagInstance->setUpdateDate(new DateTime($tag['update_date']));
        }

        return $tagInstance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
