<?php namespace Stage\Video\Sticker\Group;

use PDO;
use DateTime;
use Exception;
use Stage\Helper;
use Stage\Stage;
use Stage\Paginator;

class Group
{
    const TYPE_CONTENT = 0;
    const TYPE_MENU = 1;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $stickerType;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $token;

    /**
     * @var
     */
    private $metaUrl;

    /**
     * @var
     */
    private $priority;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $countriesEnumeration;

    /**
     * @var
     */
    private $excluded;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'stickerType', 'name', 'nameStripped', 'token', 'metaUrl', 'priority', 'active', 'total', 'countriesEnumeration', 'excluded', 'registerDate', 'updateDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getStickerType()
    {
        return $this->stickerType;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getMetaUrl()
    {
        return $this->metaUrl;
    }

    public function getPriority()
    {
        return $this->priority;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getCountriesEnumeration()
    {
        return $this->countriesEnumeration;
    }

    public function getExcluded()
    {
        return $this->excluded;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setStickerType($type)
    {
        $this->stickerType = (int)$type;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setMetaUrl($metaUrl)
    {
        $this->metaUrl = $metaUrl;
    }

    public function setPriority($priority)
    {
        $this->priority = (int)$priority;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setTotal($total)
    {
        $this->total = (int)$total;
    }

    public function setCountriesEnumeration($isos)
    {
        $this->countriesEnumeration = $isos;
    }

    public function setExcluded($excluded)
    {
        $this->excluded = (bool) $excluded;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * find
     *
     * Find a Group
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "VIDEO_STICKERS_GROUP";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Group id $id not exists.", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a Group
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "VIDEO_STICKERS_GROUPS";
            $method = "SET";
            $parameters = [[$this->getStickerType(), PDO::PARAM_INT],
                [$this->getId(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->getToken(), PDO::PARAM_STR],
                [$this->getMetaUrl(), PDO::PARAM_STR],
                [$this->getPriority(), PDO::PARAM_INT], // 0 - low, 1 - medium, 2 - high
                [$this->getActive(), PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_group'])) {
                $this->setId($data[0]['id_group']);
                if (isset($data[0]) && isset($data[0]['register_date']) && Helper::isValidDate($data[0]['register_date'], "Y-m-d H:i:s")) {
                    $this->setRegisterDate(new DateTime($data[0]['register_date']));
                }

                if (isset($data[0]) && isset($data[0]['update_date']) && Helper::isValidDate($data[0]['update_date'], "Y-m-d H:i:s")) {
                    $this->setUpdateDate(new DateTime($data[0]['update_date']));
                }
                return $this;
            } else {
                throw new Exception("Error saving Group", $data[0]['return_code']);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * countries
     *
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public function countries()
    {
        // table: video_stickers_groups_countries
        try {
            $name = "VIDEO_STICKERS_GROUP_COUNTRIES";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $countries = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($country) {
                return Country::createInstance($country);
            }, $countries));
            $result->setCount(sizeof($countries));
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachCountry
     *
     * @param $iso
     * @param $exclude
     * @return bool
     */
    public function attachCountry($iso, $exclude)
    {
        // table: video_stickers_groups_countries
        try {
            $name = "VIDEO_STICKERS_GROUPS_COUNTRIES";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$iso, PDO::PARAM_STR],
                [$exclude, PDO::PARAM_BOOL]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['id']);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachAllCountries
     *
     * Detach all Countries attached to this Group
     * @throws Exception
     */
    public function detachAllCountries()
    {
        $name = "VIDEO_STICKERS_GROUPS_COUNTRIES";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $result = Stage::esp($parameters, $name, $method);
        return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
    }

    /**
     * query
     *
     * Search Stickers
     *
     * @param Query $q
     * @param bool $count
     * @return \Stage\Post\MyArrayObject
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $type = $q->type;
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {

            if ($count == true) {
                $parameters = [
                    [$type, PDO::PARAM_INT],
                    [$field, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "VIDEO_STICKERS_GROUPS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$type, PDO::PARAM_INT],
                [$field, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $name = "VIDEO_STICKERS_GROUPS";

            $data = Stage::esp($parameters, $name, $method);
            $result = new \Stage\Post\MyArrayObject(array_map(function ($label) {
                return self::createInstance($label);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Group
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "VIDEO_STICKERS_GROUP";
            $method = "DEL";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * urlNum
     *
     * @param $idGroup
     * @param $stickerType
     * @param $metaUrl
     * @param $priority
     * @return int
     * @throws Exception
     */
    public static function urlNum($idGroup, $stickerType, $metaUrl, $priority)
    {
        try {
            $name = "VIDEO_STICKERS_GROUP_URL_NUM";
            $method = "GET";
            $parameters = [[$idGroup, PDO::PARAM_INT],
                [$stickerType, PDO::PARAM_INT],
                [$metaUrl, PDO::PARAM_STR],
                [$priority, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param int $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();

        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param array $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    /**
     * createInstance
     *
     * @param $tag
     */
    public static function createInstance($tag)
    {
        $instance = new self();
        $instance->setId($tag['id_group']);
        $instance->setStickerType($tag['sticker_type']); // 0 - content, 1 - menu
        $instance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $instance->setNameStripped(html_entity_decode($tag['name_stripped'], ENT_COMPAT, 'UTF-8'));
        $instance->setToken($tag['token']);
        $instance->setMetaUrl($tag['meta_url']);
        $instance->setPriority($tag['priority']);
        $instance->setActive($tag['active']);
        $instance->setTotal(isset($tag['total']) ? (int)$tag['total'] : 0);

        if (isset($tag['countries_grouped'])) {
            $instance->setCountriesEnumeration($tag['countries_grouped']);
        }

        $instance->setExcluded($tag['exclude']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        if (isset($data['update_date']) && Helper::isValidDate($data['update_date'], "Y-m-d H:i:s")) {
            $instance->setUpdateDate(new DateTime($data['update_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
