<?php namespace Stage\Video\Sticker;

use PDO;
use DateTime;
use Exception;
use Stage\Helper;
use Stage\Paginator;
use Stage\Post\MyArrayObject;
use Stage\Stage;

class Sticker
{
    // Possible values for the type of stickers
    const TYPE_VIDEO_COMPOSED = 1; // the items in the sticker is of type VideoComposed
    const TYPE_PLAYLISTS = 2; // the items in the sticker is of type Playlists
    const TYPE_CHANNELS = 3; // the items in the sticker is of type Channels
    const TYPE_RSS_FEED = 6;
    const TYPE_MENU = 100;

    // Possible values for the state of a sticker
    const STATE_SCHEDULE = 4;
    const STATE_ONLINE = 5;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idType;

    /**
     * @var
     */
    private $format;

    /**
     * @var
     */
    private $variant;

    /**
     * @var
     */
    private $idGroup = -1;

    /**
     * @var
     */
    private $stickerType;

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $nameStripped;

    /**
     * @var
     */
    private $token;

    /**
     * @var
     */
    private $maximumItems;

    /**
     * @var
     */
    private $numberOfRows;

    /**
     * @var
     */
    private $displayIn;

    /**
     * @var
     */
    private $idFormat;

    /**
     * @var
     */
    private $link;

    /**
     * @var
     */
    private $autoFeedEnabled;

    /**
     * @var
     */
    private $autoFeedSort;

    /**
     * @var
     */
    private $autoFeedSortSlug;

    /**
     * @var
     */
    private $autoFeedSourceType;

    /**
     * @var
     */
    private $color;

    /**
     * @var
     */
    private $active;

    /**
     * @var
     */
    private $position;

    /**
     * @var
     */
    private $total;

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $image;

    /**
     * @var
     */
    private $imageAlign;

    /**
     * @var
     */
    private $imageWidth;

    /**
     * @var
     */
    private $imageHeight;

    /**
     * @var
     */
    private $idAsset;

    /**
     * @var
     */
    private $registerDate;

    /**
     * @var
     */
    private $beginDate;

    /**
     * @var
     */
    private $endDate;

    /**
     * @var
     */
    private $updateDate;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id', 'idType', 'format', 'variant', 'idGroup', 'stickerType', 'name', 'nameStripped', 'token', 'image', 'imageAlign', 'imageWidth', 'imageHeight', 'idAsset', 'maximumItems', 'numberOfRows', 'displayIn', 'idFormat', 'link', 'autoFeedEnabled', 'autoFeedSort', 'autoFeedSortSlug', 'autoFeedSourceType', 'color', 'active', 'position', 'total', 'state', 'registerDate', 'updateDate', 'beginDate', 'endDate');

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = array('');

    /**
     * The attributes that should be visible in arrays.
     *
     * @var array
     */
    protected $visible = array();

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = array();


    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = array('id');    // The attributes that cannot be setted on the model


    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdType()
    {
        return $this->idType;
    }

    public function getFormat()
    {
        return $this->format;
    }

    public function getVariant()
    {
        return $this->variant;
    }

    public function getIdGroup()
    {
        return $this->idGroup;
    }

    public function getStickerType()
    {
        return $this->stickerType;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getNameStripped()
    {
        return $this->nameStripped;
    }

    public function getToken()
    {
        return $this->token;
    }

    public function getMaximumItems()
    {
        return $this->maximumItems;
    }

    public function getNumberOfRows()
    {
        return $this->numberOfRows;
    }

    public function getDisplayIn()
    {
        return $this->displayIn;
    }

    public function getIdFormat()
    {
        return $this->idFormat;
    }

    public function getLink()
    {
        return $this->link;
    }

    public function isAutoFeedEnabled()
    {
        return (bool)$this->autoFeedEnabled;
    }

    public function getAutoFeedSort()
    {
        return $this->autoFeedSort;
    }

    public function getAutoFeedSortSlug()
    {
        return $this->autoFeedSortSlug;
    }

    public function getAutoFeedSourceType()
    {
        return $this->autoFeedSourceType;
    }

    public function getColor()
    {
        return $this->color;
    }

    public function getActive()
    {
        return $this->active;
    }

    public function getPosition()
    {
        return $this->position;
    }

    public function getTotal()
    {
        return $this->total;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getImage()
    {
        return $this->image;
    }

    public function getImageAlign()
    {
        return $this->imageAlign;
    }

    public function getImageWidth()
    {
        return $this->imageWidth;
    }

    public function getImageHeight()
    {
        return $this->imageHeight;
    }

    public function getIdAsset()
    {
        return $this->idAsset;
    }

    public function getBeginDate()
    {
        return $this->beginDate;
    }

    public function getEndDate()
    {
        return $this->endDate;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function getUpdateDate()
    {
        return $this->updateDate;
    }

    private function setId($id)
    {
        $this->id = (int)$id;
    }

    public function setIdType($id)
    {
        $this->idType = (int)$id;
    }

    public function setFormat($format)
    {
        $this->format = $format;
    }

    public function setVariant($variant)
    {
        $this->variant = $variant;
    }

    public function setIdGroup($id)
    {
        $this->idGroup = (int)$id;
    }

    public function setStickerType($type)
    {
        $this->stickerType = (int)$type;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setNameStripped($name)
    {
        $this->nameStripped = $name;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setMaximumItems($total)
    {
        $this->maximumItems = $total;
    }

    public function setNumberOfRows($total)
    {
        $this->numberOfRows = $total;
    }

    public function setDisplayIn($in)
    {
        $this->displayIn = $in;
    }

    public function setIdFormat($idFormat)
    {
        $this->idFormat = $idFormat;
    }

    public function setLink($link)
    {
        $this->link = $link;
    }

    public function setAutoFeedEnabled($feed)
    {
        $this->autoFeedEnabled = (bool)$feed;
    }

    public function setAutoFeedSort($sort)
    {
        $this->autoFeedSort = (int)$sort;
    }

    public function setAutoFeedSortSlug($slug)
    {
        $this->autoFeedSortSlug = $slug;
    }

    public function setAutoFeedSourceType($type)
    {
        $this->autoFeedSourceType = $type;
    }

    public function setColor($color)
    {
        $this->color = $color;
    }

    public function setActive($active)
    {
        $this->active = (bool)$active;
    }

    public function setPosition($position)
    {
        $this->position = (int)$position;
    }

    public function setTotal($total)
    {
        $this->total = (int)$total;
    }

    public function setState($state)
    {
        $this->state = $state;
    }

    public function setImage($image)
    {
        $this->image = $image;
    }

    public function setImageAlign($imageAlign)
    {
        $this->imageAlign = $imageAlign;
    }

    public function setImageWidth($imageWidth)
    {
        $this->imageWidth = (int)$imageWidth;
    }

    public function setImageHeight($height)
    {
        $this->imageHeight = (int)$height;
    }

    public function setIdAsset($id)
    {
        $this->idAsset = (int)$id;
    }

    public function setRegisterDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->registerDate = "";
        } else {
            //$date = new DateTime($date);
            $this->registerDate = $date->format(DateTime::ATOM);
        }
    }

    public function setBeginDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->beginDate = "";
        } else {
            //$date = new DateTime($date);
            $this->beginDate = $date->format(DateTime::ATOM);
        }
    }

    public function setEndDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->endDate = "";
        } else {
            //$date = new DateTime($date);
            $this->endDate = $date->format(DateTime::ATOM);
        }
    }

    public function setUpdateDate($date)
    {
        // Define how is the date format
        if ($date == null) {
            $this->updateDate = "";
        } else {
            //$date = new DateTime($date);
            $this->updateDate = $date->format(DateTime::ATOM);
        }
    }

    /**
     * @return array
     */
    public static function sizeof()
    {
        $name = "VIDEO_STICKERS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * save
     *
     * Save a tag to the database and return a Tag instance
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        try {
            $name = "VIDEO_STICKER";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$this->getIdGroup(), PDO::PARAM_INT],
                [$this->getIdType(), PDO::PARAM_INT],
                [$this->getPosition(), PDO::PARAM_INT],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getNameStripped(), PDO::PARAM_STR],
                [$this->getToken(), PDO::PARAM_STR],
                [$this->getImage(), PDO::PARAM_STR],
                [$this->getImageAlign(), PDO::PARAM_STR],
                [$this->getMaximumItems(), PDO::PARAM_INT],
                [$this->getNumberOfRows(), PDO::PARAM_INT],
                [$this->getDisplayIn(), PDO::PARAM_STR],    // all 1 | tv 2 | desktop 3 | tablet 4 | mobile 5
                [$this->getIdFormat(), PDO::PARAM_INT],
                [$this->getLink(), PDO::PARAM_STR],
                [$this->isAutoFeedEnabled(), PDO::PARAM_INT],
                [$this->getAutoFeedSort(), PDO::PARAM_STR],
                [$this->getAutoFeedSourceType(), PDO::PARAM_INT],
                [$this->getColor(), PDO::PARAM_STR],
                [$this->getActive(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);

            $this->setId($data[0]['id_sticker']);

        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * delete
     *
     * Delete a Sticker
     *
     * @return bool
     * @throws Exception
     */
    public function delete()
    {
        try {
            $name = "VIDEO_STICKER";
            $method = "DEL";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == "0";
        } catch (Exception $e) {
            throw $e;
        };
    }

    /**
     * @return $this
     * @throws Exception
     */
    public function deleteRelated()
    {
        try {
            $name = "VIDEO_STICKERS_RELATED";
            $method = "DEL";
            $parameters = [ [$this->getId(), PDO::PARAM_INT]];
            Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * find
     *
     * Finds a Label
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id)
    {
        try {
            $name = "VIDEO_STICKER";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Sticker id $id not exists.", $data[0]['return_code']);
            } else {
                return static::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * all
     *
     * Returns all Tags
     *
     * @return array
     * @throws Exception
     */
    public static function all()
    {
        $q = new Query();
        $q->by = Query::ALL;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        return static::query($q);
    }

    /**
     * query
     *
     * Search Stickers
     *
     * @return array
     * @throws Exception
     */
    public static function query(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        $method = "GET";
        $field = $q->field;
        $idGroup = $q->idGroup;
        $query = $q->query;
        $filter = $q->filter;
        $order = $q->order;
        $orderBy = $q->orderBy;

        try {

            if ($count == true) {
                $parameters = [[$field, PDO::PARAM_INT],
                    [$idGroup, PDO::PARAM_INT],
                    [$query, PDO::PARAM_STR],
                    [$filter, PDO::PARAM_STR]];

                $name = "VIDEO_STICKERS_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$field, PDO::PARAM_INT],
                [$idGroup, PDO::PARAM_INT],
                [$query, PDO::PARAM_STR],
                [$filter, PDO::PARAM_STR],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT],
                [$order, PDO::PARAM_INT],
                [$orderBy, PDO::PARAM_INT]];

            $name = "VIDEO_STICKERS";

            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($sticker) {
                return self::createInstance($sticker);
            }, $data));

            if (isset($p)) {
                $result->setCount($total);
                $result->setPaginator($p);
            }
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * composes
     *
     * @param Query $q
     * @param bool $count
     * @return MyArrayObject
     * @throws Exception
     */
    public static function composes(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        // table: video_composed_stickers_related
        // retorna videos com a position
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $method = "GET";
            if ($count == true) {
                $parameters = [[$q->field, PDO::PARAM_INT],
                    [$q->sndField, PDO::PARAM_INT],
                    [$q->trdField, PDO::PARAM_INT],
                    [$q->query, PDO::PARAM_STR],
                    [$q->filter, PDO::PARAM_STR],
                    [$q->sndFilter, PDO::PARAM_STR],
                    [$q->trdFilter, PDO::PARAM_STR]];

                $name = "VIDEO_STICKERS_RELATED_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$q->field, PDO::PARAM_INT],
                [$q->sndField, PDO::PARAM_INT],
                [$q->trdField, PDO::PARAM_INT],
                [$q->query, PDO::PARAM_STR],
                [$q->filter, PDO::PARAM_STR],
                [$q->sndFilter, PDO::PARAM_STR],
                [$q->trdFilter, PDO::PARAM_STR],
                [$q->order, PDO::PARAM_INT],
                [$q->orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "VIDEO_STICKERS_RELATED";

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_sticker'])) {
                $result = new MyArrayObject(array_map(function ($post) {
                    return Compose::createInstance($post);
                }, $data));

                if (isset($p)) {
                    $result->setCount($total);
                    $result->setPaginator($p);
                } else {
                    $result->setCount(sizeof($data));
                }

                return $result;
            } else {
                $result = new MyArrayObject([]);
                if (isset($p)) {
                    $result->setCount($total);
                    $result->setPaginator($p);
                }
                return $result;
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * playlists
     *
     * @param Query $q
     * @param bool $count
     * @return MyArrayObject
     * @throws Exception
     */
    public static function playlists(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        // table: playlists_stickers_related
        // retorna videos com a position
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $method = "GET";
            if ($count == true) {
                $parameters = [[$q->field, PDO::PARAM_INT],
                    [$q->sndField, PDO::PARAM_INT],
                    [$q->trdField, PDO::PARAM_INT],
                    [$q->query, PDO::PARAM_STR],
                    [$q->filter, PDO::PARAM_STR],
                    [$q->sndFilter, PDO::PARAM_STR],
                    [$q->trdFilter, PDO::PARAM_STR]];

                $name = "VIDEO_PLAYLISTS_STICKERS_RELATED_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$q->field, PDO::PARAM_INT],
                [$q->sndField, PDO::PARAM_INT],
                [$q->trdField, PDO::PARAM_INT],
                [$q->query, PDO::PARAM_STR],
                [$q->filter, PDO::PARAM_STR],
                [$q->sndFilter, PDO::PARAM_STR],
                [$q->trdFilter, PDO::PARAM_STR],
                [$q->order, PDO::PARAM_INT],
                [$q->orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "VIDEO_PLAYLISTS_STICKERS_RELATED";

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_sticker'])) {
                $result = new MyArrayObject(array_map(function ($post) {
                    return Playlist::createInstance($post);
                }, $data));

                if (isset($p)) {
                    $result->setCount($total);
                    $result->setPaginator($p);
                } else {
                    $result->setCount(sizeof($data));
                }

                return $result;
            } else {
                $result = new MyArrayObject([]);
                if (isset($p)) {
                    $result->setCount($total);
                    $result->setPaginator($p);
                }
                return $result;
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * channels
     *
     * @param Query $q
     * @param bool $count
     * @return MyArrayObject
     * @throws Exception
     */
    public static function channels(Query $q, $count = false, $page = 1, $perPage = 15)
    {
        // table: channels_stickers_related
        // retorna videos com a position
        $field = $q->field;
        $query = $q->query;
        $filter = $q->filter;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $method = "GET";
            if ($count == true) {
                $parameters = [[$q->field, PDO::PARAM_INT],
                    [$q->sndField, PDO::PARAM_INT],
                    [$q->trdField, PDO::PARAM_INT],
                    [$q->query, PDO::PARAM_STR],
                    [$q->filter, PDO::PARAM_STR],
                    [$q->sndFilter, PDO::PARAM_STR],
                    [$q->trdFilter, PDO::PARAM_STR]];

                $name = "VIDEO_CHANNELS_STICKERS_RELATED_COUNT";
                $data = Stage::esp($parameters, $name, $method);
                $total = isset($data[0]) && $data[0]['total'] ? (int)$data[0]['total'] : 0;

                // Create a new Paginator
                $p = new Paginator($total, $perPage);
                $p->page($page);
                $q->from = $p->from();
                $q->to = $p->to();
            }

            $limit_a = isset($p) ? $p->from() : $q->from;
            $limit_b = isset($p) ? $p->to() : $q->to;

            $parameters = [[$q->field, PDO::PARAM_INT],
                [$q->sndField, PDO::PARAM_INT],
                [$q->trdField, PDO::PARAM_INT],
                [$q->query, PDO::PARAM_STR],
                [$q->filter, PDO::PARAM_STR],
                [$q->sndFilter, PDO::PARAM_STR],
                [$q->trdFilter, PDO::PARAM_STR],
                [$q->order, PDO::PARAM_INT],
                [$q->orderBy, PDO::PARAM_INT],
                [$limit_a, PDO::PARAM_INT],
                [$limit_b, PDO::PARAM_INT]];

            $name = "VIDEO_CHANNELS_STICKERS_RELATED";

            $data = Stage::esp($parameters, $name, $method);
            if (isset($data[0]) && isset($data[0]['id_sticker'])) {
                $result = new MyArrayObject(array_map(function ($post) {
                    return Channel::createInstance($post);
                }, $data));

                if (isset($p)) {
                    $result->setCount($total);
                    $result->setPaginator($p);
                } else {
                    $result->setCount(sizeof($data));
                }

                return $result;
            } else {
                $result = new MyArrayObject([]);
                if (isset($p)) {
                    $result->setCount($total);
                    $result->setPaginator($p);
                }
                return $result;
            }

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $url
     * @return array
     * @throws Exception
     */
    public static function slugExists($id, $url)
    {
        try {
            $name = "VIDEO_STICKERS_NAME_NUM";
            $method = "GET";
            $parameters = [[$id, PDO::PARAM_INT],
                           [$url, PDO::PARAM_STR]];
            return Stage::esp($parameters, $name, $method);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * attachCustomFieldVariant
     *
     * @param $idKey // is the id_key from the table video_stickers_custom_fields_keys
     * @param $variantValue
     * @param $idPosition
     * @return CustomFieldVariant
     * @throws Exception
     */
    public function attachCustomFieldVariant($idKey, $variantValue, $idPosition)
    {
        // table: video_stickers_custom_fields_keys
        $idSticker = $this->getId();
        $spname = "VIDEO_STICKERS_CUSTOM_FIELDS_VARIANTS";
        $method = "SET";
        $parameters = [[$idSticker, PDO::PARAM_INT],
            [$idKey, PDO::PARAM_INT],
            [$variantValue, PDO::PARAM_STR],
            [$idPosition, PDO::PARAM_STR]];

        try {
            $data = Stage::esp($parameters, $spname, $method);
            if ((isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") || !isset($data[0])) {
                throw new Exception("The Key not exists", $data[0]['return_code']);
            } else {
                return CustomFieldVariant::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachCustomFieldsVariant
     *
     * @param $idKey
     * @return bool
     * @throws Exception
     */
    public function detachCustomFieldsVariant($idKey)
    {
        // table: video_stickers_custom_fields_keys
        $idSticker = $this->getId();
        $name = "VIDEO_STICKERS_CUSTOM_FIELD_VARIANT";
        $method = "DEL";
        $parameters = [[$idSticker, PDO::PARAM_INT], [$idKey, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
    }

    /**
     * customFieldsVariants
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function customFieldsVariants()
    {
        $idSticker = $this->getId();
        $name = "VIDEO_STICKERS_CUSTOM_FIELDS_VARIANTS_RELATED";
        $method = "GET";
        $parameters = [[$idSticker, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);

        $result = new MyArrayObject(array_filter(array_map(function ($customField) {
            return isset($customField['id_variant']) ? CustomFieldVariant::createInstance($customField) : [];
        }, $data), function ($item) {
            return $item;
        }));
        $result->setCount($result->count());
        return $result;
    }

    /**
     * formats
     *
     * @param $field
     * @return MyArrayObject
     * @throws Exception
     */
    public static function formats($field)
    {
        // table: video_stickers_formats
        try {
            $name = "VIDEO_STICKERS_FORMATS";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($format) {
                return Format::createInstance($format);
            }, $data));
            $result->setCount(sizeof($data));
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * types
     *
     * @param $field
     * @return MyArrayObject
     * @throws Exception
     */
    public static function types($field)
    {
        // table: video_stickers_types
        try {
            $name = "VIDEO_STICKERS_TYPES";
            $method = "GET";
            $parameters = [[$field, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($format) {
                return Type::createInstance($format);
            }, $data));
            $result->setCount(sizeof($data));
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * feedsSources
     *
     * @return MyArrayObject
     * @throws Exception
     */
    public function feedsSources()
    {
        // table: video_stickers_feeds_sources
        try {
            $name = "VIDEO_STICKER_FEED_SOURCES";
            $method = "GET";
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            $result = new MyArrayObject(array_map(function ($format) {
                return SourceType::createInstance($format);
            }, $data));
            $result->setCount(sizeof($data));
            return $result;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * detachAllFeedsSources
     *
     * @return bool
     * @throws Exception
     */
    public function detachAllFeedsSources()
    {
        $name = "VIDEO_STICKERS_FEED_SOURCES";
        $method = "DEL";
        $parameters = [[$this->getId(), PDO::PARAM_INT]];
        $result = Stage::esp($parameters, $name, $method);
        return isset($result[0]) && isset($result[0]['return_code']) && $result[0]['return_code'] == "0";
    }

    /**
     * attachFeedSource
     *
     * @param $type
     * @param $id
     * @return bool
     * @throws Exception
     */
    public function attachFeedSource($type, $id)
    {
        try {
            $name = "VIDEO_STICKER_FEED_SOURCE";
            $method = "SET";
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$type, PDO::PARAM_INT],
                [$id, PDO::PARAM_INT]];

            $data = Stage::esp($parameters, $name, $method);
            return isset($data[0]) && isset($data[0]['return_code']) && $data[0]['return_code'] == 0;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->getArrayableAttributes();
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" || $attribute == "nameStripped" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * Get an attribute array of all arrayable attributes.
     *
     * @return array
     */
    protected function getArrayableAttributes()
    {
        return $this->getArrayableItems($this->attributes);
    }

    /**
     * Get an attribute array of all arrayable values.
     *
     * @param  array  $values
     * @return array
     */
    protected function getArrayableItems(array $values)
    {
        if (count($this->visible) > 0) {
            return array_intersect_key($values, array_flip($this->visible));
        }

        /*
        print_r(array_flip($this->hidden));
        print_r(array_flip($values));
        print_r(array_diff_key(array_flip($values),array_flip($this->hidden)));
        */
        return array_flip(array_diff_key(array_flip($values), array_flip($this->hidden)));
    }

    public function __clone()
    {
        trigger_error('Clone is not allowed.', E_USER_ERROR);
    }

    /**
     * createInstance
     *
     * @param $data
     */
    public static function createInstance($tag)
    {
        $tagInstance = new static();
        $tagInstance->setId($tag['id_sticker']);
        $tagInstance->setIdType($tag['id_type']); // self::TYPE_POSTS, self::TYPE_DOSSIES, ...
        $tagInstance->setFormat($tag['format']);
        $tagInstance->setVariant($tag['variant']);
        $tagInstance->setIdGroup($tag['id_group']); // Its the idGroup of this sticker
        $tagInstance->setStickerType($tag['sticker_type']); // Its the sticker group type of which this sticker in inserted in 0 - content, 1 - menu
        $tagInstance->setName(html_entity_decode($tag['name'], ENT_COMPAT, 'UTF-8'));
        $tagInstance->setNameStripped($tag['name_stripped']);
        $tagInstance->setToken($tag['token']);
        $tagInstance->setImage(isset($tag['image']) ? $tag['image'] : "");
        $tagInstance->setImageAlign(isset($tag['image_align']) ? $tag['image_align'] : "");
        $tagInstance->setImageWidth($tag['image_width']);
        $tagInstance->setImageHeight($tag['image_height']);
        $tagInstance->setIdAsset($tag['id_asset']);
        $tagInstance->setMaximumItems($tag['maximum_items']);
        $tagInstance->setNumberOfRows($tag['number_of_rows']);
        $tagInstance->setDisplayIn($tag['display_in']);
        $tagInstance->setIdFormat($tag['id_format']);
        $tagInstance->setLink($tag['link']);
        $tagInstance->setAutoFeedEnabled($tag['auto_feed_enabled']);
        $tagInstance->setAutoFeedSort($tag['auto_feed_sort']); // 1 - publishedAt, 2 - MostViewed
        $tagInstance->setAutoFeedSortSlug($tag['auto_feed_sort_slug']); // publishedAt, MostViewed
        $tagInstance->setAutoFeedSourceType($tag['auto_feed_source_type']); // 2 Dossiers | 3 Collections | 5 Authors | 10 Posts Tags | 20 Dossiers Tags | 30 Collections Tags
        $tagInstance->setColor($tag['color']);
        $tagInstance->setActive($tag['active']);
        $tagInstance->setPosition(isset($tag['position']) ? (int)$tag['position'] : 0);
        $tagInstance->setTotal(isset($tag['total']) ? (int)$tag['total'] : 0);
        $tagInstance->setState($tag['state']);

        if (isset($tag['register_date']) && Helper::isValidDate($tag['register_date'], "Y-m-d H:i:s")) {
            $tagInstance->setRegisterDate(new DateTime($tag['register_date']));
        }

        if (isset($tag['update_date']) && Helper::isValidDate($tag['update_date'], "Y-m-d H:i:s")) {
            $tagInstance->setUpdateDate(new DateTime($tag['update_date']));
        }

        return $tagInstance;
    }
}
