<?php namespace Stage\Video\Thumbnail;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class Thumbnail
{

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idVideo;

    /**
     * Filename of the thumbnail
     * @var array
     */
    private $filename;

    /**
     * Bool
     * @var array
     */
    private $isDefault;

    /**
     * @var
     */
    private $seconds = 0;

    /**
     * @var string
     */
    private $registerDate = "";

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','idVideo','filename','isDefault','seconds','registerDate');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdVideo()
    {
        return $this->idVideo;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function isDefault($default = null)
    {
        if (is_bool($default)) {
            $this->isDefault = $default;
        }

        return $this->isDefault;
    }

    public function getSeconds()
    {
        return $this->seconds;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdVideo($id)
    {
        $this->idVideo = (int) $id;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setSeconds($seconds)
    {
        $this->seconds = $seconds;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    /**
     * find
     *
     * @param $id
     * @return mixed
     * @throws Exception
     */
    public static function find($id)
    {
        $query = $id;
        $field = 3;
        $limit_a = 0;
        $limit_b = -1;

        try {
            $method = "GET";
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "VIDEO_THUMBNAILS";
            $data = Stage::esp($parameters, $name, $method);

            if (!sizeof($data)) {
                throw new Exception("The id thumbnail " . $id . " was not found.", -1);
            }

            return array_map(function ($thumbnail) {
                return self::createInstance($thumbnail);
            }, $data)[0];
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * query
     *
     * Query Thumbnail's
     *
     * @return $this
     * @throws Exception
     */
    public static function query(Query $q)
    {
        $query   = $q->term;
        $field   = $q->by;
        $limit_a = $q->from;
        $limit_b = $q->to;

        try {
            $method = "GET";
            $parameters = [ [$field, PDO::PARAM_INT],
                            [$query, PDO::PARAM_INT],
                            [$limit_a, PDO::PARAM_INT],
                            [$limit_b, PDO::PARAM_INT]];

            $name = "VIDEO_THUMBNAILS";
            $data = Stage::esp($parameters, $name, $method);

            return array_map(function ($thumbnail) {
                return self::createInstance($thumbnail);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$this->getIdVideo(), PDO::PARAM_INT],
                       [$this->getFilename(), PDO::PARAM_STR],
                       [$this->isDefault(), PDO::PARAM_BOOL],
                       [$this->getSeconds(), PDO::PARAM_INT]];

        try {
            $name = "VIDEO_THUMBNAIL";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && ($data[0]['return_code']) == "-2") {
                throw new Exception("The id video " . $this->getIdVideo() . " was not found.", $data[0]['return_code']);
            } elseif (isset($data[0]['id_thumbnail']) && $data[0]['id_thumbnail'] >= 0) {
                $idThumbnail = $data[0]['id_thumbnail'];
                $this->setId($idThumbnail);
            }

            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Thumbnail
     */
    public static function createInstance($data)
    {
        $thumbnail = new self();
        $thumbnail->setId($data['id_thumbnail']);
        $thumbnail->setIdVideo($data['id_video']);
        $thumbnail->setFilename($data['thumbnail']);
        $thumbnail->isDefault((bool)$data['is_default']);
        $thumbnail->setSeconds($data['seconds']);

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $thumbnail->setRegisterDate(new DateTime($data['register_date']));
        }

        return $thumbnail;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
