<?php namespace Stage\Video;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;

class Track
{
    const SUBTITLE = 1;
    const CAPTIONS = 2;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idVideo = null;

    /**
     * @var
     */
    private $iso = "";

    /**
     * @var
     */
    private $name;

    /**
     * @var
     */
    private $filename;

    /**
     *
     * @var string
     */
    private $extension;

    /**
     *
     * @var int
     */
    private $size;

    /**
     *
     * @var string
     */
    private $type;

    /**
     *
     * @var string
     */
    private $label;

    /**
     *
     * @var array
     */
    private $isDefault = false;

    /**
     * @var
     */
    private $registerDate = 0;

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','isDefault','iso','name','filename','extension','size','registerDate','type','label');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdVideo()
    {
        return $this->idVideo;
    }

    public function getIsoLanguage()
    {
        return $this->iso;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getExtension()
    {
        return $this->extension;
    }

    public function getSize()
    {
        return $this->size;
    }

    public function getLabel()
    {
        return $this->label;
    }

    public function getType()
    {
        return $this->type;
    }

    public function isDefault()
    {
        return (bool) $this->isDefault;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIsoLanguage($sec)
    {
        $this->iso = $sec;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setLabel($label)
    {
        $this->label = $label;
    }

    public function setExtension($sec)
    {
        $this->extension = $sec;
    }

    public function setSize($size)
    {
        $this->size = $size;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function setIdVideo($idVideo)
    {
        $this->idVideo = $idVideo;
    }

    public function setIsDefault($is)
    {
        $this->isDefault = (bool) $is;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    /**
     * save
     *
     * @param $idVideo
     * @throws Exception
     * @return Track
     */
    public function save($idVideo)
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT],
                [$idVideo, PDO::PARAM_INT],
                [$this->getIsoLanguage(), PDO::PARAM_STR],
                [$this->getName(), PDO::PARAM_STR],
                [$this->getFilename(), PDO::PARAM_STR],
                [$this->getExtension(), PDO::PARAM_STR],
                [$this->getSize(), PDO::PARAM_INT],
                [$this->getLabel(), PDO::PARAM_STR],
                [$this->getType(), PDO::PARAM_STR],
                [$this->isDefault(), PDO::PARAM_BOOL]];

            $name = "VIDEO_VTT";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0])) {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * find
     *
     * @param $idVideo
     * @throws Exception
     */
    public static function find($idVideo)
    {
        try {
            //echo $idVideo;die();
            $parameters = [[$idVideo, PDO::PARAM_INT]];
            $name = "VIDEO_VTT";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            //print_r($data); die();
            return array_map(function ($sp) {
                return self::createInstance($sp);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * @param $idVideo
     * @return bool
     * @throws Exception
     */
    public static function delete($idVideo, $soft = false, $idTrack = null)
    {
        try {
            if ($soft) {
                $parameters = [[$idTrack, PDO::PARAM_INT],
                    [$soft, PDO::PARAM_BOOL]];
                $name = "VIDEO_VTT_DELETED";
                $method = "SET";
                $data = Stage::esp($parameters, $name, $method);
                return isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : false;
            } else {
                $parameters = [[$idTrack, PDO::PARAM_INT]];
                $name = "VIDEO_VTT";
                $method = "DEL";
                $data = Stage::esp($parameters, $name, $method);
                return isset($data[0]['return_code']) && $data[0]['return_code'] == 0 ? true : false;
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * deleted
     *
     * @throws Exception
     */
    public static function deleted()
    {
        try {
            $parameters = [];
            $name = "VIDEO_VTT_DELETED";
            $method = "GET";
            $data = Stage::esp($parameters, $name, $method);
            return array_map(function ($sp) {
                return self::createInstance($sp);
            }, $data);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Track
     */
    public static function createInstance($data)
    {
        $instance = new self();
        if (isset($data['id_video'])) {
            $instance->setIdVideo($data['id_video']);
        }
        $instance->setId($data['id_vtt']);
        $instance->setIsoLanguage($data['iso_language']);
        $instance->setFilename($data['filename']);
        $instance->setName($data['name']);
        $instance->setExtension($data['extension']);
        $instance->setLabel($data['label']);
        $instance->setType($data['vtt_type']);
        $instance->setSize($data['size']);
        $instance->setIsDefault($data['is_default']);

        if (isset($data['register_date']) && Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $instance->setRegisterDate(new DateTime($data['register_date']));
        }

        return $instance;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
