<?php namespace Stage\Video;

use Stage\Helper;
use Stage\Stage;
use PDO;
use DateTime;
use Exception;
use Stage\Video\Thumbnail\Thumbnail;
use Stage\Video\Thumbnail\Query as ThumbnailQuery;

class Video
{
    const STATE2 = 2;
    const STATE3 = 3;
    const STATE4 = 4;
    const STATE5 = 5;

    /**
     * @var
     */
    private $id = -1;

    /**
     * @var
     */
    private $idDirectory;

    /**
     * Original name of the video
     * @var array
     */
    private $name;

    /**
     * Filename
     * @var array
     */
    private $filename;

    /**
     * The video duration in seconds
     * @var
     */
    private $duration;

    /**
     * @var
     */
    private $extension;

    /**
     * @var
     */
    private $mimeType;

    /**
     * Video filesize in bytes
     * @var
     */
    private $size;

    /**
     * @var
     */
    private $bitrate;

    /**
     * @var
     */
    private $width;

    /**
     * @var
     */
    private $height;

    /**
     * @var
     */
    private $ratio;

    /**
     * @var
     */
    private $state;

    /**
     * @var
     */
    private $codecAudio;

    /**
     * @var
     */
    private $codecVideo;

    /**
     * @var
     */
    private $audioBitrate;

    /**
     * @var string
     */
    private $defaultThumbnail = "";

    /**
     * @var string
     */
    private $registerDate = "";

    /**
     * @var boolean
     */
    private $hasPreview = false;

    /**
     * @var integer
     */
    private $framerate = 0;

    /**
     * @var integer
     */
    private $framecount = 0;

    /**
     * @var integer
     */
    private $idStorage = "";

    /**
     * @var integer
     */
    private $idCdn = "";

    /**
     * @var
     */
    private $tags = [];

    /**
     * The model's attributes.
     *
     * @var array
     */
    protected $attributes = array('id','idDirectory','name','filename','duration','extension','mimeType','size','bitrate','width','height','ratio','state','defaultThumbnail','registerDate','codecAudio','codecVideo','audioBitrate','hasPreview', 'framerate', 'framecount', 'idStorage', 'idCdn');

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = array();

    public function __construct()
    {
    }

    public function getId()
    {
        return $this->id;
    }

    public function getIdDirectory()
    {
        return $this->idDirectory;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getFilename()
    {
        return $this->filename;
    }

    public function getDuration()
    {
        return $this->duration;
    }

    public function getExtension()
    {
        return $this->extension;
    }

    public function getMimeType()
    {
        return $this->mimeType;
    }

    public function getSize()
    {
        return (int) $this->size;
    }

    public function getBitrate()
    {
        return $this->bitrate;
    }

    public function getWidth()
    {
        return (int) $this->width;
    }

    public function getHeight()
    {
        return (int) $this->height;
    }

    public function getRatio()
    {
        return $this->ratio;
    }

    public function getState()
    {
        return $this->state;
    }

    public function getCodecAudio()
    {
        return $this->codecAudio;
    }

    public function getCodecVideo()
    {
        return $this->codecVideo;
    }

    public function getAudioBitrate()
    {
        return $this->audioBitrate ? $this->audioBitrate : 0;
    }

    public function getDefaultThumbnail()
    {
        return $this->defaultThumbnail;
    }

    public function getRegisterDate()
    {
        return $this->registerDate;
    }

    public function hasPreview()
    {
        return $this->hasPreview;
    }

    public function getFramerate()
    {
        return $this->framerate;
    }

    public function getFramecount()
    {
        return $this->framecount;
    }

    public function getIdStorage()
    {
        return $this->idStorage;
    }

    public function getIdCdn()
    {
        return $this->idCdn;
    }

    private function setId($id)
    {
        $this->id = (int) $id;
    }

    public function setIdDirectory($id)
    {
        $this->idDirectory = $id;
    }

    public function setName($name)
    {
        $this->name = $name;
    }

    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    public function setDuration($seconds)
    {
        $this->duration = $seconds;
    }

    public function setExtension($extension)
    {
        $this->extension = strtolower(trim($extension));
    }

    public function setMimeType($mimeType)
    {
        $this->mimeType = strtolower(trim($mimeType));
    }

    public function setSize($size)
    {
        $this->size = (int)$size;
    }

    public function setBitrate($bitrate)
    {
        $this->bitrate = $bitrate;
    }

    public function setWidth($width)
    {
        $this->width = (int)$width;
    }

    public function setHeight($height)
    {
        $this->height = (int)$height;
    }

    public function setRatio($ratio)
    {
        $this->ratio = $ratio;
    }

    public function setState($state)
    {
        $this->state = (int) $state;
    }

    public function setCodecAudio($codec)
    {
        $this->codecAudio = $codec;
    }

    public function setCodecVideo($codec)
    {
        $this->codecVideo = $codec;
    }

    public function setAudioBitrate($bitrate)
    {
        $this->audioBitrate = $bitrate;
    }

    public function setDefaultThumbnail($thumbnail)
    {
        $this->defaultThumbnail = $thumbnail;
    }

    private function setRegisterDate(DateTime $date)
    {
        $this->registerDate = $date->format(DateTime::ATOM);
    }

    private function setHasPreview($has)
    {
        $this->hasPreview = (boolean) $has;
    }

    public function setFramerate($framerate)
    {
        $this->framerate = $framerate;
    }

    public function setFramecount($framecount)
    {
        $this->framecount = $framecount;
    }

    public function setIdStorage($id)
    {
        return $this->idStorage = $id;
    }

    public function setIdCdn($id)
    {
        $this->idCdn = $id;
    }

    /**
     *
     * sizeof
     *
     * @return array
     */
    public static function summarize()
    {
        $name = "VIDEO_SUMMARIZE";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     *
     * sizeof
     *
     * @return array
     */
    public static function sizeof()
    {
        $name = "VIDEOS_NUM";
        $method = "GET";
        $parameters = [];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * dates
     *
     * @return array
     */
    public static function dates($idQuery = 0)
    {
        $name = "VIDEO_VIDEOS_DATES";
        $method = "GET";
        $parameters = [[$idQuery, PDO::PARAM_INT]];
        $data = Stage::esp($parameters, $name, $method);
        return $data;
    }

    /**
     * systemTags
     *
     * @param int $field
     * @param int $limit_a In bytes
     * @param int $limit_b In bytes
     * @return array
     */
    public static function systemTags($field = 1, $idQuery = 0, $limit_a = 0, $limit_b = 0)
    {
        $parameters = [[$field, PDO::PARAM_INT],
                        [$idQuery, PDO::PARAM_INT],
                        [$limit_a, PDO::PARAM_INT],
                        [$limit_b, PDO::PARAM_INT]];

        $name = "VIDEO_VIDEOS_SYSTEM_TAGS";
        $method = "GET";
        $tags = Stage::esp($parameters, $name, $method);

        return $tags;
    }

    /**
     * find
     *
     * Find a Video
     *
     * @return $this
     * @throws Exception
     */
    public static function find($id, $by = 'id')
    {
        try {
            $name = "VIDEO";
            $method = "GET";

            $field = $by == 'id' ? 0 : 1;
            $parameters = [[$field, PDO::PARAM_INT],
                           [$id, PDO::PARAM_STR]];
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && isset($data[0]['return_code']) == "-2") {
                throw new Exception("The Video id $id not exists", $data[0]['return_code']);
            } else {
                return self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * delete
     *
     * Delete a Video from the database or mark it to be deleted.
     * The video is only removed from database if softDelete is true and the video was already marked to be deleted
     *
     * @param $idAsset
     * @param bool $softDelete
     * @param bool $deleted
     * @throws Exception
     */
    public static function delete($idVideo, $softDelete = true, $deleted = true)
    {
        if ($softDelete) {
            $name = "VIDEO_DELETED";    // Only mark the video has deleted and not remove from the database
            $parameters = [ [$idVideo, PDO::PARAM_INT],
                            [$deleted, PDO::PARAM_BOOL]];
            $method = "SET";
        } else {
            $name = "VIDEO";            // Remove from the database
            $parameters = [[$idVideo, PDO::PARAM_INT]];
            $method = "DEL";
        }
        try {
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['return_code']) && $data[0]['return_code'] == "-2") {
                throw new Exception("The Video id $idVideo not exists in database", $data[0]['return_code']);
            }

            return $idVideo;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * save
     *
     * Save a video
     *
     * @return $this
     * @throws Exception
     */
    public function save()
    {
        $parameters = [[$this->getId(), PDO::PARAM_INT],
                       [$this->getIdDirectory(), PDO::PARAM_INT],
                       [$this->getName(), PDO::PARAM_STR],
                       [$this->getFilename(), PDO::PARAM_STR],
                       [$this->getDuration(), PDO::PARAM_INT],
                       [$this->getExtension(), PDO::PARAM_STR],
                       [$this->getMimeType(), PDO::PARAM_STR],
                       [$this->getSize(), PDO::PARAM_INT],
                       [$this->getCodecVideo(), PDO::PARAM_STR],
                       [$this->getBitrate(), PDO::PARAM_INT],
                       [$this->getAudioBitrate(), PDO::PARAM_INT],
                       [$this->getCodecAudio(), PDO::PARAM_STR],
                       [$this->getFramerate(), PDO::PARAM_INT],
                       [$this->getFramecount(), PDO::PARAM_INT],
                       [$this->getWidth(), PDO::PARAM_INT],
                       [$this->getHeight(), PDO::PARAM_INT],
                       [$this->getRatio(), PDO::PARAM_STR],
                       [$this->getIdStorage(), PDO::PARAM_INT],
                       [$this->getIdCdn(), PDO::PARAM_INT]];

        try {
            $name = "VIDEO";
            $method = "SET";
            $data = Stage::esp($parameters, $name, $method);

            if (isset($data[0]['id_video']) && $data[0]['id_video'] >= 0) {
                $idVideo = $data[0]['id_video'];
                $this->setId($idVideo);

                self::createInstance($data[0]);
            }
        } catch (Exception $e) {
            throw $e;
        }

        return $this;
    }

    /**
     * allTagsRelatedWithVideos
     *
     * @return array
     * @throws Exception
     */
    public static function allTagsRelatedWithVideos()
    {
        try {
            $parameters = [[-1, PDO::PARAM_INT]];

            $name = "VIDEOS_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag\Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * thumbnails
     *
     * @return $this|mixed
     */
    public function thumbnails()
    {
        $q = new ThumbnailQuery();
        $q->term = $this->getId();
        $data = Thumbnail::query($q);

        return $data;
    }

    /**
     * withoutThumbnails
     *
     * Return all videos without associated thumbnail
     *
     * @return array
     * @throws Exception
     */
    public static function withoutThumbnails()
    {
        try {
            $parameters = [];
            $name = "VIDEO_WITHOUT_THUMBNAILS";
            $method = "GET";
            $videos = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $videos);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tags
     *
     * Return all the video tags
     * @return array
     * @throws Exception
     */
    public function tags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEOS_TAGS_RELATED";
            $method = "GET";
            $tags = Stage::esp($parameters, $name, $method);

            if (isset($tags[0]['return_code']) && ($tags[0]['return_code']) == "-2") {
                return [];
            } else {
                return array_map(function ($tag) {
                    return Tag\Tag::createInstance($tag);
                }, $tags);
            }
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * addTag
     *
     * @param Tag\Tag $tag
     * @throws Exception
     */
    public function addTag(Tag\Tag $tag)
    {
        try {
            $idTag = $tag->getId();
            $parameters = [ [$idTag, PDO::PARAM_INT],
                            [$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEOS_TO_TAGS";
            $method = "SET";
            Stage::esp($parameters, $name, $method);

            $this->tags = $this->tags();
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * removeAllTags
     *
     * @return $this
     * @throws Exception
     */
    public function removeAllTags()
    {
        try {
            $parameters = [[$this->getId(), PDO::PARAM_INT]];

            $name = "VIDEOS_FROM_TAGS";
            $method = "DEL";
            Stage::esp($parameters, $name, $method);
            $this->tags = [];
            return $this;
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * markedToDelete
     *
     * Get the videos marked to be delete from database
     * @return array
     * @throws Exception
     */
    public static function markedToDelete()
    {
        try {
            $parameters = [];
            $name = "VIDEOS_MARKED_HAS_DELETED";
            $method = "GET";
            $videos = Stage::esp($parameters, $name, $method);

            return array_map(function ($video) {
                return self::createInstance($video);
            }, $videos);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Return the Video versions
     *
     * @param null $filterBy Filter by a specific version, like, w,m,lm
     * @param null $orderBy Order by a specific database field
     * @param int $order The order, could be, SORT_ASC | SORT_DESC
     * @return array
     * @throws Exception
     */
    public function versions($filterBy = [], $orderBy = null, $order = SORT_ASC, $type = 1)
    {
        try {
            return Version::versions($this->getId(), $filterBy, $orderBy, $order, $type);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * setDefaultVersion
     *
     * @param $idVersion
     * @return Version
     * @throws Exception
     */
    public function setDefaultVersion($idVersion)
    {
        try {
            return Version::setDefaultVideoVersion($idVersion);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * createSneakPeak
     *
     * @param SneakPeek $senak
     * @return SneakPeek
     * @throws Exception
     */
    public function createSneakPeak(SneakPeek $sneak)
    {
        try {
            return $sneak->save($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * sneakPeaks
     *
     * @return SneakPeek
     * @throws Exception
     */
    public function sneakPeaks()
    {
        try {
            return SneakPeek::find($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * deleteSneakPeak
     *
     * @return SneakPeek
     * @throws Exception
     */
    public function deleteSneakPeeks()
    {
        try {
            return SneakPeek::delete($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * track
     *
     * @param Track $vtt
     * @return Track
     * @throws Exception
     */
    public function track(Track $vtt)
    {
        try {
            return $vtt->save($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tracks
     *
     * findVtt
     * @return array
     * @throws Exception
     */
    public function tracks()
    {
        try {
            return Track::find($this->getId());
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * tracksDeleted
     *
     * @return array
     * @throws Exception
     */
    public function tracksDeleted()
    {
        try {
            return Track::deleted();
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param bool $soft
     * @param null $idTrack
     * @return bool
     * @throws Exception
     */
    public function deleteTrack($soft = false, $idTrack = null)
    {
        try {
            return Track::delete($this->getId(), $soft, $idTrack);
        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * Convert the model instance to JSON.
     *
     * @param  int  $options
     * @return string
     */
    public function toJson()
    {
        return json_encode($this->toArray());
    }

    /**
     * Convert the model instance to an array.
     *
     * @return array
     */
    public function toArray()
    {
        $attributes = $this->attributesToArray();
        return $attributes;
    }

    /**
     * Convert the model's attributes to an array.
     *
     * @return array
     */
    public function attributesToArray()
    {
        $attributes = $this->attributes;
        $arr = array();

        foreach ($attributes as $attribute) {
            $arr[$attribute] = $attribute == "name" ? html_entity_decode($this->$attribute, ENT_COMPAT, 'UTF-8') : $this->$attribute;
        }

        return $arr;
    }

    /**
     * createInstance
     *
     * @param $data
     * @return Video
     */
    public static function createInstance($data)
    {
        $video = new self();
        $video->setId($data['id_video']);
        $video->setIdDirectory($data['id_directory']);
        $video->setName(html_entity_decode($data['name'], ENT_COMPAT, 'UTF-8'));
        $video->setFilename($data['filename']);
        $video->setDuration($data['seconds']);
        $video->setExtension($data['extension']);
        $video->setMimeType($data['mime_type']);
        $video->setSize($data['size']);
        $video->setBitrate($data['bitrate']);
        $video->setWidth($data['width']);
        $video->setHeight($data['height']);
        $video->setRatio($data['ratio']);
        $video->setState($data['state']);
        $video->setDefaultThumbnail($data['thumbnail']);
        $video->setCodecAudio($data['audio_codec']);
        $video->setCodecVideo($data['codec']);
        $video->setAudioBitrate($data['audio_bitrate']);
        $video->setHasPreview($data['has_preview']);
        $video->setFramerate($data['framerate']);
        $video->setFramecount($data['framecount']);

        if (isset($data['id_storage'])) {
            $video->setIdStorage($data['id_storage']);
        }

        if (isset($data['id_cdn'])) {
            $video->setIdCdn($data['id_cdn']);
        }

        if (Helper::isValidDate($data['register_date'], "Y-m-d H:i:s")) {
            $video->setRegisterDate(new DateTime($data['register_date']));
        }

        return $video;
    }

    public function __toString()
    {
        return "My id is: {$this->getId()}\n";
    }
}
