<?php

use Stage\Account\UserGuest;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class AccountTeamTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testAccountTeamCreate()
    {
        $team = new \Stage\Account\Team();
        $team->setIdAccount(3);
        $team->setName("name");
        $team->setNameStripped("nameStripped");
        $team->setDescription("desc");
        $team->setPosition(1);
        $team->setActive(true);
        $team = $team->save();
        $this->assertInstanceOf('\Stage\Account\Team', $team);
        $this->assertTrue($team->getId() > 0);
    }

    public function testAccountTeamDelete()
    {
        $status = \Stage\Account\Team::delete(3, 9);
        $this->assertTrue($status);
    }

    public function testAccountTeamAddUser()
    {
        $status = \Stage\Account\Team::addUser(3, 1, 57, 1, true);
        $this->assertTrue($status);
    }

    public function testAccountTeamRemoveUser()
    {
        $status = \Stage\Account\Team::removeUser(3, 1, 57);
        $this->assertTrue($status);
    }

    public function testAccountTeamQuery()
    {
        $q = new \Stage\Account\QueryTeam();
        $q->idAccount = 3;
        $teams = \Stage\Account\Team::query($q, true);
        foreach ($teams as $team) {
            $this->assertInstanceOf('\Stage\Account\Team', $team);
        }
    }

    public function testAccountTeamQueryUsers()
    {
        $q = new \Stage\Account\QueryTeamUsers();
        $q->idAccount = 3;
        $q->idTeam = 1;
        $teams = \Stage\Account\Team::users($q, true);
        foreach ($teams as $team) {
            $this->assertInstanceOf('\Stage\Account\User', $team);
        }
    }

    public function testAccountTeamAttachGuest()
    {
        $idAccount = 3;
        $idTeam = 16;
        $name = "name";
        $email = "email@gmail.com";
        $position = 1;
        $active = true;
        $userGuest = \Stage\Account\Team::attachGuestUser($idAccount, $idTeam, $name, $email, $position, $active);
        $this->assertInstanceOf('\Stage\Account\UserGuest', $userGuest);
    }

    public function testAccountTeamDetachGuest()
    {
        $idAccount = 3;
        $idTeam = 16;
        $idGuest = 2;
        $status = \Stage\Account\Team::detachGuestUser($idAccount, $idTeam, $idGuest);
        $this->assertTrue($status);
    }

    public function testAccountTeamQueryGuestUsers()
    {
        $q = new \Stage\Account\QueryTeamGuestUsers();
        $q->idAccount = 3;
        $q->idTeam = 16;
        $teams = \Stage\Account\Team::guestUsers($q, true);
        foreach ($teams as $team) {
            $this->assertInstanceOf('\Stage\Account\UserGuest', $team);
        }
    }
}
