<?php

use Stage\Account\Account;
//use Stage\Archive\Video\Video;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;
use Stage\Ad\Tag\Tag;
use Stage\Ad\Tag\Query as QueryTag;
use Stage\Archive\Directory;
use Stage\Ad\Video\Directory as AdDirectory;
use Stage\Ad\Video\Query as AdDirectoryQuery;
use Stage\Video\Compose\Campaign\Query as ComposeCompaignQuery;
use Stage\Ad\Video\Video;

class AdsTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "192.168.70.21",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testAdsTagCreate()
    {
        $tag = new Tag();

        $name = html_entity_decode("goportugal", ENT_COMPAT, 'UTF-8');
        $tag->setName($name);
        $tag->setNameStripped("go-portugal");
        $tag->setPublic("0");

        try {
            $tag->save();
            $this->assertTrue($tag instanceof Tag);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsTagNum()
    {
        try {
            $this->assertInternalType("int", Tag::sizeof());
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsTagDelete()
    {
        try {
            $tag = Tag::find(1);
            $deletedTag = $tag->delete();
            $this->assertTrue($deletedTag instanceof Tag);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsTagFind()
    {
        try {
            $tag = Tag::find(1);
            $this->assertTrue($tag instanceof Tag);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsTagQuery()
    {
        try {
            $q = new QueryTag();
            $q->by = QueryTag::NAME;
            $q->term = "";
            $q->from = 0;
            $q->to = -1;

            $tags = Tag::query($q);

            print_r(array_map(function ($tag) {
                return $tag->toArray();
            }, $tags));
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryCreate()
    {
        try {
            $dir = new AdDirectory();
            $dir->setName(str_shuffle("new folder"));
            $dir->setDescription("new folder description");
            $dir->setLink("http://www.sapo.pt/");

            $createdDirectory = $dir->appendTo(null);   // If pass null is passing to root

            $this->assertTrue($createdDirectory instanceof AdDirectory);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryFind()
    {
        try {
            $directory = AdDirectory::find(4);
            $this->assertTrue($directory instanceof Directory);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryAll()
    {
        try {
            $directories = AdDirectory::all(true);
            print_r($directories);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryBuild()
    {
        try {
            $json = AdDirectory::build();
            print_r($json);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsDirectoryFindNode()
    {
        try {
            $tree = AdDirectory::build();
            $idDirectory = 4;
            $node = AdDirectory::findNode($tree, $idDirectory);
            print_r($node);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryChildren()
    {
        try {
            $tree = AdDirectory::build();
            $idDirectory = 4;
            $node = AdDirectory::findNode($tree, $idDirectory);

            // First check if it returned results
            if (sizeof($node)) {
                $result = AdDirectory::children([$node]);
                print_r($result);
            }
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectorySoftDelete()
    {
        try {

            /** @var Directory $directory */
            $directory = AdDirectory::find(6);

            // Only mark the directory to be deleted
            AdDirectory::delete($directory->getId());
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryHardDelete()
    {
        try {

            /** @var Directory $directory */
            $directory = AdDirectory::find(6);

            // Only mark the directory to be deleted
            AdDirectory::delete($directory->getId(), false);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryMarkedToBeDeleted()
    {
        try {
            $directories = AdDirectory::markedToDelete();
            print_r($directories);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoThumbnails()
    {
        try {
            $video = Video::find(2);
            $result = $video->thumbnails();
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoVersions()
    {
        try {
            $video = Video::find(9);
            $result = $video->versions();
            print_r($result);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoDirectoryQuery()
    {
        try {
            $q = new AdDirectoryQuery();
            $q->by = QueryTag::ALL;
            $q->term = "";
            $q->filter = "";
            $q->from = 0;
            $q->to = -1;

            $result = AdDirectory::videos($q);
            print_r($result);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoThumbnailCreate()
    {
        try {
            $q = new \Stage\Ad\Video\Thumbnail\Thumbnail();
            $q->setIdVideo(2);
            $q->setFilename("aaa.mp4");
            $q->setSeconds(9999);

            $thumbnail = $q->save();

            $this->assertTrue($thumbnail instanceof \Stage\Ad\Video\Thumbnail\Thumbnail);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoThumbnailQuery()
    {
        // TODO
        try {
            $q = new \Stage\Ad\Video\Thumbnail\Query();
            $q->term = "";
            $q->by = 0;
            $q->from = 0;
            $q->to = -1;

            $result = \Stage\Ad\Video\Thumbnail\Thumbnail::query($q);

            print_r($result);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdsVideoThumbnailFind()
    {
        try {
            $result = \Stage\Ad\Video\Thumbnail\Thumbnail::find(1);
            $this->assertTrue($result instanceof \Stage\Ad\Video\Thumbnail\Thumbnail);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdVideoVersionCreate()
    {
        try {
            $q = new \Stage\Ad\Video\Version();
            $q->setIdVideo(2);
            $q->setFilename("cyz.mp4");
            $q->setExtension("mp4");
            $q->setWidth(800);
            $q->setHeight(600);
            $q->setVersion(720);
            $q->setVersionType(\Stage\Ad\Video\Version::WEB);
            $q->setSize(99999);
            $q->setState(0);
            $q->setDefaultVersion(720);
            $q->setIsDefault(true);
            $q->setBitrate(77777);

            $version = $q->save();

            $this->assertTrue($version instanceof \Stage\Ad\Video\Version);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdVideoDefaultVersion()
    {
        try {
            $result = \Stage\Ad\Video\Version::setDefaultVideoVersion(720);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (\Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdVideoCreate()
    {
        try {

            /** @var Video $clip */
            $clip = new Video();
            $id = Directory::ROOT;
            $id = Directory::find(1)->getId();
            $clip->setIdDirectory($id);
            $clip->setName("Demo");
            $clip->setFilename("xyzkwlo.mp4");
            $clip->setDuration(250);
            $clip->setExtension("mp4");
            $clip->setMimeType("video/mp4");
            $clip->setSize(523698);
            $clip->setBitrate(1);
            $clip->setWidth(800);
            $clip->setHeight(600);
            $clip->setRatio("16:9");

            $clip->save();

            $this->assertTrue($clip instanceof Video);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdVideoFind()
    {
        try {
            $clip = Video::find(2);
            $this->assertTrue($clip instanceof Video);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdVideoSoftDelete()
    {
        try {

            /** @var Video $clip */
            $clip = Video::find(2);
            Video::delete($clip->getId(), true);

            $this->assertTrue($clip instanceof Video);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdVideosTags()
    {
        try {

            /** @var Video $clip */
            $clip = Video::find(2);
            $tags = $clip->tags();
            print_r($tags);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdVideosAddTags()
    {
        try {
            $tag = Tag::find(2);

            /** @var Video $clip */
            $clip = Video::find(1);

            $tags = $clip->addTag($tag);
            print_r($tags);
        } catch (\Stage\Exception\StoredProcedure $e) {
            $this->assertTrue($e instanceof Exception);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdVideosRemoveTags()
    {
        try {

            /** @var Video $clip */
            $clip = Video::find(1);
            $tags = $clip->removeAllTags();
            print_r($tags);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoThumbnailCreate()
    {
        try {
            // TODO
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdTranscodeCreate()
    {
        try {
            $transconding = new \Stage\Ad\Video\Transcode\Preview\Transcode();
            $transconding->setIdAccount(3);
            $transconding->setAccountToken('TOKEN3');
            $transconding->setIdVideo(2);
            $transconding->setInputPath('/mn/data');
            $transconding->setOutputPath('/mn/data');
            $transconding->setFilename('video.mp4');  // Filename without extensions
            $transconding->setSource('video.mp4');
            $transconding->setWith(1920);
            $transconding->setHeight(1080);
            $transconding->setSeconds(1258);
            $transconding->setAspecRatio('16:9');
            $transconding->setAudioCodec('h264');
            $transconding->setAudioRate(123456);
            $transconding->setVideoRate(789456);
            $transconding->setVersion(1);   // TODO Check this value
            $transconding->setToH264(1);
            $transconding->setToOgg(0);
            $transconding->setToWebm(0);
            $transconding->setToWebVersion(1);
            $transconding->setToMobileVersion(1);

            $transconding->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdTranscodeNextVideo()
    {
        try {
            $job = \Stage\Ad\Video\Transcode\Preview\Transcode::next();
            $this->assertTrue($job instanceof \Stage\Video\Transcode\Job);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testExternalTagCreate()
    {
        try {
            $tag = new \Stage\Ad\External\Tag();
            $tag->setName("saop");
            $tag->setUrl('Thttp://www.sapo.pt');
            $tag->setActive(false);

            $tag->save();
            $this->assertTrue($tag instanceof \Stage\Ad\External\Tag);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testExternalTagFind()
    {
        try {
            $tag = \Stage\Ad\External\Tag::find(2);
            $this->assertTrue($tag instanceof \Stage\Ad\External\Tag);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testExternalTagDelete()
    {
        try {
            $tag = \Stage\Ad\External\Tag::find(1);
            $deletedTag = $tag->delete();
            $this->assertTrue($deletedTag instanceof \Stage\Ad\External\Tag);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testExternalTagQuery()
    {
        try {
            $q = new \Stage\Ad\External\Query();
            $q->by = \Stage\Ad\External\Query::ALL;
            $q->term = "";
            $q->filter = "";
            $q->from = 0;
            $q->to = -1;
            $result = \Stage\Ad\External\Tag::query($q);

            foreach ($result as $r) {
                $this->assertTrue($r instanceof \Stage\Ad\External\Tag);
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdCampaignCreate()
    {
        try {
            $tag = new \Stage\Ad\Campaign\Campaign();
            $tag->setName("saop");
            $tag->setBeginDate(new DateTime());
            $tag->setEndDate(new DateTime());

            $tag->save();
            $this->assertTrue($tag instanceof \Stage\Ad\Campaign\Campaign);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdCampaignFind()
    {
        try {
            $tag = \Stage\Ad\Campaign\Campaign::find(1);
            $this->assertTrue($tag instanceof \Stage\Ad\Campaign\Campaign);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdCampaignDelete()
    {
        try {
            $tag = \Stage\Ad\Campaign\Campaign::find(1);
            $deleted = $tag->delete();
            $this->assertTrue($deleted instanceof \Stage\Ad\Campaign\Campaign);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdCampaignQuery()
    {
        try {
            $q = new \Stage\Ad\Campaign\Query();
            $q->by = \Stage\Ad\Campaign\Query::ALL;
            $q->term = "";
            $q->filter = "";
            $q->from = 0;
            $q->to = -1;
            $result = \Stage\Ad\Campaign\Campaign::query($q);

            foreach ($result as $r) {
                $this->assertTrue($r instanceof \Stage\Ad\Campaign\Campaign);
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdCampaignAdTag()
    {
        try {
            $tag = Tag::find(1);

            /** @var Video $clip */
            $campaign = \Stage\Ad\Campaign\Campaign::find(2);

            $tags = $campaign->addTag($tag);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            $this->assertTrue($e instanceof Exception);
        } catch (Exception $e) {
            echo $e->getMessage();
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testAdCampaignTags()
    {
        try {

            /** @var Stage\Ad\Campaign\Campaign $campaign */
            $campaign = \Stage\Ad\Campaign\Campaign::find(5);
            $tags = $campaign->tags();
            print_r($tags);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            $this->assertTrue($e instanceof Exception);
        } catch (Exception $e) {
            echo $e->getMessage();
            $this->assertTrue($e instanceof Exception);
        }
    }

    // Group
    public function testCampaignGroupCreate()
    {
        try {
            $group = new \Stage\Ad\Campaign\Group\Group();
            $group->setName("saops");
            $group->setDomains("*");
            $group->setExclude(true);

            $campaign = \Stage\Ad\Campaign\Campaign::find(2);
            $group->setCampaign($campaign);

            $group->save();
            $this->assertTrue($group instanceof \Stage\Ad\Campaign\Group\Group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCampaignGroupGet()
    {
        try {
            $campaign = \Stage\Ad\Campaign\Campaign::find(2);
            $groups = $campaign->groups();

            foreach ($groups as $group) {
                print_r($group->toArray());
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdCampaignGroupVideosCreate()
    {
        try {

            //$group = \Stage\Ad\Campaign\Group\Group::find(9,2);
            $groupVideos = new \Stage\Ad\Campaign\Group\Ad\Video();
            $groupVideos->setIdCampaignGroup(229);
            $groupVideos->setIdCampaign(88);
            $groupVideos->setIdAd(9);   // Here is the video ad id
            $groupVideos->setDisplayIn(true);
            $groupVideos->setBeginDate(new DateTime());
            $groupVideos->setEndDate(new DateTime());
            $groupVideos->setLink("");
            $action = new \Stage\Ad\Action();
            $action->setId(5);
            $groupVideos->setAction($action);
            $groupVideos->setCanSkip(true);
            $groupVideos->setSkipAfter(2);

            $groupVideos = $groupVideos->save();
            print_r($groupVideos->toArray());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCampaignGroupAdTagCreate()
    {
        try {
            $groupTag = new \Stage\Ad\Campaign\Group\Ad\Tag();
            $groupTag->setIdCampaignGroup(9);
            $groupTag->setIdCampaign(2);
            $groupTag->setDisplayIn(true);

            $groupTag->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCampaignGroupCountryCreate()
    {
        try {
            $groupCountry = new \Stage\Ad\Campaign\Group\Country();
            $groupCountry->setIdCampaignGroup(9);
            $groupCountry->setIdCampaign(2);
            $groupCountry->setIso("pt");
            $groupCountry->setExclude(false);

            $groupCountry->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdAddComposeCampaign()
    {
        try {
            $compose = \Stage\Video\Compose\Compose::find(2);
            $campaign = \Stage\Ad\Campaign\Campaign::find(2);
            $camposeCampaign = $compose->addCampaign($campaign, 1, 3);
            print_r($camposeCampaign);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAdRemoveComposeCampaign()
    {
        try {
            $compose = \Stage\Video\Compose\Compose::find(2);
            $campaign = \Stage\Ad\Campaign\Campaign::find(2);
            $compose->removeCampaigns($campaign);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetCamposeCampaigns()
    {
        try {
            $compose = \Stage\Video\Compose\Compose::find(2);

            $query = new ComposeCompaignQuery();
            $query->filter = ComposeCompaignQuery::BY_COMPOSE_ID;
            $query->by = $compose->getId();
            $query->from = 0;
            $query->to = -1;

            $campaigns = $compose->composedCampaigns($query);

            print_r($campaigns);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testSaveCampaignCode()
    {
        $code = new \Stage\Ad\Campaign\Code\Code();
        $code->setName("name");
        $code->setCode(123);
        $code->setRatio("16:9");
        $code->setAutoplay(true);
        $code->setEndCollapse(false);
        $code->setStartExpanded(true);

        try {
            $code->save(88);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCampaignCodeQuery()
    {
        $idCampaign = 88;
        $q = new \Stage\Ad\Campaign\Code\Query();
        $q->by = \Stage\Ad\Campaign\Code\Query::ALL;
        $q->filter = $idCampaign;
        $q->term = "";
        $q->from = 0;
        $q->to = -1;

        $results = \Stage\Ad\Campaign\Code\Code::query($q);
        foreach ($results as $result) {
            print_r($result->toArray());
        }
    }

    public function testCampaignGetAllAvailableCodeOptions()
    {
        $options = \Stage\Ad\Campaign\Code\Code::allOptions();
        print_r($options);
    }

    public function testCampaignGetCodeOptions()
    {
        $options = \Stage\Ad\Campaign\Code\Code::optionsBy(1, 88);
        print_r($options);
    }

    public function testCampaignSetCodeOptions()
    {
        $options = \Stage\Ad\Campaign\Code\Code::setOptionsBy(1, 88, 1, "open_value_updated");
        print_r($options);
    }

    public function testCampaignCodeClone()
    {
        $cloned = \Stage\Ad\Campaign\Code\Code::duplicate(1, 88);
        print_r($cloned);
    }

    public function testAdActionGetAll()
    {
        $data = \Stage\Ad\Action::all();
        $r = array_map(function ($action) {
            return $action->toArray();
        }, $data);
        print_r($r);
    }

    public function testAddFrequencyCap()
    {
        $fc = new \Stage\Ad\Campaign\Group\FrequencyCap();
        $fc->setIdCampaignGroup(89);
        $fc->setIdCampaign(86);
        $fc->setType(1);
        $fc->setValue(2);
        $fc->save();

        print_r($fc);
    }

    public function testAddFrequencyCapOptions()
    {
        $options = \Stage\Ad\Campaign\Group\FrequencyCap::options(86, 89);
        print_r($options);
    }

    public function testAddWeeklySchedule()
    {
        $idGgroupAdVideo = 218;   // Its the collumn id from table ads_campaigns_groups_ads_videos
        $fc = new \Stage\Ad\Campaign\Group\Ad\WeeklySchedule();
        $fc->setExclude(true);
        $fc->setBeginAt("");
        $fc->setEndAt("");
        $fc->setDayOfWeek(2);
        $fc->save($idGgroupAdVideo);

        print_r($fc);
    }

    public function testAddWeeklyScheduleOptions()
    {
        $options = \Stage\Ad\Campaign\Group\Ad\Video::weeklySchedule(218);
        print_r($options);
    }
}
