<?php

use Stage\Account\Account;
use Stage\Archive\Video\Video;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;
use Stage\Archive\Tag\Tag;
use Stage\Archive\Tag\Query as QueryTag;
use Stage\Archive\Clip\Query as QueryClip;
use Stage\Archive\Video\Query as QueryVideo;
use Stage\Archive\Directory;
use Stage\Archive\Clip\Directory as ClipDirectory;
use Stage\Archive\Video\Directory as VideoDirectory;
use Stage\Archive\Clip\Clip;
use Stage\Helper\Formating;

class ArchiveTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "192.168.70.21",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testArchiveTagCreate()
    {
        $tag = new Tag();

        $name = html_entity_decode("goportugal", ENT_COMPAT, 'UTF-8');
        $tag->setName($name);
        $tag->setNameStripped("go-portugal");
        $tag->setPublic("0");

        try {
            $tag->save();
            $this->assertTrue($tag instanceof Tag);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testArchiveTagFind()
    {
        try {
            $tag = Tag::find(1);
            $this->assertTrue($tag instanceof Tag);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testArchiveTagQuery()
    {
        try {
            $q = new QueryTag();
            $q->by = QueryTag::NAME;
            $q->term = "";
            $q->from = 0;
            $q->to = -1;

            $tags = Tag::query($q);

            print_r(array_map(function ($tag) {
                return $tag->toArray();
            }, $tags));
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testArchiveTagDelete()
    {
        try {
            $tag = Tag::find(1);
            $deletedTag = $tag->delete();
            $this->assertTrue($deletedTag instanceof Tag);
        } catch (Exception $e) {
            $this->assertTrue($e instanceof Exception);
        }
    }

    public function testArchiveVideoDirectoryCreate()
    {
        try {
            $dir = new VideoDirectory();
            $dir->setName(str_shuffle("new folder"));
            $dir->setDescription("new folder description");
            $dir->setLink("http://www.sapo.pt/");
            // $targetDirectory = Directory::find(7);            // Create on a target directory
            // print_r($targetDirectory);
            // die();

            $createdDirectory = $dir->appendTo(null);   // If pass null is passing to root
            // $dir->appendTo($targetDirectory);   // Pass a valid Directory to append

            // print_r($dir);
            // print_r($dir->toArray());
            // echo json_encode($dir->toArray());
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoDirectoryFind()
    {
        try {
            $directory = VideoDirectory::find(2);
            $this->assertTrue($directory instanceof Directory);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoDirectoryAll()
    {
        try {
            $directories = VideoDirectory::all(true);
            print_r($directories);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoDirectoryBuild()
    {
        try {
            $json = VideoDirectory::build();
            print_r($json);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveDirectoryFindNode()
    {
        try {
            $tree = VideoDirectory::build();
            $idDirectory = 3;
            $node = VideoDirectory::findNode($tree, $idDirectory);
            print_r($node);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoDirectoryChildren()
    {
        try {
            $tree = VideoDirectory::build();
            $idDirectory = 3;
            $node = VideoDirectory::findNode($tree, $idDirectory);

            // First check if it returned results
            if (sizeof($node)) {
                $result = VideoDirectory::children([$node]);
                print_r($result);
            }
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoDirectorySoftDelete()
    {
        try {

            /** @var Directory $directory */
            $directory = VideoDirectory::find(5);

            // Only mark the directory to be deleted
            VideoDirectory::delete($directory->getId());
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoDirectoryHardDelete()
    {
        try {

            /** @var Directory $directory */
            $directory = VideoDirectory::find(4);

            // Only mark the directory to be deleted
            VideoDirectory::delete($directory->getId(), false);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoDirectoryMarkedToBeDeleted()
    {
        try {
            $directories = VideoDirectory::markedToDelete();
            print_r($directories);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
            echo $e->getCode();
            print_r($e->getObject());
        } catch (\Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveClipCreate()
    {
        try {
            $clip = new Clip();
            $id = Directory::ROOT;
            $id = Directory::find(1)->getId();
            $clip->setIdDirectory($id);
            $clip->setIdVideo(1);
            $clip->setName("Demo");
            $clip->setFilename("xyzkwlo.mp4");
            $clip->setDuration(250);
            $clip->setExtension("mp4");
            $clip->setMimeType("video/mp4");
            $clip->setSize(523698);
            $clip->setBitrate(1);
            $clip->setWidth(800);
            $clip->setHeight(600);
            $clip->setRatio("16:9");

            $clip->save();

            $this->assertTrue($clip instanceof Clip);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveClipFind()
    {
        try {
            $clip = Clip::find(4);
            print_r($clip->toArray());
            $this->assertTrue($clip instanceof Clip);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveClipUpdate()
    {
        // TODO
    }

    public function testArchiveClipSoftDelete()
    {
        try {

            /** @var Clip $clip */
            $clip = Clip::find(4);
            Clip::delete($clip->getId(), true);

            $this->assertTrue($clip instanceof Clip);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveClipTags()
    {
        try {

            /** @var Clip $clip */
            $clip = Clip::find(4);
            $tags = $clip->tags();
            print_r($tags);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveClipAddTags()
    {
        try {
            $tag = Tag::find(2);
            /** @var Clip $clip */
            $clip = Clip::find(4);

            $tags = $clip->addTag($tag);
            print_r($tags);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveClipRemoveTags()
    {
        try {

            /** @var Clip $clip */
            $clip = Clip::find(1);
            $tags = $clip->removeAllTags();
            print_r($tags);
        } catch (\Stage\Exception\StoredProcedure $e) {
            print_r($e->getObject());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveDirectoryVideos()
    {
        try {
            $q = new QueryVideo();
            $q->by = QueryVideo::ALL;
            $q->term = "";
            $q->filter = "";
            $q->from = 0;
            $q->to = -1;

            // Pass id directory equal to zero to fetch in all directories, otherwise pass a valid id directory
            $idDirectory = 0;
            $videos = Directory::videos($q, $idDirectory);

            $videosArray = array_map(function ($clip) {
                return $clip->toArray();
            }, $videos);

            print_r($videosArray);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveDirectoryClips()
    {
        try {
            $q = new QueryClip();
            $q->by = QueryClip::ALL;
            $q->term = "";
            $q->filter = "";
            $q->from = 0;
            $q->to = -1;

            // Pass id directory equal to zero to fetch in all directories, otherwise pass a valid id directory
            $idDirectory = 0;
            $clips = Directory::clips($q, $idDirectory);

            $clipsArray = array_map(function ($clip) {
                return $clip->toArray();
            }, $clips);

            print_r($clipsArray);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVideoThumbnailCreate()
    {
        try {
            // TODO
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveTransferCreate()
    {
        try {
            $transfer = new \Stage\Archive\Transfer\Transfer();
            $transfer->setCode("xyz2");
            $transfer->setName("name");
            $transfer->setPassword("***");
            $transfer->setEndDate(new DateTime());

            $transfer->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }


    public function testArchiveTransferQuery()
    {
        try {
            $q = new \Stage\Archive\Transfer\Query();
            $q->term = "name";

            $data = \Stage\Archive\Transfer\Transfer::query($q);
            print_r($data);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveTransferFind()
    {
        try {
            $data = \Stage\Archive\Transfer\Transfer::find(1);
            print_r($data);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveTranscodeCreate()
    {
        try {
            $transconding = new \Stage\Archive\Video\Transcode\Transcode();
            $transconding->setIdAccount(3);
            $transconding->setAccountToken('TOKEN3');
            $transconding->setIdVideo(2);
            $transconding->setInputPath('/mn/data');
            $transconding->setOutputPath('/mn/data');
            $transconding->setFilename('video.mp4');  // Filename without extensions
            $transconding->setSource('video.mp4');
            $transconding->setWith(1920);
            $transconding->setHeight(1080);
            $transconding->setSeconds(1258);
            $transconding->setAspecRatio('16:9');
            $transconding->setVideoCodec('h264');
            $transconding->setAudioCodec('h264');
            $transconding->setAudioRate(123456);
            $transconding->setVideoRate(789456);

            $transconding->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveTranscodeNextVideo()
    {
        try {
            $job = \Stage\Archive\Video\Transcode\Transcode::next();
            print_r($job);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveCreateGroup()
    {
        try {
            $name = "GroupName";
            $group = new \Stage\Archive\Tag\Group\Group();
            $group->setName(htmlentities(trim($name), ENT_COMPAT, 'UTF-8'));

            $group->save();
            print_r($group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveFindGroup()
    {
        try {
            $group = \Stage\Archive\Tag\Group\Group::find(2);
            print_r($group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveQueryGroup()
    {
        try {
            $group = \Stage\Archive\Tag\Group\Group::build();
            print_r($group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveCreateKey()
    {
        try {
            $name = "key1";
            $public = true;

            $key = new \Stage\Archive\Tag\Key\Key();
            $key->setName(htmlentities(trim($name), ENT_COMPAT, 'UTF-8'));
            $key->setNameStripped(strtolower(str_replace(" ", "-", trim(Formating::generateNameLink($name)))));
            $key->setPublic($public);

            $key->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveFindKey()
    {
        try {
            $key = \Stage\Archive\Tag\Key\Key::find(1);
            print_r($key);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveAddVariantKeyToTag()
    {
        try {
            $tag = \Stage\Asset\Tag\Tag::find(1);
            $key = \Stage\Archive\Tag\Key\Key::find(1);
            $value = "value";

            $variant = $tag->addVariants($key->getId(), $value);
            print_r($variant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveRemoveVariantKeyFromTag()
    {
        try {
            $tag = \Stage\Archive\Tag\Tag::find(6);
            $idKey = 6;
            $data = $tag->removeVariants($idKey);

            print_r($data);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveVariantKeys()
    {
        try {
            $tag = \Stage\Archive\Tag\Tag::find(3);
            $variants = $tag->variants();

            $data = array_map(function ($variant) {
                return $variant->toArray();
            }, $variants);
            print_r($data);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testArchiveTagKeys()
    {
        $q = new \Stage\Archive\Tag\Key\Query();
        $data = \Stage\Archive\Tag\Key\Key::query($q);
        print_r($data);
    }
}
