<?php

use Service\Connection;
use Stage\Account\Account;
use Stage\Sps;
use Stage\Stage;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class AssetPlaylistCustomKeyTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }


    public function testAssetPlaylistCreateCustomKey()
    {
        try {
            $customKey = new \Stage\Asset\Playlist\CustomFieldKey();
            $customKey->setName("name");
            $customKey->setNameStripped("stripped");
            $customKey->setPublic(true);

            $result = $customKey->save();
            $this->assertInstanceOf('\Stage\Asset\Playlist\CustomFieldKey', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetPlaylistGetCustomKey()
    {
        try {
            $customKey = \Stage\Asset\Playlist\CustomFieldKey::find(8);
            $this->assertInstanceOf('\Stage\Asset\Playlist\CustomFieldKey', $customKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetPlaylistDeleteCustomKeys()
    {
        try {
            $customKey = \Stage\Asset\Playlist\CustomFieldKey::find(8);
            $status = $customKey->delete();
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetPlaylistsQueryCustomKeys()
    {
        try {
            $q = new \Stage\Asset\Playlist\CustomFieldKeyQuery();
            $result = \Stage\Asset\Playlist\CustomFieldKey::query($q);
            print_r($result->toArray());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetPlaylistsAttachCustomFieldVariant()
    {
        try {
            $playlist = \Stage\Asset\Playlist\Playlist::find(1);
            $variant = $playlist->attachCustomFieldVariant(1,"value", 1);
            $this->assertInstanceOf('\Stage\Asset\Playlist\CustomFieldVariant', $variant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetPlaylistsDetachCustomFieldVariant()
    {
        try {
            $playlist = \Stage\Asset\Playlist\Playlist::find(1);
            $status = $playlist->detachCustomFieldsVariant(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetPlaylistsCustomFieldVariants()
    {
        try {
            $playlist = \Stage\Asset\Playlist\Playlist::find(1);
            $variants = $playlist->customFieldsVariants(1);
            print_r($variants->toArray());
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

}
