<?php

use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Asset\Asset;
use Stage\Asset\CustomFieldKey;
use Stage\Asset\Album\CustomFieldKey as AlbumCustomFieldKey;
use Stage\Asset\Extensions\ItemFilter;
use Stage\Helper\Formating;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;
use Stage\Asset\Directory;
use Stage\Asset\Query;

class AssetsTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        //$stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

//        try {
//            $idAccount = 3;
//            $account = Account::find($idAccount);
//
//            // Get a Stage instance
//            $stageInstance->setAccount($account);
//        } catch (Exception $e) {
//            echo $e->getMessage();
//        }
    }

    public function tearDown()
    {
    }

    public function testAdsTree()
    {
        $json = '[
            {"id":2,"parentId":1,"name":"DT","description":"","link":"","files":"1"},
            {"children":[
                {"children":[
                    {"id":272,
                        "parentId":590,
                        "name":"autumn-winter",
                        "description":"",
                        "link":"",
                        "files":"7",
                        "children":[
                            {"children":[
                                {"id":280,
                                    "parentId":273,
                                    "name":"adventure",
                                    "description":"",
                                    "link":"",
                                    "files":"4",
                                    "children":[  
                                        {"id":284,"parentId":280,"name":"SW00182000200","description":"","link":"","files":"2"},
                                        {"id":285,"parentId":280,"name":"SW00182000204","description":"","link":"","files":"2"},
                                        {"id":286,"parentId":280,"name":"SW00182000208","description":"","link":"","files":"5"},
                                        {"id":287,"parentId":280,"name":"SW00182000224","description":"","link":"","files":"1"}]},
                                        {"children":[{"id":288,"parentId":281,"name":"CS00191000200","description":"","link":"","files":"2"},{"id":289,"parentId":281,"name":"CS00191000202","description":"","link":"","files":"1"},{"id":290,"parentId":281,"name":"CS00191000204","description":"","link":"","files":"1"},{"id":291,"parentId":281,"name":"CS00191000206","description":"","link":"","files":"2"},{"id":292,"parentId":281,"name":"CS00191000226","description":"","link":"","files":"1"}],"id":281,"parentId":273,"name":"quest","description":"","link":"","files":"5"},{"children":[{"id":293,"parentId":282,"name":"CS00192000200","description":"","link":"","files":"1"},{"id":294,"parentId":282,"name":"CS00192000202","description":"","link":"","files":"1"},{"id":295,"parentId":282,"name":"CS00192000203","description":"","link":"","files":"1"},{"id":296,"parentId":282,"name":"CS00192000204","description":"","link":"","files":"1"},{"id":297,"parentId":282,"name":"CS00192000206","description":"","link":"","files":"2"},{"id":298,"parentId":282,"name":"CS00192000207","description":"","link":"","files":"2"},{"id":299,"parentId":282,"name":"CS00192000208","description":"","link":"","files":"1"}],"id":282,"parentId":273,"name":"search","description":"","link":"","files":"7"},{"children":[{"id":300,"parentId":283,"name":"SW00203000200","description":"","link":"","files":"2"},{"id":301,"parentId":283,"name":"SW00203000202","description":"","link":"","files":"1"},{"id":302,"parentId":283,"name":"SW00203000204","description":"","link":"","files":"3"},{"id":303,"parentId":283,"name":"SW00203000206","description":"","link":"","files":"1"},{"id":304,"parentId":283,"name":"SW00203000226","description":"","link":"","files":"1"}],"id":283,"parentId":273,"name":"travel","description":"","link":"","files":"5"}],"id":273,"parentId":272,"name":"casacos","description":"","link":"","files":"4"},{"children":[{"id":305,"parentId":274,"name":"action","description":"","link":"","files":"1","children":[{"id":306,"parentId":305,"name":"PT00168000200","description":"","link":"","files":"3"}]}],"id":274,"parentId":272,"name":"corsarios","description":"","link":"","files":"1"},{"id":275,"parentId":272,"name":"cropped","description":"","link":"","files":"3","children":[{"children":[{"id":310,"parentId":307,"name":"CP00185000524","description":"","link":"","files":"2"},{"id":311,"parentId":307,"name":"CP00185000525","description":"","link":"","files":"2"},{"id":312,"parentId":307,"name":"CP00185000526","description":"","link":"","files":"1"},{"id":313,"parentId":307,"name":"CP00185000528","description":"","link":"","files":"1"},{"id":314,"parentId":307,"name":"CP00185000529","description":"","link":"","files":"1"},{"id":315,"parentId":307,"name":"CP00185000530","description":"","link":"","files":"1"}],"id":307,"parentId":275,"name":"dawn","description":"","link":"","files":"6"},{"children":[{"id":316,"parentId":308,"name":"CP00167000200","description":"","link":"","files":"1"},{"id":317,"parentId":308,"name":"CP00167000201","description":"","link":"","files":"1"},{"id":318,"parentId":308,"name":"CP00167000203","description":"","link":"","files":"1"}],"id":308,"parentId":275,"name":"escape","description":"","link":"","files":"3"},{"children":[{"id":319,"parentId":309,"name":"CP00231000534","description":"","link":"","files":"1"},{"id":320,"parentId":309,"name":"CP00231000535","description":"","link":"","files":"3"}],"id":309,"parentId":275,"name":"impression","description":"","link":"","files":"2"}]},{"children":[{"id":321,"parentId":276,"name":"chase","description":"","link":"","files":"4","children":[{"id":333,"parentId":321,"name":"LG00169000200","description":"","link":"","files":"1"},{"id":334,"parentId":321,"name":"LG00169000510","description":"","link":"","files":"1"},{"id":335,"parentId":321,"name":"LG00169000515","description":"","link":"","files":"2"},{"id":336,"parentId":321,"name":"LG00169000516","description":"","link":"","files":"2"}]},{"id":322,"parentId":276,"name":"diamond","description":"","link":"","files":"4","children":[{"id":337,"parentId":322,"name":"LG00225000509","description":"","link":"","files":"2"},{"id":338,"parentId":322,"name":"LG00225000512","description":"","link":"","files":"1"},{"id":339,"parentId":322,"name":"LG00225000517","description":"","link":"","files":"3"},{"id":340,"parentId":322,"name":"LG00225000522","description":"","link":"","files":"1"}]},{"id":323,"parentId":276,"name":"experience","description":"","link":"","files":"3","children":[{"id":341,"parentId":323,"name":"LG00230000200","description":"","link":"","files":"4"},{"id":342,"parentId":323,"name":"LG00230000509","description":"","link":"","files":"1"},{"id":343,"parentId":323,"name":"LG00230000510","description":"","link":"","files":"1"}]},{"id":324,"parentId":276,"name":"journey","description":"","link":"","files":"5","children":[{"id":344,"parentId":324,"name":"LG00178000509","description":"","link":"","files":"1"},{"id":345,"parentId":324,"name":"LG00178000510","description":"","link":"","files":"1"},{"id":346,"parentId":324,"name":"LG00178000511","description":"","link":"","files":"1"},{"id":347,"parentId":324,"name":"LG00178000513","description":"","link":"","files":"1"},{"id":348,"parentId":324,"name":"LG00178000517","description":"","link":"","files":"3"}]},{"children":[{"id":349,"parentId":325,"name":"LG00177000200","description":"","link":"","files":"1"},{"id":350,"parentId":325,"name":"LG00177000509","description":"","link":"","files":"1"},{"id":351,"parentId":325,"name":"LG00177000510","description":"","link":"","files":"1"},{"id":352,"parentId":325,"name":"LG00177000511","description":"","link":"","files":"3"},{"id":353,"parentId":325,"name":"LG00177000512","description":"","link":"","files":"1"},{"id":354,"parentId":325,"name":"LG00177000513","description":"","link":"","files":"1"}],"id":325,"parentId":276,"name":"magnetic","description":"","link":"","files":"6"},{"children":[{"id":355,"parentId":326,"name":"LG00194000534","description":"","link":"","files":"1"},{"id":356,"parentId":326,"name":"LG00194000535","description":"","link":"","files":"4"}],"id":326,"parentId":276,"name":"perception","description":"","link":"","files":"2"},{"children":[{"id":357,"parentId":327,"name":"LG00166000200","description":"","link":"","files":"3"}],"id":327,"parentId":276,"name":"reaction","description":"","link":"","files":"1"},{"children":[{"id":358,"parentId":328,"name":"LG00195000200","description":"","link":"","files":"1"},{"id":359,"parentId":328,"name":"LG00195000201","description":"","link":"","files":"1"},{"id":360,"parentId":328,"name":"LG00195000203","description":"","link":"","files":"2"},{"id":361,"parentId":328,"name":"LG00195000510","description":"","link":"","files":"1"}],"id":328,"parentId":276,"name":"senses","description":"","link":"","files":"4"},{"children":[{"id":362,"parentId":329,"name":"LG00179000200","description":"","link":"","files":"3"}],"id":329,"parentId":276,"name":"spirit","description":"","link":"","files":"1"},{"children":[{"id":363,"parentId":330,"name":"LG00193000509","description":"","link":"","files":"1"},{"id":364,"parentId":330,"name":"LG00193000510","description":"","link":"","files":"2"},{"id":365,"parentId":330,"name":"LG00193000512","description":"","link":"","files":"1"},{"id":366,"parentId":330,"name":"LG00193000513","description":"","link":"","files":"1"},{"id":367,"parentId":330,"name":"LG00193000514","description":"","link":"","files":"1"}],"id":330,"parentId":276,"name":"stream","description":"","link":"","files":"5"},{"children":[{"id":368,"parentId":331,"name":"LG00187000200","description":"","link":"","files":"1"},{"id":369,"parentId":331,"name":"LG00187000509","description":"","link":"","files":"1"},{"id":370,"parentId":331,"name":"LG00187000510","description":"","link":"","files":"1"},{"id":371,"parentId":331,"name":"LG00187000511","description":"","link":"","files":"3"},{"id":372,"parentId":331,"name":"LG00187000512","description":"","link":"","files":"2"},{"id":373,"parentId":331,"name":"LG00187000513","description":"","link":"","files":"1"}],"id":331,"parentId":276,"name":"unik","description":"","link":"","files":"6"},{"children":[{"id":374,"parentId":332,"name":"LG00180000200","description":"","link":"","files":"1"},{"id":375,"parentId":332,"name":"LG00180000510","description":"","link":"","files":"2"},{"id":376,"parentId":332,"name":"LG00180000512","description":"","link":"","files":"2"},{"id":377,"parentId":332,"name":"LG00180000519","description":"","link":"","files":"1"}],"id":332,"parentId":276,"name":"vision","description":"","link":"","files":"4"}],"id":276,"parentId":272,"name":"leggings","description":"","link":"","files":"12"},{"children":[{"id":392,"parentId":279,"name":"dreamy","description":"","link":"","files":"3","children":[{"id":395,"parentId":392,"name":"TS00204000227","description":"","link":"","files":"2"},{"id":396,"parentId":392,"name":"TS00204000228","description":"","link":"","files":"3"},{"id":397,"parentId":392,"name":"TS00204000229","description":"","link":"","files":"2"}]},{"id":393,"parentId":279,"name":"skill","description":"","link":"","files":"6","children":[{"id":398,"parentId":393,"name":"TS00154000200","description":"","link":"","files":"1"},{"id":399,"parentId":393,"name":"TS00154000202","description":"","link":"","files":"1"},{"id":400,"parentId":393,"name":"TS00154000204","description":"","link":"","files":"1"},{"id":401,"parentId":393,"name":"TS00154000206","description":"","link":"","files":"3"},{"id":402,"parentId":393,"name":"TS00154000208","description":"","link":"","files":"1"},{"id":403,"parentId":393,"name":"TS00154000226","description":"","link":"","files":"2"}]},{"children":[{"id":404,"parentId":394,"name":"TS00205000227","description":"","link":"","files":"2"},{"id":405,"parentId":394,"name":"TS00205000228","description":"","link":"","files":"2"},{"id":406,"parentId":394,"name":"TS00205000229","description":"","link":"","files":"1"}],"id":394,"parentId":279,"name":"utopic","description":"","link":"","files":"3"}],"id":279,"parentId":272,"name":"t-shirts","description":"","link":"","files":"3"},{"children":[{"id":378,"parentId":277,"name":"dash","description":"","link":"","files":"3","children":[{"id":380,"parentId":378,"name":"TP00209000200","description":"","link":"","files":"4"},{"id":381,"parentId":378,"name":"TP00209000206","description":"","link":"","files":"1"},{"id":382,"parentId":378,"name":"TP00209000208","description":"","link":"","files":"1"}]},{"children":[{"id":383,"parentId":379,"name":"TP00159000202","description":"","link":"","files":"2"},{"id":384,"parentId":379,"name":"TP00159000204","description":"","link":"","files":"1"},{"id":385,"parentId":379,"name":"TP00159000206","description":"","link":"","files":"1"},{"id":386,"parentId":379,"name":"TP00159000226","description":"","link":"","files":"3"}],"id":379,"parentId":277,"name":"unlimited","description":"","link":"","files":"4"}],"id":277,"parentId":272,"name":"tops","description":"","link":"","files":"2"},{"children":[{"id":387,"parentId":278,"name":"conception","description":"","link":"","files":"1","children":[{"id":389,"parentId":387,"name":"TP00222000535","description":"","link":"","files":"4"}]},{"id":388,"parentId":278,"name":"jiana","description":"","link":"","files":"3","children":[{"id":606,"parentId":388,"name":"TP00161000200","description":"","link":"","files":"1"},{"id":390,"parentId":388,"name":"TP00161000201","description":"","link":"","files":"3"},{"id":391,"parentId":388,"name":"TP00161000203","description":"","link":"","files":"2"}]}],"id":278,"parentId":272,"name":"tops-curtos","description":"","link":"","files":"2"}]},{"children":[{"id":414,"parentId":407,"name":"calcoes","description":"","link":"","files":"1","children":[{"id":561,"parentId":414,"name":"piconia","description":"","link":"","files":"1","children":[{"id":562,"parentId":561,"name":"SH00116000036","description":"","link":"","files":"3"}]}]},{"children":[{"id":420,"parentId":409,"name":"alysia","description":"","link":"","files":"2","children":[{"id":421,"parentId":420,"name":"CS00098000018","description":"","link":"","files":"2"},{"id":422,"parentId":420,"name":"CS00098000036","description":"","link":"","files":"3"}]},{"children":[{"id":424,"parentId":423,"name":"CS00100000001","description":"","link":"","files":"3"}],"id":423,"parentId":409,"name":"itesya","description":"","link":"","files":"1"}],"id":409,"parentId":407,"name":"casacos","description":"","link":"","files":"2"},{"children":[{"id":417,"parentId":408,"name":"brysa","description":"","link":"","files":"2","children":[{"id":418,"parentId":417,"name":"CT00099000011","description":"","link":"","files":"1"},{"id":419,"parentId":417,"name":"CT00099000012","description":"","link":"","files":"3"}]}],"id":408,"parentId":407,"name":"coletes","description":"","link":"","files":"1"},{"children":[{"id":567,"parentId":415,"name":"corema","description":"","link":"","files":"2","children":[{"id":568,"parentId":567,"name":"PT00119000001","description":"","link":"","files":"3"},{"id":569,"parentId":567,"name":"PT00119000004","description":"","link":"","files":"1"}]},{"id":566,"parentId":415,"name":"dracas","description":"","link":"","files":"1","children":[{"id":570,"parentId":566,"name":"PT00118000038","description":"","link":"","files":"3"}]},{"id":563,"parentId":415,"name":"erica","description":"","link":"","files":"3","children":[{"id":571,"parentId":563,"name":"PT00087000019","description":"","link":"","files":"1"},{"id":572,"parentId":563,"name":"PT00087000020","description":"","link":"","files":"3"},{"id":573,"parentId":563,"name":"PT00087000039","description":"","link":"","files":"1"}]},{"id":565,"parentId":415,"name":"higia","description":"","link":"","files":"4","children":[{"id":574,"parentId":565,"name":"PT00117000019","description":"","link":"","files":"1"},{"id":595,"parentId":565,"name":"PT00117000020","description":"","link":"","files":"1"},{"id":575,"parentId":565,"name":"PT00117000022","description":"","link":"","files":"3"},{"id":596,"parentId":565,"name":"PT00117000036","description":"","link":"","files":"1"}]},{"id":564,"parentId":415,"name":"pecan","description":"","link":"","files":"3","children":[{"id":576,"parentId":564,"name":"PT00113000019","description":"","link":"","files":"2"},{"id":577,"parentId":564,"name":"PT00113000020","description":"","link":"","files":"1"},{"id":578,"parentId":564,"name":"PT00113000035","description":"","link":"","files":"2"}]}],"id":415,"parentId":407,"name":"corsarios","description":"","link":"","files":"5"},{"children":[{"id":579,"parentId":416,"name":"anona","description":"","link":"","files":"2","children":[{"id":582,"parentId":579,"name":"CP00094000054","description":"","link":"","files":"1"},{"id":583,"parentId":579,"name":"CP00094000055","description":"","link":"","files":"3"}]},{"children":[{"id":584,"parentId":580,"name":"CP00114000041","description":"","link":"","files":"3"},{"id":585,"parentId":580,"name":"CP00114000043","description":"","link":"","files":"1"}],"id":580,"parentId":416,"name":"olaia","description":"","link":"","files":"2"},{"children":[{"id":586,"parentId":581,"name":"CP00115000019","description":"","link":"","files":"2"},{"id":587,"parentId":581,"name":"CP00115000020","description":"","link":"","files":"1"},{"id":588,"parentId":581,"name":"CP00115000035","description":"","link":"","files":"1"},{"id":589,"parentId":581,"name":"CP00115000036","description":"","link":"","files":"2"}],"id":581,"parentId":416,"name":"wysteria","description":"","link":"","files":"4"}],"id":416,"parentId":407,"name":"cropped","description":"","link":"","files":"3"},{"children":[{"id":515,"parentId":413,"name":"bacatas","description":"","link":"","files":"4","children":[{"id":520,"parentId":515,"name":"LG00093000020","description":"","link":"","files":"2"},{"id":521,"parentId":515,"name":"LG00093000035","description":"","link":"","files":"2"},{"id":522,"parentId":515,"name":"LG00093000036","description":"","link":"","files":"2"},{"id":523,"parentId":515,"name":"LG00093000039","description":"","link":"","files":"1"}]},{"id":514,"parentId":413,"name":"faya","description":"","link":"","files":"4","children":[{"id":524,"parentId":514,"name":"LG00120000020","description":"","link":"","files":"1"},{"id":525,"parentId":514,"name":"LG00120000035","description":"","link":"","files":"2"},{"id":526,"parentId":514,"name":"LG00120000036","description":"","link":"","files":"1"},{"id":527,"parentId":514,"name":"LG00120000037","description":"","link":"","files":"2"}]},{"id":512,"parentId":413,"name":"gardena","description":"","link":"","files":"4","children":[{"id":528,"parentId":512,"name":"LG00077000047","description":"","link":"","files":"2"},{"id":529,"parentId":512,"name":"LG00077000049","description":"","link":"","files":"2"},{"id":530,"parentId":512,"name":"LG00077000051","description":"","link":"","files":"2"},{"id":531,"parentId":512,"name":"LG00077000053","description":"","link":"","files":"1"}]},{"id":513,"parentId":413,"name":"hera","description":"","link":"","files":"4","children":[{"id":532,"parentId":513,"name":"LG00075000019","description":"","link":"","files":"1"},{"id":533,"parentId":513,"name":"LG00075000035","description":"","link":"","files":"1"},{"id":534,"parentId":513,"name":"LG00075000036","description":"","link":"","files":"3"},{"id":535,"parentId":513,"name":"LG00075000037","description":"","link":"","files":"1"}]},{"id":516,"parentId":413,"name":"juni","description":"","link":"","files":"1","children":[{"id":536,"parentId":516,"name":"LG00078000043","description":"","link":"","files":"3"}]},{"children":[{"id":537,"parentId":517,"name":"LG00070000040","description":"","link":"","files":"3"}],"id":517,"parentId":413,"name":"magnolia","description":"","link":"","files":"1"},{"children":[{"id":538,"parentId":511,"name":"LG00083000025","description":"","link":"","files":"3"},{"id":539,"parentId":511,"name":"LG00083000036","description":"","link":"","files":"1"}],"id":511,"parentId":413,"name":"marja","description":"","link":"","files":"2"},{"children":[{"id":540,"parentId":509,"name":"LG00092000019","description":"","link":"","files":"1"},{"id":541,"parentId":509,"name":"LG00092000022","description":"","link":"","files":"2"},{"id":542,"parentId":509,"name":"LG00092000036","description":"","link":"","files":"2"},{"id":543,"parentId":509,"name":"LG00092000037","description":"","link":"","files":"1"},{"id":544,"parentId":509,"name":"LG00092000039","description":"","link":"","files":"1"}],"id":509,"parentId":413,"name":"melia","description":"","link":"","files":"5"},{"children":[{"id":545,"parentId":518,"name":"LG00071000045","description":"","link":"","files":"3"},{"id":546,"parentId":518,"name":"LG00071000046","description":"","link":"","files":"3"},{"id":547,"parentId":518,"name":"LG00071000052","description":"","link":"","files":"3"}],"id":518,"parentId":413,"name":"palma","description":"","link":"","files":"3"},{"children":[{"id":548,"parentId":510,"name":"LG00095000054","description":"","link":"","files":"1"},{"id":549,"parentId":510,"name":"LG00095000057","description":"","link":"","files":"3"},{"id":550,"parentId":510,"name":"LG00095000059","description":"","link":"","files":"1"}],"id":510,"parentId":413,"name":"pataluga","description":"","link":"","files":"3"},{"children":[{"id":551,"parentId":508,"name":"LG00088000020","description":"","link":"","files":"3"},{"id":552,"parentId":508,"name":"LG00088000035","description":"","link":"","files":"1"},{"id":553,"parentId":508,"name":"LG00088000036","description":"","link":"","files":"1"},{"id":554,"parentId":508,"name":"LG00088000037","description":"","link":"","files":"1"},{"id":555,"parentId":508,"name":"LG00088000039","description":"","link":"","files":"1"}],"id":508,"parentId":413,"name":"prunny","description":"","link":"","files":"5"},{"children":[{"id":556,"parentId":519,"name":"LG00091000018","description":"","link":"","files":"1"},{"id":557,"parentId":519,"name":"LG00091000019","description":"","link":"","files":"2"},{"id":558,"parentId":519,"name":"LG00091000020","description":"","link":"","files":"2"},{"id":559,"parentId":519,"name":"LG00091000035","description":"","link":"","files":"2"},{"id":560,"parentId":519,"name":"LG00091000036","description":"","link":"","files":"1"}],"id":519,"parentId":413,"name":"vidalia","description":"","link":"","files":"5"}],"id":413,"parentId":407,"name":"leggings","description":"","link":"","files":"12"},{"children":[{"id":427,"parentId":410,"name":"furna","description":"","link":"","files":"3","children":[{"id":428,"parentId":427,"name":"MC00112000019","description":"","link":"","files":"1"},{"id":429,"parentId":427,"name":"MC00112000036","description":"","link":"","files":"3"},{"id":430,"parentId":427,"name":"MC00112000037","description":"","link":"","files":"1"}]},{"id":426,"parentId":410,"name":"magma","description":"","link":"","files":"3","children":[{"id":431,"parentId":426,"name":"MC00090000041","description":"","link":"","files":"2"},{"id":432,"parentId":426,"name":"MC00090000042","description":"","link":"","files":"3"},{"id":433,"parentId":426,"name":"MC00090000043","description":"","link":"","files":"2"}]},{"children":[{"id":434,"parentId":425,"name":"MC00073000001","description":"","link":"","files":"3"}],"id":425,"parentId":410,"name":"zelina","description":"","link":"","files":"1"}],"id":410,"parentId":407,"name":"macacoes","description":"","link":"","files":"3"},{"children":[{"id":437,"parentId":411,"name":"bersya","description":"","link":"","files":"1","children":[{"id":442,"parentId":437,"name":"TP00065000137","description":"","link":"","files":"3"}]},{"id":436,"parentId":411,"name":"elina","description":"","link":"","files":"4","children":[{"id":443,"parentId":436,"name":"TS00105000001","description":"","link":"","files":"1"},{"id":444,"parentId":436,"name":"TS00105000010","description":"","link":"","files":"1"},{"id":445,"parentId":436,"name":"TS00105000011","description":"","link":"","files":"1"},{"id":446,"parentId":436,"name":"TS00105000012","description":"","link":"","files":"2"}]},{"id":435,"parentId":411,"name":"histri","description":"","link":"","files":"2","children":[{"id":447,"parentId":435,"name":"TS00084000001","description":"","link":"","files":"3"},{"id":448,"parentId":435,"name":"TS00084000012","description":"","link":"","files":"2"}]},{"id":438,"parentId":411,"name":"lampas","description":"","link":"","files":"2","children":[{"id":449,"parentId":438,"name":"TP00085000001","description":"","link":"","files":"2"},{"id":450,"parentId":438,"name":"TP00085000011","description":"","link":"","files":"3"}]}],"id":411,"parentId":407,"name":"t-shirts","description":"","link":"","files":"4"},{"children":[{"id":441,"parentId":412,"name":"armata","description":"","link":"","files":"2","children":[{"id":593,"parentId":441,"name":"TP00104000009","description":"","link":"","files":"2"},{"id":464,"parentId":441,"name":"TP00104000019","description":"","link":"","files":"2"}]},{"id":462,"parentId":412,"name":"aster","description":"","link":"","files":"5","children":[{"id":465,"parentId":462,"name":"TP00097000010","description":"","link":"","files":"2"},{"id":466,"parentId":462,"name":"TP00097000011","description":"","link":"","files":"1"},{"id":467,"parentId":462,"name":"TP00097000012","description":"","link":"","files":"1"},{"id":468,"parentId":462,"name":"TP00097000018","description":"","link":"","files":"1"},{"id":469,"parentId":462,"name":"TP00097000039","description":"","link":"","files":"1"}]},{"id":455,"parentId":412,"name":"atlantis","description":"","link":"","files":"2","children":[{"id":470,"parentId":455,"name":"TP00079000008","description":"","link":"","files":"1"},{"id":471,"parentId":455,"name":"TP00079000011","description":"","link":"","files":"3"}]},{"id":439,"parentId":412,"name":"callista","description":"","link":"","files":"2","children":[{"id":472,"parentId":439,"name":"TP00103000001","description":"","link":"","files":"3"},{"id":594,"parentId":439,"name":"TP00103000012","description":"","link":"","files":"1"}]},{"id":453,"parentId":412,"name":"dasy","description":"","link":"","files":"1","children":[{"id":473,"parentId":453,"name":"TP00122000001","description":"","link":"","files":"1"}]},{"id":454,"parentId":412,"name":"dolfana","description":"","link":"","files":"4","children":[{"id":474,"parentId":454,"name":"TP00081000001","description":"","link":"","files":"1"},{"id":475,"parentId":454,"name":"TP00081000011","description":"","link":"","files":"1"},{"id":476,"parentId":454,"name":"TP00081000012","description":"","link":"","files":"1"},{"id":477,"parentId":454,"name":"TP00081000018","description":"","link":"","files":"2"}]},{"id":459,"parentId":412,"name":"hidra","description":"","link":"","files":"1","children":[{"id":478,"parentId":459,"name":"TP00033000001","description":"","link":"","files":"3"}]},{"id":440,"parentId":412,"name":"lagoa","description":"","link":"","files":"4","children":[{"id":479,"parentId":440,"name":"TP00063000126","description":"","link":"","files":"1"},{"id":480,"parentId":440,"name":"TP00063000127","description":"","link":"","files":"1"},{"id":481,"parentId":440,"name":"TP00063000128","description":"","link":"","files":"1"},{"id":482,"parentId":440,"name":"TP00063000129","description":"","link":"","files":"3"}]},{"id":463,"parentId":412,"name":"marina","description":"","link":"","files":"2","children":[{"id":483,"parentId":463,"name":"TP00101000041","description":"","link":"","files":"2"},{"id":484,"parentId":463,"name":"TP00101000043","description":"","link":"","files":"3"}]},{"id":452,"parentId":412,"name":"naria","description":"","link":"","files":"1","children":[{"id":485,"parentId":452,"name":"TP00102000018","description":"","link":"","files":"3"}]},{"id":457,"parentId":412,"name":"nidaria","description":"","link":"","files":"4","children":[{"id":486,"parentId":457,"name":"TP00076000019","description":"","link":"","files":"2"},{"id":487,"parentId":457,"name":"TP00076000035","description":"","link":"","files":"2"},{"id":488,"parentId":457,"name":"TP00076000036","description":"","link":"","files":"2"},{"id":489,"parentId":457,"name":"TP00076000037","description":"","link":"","files":"1"}]},{"id":461,"parentId":412,"name":"padina","description":"","link":"","files":"5","children":[{"id":490,"parentId":461,"name":"TP00082000001","description":"","link":"","files":"1"},{"id":491,"parentId":461,"name":"TP00082000003","description":"","link":"","files":"1"},{"id":492,"parentId":461,"name":"TP00082000005","description":"","link":"","files":"1"},{"id":493,"parentId":461,"name":"TP00082000008","description":"","link":"","files":"2"},{"id":494,"parentId":461,"name":"TP00082000019","description":"","link":"","files":"2"}]},{"id":460,"parentId":412,"name":"petula","description":"","link":"","files":"1","children":[{"id":495,"parentId":460,"name":"TP00032000105","description":"","link":"","files":"2"}]},{"id":458,"parentId":412,"name":"tasya","description":"","link":"","files":"7","children":[{"id":496,"parentId":458,"name":"TP00110000001","description":"","link":"","files":"1"},{"id":497,"parentId":458,"name":"TP00110000005","description":"","link":"","files":"1"},{"id":498,"parentId":458,"name":"TP00110000089","description":"","link":"","files":"1"},{"id":499,"parentId":458,"name":"TP00110000090","description":"","link":"","files":"1"},{"id":500,"parentId":458,"name":"TP00110000091","description":"","link":"","files":"1"},{"id":501,"parentId":458,"name":"TP00110000092","description":"","link":"","files":"1"},{"id":502,"parentId":458,"name":"TP00110000093","description":"","link":"","files":"1"}]},{"children":[{"id":503,"parentId":451,"name":"TP00108000001","description":"","link":"","files":"3"}],"id":451,"parentId":412,"name":"valonia","description":"","link":"","files":"1"},{"children":[{"id":504,"parentId":456,"name":"TP00074000010","description":"","link":"","files":"2"},{"id":505,"parentId":456,"name":"TP00074000012","description":"","link":"","files":"1"},{"id":506,"parentId":456,"name":"TP00074000019","description":"","link":"","files":"1"},{"id":507,"parentId":456,"name":"TP00074000037","description":"","link":"","files":"2"}],"id":456,"parentId":412,"name":"zomaria","description":"","link":"","files":"4"}],"id":412,"parentId":407,"name":"tops","description":"","link":"","files":"16"}],"id":407,"parentId":590,"name":"outlet","description":"","link":"","files":"9"},{"children":[{"id":202,"parentId":201,"name":"calcoes","description":"","link":"","files":"2","children":[{"id":203,"parentId":202,"name":"deline","description":"","link":"","files":"4","children":[{"id":205,"parentId":203,"name":"SH00235000510","description":"","link":"","files":"3"},{"id":206,"parentId":203,"name":"SH00235000520","description":"","link":"","files":"1"},{"id":207,"parentId":203,"name":"SH00235000544","description":"","link":"","files":"1"},{"id":208,"parentId":203,"name":"SH00235000545","description":"","link":"","files":"1"}]},{"children":[{"id":209,"parentId":204,"name":"SH00233000200","description":"","link":"","files":"1"},{"id":210,"parentId":204,"name":"SH00233000510","description":"","link":"","files":"1"},{"id":211,"parentId":204,"name":"SH00233000512","description":"","link":"","files":"3"},{"id":212,"parentId":204,"name":"SH00233000514","description":"","link":"","files":"1"},{"id":213,"parentId":204,"name":"SH00233000517","description":"","link":"","files":"1"},{"id":214,"parentId":204,"name":"SH00233000519","description":"","link":"","files":"1"}],"id":204,"parentId":202,"name":"shape","description":"","link":"","files":"6"}]},{"children":[{"id":216,"parentId":215,"name":"gentle","description":"","link":"","files":"8","children":[{"id":224,"parentId":216,"name":"SC00238000200","description":"","link":"","files":"1"},{"id":225,"parentId":216,"name":"SC00238000202","description":"","link":"","files":"1"},{"id":226,"parentId":216,"name":"SC00238000204","description":"","link":"","files":"1"},{"id":227,"parentId":216,"name":"SC00238000205","description":"","link":"","files":"1"},{"id":228,"parentId":216,"name":"SC00238000206","description":"","link":"","files":"1"},{"id":229,"parentId":216,"name":"SC00238000207","description":"","link":"","files":"1"},{"id":230,"parentId":216,"name":"SC00238000208","description":"","link":"","files":"2"},{"id":231,"parentId":216,"name":"SC00238000224","description":"","link":"","files":"1"}]},{"id":217,"parentId":215,"name":"idol","description":"","link":"","files":"2","children":[{"id":232,"parentId":217,"name":"SC00199000216","description":"","link":"","files":"1"},{"id":233,"parentId":217,"name":"SC00199000217","description":"","link":"","files":"1"}]},{"id":218,"parentId":215,"name":"jewel","description":"","link":"","files":"4","children":[{"id":234,"parentId":218,"name":"SC00198000200","description":"","link":"","files":"1"},{"id":235,"parentId":218,"name":"SC00198000201","description":"","link":"","files":"1"},{"id":236,"parentId":218,"name":"SC00198000203","description":"","link":"","files":"1"},{"id":237,"parentId":218,"name":"SC00198000208","description":"","link":"","files":"1"}]},{"id":219,"parentId":215,"name":"play","description":"","link":"","files":"3","children":[{"id":238,"parentId":219,"name":"SC00197000510","description":"","link":"","files":"1"},{"id":239,"parentId":219,"name":"SC00197000520","description":"","link":"","files":"3"},{"id":240,"parentId":219,"name":"SC00197000521","description":"","link":"","files":"1"}]},{"id":220,"parentId":215,"name":"sand","description":"","link":"","files":"2","children":[{"id":241,"parentId":220,"name":"SC00236000219","description":"","link":"","files":"2"},{"id":242,"parentId":220,"name":"SC00236000221","description":"","link":"","files":"1"}]},{"children":[{"id":243,"parentId":221,"name":"SC00237000201","description":"","link":"","files":"1"},{"id":244,"parentId":221,"name":"SC00237000203","description":"","link":"","files":"1"},{"id":245,"parentId":221,"name":"SC00237000224","description":"","link":"","files":"1"}],"id":221,"parentId":215,"name":"slash","description":"","link":"","files":"3"},{"children":[{"id":246,"parentId":222,"name":"SC00201000200","description":"","link":"","files":"1"},{"id":247,"parentId":222,"name":"SC00201000201","description":"","link":"","files":"1"},{"id":248,"parentId":222,"name":"SC00201000203","description":"","link":"","files":"1"},{"id":249,"parentId":222,"name":"SC00201000208","description":"","link":"","files":"1"}],"id":222,"parentId":215,"name":"smash","description":"","link":"","files":"4"},{"children":[{"id":250,"parentId":223,"name":"SC00200000200","description":"","link":"","files":"1"},{"id":251,"parentId":223,"name":"SC00200000201","description":"","link":"","files":"2"},{"id":252,"parentId":223,"name":"SC00200000203","description":"","link":"","files":"1"},{"id":253,"parentId":223,"name":"SC00200000208","description":"","link":"","files":"1"},{"id":254,"parentId":223,"name":"SC00200000224","description":"","link":"","files":"1"},{"id":255,"parentId":223,"name":"SC00200000226","description":"","link":"","files":"1"}],"id":223,"parentId":215,"name":"sway","description":"","link":"","files":"6"}],"id":215,"parentId":201,"name":"saias","description":"","link":"","files":"8"},{"children":[{"id":264,"parentId":263,"name":"haste","description":"","link":"","files":"1","children":[{"id":265,"parentId":264,"name":"TS00208000200","description":"","link":"","files":"2"}]}],"id":263,"parentId":201,"name":"t-shirts","description":"","link":"","files":"1"},{"children":[{"id":257,"parentId":256,"name":"flux","description":"","link":"","files":"2","children":[{"id":259,"parentId":257,"name":"TP00211000200","description":"","link":"","files":"2"},{"id":599,"parentId":257,"name":"TP00211000207","description":"","link":"","files":"1"}]},{"id":258,"parentId":256,"name":"serene","description":"","link":"","files":"3","children":[{"id":260,"parentId":258,"name":"TP00217000539","description":"","link":"","files":"2"},{"id":261,"parentId":258,"name":"TP00217000541","description":"","link":"","files":"1"},{"id":262,"parentId":258,"name":"TP00217000542","description":"","link":"","files":"1"}]}],"id":256,"parentId":201,"name":"tops","description":"","link":"","files":"2"},{"children":[{"id":267,"parentId":266,"name":"vally","description":"","link":"","files":"4","children":[{"id":268,"parentId":267,"name":"VT00239000200","description":"","link":"","files":"1"},{"id":269,"parentId":267,"name":"VT00239000202","description":"","link":"","files":"1"},{"id":270,"parentId":267,"name":"VT00239000206","description":"","link":"","files":"4"},{"id":271,"parentId":267,"name":"VT00239000208","description":"","link":"","files":"1"}]}],"id":266,"parentId":201,"name":"vestidos","description":"","link":"","files":"1"}],"id":201,"parentId":590,"name":"paddle-tennis","description":"","link":"","files":"5"},{"children":[{"id":7,"parentId":6,"name":"casacos","description":"","link":"","files":"1","children":[{"children":[{"id":9,"parentId":8,"name":"CS00202000200","description":"","link":"","files":"2"},{"id":10,"parentId":8,"name":"CS00202000203","description":"","link":"","files":"2"},{"id":11,"parentId":8,"name":"CS00202000208","description":"","link":"","files":"2"},{"id":12,"parentId":8,"name":"CS00202000224","description":"","link":"","files":"3"}],"id":8,"parentId":7,"name":"sparkle","description":"","link":"","files":"4"}]},{"children":[{"id":82,"parentId":81,"name":"accolade","description":"","link":"","files":"6","children":[{"id":86,"parentId":82,"name":"PT00183000524","description":"","link":"","files":"1"},{"id":87,"parentId":82,"name":"PT00183000525","description":"","link":"","files":"1"},{"id":88,"parentId":82,"name":"PT00183000526","description":"","link":"","files":"1"},{"id":89,"parentId":82,"name":"PT00183000528","description":"","link":"","files":"3"},{"id":90,"parentId":82,"name":"PT00183000529","description":"","link":"","files":"1"},{"id":91,"parentId":82,"name":"PT00183000530","description":"","link":"","files":"2"}]},{"id":83,"parentId":81,"name":"gala","description":"","link":"","files":"4","children":[{"id":92,"parentId":83,"name":"PT00234000200","description":"","link":"","files":"1"},{"id":93,"parentId":83,"name":"PT00234000509","description":"","link":"","files":"2"},{"id":94,"parentId":83,"name":"PT00234000510","description":"","link":"","files":"1"},{"id":95,"parentId":83,"name":"PT00234000513","description":"","link":"","files":"2"}]},{"id":84,"parentId":81,"name":"lumy","description":"","link":"","files":"3","children":[{"id":96,"parentId":84,"name":"PT00174000200","description":"","link":"","files":"1"},{"id":97,"parentId":84,"name":"PT00174000512","description":"","link":"","files":"1"},{"id":98,"parentId":84,"name":"PT00174000519","description":"","link":"","files":"3"}]},{"children":[{"id":99,"parentId":85,"name":"PT00175000504","description":"","link":"","files":"2"},{"id":100,"parentId":85,"name":"PT00175000510","description":"","link":"","files":"1"},{"id":101,"parentId":85,"name":"PT00175000520","description":"","link":"","files":"1"},{"id":102,"parentId":85,"name":"PT00175000521","description":"","link":"","files":"3"}],"id":85,"parentId":81,"name":"world","description":"","link":"","files":"4"}],"id":81,"parentId":6,"name":"corsarios","description":"","link":"","files":"4"},{"children":[{"id":104,"parentId":103,"name":"brume","description":"","link":"","files":"5","children":[{"id":109,"parentId":104,"name":"CP00173000200","description":"","link":"","files":"1"},{"id":110,"parentId":104,"name":"CP00173000509","description":"","link":"","files":"1"},{"id":111,"parentId":104,"name":"CP00173000510","description":"","link":"","files":"2"},{"id":112,"parentId":104,"name":"CP00173000513","description":"","link":"","files":"1"},{"id":113,"parentId":104,"name":"CP00173000523","description":"","link":"","files":"1"}]},{"children":[{"id":114,"parentId":105,"name":"CP00176000509","description":"","link":"","files":"1"},{"id":115,"parentId":105,"name":"CP00176000510","description":"","link":"","files":"2"},{"id":116,"parentId":105,"name":"CP00176000512","description":"","link":"","files":"2"},{"id":117,"parentId":105,"name":"CP00176000513","description":"","link":"","files":"1"},{"id":118,"parentId":105,"name":"CP00176000517","description":"","link":"","files":"1"}],"id":105,"parentId":103,"name":"hunt","description":"","link":"","files":"5"},{"children":[{"id":119,"parentId":106,"name":"CP00196000200","description":"","link":"","files":"2"}],"id":106,"parentId":103,"name":"magny","description":"","link":"","files":"1"},{"children":[{"id":120,"parentId":107,"name":"CP00184000510","description":"","link":"","files":"1"},{"id":121,"parentId":107,"name":"CP00184000512","description":"","link":"","files":"1"},{"id":122,"parentId":107,"name":"CP00184000513","description":"","link":"","files":"1"},{"id":123,"parentId":107,"name":"CP00184000517","description":"","link":"","files":"2"}],"id":107,"parentId":103,"name":"risk","description":"","link":"","files":"4"},{"children":[{"id":124,"parentId":108,"name":"CP00232000510","description":"","link":"","files":"1"},{"id":125,"parentId":108,"name":"CP00232000512","description":"","link":"","files":"1"},{"id":126,"parentId":108,"name":"CP00232000513","description":"","link":"","files":"1"},{"id":127,"parentId":108,"name":"CP00232000517","description":"","link":"","files":"2"}],"id":108,"parentId":103,"name":"wager","description":"","link":"","files":"4"}],"id":103,"parentId":6,"name":"cropped","description":"","link":"","files":"5"},{"children":[{"id":14,"parentId":13,"name":"bewitch","description":"","link":"","files":"5","children":[{"id":15,"parentId":14,"name":"LG00186000200","description":"","link":"","files":"1"},{"id":598,"parentId":14,"name":"LG00186000509","description":"","link":"","files":"1"},{"id":16,"parentId":14,"name":"LG00186000510","description":"","link":"","files":"1"},{"id":18,"parentId":14,"name":"LG00186000512","description":"","link":"","files":"1"},{"id":17,"parentId":14,"name":"LG00186000513","description":"","link":"","files":"2"}]},{"id":20,"parentId":13,"name":"dazzle","description":"","link":"","files":"1","children":[{"id":21,"parentId":20,"name":"LG00171000222","description":"","link":"","files":"4"}]},{"id":22,"parentId":13,"name":"extent","description":"","link":"","files":"4","children":[{"id":23,"parentId":22,"name":"LG00223000509","description":"","link":"","files":"2"},{"id":24,"parentId":22,"name":"LG00223000510","description":"","link":"","files":"1"},{"id":25,"parentId":22,"name":"LG00223000513","description":"","link":"","files":"1"},{"id":26,"parentId":22,"name":"LG00223000522","description":"","link":"","files":"1"}]},{"id":27,"parentId":13,"name":"focus","description":"","link":"","files":"4","children":[{"id":28,"parentId":27,"name":"LG00170000504","description":"","link":"","files":"1"},{"id":29,"parentId":27,"name":"LG00170000510","description":"","link":"","files":"2"},{"id":30,"parentId":27,"name":"LG00170000514","description":"","link":"","files":"3"},{"id":31,"parentId":27,"name":"LG00170000518","description":"","link":"","files":"1"}]},{"id":32,"parentId":13,"name":"laurel","description":"","link":"","files":"1","children":[{"id":44,"parentId":32,"name":"LG00226000518","description":"","link":"","files":"1"}]},{"children":[{"id":45,"parentId":33,"name":"LG00163000505","description":"","link":"","files":"1"},{"id":46,"parentId":33,"name":"LG00163000506","description":"","link":"","files":"1"},{"id":47,"parentId":33,"name":"LG00163000507","description":"","link":"","files":"1"},{"id":48,"parentId":33,"name":"LG00163000508","description":"","link":"","files":"1"}],"id":33,"parentId":13,"name":"limit","description":"","link":"","files":"4"},{"children":[{"id":49,"parentId":34,"name":"LG00188000532","description":"","link":"","files":"1"},{"id":50,"parentId":34,"name":"LG00188000533","description":"","link":"","files":"1"},{"id":51,"parentId":34,"name":"LG00188000537","description":"","link":"","files":"1"}],"id":34,"parentId":13,"name":"merizon","description":"","link":"","files":"3"},{"children":[{"id":52,"parentId":35,"name":"LG00228000211","description":"","link":"","files":"1"},{"id":53,"parentId":35,"name":"LG00228000212","description":"","link":"","files":"1"},{"id":54,"parentId":35,"name":"LG00228000213","description":"","link":"","files":"2"},{"id":55,"parentId":35,"name":"LG00228000215","description":"","link":"","files":"1"}],"id":35,"parentId":13,"name":"mild","description":"","link":"","files":"4"},{"children":[{"id":56,"parentId":36,"name":"LG00172000218","description":"","link":"","files":"3"},{"id":57,"parentId":36,"name":"LG00172000219","description":"","link":"","files":"2"}],"id":36,"parentId":13,"name":"mizzy","description":"","link":"","files":"2"},{"children":[{"id":58,"parentId":37,"name":"LG00165000200","description":"","link":"","files":"1"},{"id":59,"parentId":37,"name":"LG00165000510","description":"","link":"","files":"2"},{"id":60,"parentId":37,"name":"LG00165000514","description":"","link":"","files":"3"}],"id":37,"parentId":13,"name":"percy","description":"","link":"","files":"3"},{"children":[{"id":61,"parentId":38,"name":"LG00229000220","description":"","link":"","files":"1"},{"id":62,"parentId":38,"name":"LG00229000221","description":"","link":"","files":"2"}],"id":38,"parentId":13,"name":"posh","description":"","link":"","files":"2"},{"children":[{"id":63,"parentId":39,"name":"LG00190000509","description":"","link":"","files":"1"},{"id":64,"parentId":39,"name":"LG00190000510","description":"","link":"","files":"1"},{"id":65,"parentId":39,"name":"LG00190000513","description":"","link":"","files":"1"}],"id":39,"parentId":13,"name":"prime","description":"","link":"","files":"3"},{"children":[{"id":66,"parentId":40,"name":"LG00162000500","description":"","link":"","files":"2"},{"id":67,"parentId":40,"name":"LG00162000501","description":"","link":"","files":"1"},{"id":68,"parentId":40,"name":"LG00162000502","description":"","link":"","files":"1"},{"id":69,"parentId":40,"name":"LG00162000503","description":"","link":"","files":"1"}],"id":40,"parentId":13,"name":"routine","description":"","link":"","files":"4"},{"children":[{"id":70,"parentId":41,"name":"LG00227000200","description":"","link":"","files":"1"},{"id":71,"parentId":41,"name":"LG00227000509","description":"","link":"","files":"1"},{"id":72,"parentId":41,"name":"LG00227000510","description":"","link":"","files":"1"},{"id":73,"parentId":41,"name":"LG00227000512","description":"","link":"","files":"2"},{"id":74,"parentId":41,"name":"LG00227000513","description":"","link":"","files":"1"}],"id":41,"parentId":13,"name":"spell","description":"","link":"","files":"5"},{"children":[{"id":75,"parentId":42,"name":"LG00189000200","description":"","link":"","files":"1"},{"id":76,"parentId":42,"name":"LG00189000201","description":"","link":"","files":"4"},{"id":77,"parentId":42,"name":"LG00189000208","description":"","link":"","files":"1"}],"id":42,"parentId":13,"name":"trim","description":"","link":"","files":"3"},{"children":[{"id":78,"parentId":43,"name":"LG00224000510","description":"","link":"","files":"1"},{"id":79,"parentId":43,"name":"LG00224000512","description":"","link":"","files":"1"},{"id":80,"parentId":43,"name":"LG00224000517","description":"","link":"","files":"3"}],"id":43,"parentId":13,"name":"twist","description":"","link":"","files":"3"}],"id":13,"parentId":6,"name":"leggings","description":"","link":"","files":"16"},{"children":[{"id":129,"parentId":128,"name":"edge","description":"","link":"","files":"1","children":[{"id":130,"parentId":129,"name":"MC00240000200","description":"","link":"","files":"2"}]}],"id":128,"parentId":6,"name":"macacoes","description":"","link":"","files":"1"},{"children":[{"id":132,"parentId":131,"name":"brink","description":"","link":"","files":"2","children":[{"id":144,"parentId":132,"name":"TP00212000200","description":"","link":"","files":"1"},{"id":145,"parentId":132,"name":"TP00212000208","description":"","link":"","files":"2"}]},{"id":133,"parentId":131,"name":"charisma","description":"","link":"","files":"2","children":[{"id":146,"parentId":133,"name":"TP00216000200","description":"","link":"","files":"1"},{"id":147,"parentId":133,"name":"TP00216000230","description":"","link":"","files":"2"}]},{"id":134,"parentId":131,"name":"charm","description":"","link":"","files":"2","children":[{"id":148,"parentId":134,"name":"TP00214000200","description":"","link":"","files":"2"},{"id":600,"parentId":134,"name":"TP00214000208","description":"","link":"","files":"1"}]},{"id":135,"parentId":131,"name":"dauntless","description":"","link":"","files":"3","children":[{"id":149,"parentId":135,"name":"TP00219000204","description":"","link":"","files":"2"},{"id":150,"parentId":135,"name":"TP00219000206","description":"","link":"","files":"1"},{"id":151,"parentId":135,"name":"TP00219000208","description":"","link":"","files":"1"}]},{"id":136,"parentId":131,"name":"enchant","description":"","link":"","files":"2","children":[{"id":152,"parentId":136,"name":"TP00157000211","description":"","link":"","files":"2"},{"id":153,"parentId":136,"name":"TP00157000213","description":"","link":"","files":"2"}]},{"id":137,"parentId":131,"name":"force","description":"","link":"","files":"4","children":[{"id":154,"parentId":137,"name":"TP00160000200","description":"","link":"","files":"1"},{"id":155,"parentId":137,"name":"TP00160000202","description":"","link":"","files":"1"},{"id":156,"parentId":137,"name":"TP00160000206","description":"","link":"","files":"1"},{"id":157,"parentId":137,"name":"TP00160000208","description":"","link":"","files":"2"}]},{"id":138,"parentId":131,"name":"linger","description":"","link":"","files":"5","children":[{"id":158,"parentId":138,"name":"TP00215000200","description":"","link":"","files":"2"},{"id":159,"parentId":138,"name":"TP00215000202","description":"","link":"","files":"2"},{"id":160,"parentId":138,"name":"TP00215000204","description":"","link":"","files":"1"},{"id":161,"parentId":138,"name":"TP00215000205","description":"","link":"","files":"1"},{"id":162,"parentId":138,"name":"TP00215000206","description":"","link":"","files":"1"}]},{"id":139,"parentId":131,"name":"origin","description":"","link":"","files":"4","children":[{"id":163,"parentId":139,"name":"TP00155000211","description":"","link":"","files":"2"},{"id":164,"parentId":139,"name":"TP00155000212","description":"","link":"","files":"2"},{"id":165,"parentId":139,"name":"TP00155000213","description":"","link":"","files":"1"},{"id":166,"parentId":139,"name":"TP00155000215","description":"","link":"","files":"1"}]},{"id":140,"parentId":131,"name":"poise","description":"","link":"","files":"4","children":[{"id":167,"parentId":140,"name":"TP00213000200","description":"","link":"","files":"1"},{"id":168,"parentId":140,"name":"TP00213000205","description":"","link":"","files":"1"},{"id":169,"parentId":140,"name":"TP00213000206","description":"","link":"","files":"1"},{"id":170,"parentId":140,"name":"TP00213000207","description":"","link":"","files":"1"}]},{"id":141,"parentId":131,"name":"special","description":"","link":"","files":"3","children":[{"id":171,"parentId":141,"name":"TP00158000202","description":"","link":"","files":"1"},{"id":172,"parentId":141,"name":"TP00158000204","description":"","link":"","files":"1"},{"id":173,"parentId":141,"name":"TP00158000208","description":"","link":"","files":"1"}]},{"id":142,"parentId":131,"name":"swank","description":"","link":"","files":"5","children":[{"id":174,"parentId":142,"name":"TP00218000200","description":"","link":"","files":"1"},{"id":175,"parentId":142,"name":"TP00218000201","description":"","link":"","files":"3"},{"id":176,"parentId":142,"name":"TP00218000202","description":"","link":"","files":"1"},{"id":177,"parentId":142,"name":"TP00218000204","description":"","link":"","files":"1"},{"id":178,"parentId":142,"name":"TP00218000208","description":"","link":"","files":"2"}]},{"children":[{"id":179,"parentId":143,"name":"TP00210000200","description":"","link":"","files":"1"},{"id":180,"parentId":143,"name":"TP00210000208","description":"","link":"","files":"3"}],"id":143,"parentId":131,"name":"verge","description":"","link":"","files":"2"}],"id":131,"parentId":6,"name":"tops","description":"","link":"","files":"12"},{"children":[{"id":182,"parentId":181,"name":"fearless","description":"","link":"","files":"3","children":[{"id":185,"parentId":182,"name":"TP00181000200","description":"","link":"","files":"2"},{"id":186,"parentId":182,"name":"TP00181000510","description":"","link":"","files":"2"},{"id":187,"parentId":182,"name":"TP00181000514","description":"","link":"","files":"1"}]},{"id":183,"parentId":181,"name":"feather","description":"","link":"","files":"3","children":[{"id":188,"parentId":183,"name":"TP00220000202","description":"","link":"","files":"1"},{"id":189,"parentId":183,"name":"TP00220000204","description":"","link":"","files":"2"},{"id":190,"parentId":183,"name":"TP00220000208","description":"","link":"","files":"1"}]},{"id":184,"parentId":181,"name":"intrepid","description":"","link":"","files":"1","children":[{"id":191,"parentId":184,"name":"TP00221000200","description":"","link":"","files":"2"}]}],"id":181,"parentId":6,"name":"tops-curtos","description":"","link":"","files":"3"},{"children":[{"id":193,"parentId":192,"name":"allure","description":"","link":"","files":"2","children":[{"id":196,"parentId":193,"name":"TS00207000200","description":"","link":"","files":"1"},{"id":197,"parentId":193,"name":"TS00207000208","description":"","link":"","files":"3"}]},{"id":194,"parentId":192,"name":"moment","description":"","link":"","files":"2","children":[{"id":198,"parentId":194,"name":"TS00156000202","description":"","link":"","files":"1"},{"id":199,"parentId":194,"name":"TS00156000206","description":"","link":"","files":"2"}]},{"id":195,"parentId":192,"name":"razzle","description":"","link":"","files":"2","children":[{"id":200,"parentId":195,"name":"TS00206000200","description":"","link":"","files":"3"},{"id":597,"parentId":195,"name":"TS00206000208","description":"","link":"","files":"1"}]}],"id":192,"parentId":6,"name":"tshirts","description":"","link":"","files":"3"}],"id":6,"parentId":590,"name":"spring-summer","description":"","link":"","files":"8"}],"id":590,"parentId":3,"name":"collections","description":"","link":"","files":"4"},{"id":591,"parentId":3,"name":"marketing-content","description":"","link":"","files":"25"}],"id":3,"parentId":1,"name":"Loja Online - 2018","description":"","link":"","files":"2"},{"children":[{"children":[{"id":797,"parentId":610,"name":"basics","description":"","link":"","files":"10","children":[{"id":835,"parentId":797,"name":"calcas","description":"","link":"","files":"1","children":[{"children":[{"id":837,"parentId":836,"name":"CL00249200","description":"","link":"","files":"3"},{"id":840,"parentId":836,"name":"CL00249208","description":"","link":"","files":"0"},{"id":839,"parentId":836,"name":"CL00249210","description":"","link":"","files":"0"},{"id":838,"parentId":836,"name":"CL00249225","description":"","link":"","files":"3"}],"id":836,"parentId":835,"name":"comfy","description":"","link":"","files":"4"}]},{"children":[{"id":802,"parentId":798,"name":"active","description":"","link":"","files":"2","children":[{"id":803,"parentId":802,"name":"SH00235200","description":"","link":"","files":"4"},{"id":804,"parentId":802,"name":"SH00235225","description":"","link":"","files":"0"}]},{"id":799,"parentId":798,"name":"alive","description":"","link":"","files":"2","children":[{"id":800,"parentId":799,"name":"CJ00126200","description":"","link":"","files":"3"},{"id":801,"parentId":799,"name":"CJ00126225","description":"","link":"","files":"3"}]}],"id":798,"parentId":797,"name":"calcoes","description":"","link":"","files":"2"},{"id":856,"parentId":797,"name":"casacos","description":"","link":"","files":"1","children":[{"children":[{"id":858,"parentId":857,"name":"CS00192200","description":"","link":"","files":"3"},{"id":861,"parentId":857,"name":"CS00192208","description":"","link":"","files":"3"},{"id":860,"parentId":857,"name":"CS00192210","description":"","link":"","files":"0"},{"id":859,"parentId":857,"name":"CS00192225","description":"","link":"","files":"3"}],"id":857,"parentId":856,"name":"search","description":"","link":"","files":"4"}]},{"id":815,"parentId":797,"name":"corsarios","description":"","link":"","files":"1","children":[{"children":[{"id":817,"parentId":816,"name":"PT00292200","description":"","link":"","files":"4"},{"id":819,"parentId":816,"name":"PT00292210","description":"","link":"","files":"0"},{"id":818,"parentId":816,"name":"PT00292225","description":"","link":"","files":"0"}],"id":816,"parentId":815,"name":"sparkle","description":"","link":"","files":"3"}]},{"id":810,"parentId":797,"name":"cropped","description":"","link":"","files":"1","children":[{"children":[{"id":812,"parentId":811,"name":"CP00196200","description":"","link":"","files":"3"},{"id":814,"parentId":811,"name":"CP00196210","description":"","link":"","files":"0"},{"id":813,"parentId":811,"name":"CP00196225","description":"","link":"","files":"4"}],"id":811,"parentId":810,"name":"magny","description":"","link":"","files":"3"}]},{"children":[{"id":830,"parentId":820,"name":"genuine","description":"","link":"","files":"4","children":[{"id":831,"parentId":830,"name":"LG00044200","description":"","link":"","files":"3"},{"id":834,"parentId":830,"name":"LG00044208","description":"","link":"","files":"0"},{"id":833,"parentId":830,"name":"LG00044210","description":"","link":"","files":"3"},{"id":832,"parentId":830,"name":"LG00044225","description":"","link":"","files":"3"}]},{"children":[{"id":822,"parentId":821,"name":"LG00066200","description":"","link":"","files":"1"},{"id":825,"parentId":821,"name":"LG00066208","description":"","link":"","files":"0"},{"id":824,"parentId":821,"name":"LG00066210","description":"","link":"","files":"3"},{"id":823,"parentId":821,"name":"LG00066225","description":"","link":"","files":"3"}],"id":821,"parentId":820,"name":"lotus","description":"","link":"","files":"4"},{"children":[{"id":827,"parentId":826,"name":"LG00080200","description":"","link":"","files":"1"},{"id":829,"parentId":826,"name":"LG00080210","description":"","link":"","files":"2"},{"id":828,"parentId":826,"name":"LG00080225","description":"","link":"","files":"1"}],"id":826,"parentId":820,"name":"vanilla","description":"","link":"","files":"3"}],"id":820,"parentId":797,"name":"leggings","description":"","link":"","files":"3"},{"children":[{"id":806,"parentId":805,"name":"jewel","description":"","link":"","files":"3","children":[{"id":807,"parentId":806,"name":"SC00198200","description":"","link":"","files":"1"},{"id":809,"parentId":806,"name":"SC00198208","description":"","link":"","files":"1"},{"id":808,"parentId":806,"name":"SC00198225","description":"","link":"","files":"1"}]}],"id":805,"parentId":797,"name":"saias","description":"","link":"","files":"1"},{"children":[{"id":853,"parentId":852,"name":"CS00192200","description":"","link":"","files":"0"},{"id":854,"parentId":852,"name":"CS00192225","description":"","link":"","files":"0"},{"id":855,"parentId":852,"name":"CS00192225","description":"","link":"","files":"0"}],"id":852,"parentId":797,"name":"search","description":"","link":"","files":"3"},{"children":[{"id":848,"parentId":847,"name":"ease","description":"","link":"","files":"3","children":[{"id":849,"parentId":848,"name":"TS00365200","description":"","link":"","files":"4"},{"id":851,"parentId":848,"name":"TS00365208","description":"","link":"","files":"1"},{"id":850,"parentId":848,"name":"TS00365225","description":"","link":"","files":"0"}]}],"id":847,"parentId":797,"name":"t-shirts","description":"","link":"","files":"1"},{"children":[{"id":842,"parentId":841,"name":"fire","description":"","link":"","files":"4","children":[{"id":843,"parentId":842,"name":"TP00244200","description":"","link":"","files":"4"},{"id":846,"parentId":842,"name":"TP00244208","description":"","link":"","files":"4"},{"id":845,"parentId":842,"name":"TP00244210","description":"","link":"","files":"0"},{"id":844,"parentId":842,"name":"TP00244225","description":"","link":"","files":"3"}]}],"id":841,"parentId":797,"name":"tops","description":"","link":"","files":"1"}]},{"id":773,"parentId":610,"name":"padel-tenis","description":"","link":"","files":"3","children":[{"children":[{"id":775,"parentId":774,"name":"charming","description":"","link":"","files":"2","children":[{"id":776,"parentId":775,"name":"SC00318539","description":"","link":"","files":"3"},{"id":777,"parentId":775,"name":"SC00318559","description":"","link":"","files":"3"}]},{"id":780,"parentId":774,"name":"cute","description":"","link":"","files":"2","children":[{"id":781,"parentId":780,"name":"SC00322208","description":"","link":"","files":"3"},{"id":782,"parentId":780,"name":"SC00322239","description":"","link":"","files":"4"}]},{"id":778,"parentId":774,"name":"framing","description":"","link":"","files":"1","children":[{"id":779,"parentId":778,"name":"SC00337200","description":"","link":"","files":"0"}]},{"id":783,"parentId":774,"name":"peppery","description":"","link":"","files":"2","children":[{"id":784,"parentId":783,"name":"SC00321543","description":"","link":"","files":"3"},{"id":785,"parentId":783,"name":"SC00321559","description":"","link":"","files":"3"}]}],"id":774,"parentId":773,"name":"saias","description":"","link":"","files":"4"},{"children":[{"id":794,"parentId":793,"name":"salty","description":"","link":"","files":"2","children":[{"id":795,"parentId":794,"name":"TS00316510","description":"","link":"","files":"3"},{"id":796,"parentId":794,"name":"TS00316578","description":"","link":"","files":"0"}]}],"id":793,"parentId":773,"name":"t-shirts","description":"","link":"","files":"1"},{"children":[{"id":790,"parentId":786,"name":"cheers","description":"","link":"","files":"2","children":[{"id":791,"parentId":790,"name":"TP00317200","description":"","link":"","files":"0"},{"id":792,"parentId":790,"name":"TP00317208","description":"","link":"","files":"2"}]},{"id":787,"parentId":786,"name":"runaway","description":"","link":"","files":"2","children":[{"id":789,"parentId":787,"name":"TP00291539","description":"","link":"","files":"2"},{"id":788,"parentId":787,"name":"TP00291559","description":"","link":"","files":"2"}]}],"id":786,"parentId":773,"name":"tops","description":"","link":"","files":"2"}]},{"children":[{"id":759,"parentId":611,"name":"calcas","description":"","link":"","files":"2","children":[{"children":[{"id":763,"parentId":762,"name":"CL00324246","description":"","link":"","files":"4"},{"id":764,"parentId":762,"name":"CL00324247","description":"","link":"","files":"3"}],"id":762,"parentId":759,"name":"cozy","description":"","link":"","files":"2"},{"children":[{"id":761,"parentId":760,"name":"CL00350200","description":"","link":"","files":"5"}],"id":760,"parentId":759,"name":"embrace","description":"","link":"","files":"1"}]},{"children":[{"id":721,"parentId":720,"name":"brave","description":"","link":"","files":"1","children":[{"id":722,"parentId":721,"name":"CJ00332200","description":"","link":"","files":"3"}]},{"children":[{"id":724,"parentId":723,"name":"CJ00349200","description":"","link":"","files":"3"}],"id":723,"parentId":720,"name":"sailor","description":"","link":"","files":"1"}],"id":720,"parentId":611,"name":"calcoes","description":"","link":"","files":"2"},{"id":682,"parentId":611,"name":"camisolas","description":"","link":"","files":"1","children":[{"id":683,"parentId":682,"name":"prestige","description":"","link":"","files":"4","children":[{"id":684,"parentId":683,"name":"SW00363200","description":"","link":"","files":"4"},{"id":685,"parentId":683,"name":"SW00363237","description":"","link":"","files":"1"},{"id":686,"parentId":683,"name":"SW00363246","description":"","link":"","files":"2"},{"id":687,"parentId":683,"name":"SW00363247","description":"","link":"","files":"0"}]}]},{"id":673,"parentId":611,"name":"casacos","description":"","link":"","files":"2","children":[{"children":[{"id":677,"parentId":676,"name":"CS00330200","description":"","link":"","files":"2"},{"id":678,"parentId":676,"name":"CS00330544","description":"","link":"","files":"0"},{"id":679,"parentId":676,"name":"CS00330546","description":"","link":"","files":"1"},{"id":680,"parentId":676,"name":"CS00330547","description":"","link":"","files":"2"},{"id":681,"parentId":676,"name":"CS00330548","description":"","link":"","files":"1"}],"id":676,"parentId":673,"name":"dazzling","description":"","link":"","files":"5"},{"children":[{"id":675,"parentId":674,"name":"CS00250234","description":"","link":"","files":"4"}],"id":674,"parentId":673,"name":"insurgent","description":"","link":"","files":"1"}]},{"id":688,"parentId":611,"name":"corsarios","description":"","link":"","files":"3","children":[{"id":689,"parentId":688,"name":"demanding","description":"","link":"","files":"5","children":[{"id":690,"parentId":689,"name":"PT00300544","description":"","link":"","files":"2"},{"id":691,"parentId":689,"name":"PT00300546","description":"","link":"","files":"1"},{"id":692,"parentId":689,"name":"PT00300547","description":"","link":"","files":"2"},{"id":693,"parentId":689,"name":"PT00300548","description":"","link":"","files":"3"},{"id":694,"parentId":689,"name":"PT00300558","description":"","link":"","files":"2"}]},{"id":695,"parentId":688,"name":"move","description":"","link":"","files":"3","children":[{"id":696,"parentId":695,"name":"PT00253200","description":"","link":"","files":"3"},{"id":697,"parentId":695,"name":"PT00253546","description":"","link":"","files":"3"},{"id":698,"parentId":695,"name":"PT00253547","description":"","link":"","files":"3"}]},{"children":[{"id":700,"parentId":699,"name":"PT00303200","description":"","link":"","files":"3"},{"id":701,"parentId":699,"name":"PT00303246","description":"","link":"","files":"3"}],"id":699,"parentId":688,"name":"sharp","description":"","link":"","files":"2"}]},{"id":702,"parentId":611,"name":"cropped","description":"","link":"","files":"6","children":[{"children":[{"id":705,"parentId":703,"name":"CP00338546","description":"","link":"","files":"0"},{"id":704,"parentId":703,"name":"CP00338547","description":"","link":"","files":"3"}],"id":703,"parentId":702,"name":"fair","description":"","link":"","files":"2"},{"children":[{"id":707,"parentId":706,"name":"CP00299254","description":"","link":"","files":"3"}],"id":706,"parentId":702,"name":"flash","description":"","link":"","files":"1"},{"children":[{"id":709,"parentId":708,"name":"CP00271200","description":"","link":"","files":"3"},{"id":710,"parentId":708,"name":"CP00271236","description":"","link":"","files":"3"}],"id":708,"parentId":702,"name":"flee","description":"","link":"","files":"2"},{"children":[{"id":715,"parentId":714,"name":"CP00298200","description":"","link":"","files":"3"}],"id":714,"parentId":702,"name":"provocative","description":"","link":"","files":"1"},{"children":[{"id":712,"parentId":711,"name":"CP00280200","description":"","link":"","files":"0"},{"id":713,"parentId":711,"name":"CP00280546","description":"","link":"","files":"3"}],"id":711,"parentId":702,"name":"resfeber","description":"","link":"","files":"2"},{"children":[{"id":717,"parentId":716,"name":"CP00270510","description":"","link":"","files":"3"},{"id":718,"parentId":716,"name":"CP00270545","description":"","link":"","files":"3"},{"id":719,"parentId":716,"name":"CP00270556","description":"","link":"","files":"3"}],"id":716,"parentId":702,"name":"roam","description":"","link":"","files":"3"}]},{"children":[{"id":742,"parentId":725,"name":"bold","description":"","link":"","files":"2","children":[{"id":743,"parentId":742,"name":"LG00260545","description":"","link":"","files":"0"},{"id":744,"parentId":742,"name":"LG00260551","description":"","link":"","files":"3"}]},{"id":728,"parentId":725,"name":"celebration","description":"","link":"","files":"1","children":[{"id":729,"parentId":728,"name":"LG00339257","description":"","link":"","files":"3"}]},{"id":738,"parentId":725,"name":"energy","description":"","link":"","files":"3","children":[{"id":739,"parentId":738,"name":"LG00278510","description":"","link":"","files":"0"},{"id":740,"parentId":738,"name":"LG00278545","description":"","link":"","files":"3"},{"id":741,"parentId":738,"name":"LG00278555","description":"","link":"","files":"3"}]},{"id":748,"parentId":725,"name":"evasion","description":"","link":"","files":"3","children":[{"id":749,"parentId":748,"name":"LG00254510","description":"","link":"","files":"2"},{"id":750,"parentId":748,"name":"LG00254544","description":"","link":"","files":"3"},{"id":751,"parentId":748,"name":"LG00254545","description":"","link":"","files":"3"}]},{"id":757,"parentId":725,"name":"fantasy","description":"","link":"","files":"1","children":[{"id":758,"parentId":757,"name":"LG00268249","description":"","link":"","files":"3"}]},{"children":[{"id":727,"parentId":726,"name":"LG00348200","description":"","link":"","files":"4"}],"id":726,"parentId":725,"name":"lovely","description":"","link":"","files":"1"},{"children":[{"id":731,"parentId":730,"name":"LG00306200","description":"","link":"","files":"3"},{"id":732,"parentId":730,"name":"LG00306246","description":"","link":"","files":"0"}],"id":730,"parentId":725,"name":"mockingbird","description":"","link":"","files":"2"},{"children":[{"id":746,"parentId":745,"name":"LG00277510","description":"","link":"","files":"3"},{"id":747,"parentId":745,"name":"LG00277568","description":"","link":"","files":"0"}],"id":745,"parentId":725,"name":"moxie","description":"","link":"","files":"2"},{"children":[{"id":734,"parentId":733,"name":"LG00265200","description":"","link":"","files":"3"}],"id":733,"parentId":725,"name":"no-drama","description":"","link":"","files":"1"},{"children":[{"id":736,"parentId":735,"name":"LG00333250","description":"","link":"","files":"4"},{"id":737,"parentId":735,"name":"LG00333251","description":"","link":"","files":"3"}],"id":735,"parentId":725,"name":"oblique","description":"","link":"","files":"2"},{"children":[{"id":772,"parentId":771,"name":"LG00373200","description":"","link":"","files":"4"}],"id":771,"parentId":725,"name":"rise","description":"","link":"","files":"1"},{"children":[{"id":769,"parentId":768,"name":"LG00272571","description":"","link":"","files":"2"},{"id":770,"parentId":768,"name":"LG00272574","description":"","link":"","files":"2"}],"id":768,"parentId":725,"name":"silhouette","description":"","link":"","files":"2"},{"children":[{"id":753,"parentId":752,"name":"LG00256544","description":"","link":"","files":"0"},{"id":754,"parentId":752,"name":"LG00256546","description":"","link":"","files":"3"},{"id":755,"parentId":752,"name":"LG00256547","description":"","link":"","files":"0"},{"id":756,"parentId":752,"name":"LG00256548","description":"","link":"","files":"3"}],"id":752,"parentId":725,"name":"speed","description":"","link":"","files":"4"}],"id":725,"parentId":611,"name":"leggings","description":"","link":"","files":"13"},{"children":[{"id":766,"parentId":765,"name":"elegance","description":"","link":"","files":"1","children":[{"id":767,"parentId":766,"name":"MC00364510","description":"","link":"","files":"5"}]}],"id":765,"parentId":611,"name":"macacoes","description":"","link":"","files":"1"},{"children":[{"id":660,"parentId":659,"name":"clearity","description":"","link":"","files":"3","children":[{"id":661,"parentId":660,"name":"TS00310510","description":"","link":"","files":"3"},{"id":662,"parentId":660,"name":"TS00310539","description":"","link":"","files":"3"},{"id":663,"parentId":660,"name":"TS00310556","description":"","link":"","files":"3"}]},{"id":668,"parentId":659,"name":"hiatus","description":"","link":"","files":"4","children":[{"id":669,"parentId":668,"name":"TS00311200","description":"","link":"","files":"0"},{"id":670,"parentId":668,"name":"TS00311208","description":"","link":"","files":"1"},{"id":671,"parentId":668,"name":"TS00311237","description":"","link":"","files":"0"},{"id":672,"parentId":668,"name":"TS00311247","description":"","link":"","files":"2"}]},{"id":664,"parentId":659,"name":"imagine","description":"","link":"","files":"3","children":[{"id":665,"parentId":664,"name":"TS00252200","description":"","link":"","files":"4"},{"id":666,"parentId":664,"name":"TS00252208","description":"","link":"","files":"3"},{"id":667,"parentId":664,"name":"TS00252237","description":"","link":"","files":"3"}]}],"id":659,"parentId":611,"name":"t-shirts","description":"","link":"","files":"3"},{"children":[{"id":628,"parentId":612,"name":"balance","description":"","link":"","files":"4","children":[{"id":629,"parentId":628,"name":"TP00313200","description":"","link":"","files":"2"},{"id":630,"parentId":628,"name":"TP00313208","description":"","link":"","files":"1"},{"id":631,"parentId":628,"name":"TP00313239","description":"","link":"","files":"0"},{"id":632,"parentId":628,"name":"TP00313240","description":"","link":"","files":"4"}]},{"id":643,"parentId":612,"name":"breakout","description":"","link":"","files":"4","children":[{"id":644,"parentId":643,"name":"TP00287200","description":"","link":"","files":"3"},{"id":645,"parentId":643,"name":"TP00287208","description":"","link":"","files":"1"},{"id":646,"parentId":643,"name":"TP00287240","description":"","link":"","files":"0"},{"id":647,"parentId":643,"name":"TP00287246","description":"","link":"","files":"0"}]},{"id":651,"parentId":612,"name":"care","description":"","link":"","files":"5","children":[{"id":652,"parentId":651,"name":"TP00327200","description":"","link":"","files":"3"},{"id":653,"parentId":651,"name":"TP00327206","description":"","link":"","files":"3"},{"id":654,"parentId":651,"name":"TP00327208","description":"","link":"","files":"2"},{"id":655,"parentId":651,"name":"TP00327239","description":"","link":"","files":"2"},{"id":656,"parentId":651,"name":"TP00327246","description":"","link":"","files":"2"}]},{"id":633,"parentId":612,"name":"exposure","description":"","link":"","files":"2","children":[{"id":634,"parentId":633,"name":"TP00307200","description":"","link":"","files":"0"},{"id":635,"parentId":633,"name":"TP00307208","description":"","link":"","files":"1"}]},{"id":657,"parentId":612,"name":"iconic","description":"","link":"","files":"1","children":[{"id":658,"parentId":657,"name":"TP00293200","description":"","link":"","files":"4"}]},{"id":618,"parentId":612,"name":"incredible","description":"","link":"","files":"1","children":[{"id":619,"parentId":618,"name":"TP00346200","description":"","link":"","files":"4"}]},{"id":616,"parentId":612,"name":"pilot","description":"","link":"","files":"1","children":[{"id":617,"parentId":616,"name":"TP00347200","description":"","link":"","files":"4"}]},{"id":625,"parentId":612,"name":"release","description":"","link":"","files":"2","children":[{"id":626,"parentId":625,"name":"TP00290200","description":"","link":"","files":"3"},{"id":627,"parentId":625,"name":"TP00290239","description":"","link":"","files":"3"}]},{"id":613,"parentId":612,"name":"royal","description":"","link":"","files":"2","children":[{"id":615,"parentId":613,"name":"TP00297547","description":"","link":"","files":"2"},{"id":614,"parentId":613,"name":"TP00297569","description":"","link":"","files":"1"}]},{"id":623,"parentId":612,"name":"sassy","description":"","link":"","files":"1","children":[{"id":624,"parentId":623,"name":"TP00284200","description":"","link":"","files":"4"}]},{"id":638,"parentId":612,"name":"sunny","description":"","link":"","files":"4","children":[{"id":639,"parentId":638,"name":"TP00302200","description":"","link":"","files":"2"},{"id":640,"parentId":638,"name":"TP00302237","description":"","link":"","files":"0"},{"id":641,"parentId":638,"name":"TP00302245","description":"","link":"","files":"3"},{"id":642,"parentId":638,"name":"TP00302247","description":"","link":"","files":"3"}]},{"id":636,"parentId":612,"name":"tenacity","description":"","link":"","files":"1","children":[{"id":637,"parentId":636,"name":"TP00285200","description":"","link":"","files":"0"}]},{"children":[{"id":649,"parentId":648,"name":"TP00282238","description":"","link":"","files":"0"},{"id":650,"parentId":648,"name":"TP00282240","description":"","link":"","files":"0"}],"id":648,"parentId":612,"name":"unravel","description":"","link":"","files":"2"},{"children":[{"id":621,"parentId":620,"name":"TP00295250","description":"","link":"","files":"3"},{"id":622,"parentId":620,"name":"TP00295251","description":"","link":"","files":"2"}],"id":620,"parentId":612,"name":"zigged","description":"","link":"","files":"2"}],"id":612,"parentId":611,"name":"tops","description":"","link":"","files":"14"}],"id":611,"parentId":610,"name":"spring-summer","description":"","link":"","files":"10"}],"id":610,"parentId":609,"name":"collections","description":"","link":"","files":"3"},{"id":862,"parentId":609,"name":"marketing-content","description":"","link":"","files":"12"}],"id":609,"parentId":1,"name":"Loja Online - 2019","description":"","link":"","files":"2"},{"children":[{"id":608,"parentId":607,"name":"Instagram","description":"","link":"","files":"250"}],"id":607,"parentId":1,"name":"Marketing","description":"","link":"","files":"1"}]';


        $tree = [
            [
                'id' => 2,
                'parentId' => 1,
                'children' => [
                    [
                        'id' => 3,
                        'parentId' => 2,
                        'children' => [
                            [
                                'id' => 6,
                                'parentId' => 3,
                                'children' => [
                                    [
                                        'id' => 7,
                                        'parentId' => 6,
                                        'children' => [
                                            [
                                                'id' => 8,
                                                'parentId' => 7,
                                                'children' => []
                                            ]
                                        ]
                                    ]
                                ]
                            ]
                        ]
                    ],
                    [
                        'id' => 4,
                        'parentId' => 2,
                        'children' => [
                            [
                                'id' => 11,
                                'parentId' => 4,
                                'children' => []
                            ]
                        ]
                    ],
                    [
                        'id' => 5,
                        'parentId' => 2,
                    ],
                ]
            ],
            [
                'id' => 10,
                'parentId' => 1,
            ],
            [
                'id' => 12,
                'parentId' => 1,
            ]
        ];

        //print_r($tree);
        //print_r($tree);
//     609 > > 610 > 797 > 798 > 802 > 804
        $tree = Directory::build();
        // print_r($tree);

        $node = $this->findNode($tree, 804);
        //$node = $this->findNode($tree, 8);
        //$node = Directory::findNode(json_decode($json), 8 );
        print_r($node);
    }

    // https://stackoverflow.com/questions/39860090/find-element-by-id-in-tree-data-structure
    public function findNode($tree, $idDirectory)
    {
        $temp = [];
        $found = false;
        foreach ($tree as $node) {
            // 609 > > 610 > 797 > 798 > 802 > 804
            //echo $node['id']. " | ";
            if ($node['id'] == $idDirectory) {   //print_r($node);
                //die();
                // $found = true;
                return $node;
            } elseif (isset($node['children'])) {
                $temp = $this->findNode($node['children'], $idDirectory);
                if ($temp) {
                    return $temp;
                }
                //print_r($temp);
//                if( sizeof($temp) )
//                    return $node;
            }

//            if( $found ) {
//                print_r($node);
//                die();
//            }
        }

//        print_r($temp);
        return $temp;
    }

    public function testDirectoryTreeNodeChildren()
    {
        $tree = Directory::build();
    }

    public function testDirectoryQuery()
    {
        // Get all children of the idDirectory = 3
        $q = new \Stage\Asset\DirectoryQuery();
        $q->idDirectory = 3;
        $q->by = 11;
        $directories = Directory::query($q, false, 0, 15);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Directory', $directories);
    }

    public function testAssetLabelCreate()
    {
        try {
            $label = new \Stage\Asset\Label\Label();
            $name = html_entity_decode("goportugal", ENT_COMPAT, 'UTF-8');
            $label->setName($name);
            $label->setNameStripped("go-portugal");
            $label->setPublic("0");

            $label->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetLabelFind()
    {
        try {
            $label = \Stage\Asset\Label\Label::find(1);
            print_r($label);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetLabelQuery()
    {
        try {
            $query = new \Stage\Asset\Label\Query();
            $labels = \Stage\Asset\Label\Label::query($query);
            print_r($labels);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetAlbumTagCreate()
    {
        try {
            $tag = new \Stage\Asset\Album\Tag\Tag();
            $name = html_entity_decode("goportugal", ENT_COMPAT, 'UTF-8');
            $tag->setName($name);
            $tag->setNameStripped("go-portugal");
            $tag->setPublic(false);

            $tag->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetAlbumTagFind()
    {
        try {
            $tag = \Stage\Asset\Album\Tag\Tag::find(1);
            print_r($tag);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetAlbumTags()
    {
        try {
            $album = \Stage\Asset\Album\Album::find(1);
            $tags = $album->tags();
            print_r($tags);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetGalleryTags()
    {
        try {
            $gallery = \Stage\Asset\Gallery\Gallery::find(1);
            $tags = $gallery->tags();
            print_r($tags);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetGalleryAddTags()
    {
        try {
            $gallery = \Stage\Asset\Gallery\Gallery::find(1);
            $tag = \Stage\Asset\Gallery\Tag\Tag::find(1);
            $gallery->addTag($tag);
            print_r($gallery);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetCreateGroup()
    {
        try {
            $name = "GroupName";
            $group = new \Stage\Asset\Tag\Group\Group();
            $group->setName(htmlentities(trim($name), ENT_COMPAT, 'UTF-8'));

            $group->save();
            print_r($group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetFindGroup()
    {
        try {
            $group = \Stage\Asset\Tag\Group\Group::find(5);
            print_r($group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetQueryGroup()
    {
        try {
            $group = \Stage\Asset\Tag\Group\Group::build();
            print_r($group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetCreateKey()
    {
        try {
            $name = "key1";
            $public = true;

            $key = new \Stage\Asset\Tag\Key\Key();
            $key->setName(htmlentities(trim($name), ENT_COMPAT, 'UTF-8'));
            $key->setNameStripped(strtolower(str_replace(" ", "-", trim(Formating::generateNameLink($name)))));
            $key->setPublic($public);

            $key->save();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetFindKey()
    {
        try {
            $key = \Stage\Asset\Tag\Key\Key::find(3);
            print_r($key);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetAddVariantKeyToTag()
    {
        try {
            $tag = \Stage\Asset\Tag\Tag::find(1);
            $key = \Stage\Asset\Tag\Key\Key::find(3);
            $value = "value";

            $variant = $tag->addVariants($key->getId(), $value);
            print_r($variant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetRemoveVariantKeyFromTag()
    {
        try {
            $tag = \Stage\Asset\Tag\Tag::find(1);
            $idKey = 3;
            $data = $tag->removeVariants($idKey);

            print_r($data);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetVariantKeys()
    {
        try {
            $tag = \Stage\Asset\Tag\Tag::find(1);
            $variants = $tag->variants();

            $data = array_map(function ($variant) {
                return $variant->toArray();
            }, $variants);
            print_r($data);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetTagKeys()
    {
        $q = new \Stage\Asset\Tag\Key\Query();
        $data = \Stage\Asset\Tag\Key\Key::query($q);
    }

    public function testAssetDuplicateAlbum()
    {
        // TODO
        $album = \Stage\Asset\Album\Album::find(3);

        $idDirectory = 4;
        $album->duplicate($idDirectory);
    }

    public function testAssetSync()
    {
        // TODO
        $idAlbuma = 2;
        $idAlbumb = 3;
        $type = 1;
        $data = \Stage\Asset\Album\Album::sync($idAlbuma, $idAlbumb, $type);
        print_r($data);
    }

    public function testGetEmbeds()
    {
        $q = new \Stage\Asset\Embed\Query();
        $results = \Stage\Asset\Embed\Embed::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Embed\Embed', $results);
    }

    public function testGetEmbedById()
    {
        $result = \Stage\Asset\Embed\Embed::find(1);
        $this->assertInstanceOf('\Stage\Asset\Embed\Embed', $result);
    }

    public function testSaveNewEmbed()
    {
        $embed = new \Stage\Asset\Embed\Embed();
        $embed->setSourceType(1);
        $embed->setSourceId(1);
        $embed->setFormat(1);
        $embed->setType('g');
        $embed->setVariant("1");
        $embed->setName("name1");
        $embed->setActive(true);

        try {
            $result = $embed->save();
            $this->assertTrue($result instanceof \Stage\Asset\Embed\Embed);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetEmbedAssets()
    {
        $field = 1; // its the id of embed from the table assets_embeds_assets_related
        $filter = 1; // filter by id embed
        $result = \Stage\Asset\Embed\Embed::assets($field, $filter);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Embed\Asset', $result);
    }

    public function testAttachAssetToEmbed()
    {
        $asset = new \Stage\Asset\Embed\Asset();
        $asset->setId(1); // its the id embedd
        $asset->setIdAsset(6);
        $asset->setPosition(1);
        $asset->setHeadline("headline");
        $asset->setTitle("title");
        $asset->setDescription("desc");
        $asset->setActionName("actiom_name");
        $asset->setLink("lonk");
        $result = \Stage\Asset\Embed\Embed::attachAsset($asset);
        $this->assertTrue($result);
    }

    public function testAttachOptionToEmbed()
    {
        $idEmbed = 1;
        $idOption = 2;
        $optionValue = "cenas";
        $result = \Stage\Asset\Embed\Embed::attachOption($idEmbed, $idOption, $optionValue);
        $this->assertTrue($result);
    }

    public function testGetOptionsFromEmbed()
    {
        $field = 1;
        $filter = 1;
        $result = \Stage\Asset\Embed\Embed::options($field, $filter);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Embed\Option', $result);
    }

    public function testGetVariantsFromEmbed()
    {
        $type = 'g';
        $variant = '1';
        $result = \Stage\Asset\Embed\Embed::variants($type, $variant);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Embed\OptionVariant', $result);
    }

    public function testAssetsGetEmbedSettings()
    {
        $results = \Stage\Asset\Embed\Embed::settings(1);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Embed\Setting', $results);
    }

    public function testAssetsSetChecksum()
    {
        $hash = "gibberish";
        $idAsset = 4;
        $status = \Stage\Asset\Asset::checksum($idAsset, $hash);
        $this->assertTrue($status);
    }

    public function testInImageAdvertisement()
    {
        $results = \Stage\Asset\Asset::inImageAdvertisement(1, 1);
        $this->assertCount(3, $results);
    }

    public function testAssetCreatePlaylist()
    {
        try {
            $playlist = new \Stage\Asset\Playlist\Playlist();
            $playlist->setIdDirectory(1);
            $playlist->setName("name1");
            $playlist->setDescription("desc");
            $playlist->setLink("link");
            $playlist->save();
            $this->assertInstanceOf('\Stage\Asset\Playlist\Playlist', $playlist);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsFindPlaylist()
    {
        try {
            $playlist = \Stage\Asset\Playlist\Playlist::find(0, 5);
            $this->assertInstanceOf('\Stage\Asset\Playlist\Playlist', $playlist);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsClonePlaylist()
    {
        try {
            $playlist = \Stage\Asset\Playlist\Playlist::find(3);
            $playlist->duplicate(1);
            $this->assertInstanceOf('\Stage\Asset\Playlist\Playlist', $playlist);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsDeletePlaylist()
    {
        try {
            $playlist = \Stage\Asset\Playlist\Playlist::find(6);
            $playlist->delete();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsAttachPlaylist()
    {
        try {
            $asset = \Stage\Asset\Asset::find(6);
            $status = $asset->attachPlaylist(1, 2);
            $this->assertTrue($status);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsDetachPlaylist()
    {
        try {
            $asset = \Stage\Asset\Asset::find(6);
            $status = $asset->detachPlaylist(1);
            $this->assertTrue($status);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsQueryAssetsInPlaylist()
    {
        try {
            $query = new \Stage\Asset\Playlist\AssetsQuery();
            $assets = \Stage\Asset\Playlist\Playlist::assets(1, $query, true);
            foreach ($assets as $asset) {
                $this->assertInstanceOf('\Stage\Asset\Asset', $asset);
            }
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsQueryPlaylists()
    {
        try {
            $query = new \Stage\Asset\Playlist\Query();
            $playlists = \Stage\Asset\Playlist\Playlist::query($query, true);
            foreach ($playlists as $playlist) {
                $this->assertInstanceOf('\Stage\Asset\Playlist\Playlist', $playlist);
            }
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsPlaylistsMetaUrl()
    {
        try {
            $result = \Stage\Asset\Playlist\Playlist::metaUrlExist(1, "cenas");
            print_r($result);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsPlaylistsFindDirectory()
    {
        try {
            $directories = \Stage\Asset\Playlist\Directory::find(1);
            print_r($directories);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsPlaylistsCreateDirectory()
    {
        try {
            $directory = new \Stage\Asset\Playlist\Directory();
            $directory->setName("name1");
            $directory->setDescription("desc");
            $directory->setLink("http://link.to");
            $newDirectory = $directory->appendTo();
            print_r($newDirectory);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsPlaylistsRemoveDirectory()
    {
        try {
            $idDirectory = \Stage\Asset\Playlist\Directory::delete(5, false);
            print_r($idDirectory);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsPlaylistsDirectories()
    {
        try {
            $q = new \Stage\Asset\Playlist\DirectoryQuery();
            $directories = \Stage\Asset\Playlist\Directory::query($q, false);
            print_r($directories);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsPlaylistsDirectoriesTree()
    {
        try {
            $directories = \Stage\Asset\Playlist\Directory::build();
            print_r($directories);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetsPlaylistsSync()
    {
        try {
            $result = \Stage\Asset\Playlist\Playlist::sync(1, 2, 1);
            print_r($result->toArray());
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachAssetVariantKey()
    {
        try {
            $name = 'name';
            $nameStripped = 'name_s';
            $public = true;
            $ifExists = 0;
            $idKey = -1; // it's the id key from table posts_messages_custom_fields_keys, or -1 to create
            $customFieldVariant = Asset::attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists);
            $this->assertInstanceOf('\Stage\Asset\CustomFieldKey', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachAssetVariantKey()
    {
        try {
            $idKey = 10;
            $status = Asset::detachVariantKey($idKey);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetFindVariantKey()
    {
        try {
            $idKey = 9;
            $customVariantKey = Asset::variantKey($idKey);
            $this->assertInstanceOf('\Stage\Asset\CustomFieldKey', $customVariantKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetQueryVariantKeys()
    {
        $q = new \Stage\Asset\CustomFieldQuery();
        $data = Asset::variantKeys($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\CustomFieldKey', $data);
    }

    public function testAssetQueryAllVariantKeys()
    {
        $data = CustomFieldKey::all();
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\CustomFieldKey', $data);
    }

    ////
    public function testAttachAssetAlbumCustomFieldVariant()
    {
        try {
            $album = \Stage\Asset\Album\Album::find(4);
            $idKey = 1; // its the id key from table albums_custom_fields_keys
            $customFieldVariant = $album->attachCustomFieldVariant($idKey, "name1", 1);
            $this->assertInstanceOf('\Stage\Asset\Album\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetCustomAlbumFieldVariants()
    {
        try {
            $album = Album::find(4);
            $customFieldVariant = $album->customFieldsVariants();
            $this->assertContainsOnly('\Stage\Asset\Album\CustomFieldVariant', $customFieldVariant);
            $this->assertTrue($customFieldVariant->count() > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachAssetAlbumCustomFieldVariants()
    {
        try {
            $album = Album::find(4);
            $status = $album->detachCustomFieldsVariant(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachAssetAlbumVariantKey()
    {
        try {
            $name = 'name';
            $nameStripped = 'name_s';
            $public = true;
            $ifExists = 1;
            $idKey = -1; // it's the id key from table posts_messages_custom_fields_keys, or -1 to create
            $customFieldVariant = Album::attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists);
            $this->assertInstanceOf('\Stage\Asset\Album\CustomFieldKey', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachAssetAlbumVariantKey()
    {
        try {
            $idKey = 3;
            $status = Album::detachVariantKey($idKey);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetAlbumFindVariantKey()
    {
        try {
            $idKey = 2;
            $customVariantKey = Album::variantKey($idKey);
            $this->assertInstanceOf('\Stage\Asset\Album\CustomFieldKey', $customVariantKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAssetAlbumQueryVariantKeys()
    {
        $q = new \Stage\Asset\Album\CustomFieldQuery();
        $data = Album::variantKeys($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Album\CustomFieldKey', $data);
    }

    public function testAssetAlbumQueryAllVariantKeys()
    {
        $data = AlbumCustomFieldKey::all();
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Album\CustomFieldKey', $data);
    }

    public function testPublishAssetTags()
    {
        $tags = \Stage\Support\Asset::assetTags(4);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Tag\Tag', $tags);
    }

    public function testExternalAssetCreate()
    {
        $idAsset = -1;
        $idDirectory = 1;
        $nameAsset = "some";
        $filename = "filename";
        $extension = "png";
        $mimetype = "image/png";
        $size = 125;
        $width = 800;
        $height = 480;
        $idStorage = 1;
        $idCdn = 1;

        $asset = \Stage\External\Asset::create($idAsset, $idDirectory, $nameAsset, $filename, $extension, $mimetype, $size, $width, $height, $idStorage, $idCdn);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Asset', $asset);
    }

    public function testExternalAssetTagCreate()
    {
        $id = -1;
        $parentId = 1;
        $name = "tag1";
        $nameStripped = "tag1";
        $public = true;
        $ifExists = 1;

        $tag = \Stage\External\Asset::createTag($id, $parentId, $name, $nameStripped, $public, $ifExists);
        $this->assertTrue($tag->getId() > 0);
        $this->assertInstanceOf('\Stage\Asset\Tag\Tag', $tag);
    }

    public function testExternalAssetTagAdd()
    {
        $idTag = 2;
        $idAsset = 12;

        $status = \Stage\External\Asset::addTag($idTag, $idAsset);
        $this->assertTrue($status);
    }

    public function testExternalAssetFind()
    {
        $idUser = 999;
        $field = \Stage\Asset\Asset::ID;
        $query = "4"; // it's the idAsset

        $asset = \Stage\External\Asset::find($idUser, $field, $query);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Asset', $asset);
    }

    public function testExternalShareAndDownloadQuery()
    {
        try {
            $query = new \Stage\Asset\Download\Query();
            $query->by = \Stage\Asset\Download\Query::ALL;
            $query->term = "";
            $query->filter = "";
            $query->from = 0;
            $query->to = 100;

            $downloads = \Stage\External\Asset::shareAndDownloadQuery($query);
            $this->assertTrue(sizeof($downloads) > 0);
            $this->assertContainsOnlyInstancesOf('Stage\Asset\Download\Download', $downloads);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testExternalShareAndDownloadCountQuery()
    {
        try {
            $query = new \Stage\Asset\Download\Query();
            $query->by = \Stage\Asset\Download\Query::ALL;
            $query->term = "";
            $query->filter = "";
            $query->from = 0;
            $query->to = 100;

            $total = \Stage\External\Asset::shareAndDownloadQueryCount($query);
            $this->assertTrue($total > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testExternalAssetTransfersQueryAssets()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $q->idTransfer = 1;
        $assets = \Stage\External\Asset::transfersQueryAssets($q);
        foreach ($assets as $asset) {
            $this->assertInstanceOf('\Stage\Asset\Asset', $asset);
        }
    }

    public function testExternalAssetTransfersQueryDirectories()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $q->idTransfer = 1;
        $directories = \Stage\External\Asset::transfersQueryDirectories($q);
        foreach ($directories as $directory) {
            $this->assertInstanceOf('\Stage\Asset\Transfer\Directory', $directory);
        }
    }

    public function testExternalAssetDirectoryQuery()
    {
        // Get all children of the idDirectory = 3
        $q = new \Stage\Asset\DirectoryQuery();
        $q->idDirectory = 3;
        $q->by = 11;
        $directories = \Stage\External\Asset::assetsDirectoriesQuery($q, false, 0, 15);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Directory', $directories);
    }

    public function testExternalAssetTransfersFindByCode()
    {
        $code = "code2";
        $transfer = \Stage\External\Asset::findTransferByCode($code);
        $this->assertInstanceOf('\Stage\Asset\Transfer\Transfer', $transfer);
    }

    public function testExternalAssetsByDirectory()
    {
        $idDirectory = 1;
        $assets = \Stage\External\Asset::assetsByDirectory($idDirectory);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Asset', $assets);
    }

    public function testExternalAssets()
    {
        $idDirectory = 1;
        $q = new \Stage\Asset\Query();
        $assets = \Stage\External\Asset::assets($q, $idDirectory);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Asset', $assets);
    }

    public function testExternalAssetsDirectories()
    {
        $directories = \Stage\External\Asset::directories(1, 0, "", 0);
//        print_r(\Stage\External\Asset::build());
    }

    public function testExternalAssetShareAndDownloadQuery()
    {
        $result = \Stage\External\Asset::externalShareAndDownloadAssetsQuery(1, 0, -1);
        $this->assertContainsOnlyInstancesOf('\Stage\Asset\Asset', $result);
    }

    public function testExternalAssetShareAndDownloadQueryCount()
    {
        $total = \Stage\External\Asset::externalShareAndDownloadAssetsQueryCount(1);
        $this->assertTrue($total > 0);
    }
}