<?php

use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Asset\Directory;
use Stage\Asset\Transfer\Transfer;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class AssetsTransfersTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        //$stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);
    }

    public function tearDown()
    {
    }

    public function testAssetTransferCreate()
    {
        $date = new DateTime();
        $code = $date->format('H:i:s');
        $transfer = new \Stage\Asset\Transfer\Transfer();
        $transfer->setDirectoryId(1);
        $transfer->setName("name");
        $transfer->setManagementName("mangement_name");
        $transfer->setDescription("desc");
        $transfer->setCode($code);
        $transfer->setPassword("xx");
        $transfer->setIsActive(true);
        $transfer->setSettings("settings");
        $transfer->setViewOnly(true);
        $transfer->setCompressEnabled(true);
        $transfer->setVerifyIdentity(true);
        $transfer->setWaterMarkImage("image");
        $transfer->setImageOptimization("optimizations");
        $transfer->setContentType(true);

        $transfer->setEndDate($date->format('Y-m-d H:i:s'));

        $transfer->save();
        $this->assertInstanceOf('\Stage\Asset\Transfer\Transfer', $transfer);
        $this->assertTrue($transfer->getId() > 0);
    }

    public function testAssetTransfersNum()
    {
        $total = \Stage\Asset\Transfer\Transfer::sizeof();
        $this->assertTrue($total > 0);
    }

    public function testAssetTransfersQuery()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $transfers = \Stage\Asset\Transfer\Transfer::query($q);
        foreach ($transfers as $transfer) {
            $this->assertInstanceOf('\Stage\Asset\Transfer\Transfer', $transfer);
        }
    }

    public function testAssetTransfersFind()
    {
        $transfer = \Stage\Asset\Transfer\Transfer::find(18);
        $this->assertInstanceOf('\Stage\Asset\Transfer\Transfer', $transfer);
    }

    public function testAssetTransfersDelete()
    {
        $status = \Stage\Asset\Transfer\Transfer::delete(6);
        $this->assertTrue($status);
    }

    public function testAssetTransfersAttachContent()
    {
        $idTransfer = 1;
        $field = 1;
        $filter = 4; // this is the asset id when filter=1
        $position = 1;
        $childDirectories = 2;
        $returnCode = \Stage\Asset\Transfer\Transfer::attachContent($idTransfer, $field, $filter, $position, $childDirectories);
        $this->assertTrue($returnCode == 0); // success
    }

    public function testAssetTransfersDetachContent()
    {
        $idTransfer = 1;
        $field = 1;
        $filter = 4; // this is the asset id when filter=1
        $returnCode = \Stage\Asset\Transfer\Transfer::detachContent($idTransfer, $field, $filter);
        $this->assertTrue($returnCode == 0); // success
    }

    public function testAssetTransfersCreateDirectory()
    {
        $directory = new \Stage\Asset\Transfer\Directory();
        $directory->setName("name");
        $directory->setLink("link");
        $directory->setDescription("desc");
        $directory->save(\Stage\Asset\Transfer\Directory::ROOT);
        $this->assertTrue($directory->getId() > 0);
    }

    public function testAssetTransfersDirectoryDelete()
    {
        $status = \Stage\Asset\Transfer\Directory::delete(6, false);
        $this->assertTrue($status);
    }

    public function testAssetTransfersDirectoryFind()
    {
        $directory = \Stage\Asset\Transfer\Directory::find(2);
        $this->assertInstanceOf('\Stage\Asset\Transfer\Directory', $directory);
    }

    public function testAssetTransfersDirectories()
    {
        $tree = \Stage\Asset\Transfer\Directory::build();
        print_r($tree);
    }

    public function testAssetTransfersDirectoriesMarkedHasDeleted()
    {
        $deleted = \Stage\Asset\Transfer\Directory::markedToDelete();
        print_r($deleted);
    }

    public function testAssetTransfersQueryAssets()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $q->idTransfer = 1;
        $assets = \Stage\Asset\Transfer\Transfer::queryAssets($q);
        foreach ($assets as $asset) {
            $this->assertInstanceOf('\Stage\Asset\Asset', $asset);
        }
    }

    public function testAssetTransfersQueryDirectories()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $q->idTransfer = 1;
        $directories = \Stage\Asset\Transfer\Transfer::queryDirectories($q);
        foreach ($directories as $directory) {
            $this->assertInstanceOf('\Stage\Asset\Transfer\Directory', $directory);
        }
    }

    public function testAssetTransferPermissionCreate()
    {
        $transfer = new \Stage\Asset\Transfer\Permission();
        $transfer->setName("name");
        $transfer->setSettings("settings");
        $transfer->setViewOnly(true);
        $transfer->setWaterMarkImage("image");
        $transfer->setImageOptimization("optimizations");
        $transfer->save();
        $this->assertInstanceOf('\Stage\Asset\Transfer\Permission', $transfer);
        $this->assertTrue($transfer->getId() > 0);
    }

    public function testAssetTransferNoticeBodyCreate()
    {
        $transfer = new \Stage\Asset\Transfer\NoticeBody();
        $transfer->setName("name");
        $transfer->setBody("body");
        $transfer->save();
        $this->assertInstanceOf('\Stage\Asset\Transfer\NoticeBody', $transfer);
        $this->assertTrue($transfer->getId() > 0);
    }

    public function testAssetTransfersPermissionDelete()
    {
        $status = \Stage\Asset\Transfer\Permission::delete(4);
        $this->assertTrue($status);
    }

    public function testAssetTransfersNoticeBodyDelete()
    {
        $status = \Stage\Asset\Transfer\NoticeBody::delete(4);
        $this->assertTrue($status);
    }

    public function testAssetTransfersPermissionFind()
    {
        $transfer = \Stage\Asset\Transfer\Permission::find(1);
        $this->assertInstanceOf('\Stage\Asset\Transfer\Permission', $transfer);
    }

    public function testAssetTransfersNoticeBodyFind()
    {
        $transfer = \Stage\Asset\Transfer\NoticeBody::find(1);
        $this->assertInstanceOf('\Stage\Asset\Transfer\NoticeBody', $transfer);
    }

    public function testAssetTransfersPermissionQuery()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $transfers = \Stage\Asset\Transfer\Permission::query($q);
        foreach ($transfers as $transfer) {
            $this->assertInstanceOf('\Stage\Asset\Transfer\Permission', $transfer);
        }
    }

    public function testAssetTransfersNoticeBodyQuery()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $transfers = \Stage\Asset\Transfer\NoticeBody::query($q);
        foreach ($transfers as $transfer) {
            $this->assertInstanceOf('\Stage\Asset\Transfer\NoticeBody', $transfer);
        }
    }

    public function testAssetTransfersUserCreate()
    {
        $q = new \Stage\Asset\Transfer\User();
        $q->save(1,1,1,57,1,"name1", "email@gmail");
        $this->assertInstanceOf('\Stage\Asset\Transfer\User', $q);
        $this->assertTrue($q->getId() > 0);
    }

    public function testAssetTransfersUserDelete()
    {
        $status = \Stage\Asset\Transfer\User::delete(4);
        $this->assertTrue($status);
    }

    public function testAssetTransfersUsersQuery()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $transfers = \Stage\Asset\Transfer\User::query($q);
        foreach ($transfers as $transfer) {
            $this->assertInstanceOf('\Stage\Asset\Transfer\User', $transfer);
        }
    }

    public function testExternalAssetTransfersUsersQuery()
    {
        $q = new \Stage\Asset\Transfer\Query();
        $transfers = \Stage\External\Asset::externalAssetsTransfersUsers($q);
        foreach ($transfers as $transfer) {
            $this->assertInstanceOf('\Stage\Asset\Transfer\User', $transfer);
        }
    }

    public function testAssetExternalTransfersUserVerify()
    {
        $status = \Stage\External\Asset::externalSetTransfersUserVerify(1,"some@gmail.com");
        $this->assertTrue($status);
    }
}