<?php

use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Asset\Asset;
use Stage\Post\Dossier\Dossier;
use Stage\Post\Tag\Tag;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Post\Collection\Collection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class AuthorTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testCreateAuthor()
    {
        $author = new \Stage\Post\Author\Author();
        $author->save();
    }

    public function testDeleteAuthor()
    {
        $author = \Stage\Post\Author\Author::find(18);
        if( $author instanceof \Stage\Post\Author\Author ){
            $status = $author->delete();
            $this->assertTrue($status);
        }
    }

    public function testDuplicateAuthor()
    {
        $author = \Stage\Post\Author\Author::find(1);
        $author->duplicate();
    }

    public function testCreateL10nAuthor()
    {
        $l10n = new \Stage\Post\Author\L10n();
        $l10n->setIdAuthor(1);
        $l10n->setIsoLanguage("pt");
        $l10n->setName("name");
        $l10n->setEmail("email@");
        $l10n->setDescription("desc");
        $l10n->setRole("name");
        $l10n->setBiography("name");
        $l10n->setImage("image.png");
        $l10n->setMetaTitle("meta title");
        $l10n->setMetaDescription("meta desc");
        $l10n->setMetaImage("meta image");
        $l10n->setMetaUrl("meta url");
        $l10n->setImageTitle("image title");
        $l10n->setImageAlterText("image alter text");
        $l10n->setMetaKeywords("meta keywords");
        $l10n->save();
    }

    public function testGetAllL10nAuthor()
    {
        $l10ns = \Stage\Post\Author\L10n::all(1);
        print_r($l10ns);
    }

    public function testAttachAssetL10nAuthor()
    {
        $asset = Asset::find(4);
        $l10n = \Stage\Post\Author\L10n::find(4, 9);
        dd($l10n);
        $l10n->attachAsset($asset);
    }

    public function testDeleteAllAuthorL10n()
    {
        $status = \Stage\Post\Author\L10n::delete(10);
    }

    public function testQueryAllAuthors()
    {
        $q = new \Stage\Post\Author\Query();
        $result = \Stage\Post\Author\Author::query($q);
        print_r($result);
    }

    public function testInnerLanguagesAuthors()
    {
        $author = \Stage\Post\Author\Author::find(1);
        $isos = $author->innerLanguages();
        print_r($isos);
    }

    public function testL10nsAllAuthors()
    {
        $author = \Stage\Post\Author\Author::find(1);
        $l10ns = $author->l10ns();
        print_r($l10ns);
    }

    public function testL10nFindAuthors()
    {
        $authorL10n = \Stage\Post\Author\L10n::find(1, 1);
        print_r($authorL10n->toArray());
    }

    public function testKeyCreateAuthor()
    {
        $key = new \Stage\Post\Author\Key\Key();
        $key->setName("name");
        $key->setNameStripped("name -stripped");
        $key->setPublic(true);
        $key->save();
    }

    public function testKeyCreateFind()
    {
        $key = \Stage\Post\Author\Key\Key::find(1);
        print_r($key->toArray());
    }

    public function testKeyDelete()
    {
        $key = \Stage\Post\Author\Key\Key::find(3);
        print_r($key->delete());
    }

    public function testKeyQuery()
    {
        $q = new \Stage\Post\Author\Key\Query();
        $results = \Stage\Post\Author\Key\Key::query($q, true);
        print_r($results->toArray());
    }

    public function testQueryVarianteKeyAuthorL10n()
    {
        $l10n = \Stage\Post\Author\L10n::find(1, 1);
        $variantKeys = $l10n->variantKeys();
        print_r($variantKeys->toArray());
        die();
    }

    public function testAttachVarianteKeyAuthorL10n()
    {
        $l10n = \Stage\Post\Author\L10n::find(1, 1);
        $variantKeys = $l10n->attachVariantKey(1, 1, "variante vl", 1);
    }

    public function testDetachVarianteKeyAuthorL10n()
    {
        $l10n = \Stage\Post\Author\L10n::find(1, 1);
        $l10n->detachVariantKey(1);
    }
}
