<?php

use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Asset\Asset;
use Stage\Post\Dossier\Dossier;
use Stage\Post\Tag\Tag;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Post\Collection\Collection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class CollectionTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testCreateCollection()
    {
        $now = new DateTime();
        $collection = new Collection();
        $collection->setManagementName("manage_ane1m");
        $collection->setPublishDate($now);
        $collection->setBeginDate($now);
        $collection->setEndDate($now);
        $collection->setDomain("*");
        $collection->setDefaultIdDossier(0);
        $collection->setPosition(1);

        $collection->save();
    }

    public function testCollectionGetAllL10nFind()
    {
        $l10ns = \Stage\Post\Collection\L10n::all(11);
    }

    public function testCollectionFind()
    {
        $collection = Collection::find(11);
    }

    public function testCollectionSizeOf()
    {
        $total = Collection::sizeof();
    }

    public function testCollectionDuplicate()
    {
        $collection = Collection::find(11);
        $duplicated = $collection->duplicate();
    }

    public function testCollectionInnerLanguages()
    {
        $collection = Collection::find(1);
        $isos = $collection->innerLanguages();
    }

    public function testQueryCollection()
    {
        $q = new \Stage\Post\Collection\Query();
        $q->sndFilter = "";
        $q->trdFilter = "";
        $q->limitSysA = "";

        $result = Collection::query($q);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Collection\Collection', $result);
        $this->assertGreaterThan(0, $result->toArray());
    }

    public function testCreateL10nCollection()
    {
        $l10n = new \Stage\Post\Collection\L10n();
        $l10n->setIdCollection(1);
        $l10n->setIsoLanguage("fr");
        $l10n->setName("name");
        $l10n->setDescription("desc");
        $l10n->setImage("image.png");
        $l10n->setMetaTitle("metatitle");
        $l10n->setMetaDescription("meta desc");
        $l10n->setMetaImage("meta image");
        $l10n->setImageTitle("image title");
        $l10n->setImageAlterText("image alter text");
        $l10n->setMetaKeywords("meta keywords");
        $l10n->setMetaUrl("meta url");
        $l10n->setMetaWatermarkImage("meta water mark image");
        $l10n->save();

        $this->assertTrue($l10n instanceof \Stage\Post\Collection\L10n);
    }

    public function testFindL10nCollection()
    {
        $l10n = \Stage\Post\Collection\L10n::find(1, 1);
        print_r($l10n->toArray());
    }

    public function testAttachTagToL10nCollection()
    {
        $tag = Tag::find(1);
        $l10nCollection = \Stage\Post\Collection\L10n::find(1, 1);
        $result = $l10nCollection->attachTag($tag);
    }

    public function testDetachAllTagToL10nCollection()
    {
        $l10nCollection = \Stage\Post\Collection\L10n::find(1, 1);
        $result = $l10nCollection->detachAllTags();
        print_r($result);
    }

    public function testQueryCollectionDossiers()
    {
        $q = new \Stage\Post\Collection\Query();
        $q->idDossier = 1;
        $result = Collection::dossiers($q);
    }

    public function testTagCollectionCreate()
    {
        $tag = new \Stage\Post\Collection\Tag\Tag();
        $tag->setName("name");
        $tag->setNameStripped("name stipped");
        $tag->setPublic(true);
        $tag->save();
    }

    public function testTagCollectionFind()
    {
        $tag = \Stage\Post\Collection\Tag\Tag::find(1);
        print_r($tag->toArray());
    }

    public function testTagCollectionSizeOf()
    {
        $total = \Stage\Post\Collection\Tag\Tag::sizeof();
        print_r($total);
    }

    public function testTagCollectionDelete()
    {
        $tag = \Stage\Post\Collection\Tag\Tag::find(4);
        print_r($tag->delete());
    }

    public function testTagCollectionQuery()
    {
        $q = new \Stage\Post\Collection\Tag\Query();
        $data = \Stage\Post\Collection\Tag\Tag::query($q, true);
        print_r($data->count());
    }

    public function testTagCollection()
    {
        $collection = \Stage\Post\Collection\Collection::find(1);
        $tags = $collection->tags();
        print_r($tags->toArray());
    }

    public function testTagCollectionL10n()
    {
        $l10n = \Stage\Post\Collection\L10n::find(11, 18);
        $tags = $l10n->tags();
        print_r($tags->toArray());
    }

    public function testAttachDossier()
    {
        $collection = \Stage\Post\Collection\Collection::find(1);
        $status = $collection->attachDossier(1, 1);
        print_r($status);
    }

    public function testStickerAttachToCollection()
    {
        $collection = Collection::find(1);
        $now = new DateTime();
        $result = $collection->attachSticker(3, 1, null, null, "red");
        $this->assertTrue($result);
    }

    public function testStickerDetachFromCollection()
    {
        $collection = Collection::find(1);
        $result = $collection->detachSticker(3);
        $this->assertTrue($result);
    }

    public function testGetAllStickersFromCollection()
    {
        $collection = Collection::find(1);
        $result = $collection->stickers();
        $this->assertContainsOnly("\Stage\Post\Collection\Sticker\Collection", $result);
    }
}
