<?php

use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Post\Dossier\Dossier;
use Stage\Asset\Asset;
use Stage\Post\Tag\Tag;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Post\Collection\Collection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class DossierTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testDossierCreate()
    {
        $now = new DateTime();
        $dossier = new Dossier();
        $dossier->setManagementName("managem name 3");
        $dossier->setPublishDate($now);
        $dossier->setBeginDate($now);
        $dossier->setEndDate($now);
        $dossier->save();
    }

    public function testCreateDossierL10n()
    {
        $l10n = new \Stage\Post\Dossier\L10n();
        $l10n->setIsoLanguage("al");
        $l10n->setName("name");
        $l10n->setDescription("desc");
        $l10n->setImage("image.png");
        $l10n->setMetaTitle("metatitle");
        $l10n->setMetaDescription("meta desc");
        $l10n->setMetaImage("meta image");
        $l10n->setImageTitle("image title");
        $l10n->setImageAlterText("image alter text");
        $l10n->setMetaKeywords("meta keywords");
        $l10n->setMetaUrl("meta url");
        $l10n->setMetaWatermarkImage("meta water mark image");
        $l10n->save(1);
    }

    public function testFindL10nDossier()
    {
        $l10n = \Stage\Post\Dossier\L10n::find(1, 1);
        print_r($l10n->toArray());
    }

    public function testAttachTagToL10nDossier()
    {
        $tag = Tag::find(1);
        $l10nDossier = \Stage\Post\Dossier\L10n::find(1, 1);
        $result = $l10nDossier->attachTag($tag);
    }

    public function testDetachAllTagToL10nDossier()
    {
        $l10nCollection = \Stage\Post\Collection\L10n::find(1, 1);
        $result = $l10nCollection->detachAllTags();
        print_r($result);
    }

    public function testL10nDossierGetTags()
    {
        $l10n = \Stage\Post\Dossier\L10n::find(1, 1);
        $tags = $l10n->tags();
        print_r($tags->toArray());
    }

    public function testTagDossierGetAllTags()
    {
        $dossier = Dossier::find(1);
        $tags = $dossier->tags();
        print_r($tags->toArray());
    }

    public function testQueryDossiers()
    {
        $q = new \Stage\Post\Dossier\Query();
        $result = Dossier::query($q);

        print_r($result);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Dossier\Dossier', $result);
        $this->assertGreaterThan(0, $result->toArray());
    }

    public function testTagDossierCreate()
    {
        $tag = new \Stage\Post\Dossier\Tag\Tag();
        $tag->setName("name");
        $tag->setNameStripped("name stipped");
        $tag->setPublic(true);
        $tag->save();
    }

    public function testTagDossierQuery()
    {
        $q = new \Stage\Post\Dossier\Tag\Query();
        $data = \Stage\Post\Dossier\Tag\Tag::query($q, true);
        print_r($data->toArray());
    }

    public function testAttachPostToDossier()
    {
        $dossier = Dossier::find(1);
        $dossier->attachPost(22);
    }

    public function testDetachPostFromDossier()
    {
        $dossier = Dossier::find(1);
        $dossier->detachPost(22);
    }

    public function testStickerAttachToDossier()
    {
        $dossier = Dossier::find(1);
        $now = new DateTime();
        $result = $dossier->attachSticker(3, 1, null, null, "red");
        $this->assertTrue($result);
    }

    public function testStickerDetachFromDossier()
    {
        $dossier = Dossier::find(1);
        $result = $dossier->detachSticker(2);
        $this->assertTrue($result);
    }

    public function testGetAllStickersFromDossier()
    {
        $dossier = Dossier::find(1);
        $result = $dossier->stickers();
        $this->assertContainsOnly("\Stage\Post\Dossier\Sticker\Dossier", $result);
    }
}
