<?php

use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Asset\Asset;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class LabelsTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testLabelCreate()
    {
        $label = new \Stage\Post\Label\Label();
        $label->setName("name");
        $label->setNameStripped("name stripped");
        $label->setPublic(false);
        $label->save();
    }

    public function testLabelAttachToL10n()
    {
        $l10n = \Stage\Post\L10n\L10n::find(22, "pt");
        $status = $l10n->attachLabel(1);
        var_dump($status);
    }

    public function testLabelSizeOf()
    {
        $sizeof = \Stage\Post\Label\Label::sizeof();
        var_dump($sizeof);
    }

    public function testLabelQuery()
    {
        $q = new \Stage\Post\Label\Query();
        $data = \Stage\Post\Label\Label::query($q);
        print_r($data->toArray());
    }

    public function testLabelsFromL10ns()
    {
        $l10n = \Stage\Post\L10n\L10n::find(22, "pt");
        print_r($l10n->labels());
    }
}
