<?php

use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Asset\Asset;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Video\Profile\Profile;
use Stage\Video\Profile\L10n\L10n;
use Stage\Video\Profile\L10n\Body;
use Stage\Video\Profile\Metadata\Metadata;
use Stage\StoredProcedure\Manager as StoredProcedureManager;
use Stage\Video\SneakPeek;
use Stage\Video\Track;

class LiveTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "192.168.70.21",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "192.168.70.21",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function createPost()
    {
        // TODO
        // 1. Execute the SET_POSTS
        // 2. Para cada language executar SET_POSTS_L10N
        // 3. Apos executar a SET_POSTS_L10N executar para cada um a insercao das TAGS, BODY, PHOTOS

        // Data structure sent when creating or updating


        $l10n = ['id' => 12,
            'title' => '',
            'description' => '',
            'isoLanguage' => '',
            'beginDate' => '',
            'endDate' => '',
            'publishDate' => '',
            'metadata' => [
                'title' => '',
                'description' => '',
                'keywords' => '',
                'image' => [
                    'src' => '',
                    'title' => '',
                    'alt' => ''
                ]
            ],
            'body' => [
                [
                    'id'=>2,
                    'body' =>'HELLO'
                ],
                [
                    'id'=>3,
                    'body' =>'HELLO'
                ]
            ],
            'tags'=>[
                [
                    'id' => 2,
                    'name' => 'aaa',
                    'nameStripped' => 'name-stripped',
                    'public' => false,
                    'total' => 5
                ],
                [
                    'id' => 6,
                    'name' => 'bbb',
                    'nameStripped' => 'name-stripped',
                    'public' => false,
                    'total' => 5
                ]
            ]
        ];

        $req = [
            'id' => 12,
            'l10n' => [$l10n, $l10n, $l10n],
        ];

        //echo json_encode($req);


        $d1 = new DateTime("2012-07-08 11:14:15.638276");
        $d2 = new DateTime("2012-07-08 11:14:15.889342");

        $post = new Post();
        $l10n = new \Stage\Post\L10n\L10n();
        $l10en = new \Stage\Post\L10n\L10n();
        $metadata = new \Stage\Post\Metadata_todelete\Metadata();

        $l10n->setTitle("Post Title");                      // Optional
        $l10n->setDescription("Some Post Description");     // Optional
        $l10n->setIsoLanguage("pt");                        // Mandatory

        $l10n->setImage("IMAGE_PATH.png");                  // Optional

        $l10n->setBeginDate($d1);                           // Mandatory
        $l10n->setPublishDate($d1);                         // Mandatory
        $l10n->setEndDate($d2);                             // Mandatory

        $l10en->setTitle("Post Title");                     // Optional
        $l10en->setDescription("Some Post Description");    // Optional
        $l10en->setIsoLanguage("en");                       // Mandatory

        $l10en->setBeginDate($d1);                          // Mandatory
        $l10en->setPublishDate($d1);                        // Mandatory
        $l10en->setEndDate($d2);                            // Mandatory

        $metadata->setTitle("SOME METADATA TITLE");         // Optional
        $metadata->setDescription("Some meta description"); // Optional
        $metadata->setKeywords("Some Meta Keyworkds");      // Optional
        $metadata->setImage("IMAGE1252.png");               // Optional
        $metadata->setImageTitle("Some Meta Image title");  // Optional
        $metadata->setImageAlt("Some image alter text");    // Optional


        $l10n->addMetadata($metadata);
        //$l10en->addMetadata($metadata);

        $bodies = ["SOME HTML BODY 2", "SOME HTML BODY 2"]; // Optional
        foreach ($bodies as $b) {                              // Optional
            $body = new \Stage\Post\L10n\Body();            // Optional
            $body->setBody($b);                             // Optional
            $l10n->addBody($body);                          // Optional
        }

        $tags = [1,2,3,4];                                  // Optional
        foreach ($tags  as $tag) {
            $t = \Stage\Tag\Tag::find($tag);                // Optional
            $l10n->addTag($t);                              // Optional
        }

        // Add assets
        $asset1 = Asset::find(150);
        $asset2 = Asset::find(151);
        $l10n->addAsset($asset1);
        $l10n->addAsset($asset2);

        // Add albums
        $album1 = Album::find(1);
        $album2 = Album::find(2);
        $l10n->addAlbum($album1);
        $l10n->addAlbum($album2);


        // Add Post Languages as many as you want
        $post->addL10n($l10n);                              // Optional
        $post->addL10n($l10en);                             // Optional
        //$post->addL10n( clone $l10n );                    // Optional



        try {
            $post->save();
        } catch (Exception $e) {
        } catch (\Stage\Exception\StoredProcedure $e) {
        }

        print_r($post);

        echo json_encode($post->toArray());
    }

    public function testCreateProfile()
    {
        $profile = new Profile();
        $l10n = new L10n();
        $l10en = new L10n();
        $metadata = new Metadata();

        $l10n->setTitle("Profile Title Pt");                        // Optional
        $l10n->setDescription("Some Profile Description");     // Optional
        $l10n->setIsoLanguage("pt");                          // Mandatory
        $l10n->setImage("IMAGE_PATH.png");                         // Optional

        $l10en->setTitle("Profile Title En");                       // Optional
        $l10en->setDescription("Some Profile Description");    // Optional
        $l10en->setIsoLanguage("en");                         // Mandatory

        $metadata->setTitle("SOME METADATA TITLE");         // Optional
        $metadata->setDescription("Some meta description"); // Optional
        $metadata->setKeywords("Some Meta Keyworkds");      // Optional
        $metadata->setImage("IMAGE1252.png");               // Optional
        $metadata->setImageTitle("Some Meta Image title");  // Optional
        $metadata->setImageAlt("Some image alter text");    // Optional

        $l10n->addMetadata($metadata);
        //$l10en->addMetadata($metadata);

        $bodies = ["SOME HTML BODY 2", "SOME HTML BODY 2"]; // Optional
        foreach ($bodies as $b) {                              // Optional
            $body = new Body();                             // Optional
            $body->setBody($b);                             // Optional
            $l10n->addBody($body);                          // Optional
        }

        $tags = [1,2];                                  // Optional
        foreach ($tags  as $tag) {
            $t = \Stage\Video\Tag\Tag::find($tag);          // Optional
            $l10n->addTag($t);                              // Optional
        }

        // Add assets
        $asset1 = Asset::find(66);
        $asset2 = Asset::find(67);
        $l10n->addAsset($asset1);
        $l10n->addAsset($asset2);

        // Add albums
        $album1 = Album::find(1);
        $album2 = Album::find(1);
        $l10n->addAlbum($album1);
        $l10n->addAlbum($album2);

        $profile->addL10n($l10n);
        $profile->addL10n($l10en);

        // print_r($profile);
        try {
            $profile->save();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFindProfile()
    {
        try {
            $profile = Profile::find(57);
            $this->assertTrue($profile instanceof Profile);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCloneProfile()
    {
        try {
            $profile = Profile::find(50);
            $profile->duplicate();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDeleteProfile()
    {
        try {
            $profile = Profile::find(51);
            $profile->delete();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testUpdateProfile()
    {
        try {
            $profile = Profile::find(31);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testQueryProfiles()
    {
        try {
            $query = new \Stage\Video\Profile\Query();
            $query->by = \Stage\Video\Profile\Query::NAME;  // $field
            $query->sndBy = 0;                             // $sndField
            $query->trdBy = 0;                             // $trdField
            $query->term = "";                              // $query
            $query->filter = "";                            // $filter
            $query->sndFilter = "";                         // $sndFilter
            $query->trdFilter = "";                         // $trdFilter
            $query->isoLanguage = "";
            $query->from = 0;                              // $limit_a
            $query->to = -1;                                // $limit_b

            $results = Profile::query($query);
            print_r($results);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCreatePlaylist()
    {
        try {
            $playlist = new \Stage\Video\Playlist\Playlist();
            $playlist->setName("Red");
            $playlist->setManagementName("Red");
            $playlist->setRandom(true);

            $l10n = new \Stage\Video\Playlist\L10n();
            $l10n->setIsoLanguage("pt");
            $l10n->setName("Red");
            $playlist->addL10n($l10n);

            $playlist->save();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFindPlaylist()
    {
        try {
            $playlist = \Stage\Video\Playlist\Playlist::find(9);
            print_r($playlist);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testClonePlaylist()
    {
        try {
            $playlist = \Stage\Video\Playlist\Playlist::find(3);
            $playlist->duplicate();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDeletePlaylist()
    {
        try {
            $playlist = \Stage\Video\Playlist\Playlist::find(9);
            $playlist->delete();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCreateSneakPeek()
    {
        try {
            $video = \Stage\Video\Video::find(17);

            $sneak = new SneakPeek();
            $sneak->setName("Named");
            $sneak->setSecondsIn(553);
            $sneak->setSecondsOut(66);
            $sneak->setIsDefault(true);
            $video->createSneakPeak($sneak);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFindSneakPeek()
    {
        try {
            $video = \Stage\Video\Video::find(17);
            $snekPeeks = $video->sneakPeaks();
            print_r($snekPeeks);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDeleteSneakPeek()
    {
        try {
            $video = \Stage\Video\Video::find(17);
            $video->deleteSneakPeeks();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testCreateTrack()
    {
        try {
            $video = \Stage\Video\Video::find(17);

            $sneak = new Track();
            $sneak->setIsoLanguage("pt");
            $sneak->setFilename("aa.str");
            $sneak->setExtension("srt");
            $sneak->setType("vtt");
            $sneak->setLabel("1");
            $sneak->setIsDefault(true);
            $video->track($sneak);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFindTracks()
    {
        try {
            $video = \Stage\Video\Video::find(17);
            $tracks = $video->tracks();
            print_r($tracks);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testTracksDeleted()
    {
        try {
            $video = \Stage\Video\Video::find(17);
            $tracks = $video->tracksDeleted();
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testSoftDeleteTrack()
    {
        try {
            $video = \Stage\Video\Video::find(17);
            $deleted = $video->deleteTrack(true, 9);
            print_r($deleted);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPushLiveClipToQueue()
    {
        try {
            $clip = new \Stage\Live\Queue\Clip();
            $clip->setIdAccount(3);
            $clip->setAccountToken('TOKEN3');
            $clip->setCommand("");
            $clip->setServer("");
            $clip->setApplication("");
            $clip->setStream("");
            $clip->setConverterStartTime(0);
            $clip->setConverterEndTime(0);
            $clip->setConverterDuration(0);
            $clip->setConverterTimeType("");
            $clip->setConverterTolerance(0);
            $clip->setConverterOutputFilename("");
            $clip->setConverterInterleaveDelay(0);
            $clip->setConverterDebugConversions(0);
            $clip->setConverterDefaultFileDestination("");
            $clip->setState("");
            $clip->setStatusCode("");

            $return = $clip->save();
            print_r($return);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetCreateLiveClipNext()
    {
        try {
            $next = \Stage\Live\Queue\Clip::next();
            print_r($next);
        } catch (\Stage\Exception\StoredProcedure $e) {
            echo $e->getMessage();
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }
}
