<?php

use Stage\Account\Account;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class PostCustomKeyTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }


    public function testPostCreateCustomKey()
    {
        try {
            $customKey = new \Stage\Post\CustomFieldKey();
            $customKey->setName("name");
            $customKey->setNameStripped("stripped");
            $customKey->setPublic(true);

            $result = $customKey->save();
            $this->assertInstanceOf('\Stage\Post\CustomFieldKey', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostGetCustomKey()
    {
        try {
            $customKey = \Stage\Post\CustomFieldKey::find(1);
            $this->assertInstanceOf('\Stage\Post\CustomFieldKey', $customKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostDeleteCustomKeys()
    {
        try {
            $customKey = \Stage\Post\CustomFieldKey::find(3);
            $status = $customKey->delete();
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostQueryCustomKeys()
    {
        try {
            $q = new \Stage\Post\CustomFieldKeyQuery();
            $result = \Stage\Post\CustomFieldKey::query($q);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

}
