<?php

use Stage\Post\Message\CustomFieldKey;
use Stage\Post\Message\Group\Group;
use Stage\Post\Message\Group\Query;
use Stage\Post\Message\Group\Schema;
use Stage\Post\Message\Message;
use Stage\Post\Message\MessagePostsQuery;
use Stage\Post\Post;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class PostMessageGroupTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testPostsMessageGroupCreate()
    {
        try {
            $schema = new Group();
            $schema->setIdType(1);
            $schema->setName("name2");
            $schema->setNameStripped("name3");
            $schema->setDomain("domain");
            $schema->setTrackingParameters("tracking");
            $schema->save();
            $this->assertInstanceOf('\Stage\Post\Message\Group\Group', $schema);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostMessageGroupFind()
    {
        $data = Group::find(1);
        $this->assertInstanceOf('\Stage\Post\Message\Group\Group', $data);
    }

    public function testPostMessageGroupQuery()
    {
        $q = new Query();
        $q->idType = 1;
        $data = Group::query($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\Group\Group', $data);
    }

    public function testAttachPostMessageGroupCustomFieldVariant()
    {
        try {
            $group = Group::find(1);
            $idKey = 1; // it's the id key from table posts_messages_custom_fields_keys
            $customFieldVariant = $group->attachCustomFieldVariant($idKey, "name1", 1);
            $this->assertInstanceOf('\Stage\Post\Message\Group\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachPostMessageGroupCustomFieldVariant()
    {
        try {
            $group = Group::find(1);
            $status = $group->detachCustomFieldsVariant(2);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostMessageGroupQueryVariantKeys()
    {
        $q = new \Stage\Post\Message\Group\Query();
        $data = Group::variantKeys($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\CustomFieldKey', $data);
    }

    public function testPostMessageGroupFindVariantKey()
    {
        try {
            $idKey = 1;
            $customVariantKey = Group::variantKey($idKey);
            $this->assertInstanceOf('\Stage\Post\Message\CustomFieldKey', $customVariantKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachPostMessageGroupVariantKey()
    {
        try {
            $idKey = 3;
            $status = Group::detachVariantKey($idKey);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachPostMessageGroupVariantKey()
    {
        try {
            $name = 'name';
            $nameStripped = 'name_s';
            $public = true;
            $ifExists = 0;
            $idKey = -1; // it's the id key from table posts_messages_custom_fields_keys, or -1 to create
            $customFieldVariant = Group::attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists);
            $this->assertInstanceOf('\Stage\Post\Message\CustomFieldKey', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsMessageGroupSchemaCreate()
    {
        $idGroup = 1;
        $isoLanguage = 'pt';
        $macroSchema = "%a%b";

        try {
            $schema = new Schema();
            $schema->setIdGroup($idGroup);
            $schema->setIsoLanguage($isoLanguage);
            $schema->setMacroSchema($macroSchema);
            $schema->save();
            $this->assertInstanceOf('\Stage\Post\Message\Group\Schema', $schema);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsMessageGroupSchemaDelete()
    {
        try {
            $status = Schema::delete(1,5);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsMessageGroupSchemas()
    {
        try {
            $schemas = Schema::find(1,0);
            $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\Group\Schema', $schemas);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDeletePostMessageGroup()
    {
        $message = Group::find(13);
        $status = $message->delete();
        $this->assertTrue($status);
    }
}