<?php

use Stage\Post\Message\CustomFieldKey;
use Stage\Post\Message\Message;
use Stage\Post\Message\MessagePostsQuery;
use Stage\Post\Post;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class PostMessageTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testCreatePostMessage()
    {
        $now = new DateTime();
        $message = new Message();
        $message->setIdType(1);
        $message->setIsoLanguage("pt");
        $message->setManagementName("managment_name");
        $message->setTitle("title");
        $message->setMessage("message");
        $message->setImage("image.png");
        $message->setLink("link");
        $message->setBeginDate($now);
        $result = $message->save();

        $this->assertInstanceOf('\Stage\Post\Message\Message', $result);
        $this->assertTrue($result->getId() > 0);
    }

    public function testAttachPostToMessage()
    {
        $message = Message::find(1);
        $this->assertInstanceOf('\Stage\Post\Message\Message', $message);

        $status = $message->attachPost(16);
        $this->assertTrue($status);
    }

    public function testMessagesSizeOf()
    {
        $total = Message::sizeof();
        $this->assertTrue($total > 0);
    }

    public function testMessagesTypes()
    {
        $field = 1;
        $filter = 1;
        $types = Message::types($field, $filter);
        $this->assertTrue($types->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\Type', $types);
    }

    public function testPostMessageQuery()
    {
        $q = new \Stage\Post\Message\Query();
        $data = Message::query($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\Message', $data);
    }

    public function testPostMessageGetPostsAssociated()
    {
        $q = new \Stage\Post\Message\MessagePostsQuery();
        $q->idMessage = 12;
        $data = Message::posts($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\MessagePost', $data);
    }

    public function testDeletePostMessage()
    {
        $message = Message::find(5);
        $status = $message->delete();
        $this->assertTrue($status);
    }

    public function testAttachPostMessageCustomFieldVariant()
    {
        try {
            $message = Message::find(6);
            $idKey = 1; // its the id key from table posts_messages_custom_fields_keys
            $customFieldVariant = $message->attachCustomFieldVariant($idKey, "name1", 1);
            $this->assertInstanceOf('\Stage\Post\Message\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachPostMessageCustomFieldVariants()
    {
        try {
            $message = Message::find(6);
            $customFieldVariant = $message->customFieldsVariants();
            $this->assertContainsOnly('\Stage\Post\Message\CustomFieldVariant', $customFieldVariant);
            $this->assertTrue($customFieldVariant->count() > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachPostMessageCustomFieldVariant()
    {
        try {
            $message = Message::find(6);
            $status = $message->detachCustomFieldsVariant(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachPostMessageVariantKey()
    {
        try {
            $name = 'name';
            $nameStripped = 'name_s';
            $public = true;
            $ifExists = 0;
            $idKey = -1; // it's the id key from table posts_messages_custom_fields_keys, or -1 to create
            $customFieldVariant = Message::attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists);
            $this->assertInstanceOf('\Stage\Post\Message\CustomFieldKey', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachPostMessageVariantKey()
    {
        try {
            $idKey = 6;
            $status = Message::detachVariantKey($idKey);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostMessageFindVariantKey()
    {
        try {
            $idKey = 7;
            $customVariantKey = Message::variantKey($idKey);
            $this->assertInstanceOf('\Stage\Post\Message\CustomFieldKey', $customVariantKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostMessageQueryVariantKeys()
    {
        $q = new \Stage\Post\Message\Query();
        $data = Message::variantKeys($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\CustomFieldKey', $data);
    }

    public function testPostMessageQueryAllVariantKeys()
    {
        $data = CustomFieldKey::all();
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Message\CustomFieldKey', $data);
    }

    public function testPostMessageDuplicate()
    {
        $message = Message::find(1)->duplicate();
        $this->assertInstanceOf('\Stage\Post\Message\Message', $message);
    }

}
