<?php

use Stage\Post\Post;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class PostTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testTagPostQuery()
    {
        $q = new \Stage\Post\Tag\Query();
        $data = \Stage\Post\Tag\Tag::query($q, true);
        print_r($data->toArray());
    }

    public function testCreatePost()
    {
        $post = new Post();
        $post->save(Post::CREATE, 1);  // persist on database

        $now = new \DateTime();
        $l10n = new \Stage\Post\L10n\L10n();
        $l10n->setIsoLanguage("pt");
        $l10n->setDraft(true);
        $l10n->setTitle("title");
        $l10n->setDescription("description");
        $l10n->setImage("image.png");
        $l10n->setMetaTitle("meta title");
        $l10n->setMetaDescription("meta description");
        $l10n->setMetaImage("metaimage.png");
        $l10n->setMetaUrl("meta url");
        $l10n->setImageTitle("image title");
        $l10n->setImageAlterText("alter text");
        $l10n->setMetaKeywords("meta keywords");
        $l10n->setMetaPlayMark(true);
        $l10n->setMetaSync(true);
        $l10n->setPublishDate($now);
        $l10n->setEndDate($now);
        $l10n->setBeginDate($now);

        try {
            $post->attachL10n($l10n); // persist on database
        } catch (Exception $e) {
            echo $e->getMessage();
        }

        $l10n->attachBody(-1, "text", 1, "", false, false, 1, 0); // persist on database
        $l10n->attachTag(1);
        $l10n->attachAsset(4);
        $l10n->attachAuthor(1);
        $l10n->attachAlbum(3);
        $l10n->attachVideoCompose(12);

        // Add L10n
        // Add Author
        // Add Asset
        // Add Tag
        // Add Body
    }

    public function testFindPostTypes()
    {
        $types = Post::types(1);
        $this->assertContainsOnly('\Stage\Post\Type', $types);
    }

    public function testFindSinglePost()
    {
        $post = Post::find(23);
        print_r($post->toArray());
    }

    public function testDetachAllL10nFromPost()
    {
        $post = Post::find(29);
        $post->detachL10n();
    }

    public function testUpdatePost()
    {
        $post = Post::find(22);

        // Get all L10n's
        $l10ns = $post->l10ns();

        /** @var \Stage\Post\L10n\L10n $l10n */
        foreach ($l10ns as $l10n) {
            $l10n->detachBody();
            $l10n->detachAllTags();
            $l10n->detachAllAssets();
            $l10n->detachAllAuthors();
            $l10n->detachAllAlbums();
            $l10n->detachAllVideoComposes();
            $l10n->detachAllLabels();
        }
        print_r($l10ns);
    }

    public function testFindPost()
    {
        $post = Post::find(22);
        $l10ns = $post->l10ns();
        print_r($post->toArray());
        print_r($l10ns->toArray());
    }

    public function testQueryPost()
    {
        $q = new \Stage\Post\Query();
        $data = Post::query($q, true);
        print_r($data->toArray());
    }

    public function testAttachPostToL10n()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $status = $l10n->attachPost(16);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetL10nPosts()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $posts = $l10n->posts();
            $this->assertContainsOnly('\Stage\Post\Post', $posts);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachAllL10nPosts()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $status = $l10n->detachAllPosts();
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachL10nKey()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $key = $l10n->attachKey(-1, "name1", "name2", true, true);
            $this->assertInstanceOf('\Stage\Post\L10n\Key\Key', $key);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachL10nKey()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $status = $l10n->detachKey(3);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetL10nKey()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $key = $l10n->key(6);
            $this->assertInstanceOf('\Stage\Post\L10n\Key\Key', $key);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachL10nCustomFieldVariant()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $idKey = 1; // its the id key from table posts_custom_fields_keys
            $customFieldVariant = $l10n->attachCustomFieldVariant($idKey, "name1", 1);
            $this->assertInstanceOf('\Stage\Post\L10n\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetL10nCustomFieldVariants()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $customFieldVariant = $l10n->customFieldsVariants();
            $this->assertContainsOnly('\Stage\Post\L10n\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachL10nCustomFieldVariant()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $status = $l10n->detachCustomFieldsVariant(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachL10nImageOptions()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $status = $l10n->attachImageOptions("1:10", "left", "q=10", "settings");
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetL10nImageOptions()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(37, "pt");
            $options = $l10n->imageOptions();
            $this->assertContainsOnly('\Stage\Post\L10n\ImageOption', $options);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFeedExternalCreate()
    {
        $feed = new \Stage\Post\Feed\External();
        $feed->setName("name");
        $feed->setUrl("http://");

        try {
            $f = $feed->save();
            $this->assertTrue($f instanceof \Stage\Post\Feed\External);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsExternalFindOne()
    {
        try {
            $result = \Stage\Post\Feed\External::find(1);
            $this->assertInstanceOf('\Stage\Post\Feed\External', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostFeedExternalDelete()
    {
        try {
            $feed = \Stage\Post\Feed\External::find(2);
            $status = $feed->delete();
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsExternalQuery()
    {
        try {
            $q = new \Stage\Post\Feed\Query();
            $results = \Stage\Post\Feed\External::query($q);
            $this->assertContainsOnlyInstancesOf('\Stage\Post\Feed\External', $results);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFeedInternalCreate()
    {
        $feed = new \Stage\Post\Feed\Internal();
        $feed->setName("name");
        $feed->setDomain("sapo.pt");
        $feed->setLength(2);
        $feed->setType(1);
        $feed->setCanOverwrite(true);
        $feed->setTrackingParameters('utm=1');
        $feed->setImageOptimization('q=5');
        $feed->setTitle('title');
        $feed->setDescription('desc');
        $feed->setImage('image');
        $feed->setCopyright('copy');
        $feed->setNumberOfBodies(10);

        try {
            $f = $feed->save();
            print_r($f->toArray());
            $this->assertTrue($f instanceof \Stage\Post\Feed\Internal);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetTotalNumberOfFeedsInternal()
    {
        try {
            $total = \Stage\Post\Feed\Internal::sizeof();
            $this->assertTrue($total > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsQuery()
    {
        try {
            $q = new \Stage\Post\Feed\Query();
            $results = \Stage\Post\Feed\Internal::query($q);
            $this->assertContainsOnlyInstancesOf('\Stage\Post\Feed\Internal', $results);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsFindOne()
    {
        try {
            $result = \Stage\Post\Feed\Internal::find(1);
            $this->assertInstanceOf('\Stage\Post\Feed\Internal', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostFeedInternalDelete()
    {
        try {
            $feed = \Stage\Post\Feed\Internal::find(2);
            $status = $feed->delete();
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsInternalFindOne()
    {
        try {
            $result = \Stage\Post\Feed\Internal::find(1);
            $this->assertInstanceOf('\Stage\Post\Feed\Internal', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsOptions()
    {
        try {
            $result = \Stage\Post\Feed\Internal::options(1);
            $this->assertContainsOnly('\Stage\Post\Feed\Option', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsAttachOption()
    {
        try {
            $result = \Stage\Post\Feed\Internal::attachOption(1, 1, 'value');
            $this->assertTrue($result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsDetachOption()
    {
        try {
            $result = \Stage\Post\Feed\Internal::detachOption(1, 1);
            $this->assertTrue($result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsAttachSchema()
    {
        try {
            $schema = \Stage\Post\Feed\Internal::attachSchema(1, "pt", "{{post.id}}");
            print_r($schema->toArray());
            $this->assertTrue($schema instanceof \Stage\Post\Feed\Schema\Schema);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsDetachSchemaFromFeed()
    {
        try {
            $status = \Stage\Post\Feed\Internal::detachSchema(1, 3);
            print_r($status);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsGetSchemas()
    {
        try {
            $schemas = \Stage\Post\Feed\Internal::schemas(1);
            $this->assertContainsOnly('\Stage\Post\Feed\Schema\Schema', $schemas);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testFeedInternalClone()
    {
        try {
            $feedInternal = \Stage\Post\Feed\Internal::find(1);
            $feed = $feedInternal->duplicate();
            $this->assertInstanceOf('\Stage\Post\Feed\Internal', $feed);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsSchemaOptions()
    {
        try {
            $result = \Stage\Post\Feed\Schema\Schema::options();
            $this->assertContainsOnly('\Stage\Post\Feed\Schema\Option', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testGetL10nVariantsKeys()
    {
        try {
            $l10n = \Stage\Post\L10n\L10n::find(17, "pt");
            $variantKeys = $l10n->variantKeys();
            $this->assertContainsOnly('\Stage\Post\L10n\Key\Variant', $variantKeys);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testSavePostStats()
    {
        try {
            $idPost = 1;
            $idDossier = 2;
            $idCollection = 3;
            $day = "2022-11-07";
            $impressions = 5;
            $result = Post::saveStats($idPost, $idDossier, $idCollection, $day, $impressions);
            $this->assertTrue($result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testImageRatios()
    {
        try {
            $result = Post::imageRations(1);
            print_r($result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupCreate()
    {
        try {
            $now = new \DateTime();
            $group = \Stage\Post\L10n\BodyGroup\Group::save(-1, 6, 'title', $now, true, true, 'settings');
            $this->assertTrue($group instanceof \Stage\Post\L10n\BodyGroup\Group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupFind()
    {
        try {
            $group = \Stage\Post\L10n\BodyGroup\Group::find(16);
            $this->assertTrue($group instanceof \Stage\Post\L10n\BodyGroup\Group);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupDelete()
    {
        try {
            $idBodyGroup = 20;
            $status = \Stage\Post\L10n\BodyGroup\Group::delete($idBodyGroup);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyQuery()
    {
        $q = new \Stage\Post\L10n\BodyGroup\Query();
        $q->idL10n = 6;
        $data = \Stage\Post\L10n\BodyGroup\Group::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\L10n\BodyGroup\Group', $data);
    }

    public function testPostBodyGroupGetAllBodies()
    {
        $idBodyGroup = 16;
        $data = \Stage\Post\L10n\BodyGroup\Group::bodies($idBodyGroup);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\L10n\Body', $data);
    }

    public function testPostBodyGroupAttachAuthor()
    {
        $idGroup = 1;
        $idAuthor = 5;
        $result = \Stage\Post\L10n\BodyGroup\Group::attachAuthor($idGroup, $idAuthor);
    }

    public function testPostBodyGroupDetachAllAuthors()
    {
        $idGroup = 1;
        \Stage\Post\L10n\BodyGroup\Group::detachAllAuthors($idGroup);
    }

    public function testPostBodyGroupGetAuthors()
    {
        $idGroup = 1;
        $authors = \Stage\Post\L10n\BodyGroup\Group::authors($idGroup);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Author\L10n', $authors);
    }

    public function testAttachPostBodyGroupCustomFieldVariant()
    {
        try {
            $group = new \Stage\Post\L10n\BodyGroup\Group();
            $group->setId(1);
            $idKey = "1";
            $customFieldVariant = $group->attachCustomFieldVariant($idKey, "name1", 1);
            $this->assertInstanceOf('\Stage\Post\L10n\BodyGroup\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupCustomFieldVariants()
    {
        try {
            $group = new \Stage\Post\L10n\BodyGroup\Group();
            $group->setId(1);
            $customFieldVariant = $group->customFieldsVariants();
            $this->assertContainsOnly('\Stage\Post\L10n\BodyGroup\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupDetachCustomFieldVariant()
    {
        try {
            $group = new \Stage\Post\L10n\BodyGroup\Group();
            $group->setId(1);
            $status = $group->detachCustomFieldsVariant(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachPostBodyGroupMatrixCustomFieldVariant()
    {
        try {
            $group = new \Stage\Post\L10n\BodyGroup\Group();
            $idVariant = 4; // this id must exist in posts_l10n_body_groups_custom_fields_related
            $nameRow = "name1";
            $nameColumn = "namecol";
            $variantValue = 1;
            $positionRow = 2;
            $positionValue = 3;
            $customFieldVariant = $group->attachCustomFieldVariantMatrix($idVariant, $nameRow, $nameColumn, $variantValue, $positionRow, $positionValue);
            $this->assertInstanceOf('\Stage\Post\L10n\BodyGroup\CustomFieldVariantMatrix', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupMatrixCustomFieldVariants()
    {
        try {
            $group = new \Stage\Post\L10n\BodyGroup\Group();
            $idVariant = 4;
            $customFieldVariant = $group->customFieldsVariantsMatrix($idVariant);
            $this->assertContainsOnly('\Stage\Post\L10n\BodyGroup\CustomFieldVariantMatrix', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupMatrixDetachCustomFieldVariant()
    {
        try {
            $group = new \Stage\Post\L10n\BodyGroup\Group();
            $idVariant = 4;
            $status = $group->detachCustomFieldsVariantMatrix($idVariant);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostBodyGroupGetCustomFieldKeys()
    {
        $q = new \Stage\Post\L10n\BodyGroup\CustomFieldKeyQuery();
        $q->idType = 0;
        $result = \Stage\Post\L10n\BodyGroup\CustomFieldKey::query($q, true);
        $this->assertContainsOnly('\Stage\Post\L10n\BodyGroup\CustomFieldKey', $result);
    }

    public function testPostTypeColor()
    {
        try {
            $idPostType = 1;
            $types = \Stage\Post\Post::typeColors($idPostType);
            $this->assertContainsOnlyInstancesOf('\Stage\Post\TypeColor', $types);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostIdsRelated()
    {
        try {
            $ids = "16,17";
            $result = \Stage\Post\Post::idsRelated($ids);
            foreach ($result as $item) {
                $this->assertTrue(isset($item->idPost));
                $this->assertTrue(isset($item->idType));
                $this->assertTrue(isset($item->idDossier));
            }
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachPostL10nCustomFieldVariantMatrix()
    {
        try {
            /** @var \Stage\Post\L10n\L10n $l10n */
            $l10n = \Stage\Post\L10n\L10n::find(17,"pt");

            $idVariant = 9; // this id must exist in posts_l10n_body_groups_custom_fields_related
            $nameRow = "name1";
            $nameColumn = "namecol";
            $variantValue = 1;
            $positionRow = 2;
            $positionValue = 3;
            $customFieldVariant = $l10n->attachCustomFieldVariantMatrix($idVariant, $nameRow, $nameColumn, $variantValue, $positionRow, $positionValue);
            $this->assertInstanceOf('\Stage\Post\L10n\CustomFieldVariantMatrix', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostL10nCustomFieldVariantsMatrix()
    {
        try {
            /** @var \Stage\Post\L10n\L10n $l10n */
            $l10n = \Stage\Post\L10n\L10n::find(17,"pt");
            $idVariant = 9;
            $customFieldVariant = $l10n->customFieldsVariantsMatrix($idVariant);
            $this->assertContainsOnly('\Stage\Post\L10n\CustomFieldVariantMatrix', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostL10nDetachCustomFieldVariantMatrix()
    {
        try {
            /** @var \Stage\Post\L10n\L10n $l10n */
            $l10n = \Stage\Post\L10n\L10n::find(17,"pt");
            $idVariant = 9;
            $status = $l10n->detachCustomFieldsVariantMatrix($idVariant);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }
}
