<?php

use Stage\Account\Account;
use Stage\Asset\Album\Album;
use Stage\Asset\Asset;
use Stage\Post\Sticker\MenuItem\Item;
use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class StickersTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testStickersGroupCreate()
    {
        $group = new \Stage\Post\Sticker\Group\Group();
//        $group->setType(0);
        $group->setName("name");
        $group->setNameStripped("name stripped");
        $group->setToken("token");
        $group->setMetaUrl("meta_url");
        $group->setPriority(1);
        $group->setActive(true);
        $group->save();
    }

    public function testStickersGroupSetCountry()
    {
        $group = \Stage\Post\Sticker\Group\Group::find(1);
        $group->attachCountry("pt", true);
    }

    public function testStickersGroupDetachAllCountries()
    {
        $group = \Stage\Post\Sticker\Group\Group::find(1);
        $group->detachAllCountries();
    }

    public function testStickersGroupGetCountries()
    {
        $group = \Stage\Post\Sticker\Group\Group::find(1);
        $countries = $group->countries();
        print_r($countries->toArray());
    }

    public function testStickersGroupDelete()
    {
        $group = \Stage\Post\Sticker\Group\Group::find(3);
        $status = $group->delete();
        $this->assertTrue($status);
    }

    public function testStickersAttachOption()
    {
        $sticker = \Stage\Post\Sticker\Sticker::find(1);
        $sticker->attachOption(1);
    }

    public function testStickersGetFormats()
    {
        $formats = \Stage\Post\Sticker\Sticker::formats(1);
        $this->assertContainsOnly('\Stage\Post\Sticker\Format', $formats);
//        print_r($formats->toArray());
    }

    public function testStickersGetOptions()
    {
        $options = \Stage\Post\Sticker\Sticker::options(1);
        print_r($options->toArray());
    }

    public function testStickersGroupFind()
    {
        $group = \Stage\Post\Sticker\Group\Group::find(1);
        $this->assertInstanceOf('\Stage\Post\Sticker\Group\Group', $group);
//        print_r($group->toArray());
    }

    public function testStickersGroupQuery()
    {
        $q = new \Stage\Post\Sticker\Group\Query();
        $results = \Stage\Post\Sticker\Group\Group::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Sticker\Group\Group', $results);
//        print_r($results->toArray());
    }

    public function testStickerCreate()
    {
        $sticker = new \Stage\Post\Sticker\Sticker();
        $sticker->setIdGroup(1);
        $sticker->setIdType(1);
        $sticker->setPosition(1);
        $sticker->setName("name");
        $sticker->setNameStripped("name stripped");
        $sticker->setToken("token");
        $sticker->setImage("");
        $sticker->setImageAlign("");
        $sticker->setImageWidth("");
        $sticker->setMaximumItems(2);
        $sticker->setNumberOfRows(2);
        $sticker->setDisplayIn("mobile");
        $sticker->setIdFormat(1);
        $sticker->setLink("link");
        $sticker->setAutoFeed(true);
        $sticker->setAutoFeedOrder("order");
        $sticker->setAutoFeedType(1);
        $sticker->setColor("red");
        $sticker->setActive(true);
        $sticker->save();
    }

    public function testStickerAttachToPost()
    {
        $post = \Stage\Post\Post::find(11);
        $now = new DateTime();
        $result = $post->attachSticker(1, 1, $now, $now);
        var_dump($result);
    }

    public function testStickerDetachFromPost()
    {
        $post = \Stage\Post\Post::find(11);
        $post->detachSticker(1);
    }

    public function testStickerFind()
    {
        $sticker = \Stage\Post\Sticker\Sticker::find(1);
        $this->assertInstanceOf('\Stage\Post\Sticker\Sticker', $sticker);
//        print_r($sticker->toArray());
    }

    public function testStickerQuery()
    {
        $q = new \Stage\Post\Sticker\Query();
        $data = \Stage\Post\Sticker\Sticker::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Sticker\Sticker', $data);
//        print_r($data->toArray());
    }

    public function testStickerTypes()
    {
        $data = \Stage\Post\Sticker\Sticker::types(1);
        print_r($data->toArray());
    }

    public function testStickersGroupUrlNum()
    {
        $data = \Stage\Post\Sticker\Group\Group::urlNum(1, 1, "", 1);
        $this->assertTrue(is_numeric($data));
    }

    public function testStickersMenuItemCreate()
    {
        $group = new Item();
        $group->setIdSticker(1);
        $group->setName("name");
        $group->setDescription("desc");
        $group->setItemType(1);
        $group->setItemAction(1);
        $group->setItemValue("value");
        $group->setImage("image.png");
        $group->setColor("#F00");
        $group->setTrackingParameters("&r=1");
        $group->setPosition(1);
        $group->appendTo(Item::ROOT);// Default append to root directory
    }

    public function testStickersMenuItemUpdate()
    {
        $group = new Item();
        $group->setIdSticker(1);
        $group->setIdItem(2);
        $group->setName("name_");
        $group->setDescription("desc");
        $group->setItemType(1);
        $group->setItemAction(1);
        $group->setItemValue("value");
        $group->setImage("image.png");
        $group->setColor("#F00");
        $group->setTrackingParameters("&r=1");
        $group->setPosition(1);
        $group->appendTo(Item::ROOT);// Default append to root directory
    }

    public function testStickersMenuItemFind()
    {
        $result = Item::find(1, 2);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Sticker\MenuItem\Item', $result);
    }

    public function testStickersMenuItemGetAll()
    {
        $result = Item::all(1);
        print_r($result);
    }

    public function testStickersMenuItemBuildTree()
    {
        $result = Item::build(1);
        print_r($result);
    }

    public function testParseIsoLanguageFromSlug()
    {
        $slug = "pt.Collection1|en.Collection1";
        $p1 = explode("|", $slug);

        $data = [];
        foreach ($p1 as $p) {
            $explode = explode(".", $p);
            $tmp['isoLanguage'] = $explode[0];
            $tmp['slug'] = $explode[1];
            $data[] = (object)$tmp;
        }
        print_r($data);
    }

    public function testStickersMenuItemBuildTreeFindNode()
    {
        $tree = Item::build(1);
        $result = Item::findNode($tree, 5);
        print_r($result);
    }

    public function testStickersMenuItemMarkedHasDeleted()
    {
        $result = Item::markedToDelete(1);
        print_r($result);
    }

    public function testStickersMenuItemSoftDeleted()
    {
        $result = Item::delete(1, 1, true);
        print_r($result);
    }

    public function testStickersMenuItemHardDeleted()
    {
        $result = Item::delete(1, 7, false);
        print_r($result);
    }

    public function testStickersSaveCustomFieldKey()
    {
        $customField = new \Stage\Post\Sticker\CustomFieldKey();
        $customField->setName("named");
        $customField->setNameStripped("named");
        $result = $customField->save();
        $this->assertInstanceOf('\Stage\Post\Sticker\CustomFieldKey', $result);
    }

    public function testStickersDeleteCustomFieldKey()
    {
        $idKey = 2;
        $customFieldKey = \Stage\Post\Sticker\CustomFieldKey::find($idKey);
        $result = $customFieldKey->delete();
        $this->assertTrue($result);
    }

    public function testStickersAttachCustomFieldVariant()
    {
        $idKey = 1;
        $customFieldKey = \Stage\Post\Sticker\CustomFieldKey::find($idKey);
        $sticker = \Stage\Post\Sticker\Sticker::find(1);
        $result = $sticker->attachCustomFieldVariant($customFieldKey->getId(), "value", 1);
        $this->assertInstanceOf('\Stage\Post\Sticker\CustomFieldVariant', $result);
    }

    public function testStickersDetachCustomFieldKey()
    {
        $idKey = 4;
        $customFieldKey = \Stage\Post\Sticker\CustomFieldKey::find($idKey);
        $sticker = \Stage\Post\Sticker\Sticker::find(1);
        $result = $sticker->detachCustomFieldsVariant($customFieldKey->getId());
        $this->assertTrue($result);
    }

    public function testStickerCustomFieldsQuery()
    {
        $q = new  \Stage\Post\Sticker\CustomFieldKeyQuery();
        $data = \Stage\Post\Sticker\CustomFieldKey::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Sticker\CustomFieldKey', $data);
    }

    public function testStickerCustomFieldsQueryRelated()
    {
        $sticker = \Stage\Post\Sticker\Sticker::find(1);
        $variants = $sticker->customFieldsVariants();
        $this->assertContainsOnlyInstancesOf('\Stage\Post\Sticker\CustomFieldVariant', $variants);
    }
}
