<?php

use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class VideoFeedsTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testVideoFeedInternalCreate()
    {
        $feed = new \Stage\Video\Feed\Internal();
        $feed->setName("name");
        $feed->setDomain("sapo.pt");
        $feed->setLength(2);
        $feed->setType(1);
        $feed->setCanOverwrite(true);
        $feed->setTrackingParameters('utm=1');
        $feed->setImageOptimization('q=5');
        $feed->setTitle('title');
        $feed->setDescription('desc');
        $feed->setImage('image');
        $feed->setCopyright('copy');
        $feed->setNumberOfBodies(10);

        try {
            $f = $feed->save();
            print_r($f->toArray());
            $this->assertTrue($f instanceof \Stage\Video\Feed\Internal);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoGetTotalNumberOfFeedsInternal()
    {
        try {
            $total = \Stage\Video\Feed\Internal::sizeof();
            $this->assertTrue($total > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedsQuery()
    {
        try {
            $q = new \Stage\Video\Feed\Query();
            $results = \Stage\Video\Feed\Internal::query($q);
            $this->assertContainsOnlyInstancesOf('\Stage\Video\Feed\Internal', $results);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedsFindOne()
    {
        try {
            $result = \Stage\Video\Feed\Internal::find(1);
            $this->assertInstanceOf('\Stage\Video\Feed\Internal', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedInternalDelete()
    {
        try {
            $feed = \Stage\Video\Feed\Internal::find(3);
            $status = $feed->delete();
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsOptions()
    {
        try {
            $result = \Stage\Video\Feed\Internal::options(1);
            $this->assertContainsOnly('\Stage\Video\Feed\Option', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedsAttachOption()
    {
        try {
            $result = \Stage\Video\Feed\Internal::attachOption(1, 1, '2');
            $this->assertInstanceOf('\Stage\Video\Feed\Option', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsDetachOption()
    {
        try {
            $result = \Stage\Video\Feed\Internal::detachOption(1, 2);
            $this->assertTrue($result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedsAttachSchema()
    {
        try {
            $schema = \Stage\Video\Feed\Internal::attachSchema(1, -1,"pt", "{{post.id}}");
            $this->assertTrue($schema instanceof \Stage\Video\Feed\Schema\Schema);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedsDetachSchemaFromFeed()
    {
        try {
            $status = \Stage\Video\Feed\Internal::detachSchema(1, 7);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsFeedsGetSchemas()
    {
        try {
            $schemas = \Stage\Video\Feed\Internal::schemas(1);
            $this->assertContainsOnly('\Stage\Video\Feed\Schema\Schema', $schemas);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedInternalClone()
    {
        try {
            $feedInternal = \Stage\Video\Feed\Internal::find(1);
            $feed = $feedInternal->duplicate();
            $this->assertInstanceOf('\Stage\Video\Feed\Internal', $feed);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideosFeedsSchemaOptions()
    {
        try {
            $result = \Stage\Video\Feed\Schema\Schema::options();
            $this->assertContainsOnly('\Stage\Video\Feed\Schema\Option', $result);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoFeedInternalAllOptions()
    {
        try {
            $options = \Stage\Video\Feed\Internal::allOptions();
            $this->assertContainsOnly('\Stage\Video\Feed\Option', $options);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }
}
