<?php

use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\Video\Message\Group\Schema;
use Stage\Video\Message\Message;
use Stage\Video\Message\Group\Group;
use Stage\Video\Message\CustomFieldKey;
use Stage\Video\Message\MessageVideoComposedQuery;
use Stage\Video\Message\Group\Query as GroupQuery;
use Stage\StoredProcedure\Manager as StoredProcedureManager;

class VideoMessageTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testVideoMessageGroupCreate()
    {
        try {
            $schema = new Group();
            $schema->setIdType(1);
            $schema->setName("name2");
            $schema->setNameStripped("name3");
            $schema->setDomain("domain");
            $schema->setTrackingParameters("tracking");
            $schema->save();
            $this->assertInstanceOf('\Stage\Video\Message\Group\Group', $schema);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoMessageGroupFind()
    {
        $data = Group::find(1);
        $this->assertInstanceOf('\Stage\Video\Message\Group\Group', $data);
    }

    public function testVideoMessageGroupQuery()
    {
        $q = new GroupQuery();
        $q->idType = 1;
        $data = Group::query($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\Group\Group', $data);
    }

    public function testAttachVideoMessageGroupCustomFieldVariant()
    {
        try {
            $group = Group::find(1);
            $idKey = 1; // it's the id key from table posts_messages_custom_fields_keys
            $customFieldVariant = $group->attachCustomFieldVariant($idKey, "name1", 1);
            $this->assertInstanceOf('\Stage\Video\Message\Group\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachVideoMessageGroupCustomFieldVariant()
    {
        try {
            $group = Group::find(1);
            $status = $group->detachCustomFieldsVariant(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testQueryVideoMessageGroupCustomFieldVariants()
    {
        try {
            $group = Group::find(1);
            $data = $group->customFieldsVariants(1);
            $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\Group\CustomFieldVariant', $data);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoMessageGroupQueryVariantKeys()
    {
        $q = new \Stage\Video\Message\Group\Query();
        $data = Group::variantKeys($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\CustomFieldKey', $data);
    }

    public function testVideoMessageGroupFindVariantKey()
    {
        try {
            $idKey = 1;
            $customVariantKey = Group::variantKey($idKey);
            $this->assertInstanceOf('\Stage\Video\Message\CustomFieldKey', $customVariantKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachVideoMessageGroupVariantKey()
    {
        try {
            $idKey = 5;
            $status = Group::detachVariantKey($idKey);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachVideoMessageGroupVariantKey()
    {
        try {
            $name = 'name';
            $nameStripped = 'name_s';
            $public = true;
            $ifExists = 0;
            $idKey = -1; // it's the id key from table posts_messages_custom_fields_keys, or -1 to create
            $customFieldVariant = Group::attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists);
            $this->assertInstanceOf('\Stage\Video\Message\CustomFieldKey', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testPostsMessageGroupSchemaCreate()
    {
        $idGroup = 1;
        $isoLanguage = 'pt';
        $macroSchema = "%a%b";

        try {
            $schema = new Schema();
            $schema->setIdGroup($idGroup);
            $schema->setIsoLanguage($isoLanguage);
            $schema->setMacroSchema($macroSchema);
            $schema->save();
            $this->assertInstanceOf('\Stage\Video\Message\Group\Schema', $schema);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoMessageGroupSchemaDelete()
    {
        try {
            $status = Schema::delete(1,4);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoMessageGroupSchemas()
    {
        try {
            $schemas = Schema::find(1,0);
            $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\Group\Schema', $schemas);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDeleteVideoMessageGroup()
    {
        $message = Group::find(4);
        $status = $message->delete();
        $this->assertTrue($status);
    }

    public function testCreateVideoMessage()
    {
        $now = new DateTime();
        $message = new Message();
        $message->setIdGroup(1);
        $message->setIdType(1);
        $message->setIsoLanguage("pt");
        $message->setManagementName("managment_name");
        $message->setTitle("title");
        $message->setMessage("message");
        $message->setImage("image.png");
        $message->setLink("link");
        $message->setBeginDate($now);
        $result = $message->save();

        $this->assertInstanceOf('\Stage\Video\Message\Message', $result);
        $this->assertTrue($result->getId() > 0);
    }

    public function testAttachVideoComposeToMessage()
    {
        $message = Message::find(1);
        $this->assertInstanceOf('\Stage\Video\Message\Message', $message);

        $status = $message->attachVideoComposed(14);
        $this->assertTrue($status);
    }

    public function testVideoMessagesSizeOf()
    {
        $total = Message::sizeof();
        $this->assertTrue($total > 0);
    }

    public function testVideoMessagesTypes()
    {
        $field = 1;
        $filter = 1;
        $types = Message::types($field, $filter);
        $this->assertTrue($types->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\Type', $types);
    }

    public function testVideoComposedMessageQuery()
    {
        $q = new \Stage\Video\Message\Query();
        $q->idGroup = 1;
        $q->idType = 1;
        $data = Message::query($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\Message', $data);
    }

    public function testVideoMessageGetVideoComposedAssociated()
    {
        $q = new \Stage\Video\Message\MessageVideoComposedQuery();
        $q->idMessage = 1;
        $data = Message::videosComposed($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\MessageVideoComposed', $data);
    }

    public function testDeleteVideoComposedMessage()
    {
        $message = Message::find(8);
        $status = $message->delete();
        $this->assertTrue($status);
    }

    public function testAttachVideoComposedMessageCustomFieldVariant()
    {
        try {
            $message = Message::find(1);
            $idKey = 1; // it's the id key from table video_messages_custom_fields_keys
            $customFieldVariant = $message->attachCustomFieldVariant($idKey, "name1", 1);
            $this->assertInstanceOf('\Stage\Video\Message\CustomFieldVariant', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testQueryVideoComposedMessageCustomFieldVariants()
    {
        try {
            $message = Message::find(1);
            $customFieldVariant = $message->customFieldsVariants();
            $this->assertContainsOnly('\Stage\Video\Message\CustomFieldVariant', $customFieldVariant);
            $this->assertTrue($customFieldVariant->count() > 0);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachVideoComposedMessageCustomFieldVariant()
    {
        try {
            $message = Message::find(1);
            $status = $message->detachCustomFieldsVariant(1);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testAttachVideoComposedMessageVariantKey()
    {
        try {
            $name = 'name';
            $nameStripped = 'name_s';
            $public = true;
            $ifExists = 0;
            $idKey = -1; // it's the id key from table posts_messages_custom_fields_keys, or -1 to create
            $customFieldVariant = Message::attachVariantKey($idKey, $name, $nameStripped, $public, $ifExists);
            $this->assertInstanceOf('\Stage\Video\Message\CustomFieldKey', $customFieldVariant);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testDetachVideoComposedMessageVariantKey()
    {
        try {
            $idKey = 4;
            $status = Message::detachVariantKey($idKey);
            $this->assertTrue($status);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoComposedMessageFindVariantKey()
    {
        try {
            $idKey = 1;
            $customVariantKey = Message::variantKey($idKey);
            $this->assertInstanceOf('\Stage\Video\Message\CustomFieldKey', $customVariantKey);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function testVideoComposedMessageQueryVariantKeys()
    {
        $q = new \Stage\Video\Message\Query();
        $data = Message::variantKeys($q, true);
        $this->assertTrue($data->count() > 0);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\CustomFieldKey', $data);
    }

    public function testVideoComposedMessageQueryAllVariantKeys()
    {
        $data = CustomFieldKey::all();
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Message\CustomFieldKey', $data);
    }

    public function testVideoComposedMessageDuplicate()
    {
        $message = Message::find(1)->duplicate();
        $this->assertInstanceOf('\Stage\Video\Message\Message', $message);
    }

}
