<?php

use Stage\Sps;
use Stage\Stage;
use Service\Connection;
use Stage\Account\Account;
use Stage\StoredProcedure\Manager as StoredProcedureManager;
use Stage\Video\Sticker\MenuItem\Item;

class VideoStickersTest extends \PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        $stageInstance = Stage::getInstance();

        $credentials = [
            'host' => "mysqldb",
            'dbname' => "StageAccount",
            'username' => "root",
            'password' => "secret"
        ];

        $clientCredentials = [
            'host' => "mysqldb",
            'dbname' => "bd1",
            'username' => "root",
            'password' => "secret"
        ];

        $tasksCredentials = [
            'host' => "mysqldb",
            'dbname' => "SkoiyTasks",
            'username' => "root",
            'password' => "secret"
        ];

        $default = new Connection($credentials);
        $client = new Connection($clientCredentials);
        $tasks = new Connection($tasksCredentials);
        $stageInstance->addConnection('default', $default);
        $stageInstance->addConnection('client', $client);
        $stageInstance->addConnection('tasks', $tasks);

        // 3. Set the Stage stored procedures
        $stageInstance->setSps(Sps::load());

        // Set the StoredProcedure\Manager instance
        $spm = StoredProcedureManager::getInstance();

        // Set the Stored Procedure Manager
        $stageInstance->setStoredProcedureManager($spm);

        try {
            $idAccount = 3;
            $account = Account::find($idAccount);

            // Get a Stage instance
            $stageInstance->setAccount($account);
        } catch (Exception $e) {
            echo $e->getMessage();
        }
    }

    public function tearDown()
    {
    }

    public function testVideoStickersGroupCreate()
    {
        $group = new \Stage\Video\Sticker\Group\Group();
//        $group->setType(0);
        $group->setName("name");
        $group->setNameStripped("name stripped");
        $group->setToken("token");
        $group->setMetaUrl("meta_url");
        $group->setPriority(1);
        $group->setActive(true);
        $group->setStickerType(\Stage\Video\Sticker\Group\Group::TYPE_CONTENT);
        $group->save();

        $this->assertInstanceOf('\Stage\Video\Sticker\Group\Group', $group);
        $this->assertTrue($group->getId() > 0);
    }

    public function testVideoStickersGroupFind()
    {
        $group = \Stage\Video\Sticker\Group\Group::find(1);
        $this->assertInstanceOf('\Stage\Video\Sticker\Group\Group', $group);
    }

    public function testVideoStickersGroupSetCountry()
    {
        $group = \Stage\Video\Sticker\Group\Group::find(1);
        $group->attachCountry("pt", true);
    }

    public function testVideoStickersGroupDetachAllCountries()
    {
        $group = \Stage\Video\Sticker\Group\Group::find(1);
        $group->detachAllCountries();
    }

    public function testVideoStickersGroupGetCountries()
    {
        $group = \Stage\Video\Sticker\Group\Group::find(1);
        $countries = $group->countries();
        print_r($countries->toArray());
    }

    public function testVideoStickersGroupQuery()
    {
        $q = new \Stage\Video\Sticker\Group\Query();
        $results = \Stage\Video\Sticker\Group\Group::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\Group\Group', $results);
//        print_r($results->toArray());
    }

    public function testVideoStickersGroupDelete()
    {
        $group = \Stage\Video\Sticker\Group\Group::find(3);
        $status = $group->delete();
        $this->assertTrue($status);
    }

    public function testVideoStickerTypes()
    {
        $data = \Stage\Video\Sticker\Sticker::types(1);
        print_r($data->toArray());
    }

    public function testVideoStickersGroupUrlNum()
    {
        $data = \Stage\Video\Sticker\Group\Group::urlNum(1, 1, "", 1);
        $this->assertTrue(is_numeric($data));
    }

    public function testVideoStickersGetFormats()
    {
        $formats = \Stage\Video\Sticker\Sticker::formats(1);
        $this->assertContainsOnly('\Stage\Video\Sticker\Format', $formats);
//        print_r($formats->toArray());
    }

    public function testVideoStickerCreate()
    {
        $sticker = new \Stage\Video\Sticker\Sticker();
        $sticker->setIdGroup(1);
        $sticker->setIdType(1);
        $sticker->setPosition(1);
        $sticker->setName("name");
        $sticker->setNameStripped("name stripped");
        $sticker->setToken("token");
        $sticker->setImage("");
        $sticker->setImageAlign("");
        $sticker->setImageWidth("");
        $sticker->setMaximumItems(2);
        $sticker->setNumberOfRows(2);
        $sticker->setDisplayIn("mobile");
        $sticker->setIdFormat(1);
        $sticker->setLink("link");
        $sticker->setAutoFeedEnabled(true);
        $sticker->setAutoFeedSourceType(1);
        $sticker->setAutoFeedSort(1);
        $sticker->setColor("red");
        $sticker->setActive(true);
        $sticker->save();
    }

    public function testVideoStickerQuery()
    {
        $q = new  \Stage\Video\Sticker\Query();
        $data = \Stage\Video\Sticker\Sticker::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\Sticker', $data);
    }

    public function testVideoStickerDelete()
    {
        $sticker = \Stage\Video\Sticker\Sticker::find(3);
        $status = $sticker->delete();
        $this->assertTrue($status);
    }

    public function testVideoStickerAttachToVideoCompose()
    {
        $video = \Stage\Video\Compose\Compose::find(14);
        $now = new DateTime();
        $result = $video->attachSticker(1, 1, $now, $now);
        $this->assertTrue($result);
    }

    public function testVideoStickerDetachFromVideoCompose()
    {
        $video = \Stage\Video\Compose\Compose::find(14);
        $status = $video->detachSticker(1);
        $this->assertTrue($status);
    }

    public function testVideoStickersGetComposeStickers()
    {
        $compose = \Stage\Video\Compose\Compose::find(14);
        // Get all the Compose stickers
        $stickers = $compose->stickers();
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\Compose', $stickers);
    }

    public function testVideoStickerAttachToPlaylist()
    {
        $playlist = \Stage\Video\Playlist\Playlist::find(4);
        $now = new DateTime();
        $result = $playlist->attachSticker(1, 1, $now, $now);
        $this->assertTrue($result);
    }

    public function testVideoStickerDetachFromPlaylist()
    {
        $playlist = \Stage\Video\Playlist\Playlist::find(4);
        $status = $playlist->detachSticker(1);
        $this->assertTrue($status);
    }

    public function testVideoStickersGetPlaylistStickers()
    {
        $playlist = \Stage\Video\Playlist\Playlist::find(4);
        // Get all the Compose stickers
        $stickers = $playlist->stickers();
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\Playlist', $stickers);
    }

    public function testVideoStickerAttachToChannel()
    {
        $playlist = \Stage\Video\Channel\Channel::find(4);
        $now = new DateTime();
        $result = $playlist->attachSticker(1, 1, $now, $now);
        $this->assertTrue($result);
    }

    public function testVideoStickerDetachFromChannel()
    {
        $playlist = \Stage\Video\Channel\Channel::find(4);
        $status = $playlist->detachSticker(1);
        $this->assertTrue($status);
    }

    public function testVideoStickersGetChannelStickers()
    {
        $channel = \Stage\Video\Channel\Channel::find(4);
        // Get all the Channel stickers
        $stickers = $channel->stickers();
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\Channel', $stickers);
    }

    public function testVideoStickersSaveCustomFieldKey()
    {
        $customField = new \Stage\Video\Sticker\CustomFieldKey();
        $customField->setName("named");
        $customField->setNameStripped("named");
        $result = $customField->save();
        $this->assertInstanceOf('\Stage\Video\Sticker\CustomFieldKey', $result);
    }

    public function testVideoStickersDeleteCustomFieldKey()
    {
        $idKey = 5;
        $customFieldKey = \Stage\Video\Sticker\CustomFieldKey::find($idKey);
        $result = $customFieldKey->delete();
        $this->assertTrue($result);
    }

    public function testVideoStickerCustomFieldsQuery()
    {
        $q = new  \Stage\Video\Sticker\CustomFieldKeyQuery();
        $data = \Stage\Video\Sticker\CustomFieldKey::query($q, true);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\CustomFieldKey', $data);
    }

    public function testVideoStickersAttachCustomFieldVariant()
    {
        $idKey = 1;
        $customFieldKey = \Stage\Video\Sticker\CustomFieldKey::find($idKey);
        $sticker = \Stage\Video\Sticker\Sticker::find(1);
        $result = $sticker->attachCustomFieldVariant($customFieldKey->getId(), "value", 1);
        $this->assertInstanceOf('\Stage\Video\Sticker\CustomFieldVariant', $result);
    }

    public function testVideoStickersDetachCustomFieldKey()
    {
        $idKey = 4;
        $customFieldKey = \Stage\Post\Sticker\CustomFieldKey::find($idKey);
        $sticker = \Stage\Post\Sticker\Sticker::find(1);
        $result = $sticker->detachCustomFieldsVariant($customFieldKey->getId());
        $this->assertTrue($result);
    }

    public function testVideoStickersGetStickerCustomFieldsVariants()
    {
        $sticker = \Stage\Post\Sticker\Sticker::find(1);
        $result = $sticker->customFieldsVariants();
        $this->assertInstanceOf('\Stage\Post\MyArrayObject', $result);
        foreach ($result as $customFieldVariant){
            $this->assertInstanceOf('\Stage\Post\Sticker\CustomFieldVariant', $customFieldVariant);
        }
    }

    public function testVideoStickerAttachFeedSource()
    {
        $sticker = \Stage\Video\Sticker\Sticker::find(2);
        $status = $sticker->attachFeedSource(1,1);
        $this->assertTrue($status);
    }

    public function testVideoStickerGetFeedSources()
    {
        $sticker = \Stage\Video\Sticker\Sticker::find(2);
        $data = $sticker->feedsSources();
        $this->assertTrue($data instanceof ArrayObject);
    }

    public function testVideoStickerDetachAllFeedSources()
    {
        $sticker = \Stage\Video\Sticker\Sticker::find(2);
        $status = $sticker->detachAllFeedsSources();
        $this->assertTrue($status);
    }

    public function testVideoStickersMenuItemCreate()
    {
        $group = new Item();
        $group->setIdSticker(1);
        $group->setName("name");
        $group->setDescription("desc");
        $group->setItemType(1);
        $group->setItemAction(1);
        $group->setItemValue("value");
        $group->setImage("image.png");
        $group->setColor("#F00");
        $group->setTrackingParameters("&r=1");
        $group->setPosition(1);
        $group->appendTo(Item::ROOT);// Default append to root directory
    }

    public function testVideoStickersMenuItemUpdate()
    {
        $group = new Item();
        $group->setIdSticker(1);
        $group->setIdItem(2);
        $group->setName("name_");
        $group->setDescription("desc");
        $group->setItemType(1);
        $group->setItemAction(1);
        $group->setItemValue("value");
        $group->setImage("image.png");
        $group->setColor("#F00");
        $group->setTrackingParameters("&r=1");
        $group->setPosition(1);
        $group->appendTo(Item::ROOT);// Default append to root directory
    }

    public function testVideoStickersMenuItemFind()
    {
        $result = Item::find(1, 2);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\MenuItem\Item', $result);
    }

    public function testVideoStickersMenuItemGetAll()
    {
        $result = Item::all(1);
        print_r($result);
    }

    public function testVideoStickersMenuItemBuildTree()
    {
        $result = Item::build(1);
        print_r($result);
    }

    public function testSupportVideoChannelStickers()
    {
        $result = \Stage\Support\Video::stickersInChannel(1);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\Sticker', $result);
    }

    public function testSupportVideoPlaylistsStickers()
    {
        $result = \Stage\Support\Video::stickersInPlaylist(1);
        $this->assertContainsOnlyInstancesOf('\Stage\Video\Sticker\Sticker', $result);
    }
}
